% Here the consistency rules for the DEFT repository are defined.
% Before reasoning, some content of the DEFT repository is added to the prolog fact base. The corresponding code is in TuPrologConstraintsManager.
%
% All projects:					project(projectId, projectName)
% All fragments: 				fragment(fragmentId, fragmentName, fragmentType)
% Parent-Child relationship:	parent(parentFragmentId, childFragmentId)  	(note: parentFragment is a folder, but this is not asserted with an explicit rule)
% Project affiliation:			belongsToProject(fragmentId, projectId)
% Folder type					folderType(folderId, folderType)			(folderType parameter determines what kind of fragment a folder can contain, e.g. CodeFiles)
% Tutorial-Chapter relation		inTutorial(tutorialId, chapterId)



%Helper rules
%============

ancestor(X, Y) :- parent(X, Y).
ancestor(X, Z) :- parent(X, Y), ancestor(Y, Z).



%The rules to be used from outside
%=================================

%%%%%
% Checks if a project with a given name exists.
%%%%%
projectExists(NAME) :- project(_, NAME).


%%%%%
% Checks if a chapter can be added to a tutorial. A chapter can be added
% to a tutorial, if both are from the same project, and if the chapter
% has not already been added to the tutorial.
%%%%%
canContainChapter(TUT_ID, CHAP_ID) :- belongsToProject(TUT_ID, X), belongsToProject(CHAP_ID, X),
		fragment(TUT_ID, _, tutorial), fragment(CHAP_ID, _, chapter),
		not(containsChapter(TUT_ID, CHAP_ID)).



%%%%%
% Checks if a Folder can contain a specific fragment.
% A folder can only contain fragments from the same project.
% A folder can contain a fragment with a given name if it does not
% already have a fragment with that name, or if multiple fragments
% with the same name are allowed (according to rules of canContainFragmentName).
% A folder can contain a fragment if their types match (according to rules of
% canContainFragmentType).
% The fragment may not already be child of the folder.
%
% If child is a folder, it must have the same type as parent, and it must not currently be
% an ancestor of parent or even the same as parent (i.e. you can't move a folder into
% itself or one of its child folders).
%%%%%
canContainFragment(FOLD_ID, FRAG_ID) :- belongsToProject(FOLD_ID, FOLDPROJ_ID), %same project
										belongsToProject(FRAG_ID, FRAGPROJ_ID),
										FOLDPROJ_ID = FRAGPROJ_ID,
										not(parent(FOLD_ID, FRAG_ID)),			%fragment not already child of folder
										fragment(FOLD_ID, FOLD_NAME, folder),   %name and type compatible
										fragment(FRAG_ID, FRAG_NAME, FRAG_TYPE),
										canContainFragmentName(FOLD_ID, FRAG_NAME),
										canContainFragmentType(FOLD_ID, FRAG_TYPE),
										frag_type \= folder.					%fragment is no folder
																				%special case, handled
																				%individually, see below
canContainFragment(FOLD_ID, FRAG_ID) :- belongsToProject(FOLD_ID, FOLDPROJ_ID), %same project
										belongsToProject(FRAG_ID, FRAGPROJ_ID),
										FOLDPROJ_ID = FRAGPROJ_ID,
										not(parent(FOLD_ID, FRAG_ID)),			%fragment not already child of folder
										fragment(FOLD_ID, FOLD_NAME, folder),   %name and type compatible
										fragment(FRAG_ID, FRAG_NAME, folder),   %fragment is also a folder
										FOLD_ID \= FRAG_ID,						%folder and fragment are not the same
										not(ancestor(FRAG_ID, FOLD_ID)),
										folderType(FRAG_ID,FRAG_TYPEASSOC),
										canContainFragmentType(FOLD_ID, FRAG_TYPEASSOC),
										canContainFragmentName(FOLD_ID, FRAG_NAME).


canContainFragmentAtRoot(FRAG_ID) :-	belongsToProject(FRAG_ID, FRAGPROJ_ID),
										fragment(FRAG_ID, FRAG_NAME, FRAG_TYPE),
										FRAG_TYPE \= folder,
										canContainFragmentNameAtRoot(FRAGPROJ_ID, FRAG_TYPE, FRAG_NAME).
canContainFragmentAtRoot(FRAG_ID) :-	belongsToProject(FRAG_ID, FRAGPROJ_ID),
										fragment(FRAG_ID, FRAG_NAME, folder),
										folderType(FRAG_ID, FRAG_TYPEASSOC),
										canContainFragmentNameAtRoot(FRAGPROJ_ID, FRAG_TYPEASSOC, FRAG_NAME).


canContainFragmentAtRoot(PROJ_ID, FRAG_TYPE, FRAG_ID) :-		belongsToProject(FRAG_ID, PROJ_ID),
																fragment(FRAG_ID, FRAG_NAME, FRAG_TYPE),
																FRAG_TYPE \= folder,
																canContainFragmentNameAtRoot(PROJ_ID, FRAG_TYPE, FRAG_NAME).
canContainFragmentAtRoot(PROJ_ID, FRAG_TYPEASSOC, FRAG_ID) :-	belongsToProject(FRAG_ID, PROJ_ID),
																fragment(FRAG_ID, FRAG_NAME, folder),
																folderType(FRAG_ID, FRAG_TYPEASSOC),
																canContainFragmentNameAtRoot(PROJ_ID, FRAG_TYPEASSOC, FRAG_NAME).									
										

%%%%%
% Checks if a fragment with a given name can be added to a folder.
% A fragment can only be added if the folder is not a codesnippet folder (codesnippet folders are not supported atm)
% and if the folder does not already contain a fragment with that name.
%%%%%
canContainFragmentName(FOLD_ID, FRAG_NAME) :- not(folderType(FOLD_ID, codesnippet)), not(containsFragmentName(FOLD_ID, FRAG_NAME)).



%%%%%
% Checks if a project can contain a fragment of the given name at the root.
% A project can contain a fragment at the root if there is not already one of the same type and name, or if
% it is a codeSnippet (no matter if there already are codesnippets with that name).
%%%%%
canContainFragmentNameAtRoot(PROJ_ID, FRAG_TYPE, FRAG_NAME) :- not(containsFragmentNameAtRoot(PROJ_ID, FRAG_TYPE, FRAG_NAME)).
canContainFragmentNameAtRoot(_, codesnippet, _).


%%%%%
% Checks if a folder can contain a fragment of a given type.
% A folder can contain a fragment, if their types match.
%%%%%
canContainFragmentType(FOLDER_ID, FRAG_TYPE) :- folderType(FOLDER_ID, FRAG_TYPE).


containsChapter(TUT_ID, CHAP_ID) :- inTutorial(TUT_ID, CHAP_ID).

%%%%%
% Checks if there is a fragment with a given name in a folder.
%%%%%
containsFragmentName(FOLD_ID, FRAG_NAME) :- parent(FOLD_ID, FRAG_ID), fragment(FRAG_ID, FRAG_NAME, _).



%%%%%
% Checks if there is a fragment with a given type and name at the root of the project. For
% example, if the fragment type is codefile, all codefiles (first rule) and all codefile folders
% (second rule) of the project are returned.
%%%%%
containsFragmentNameAtRoot(PROJ_ID, FRAG_TYPE, FRAG_NAME) :- fragment(FRAG_ID, FRAG_NAME, FRAG_TYPE),
		not(parent(_, FRAG_ID)),
		belongsToProject(FRAG_ID, PROJ_ID),
		FRAG_TYPE \= folder.
containsFragmentNameAtRoot(PROJ_ID, FRAG_TYPE, FRAG_NAME) :- fragment(FOLDER_ID, FRAG_NAME, folder),
		not(parent(_, FOLDER_ID)),
		folderType(FOLDER_ID, FRAG_TYPE),
		belongsToProject(FOLDER_ID, PROJ_ID).
