/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.internal.ui.operations;

import ag.ion.bion.workbench.office.editor.ui.wizards.document.INewDocumentRequest;

import ag.ion.bion.officelayer.application.IOfficeApplication;

import ag.ion.bion.officelayer.document.DocumentDescriptor;
import ag.ion.bion.officelayer.document.IDocument;

import org.eclipse.jface.operation.IRunnableWithProgress;

import org.eclipse.core.resources.IFile;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;

import java.lang.reflect.InvocationTargetException;

/**
 * Operation in order to construct a new OpenOffice.org document.
 * 
 * @author Andreas Brker
 * @version $Revision$
 */
public class ConstructNewDocumentOperation implements IRunnableWithProgress {
   
  private IOfficeApplication  officeApplication     = null;
  private INewDocumentRequest newDocumentRequest    = null;
  
  private InternalThread internalThread = null;
  
  private IProgressMonitor progressMonitor = null;
     
  //----------------------------------------------------------------------------
  /**
   * Internal thread class in order to construct a OpenOffice.org document.
   * 
   * @author Andreas Brker
   */
  private class InternalThread extends Thread {
    
    private IFile     file      = null;
    private Exception exception = null;
    
    private boolean done = false;
    
    //----------------------------------------------------------------------------
    /**
     * Executes thread logic.
     * 
     * @author Andreas Brker
     */
    public void run() {
      try {
        progressMonitor.subTask(Messages.ConstructNewDocumentOperation_monitor_building_message);
        progressMonitor.worked(1);
        file = newDocumentRequest.getTargetContainer().getFile(new Path(newDocumentRequest.getDocumentName()));  
        DocumentDescriptor documentDescriptor = new DocumentDescriptor();
        documentDescriptor.setHidden(true);
        IDocument document = null;
        
        progressMonitor.worked(1);
        
        progressMonitor.subTask(Messages.ConstructNewDocumentOperation_monitor_building_interface_message);
        if(newDocumentRequest.getTemplatePath() == null)
        	document = officeApplication.getDocumentService().constructNewDocument(newDocumentRequest.getDocumentType(), documentDescriptor);
        else
        	document = officeApplication.getDocumentService().loadDocument(newDocumentRequest.getTemplatePath(), documentDescriptor);
        progressMonitor.worked(1);
        
        progressMonitor.subTask(Messages.ConstructNewDocumentOperation_monitor_storing_message);
        document.getPersistenceService().store(file.getLocation().toOSString());
        document.close();
        
        progressMonitor.worked(1);
        done = true;
      }
      catch(Exception exception) {
        this.exception = exception;
      }
    }
    //----------------------------------------------------------------------------
    /**
     * Returns constructed document file. Returns null if a document file
     * is not available.
     * 
     * @return constructed document file or null if a document file
     * is not available
     * 
     * @author Andreas Brker
     */
    public IFile getFile() {
      return file;
    }
    //----------------------------------------------------------------------------
    /**
     * Returns exception. Returns null if no exception was thrown.
     * 
     * @return exception or null if no exception was thrown
     * 
     * @author Andreas Brker
     */
    public Exception getException() {
      return exception;
    }
    //----------------------------------------------------------------------------
    /**
     * Returns information whether the thread has finished his
     * work.
     * 
     * @return information whether the thread has finished his
     * work
     * 
     * @author Andreas Brker
     */
    public boolean done() {
      if(exception != null) 
        return true;
      return done;
    }
    //----------------------------------------------------------------------------
  }
  //----------------------------------------------------------------------------
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new ConstructNewDocumentOperation.
   * 
   * @param officeApplication office application to be used
   * @param newDocumentRequest new document request to be used
   * 
   * @throws IllegalArgumentException if the submitted office application or new document
   *  request is not valid
   * 
   * @author Andreas Brker
   */
  public ConstructNewDocumentOperation(IOfficeApplication officeApplication, INewDocumentRequest newDocumentRequest) throws IllegalArgumentException {
    if(officeApplication == null)
      throw new IllegalArgumentException(Messages.ConstructNewDocumentOperation_exception_application_invalid_message);
    this.officeApplication = officeApplication;
    
    if(newDocumentRequest == null)
      throw new IllegalArgumentException(Messages.ConstructNewDocumentOperation_exception_document_request_invalid_message);
    this.newDocumentRequest = newDocumentRequest;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns workpace file reference to the new constructed
   * document. Returns null if a workspace file reference is not available.
   * 
   * @return workpace file reference to the new constructed
   * document or null if a workspace file reference is not available
   * 
   * @author Andreas Brker
   */
  public IFile getDocumentFile() {
    return internalThread.getFile();
  }  
  //----------------------------------------------------------------------------
  /**
   * Runs this operation.  Progress should be reported to the given progress monitor.
   * This method is usually invoked by an <code>IRunnableContext</code>'s <code>run</code> method,
   * which supplies the progress monitor.
   * A request to cancel the operation should be honored and acknowledged 
   * by throwing <code>InterruptedException</code>.
   *
   * @param progressMonitor the progress monitor to use to display progress and receive
   *   requests for cancelation
   *   
   * @exception InvocationTargetException if the run method must propagate a checked exception,
   *  it should wrap it inside an <code>InvocationTargetException</code>; runtime exceptions are automatically
   *  wrapped in an <code>InvocationTargetException</code> by the calling context
   * @exception InterruptedException if the operation detects a request to cancel, 
   *  using <code>IProgressMonitor.isCanceled()</code>, it should exit by throwing 
   *  <code>InterruptedException</code>
   * 
   * @author Andreas Brker
   */
  public void run(IProgressMonitor progressMonitor) throws InvocationTargetException, InterruptedException {
    this.progressMonitor = progressMonitor;
    progressMonitor.beginTask(Messages.ConstructNewDocumentOperation_monitor_constructing_document_message, 6);
    internalThread = new InternalThread();   
    internalThread.start();      
    while(!internalThread.done()) {      
      Thread.sleep(150);
      if(progressMonitor.isCanceled()) {
        /**
         * This method is deprecated, but ...
         */
        internalThread.stop();
        progressMonitor.done();
        throw new InterruptedException(Messages.ConstructNewDocumentOperation_exception_interrupted_message);
      }
    }
    
    if(internalThread.getException()!= null) {
      progressMonitor.done();
      throw new InvocationTargetException(internalThread.getException());
    }
    
    progressMonitor.worked(1);
    
    try {
      progressMonitor.subTask(Messages.ConstructNewDocumentOperation_monitor_refreshing_explorer_message);
      newDocumentRequest.getTargetContainer().refreshLocal(1, progressMonitor);
    }
    catch(Exception exception) {
      //do not consume - only the refresh fails
    }
    progressMonitor.worked(1);
    
    progressMonitor.subTask(Messages.ConstructNewDocumentOperation_monitor_completed_message);
    progressMonitor.done();
  }
  //----------------------------------------------------------------------------
  
}
