/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.editors;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;

import org.eclipse.jface.dialogs.ErrorDialog;

import org.eclipse.ui.IEditorPart;

/**
 * Syncronizer for editor file resources.
 * 
 * @author Markus Krger
 * @version $Revision$
 */
public class EditorFileSynchronizer implements IResourceChangeListener, IResourceDeltaVisitor, IEditorFileSynchronizer {
  
  protected IFile       file          = null;
  protected IEditorPart editorPart    = null;
  
  protected boolean     isInstalled   = false;
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new EditorFileSynchronizer.
   * 
   * @param file the editor's file resource to be used
   * @param editorPart the editor this synchronizer is used for
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  public EditorFileSynchronizer(IFile file, IEditorPart editorPart) {
    assert file != null;
    assert editorPart != null;
    this.file = file;
    this.editorPart = editorPart;
  }
  //----------------------------------------------------------------------------
  /**
   * Installs the synchronizer.
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  public void install() {
    getFile().getWorkspace().addResourceChangeListener(this);
    isInstalled= true;
  }
  //----------------------------------------------------------------------------
  /**
   * Uninstalls the synchronizer.
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  public void uninstall() {
    getFile().getWorkspace().removeResourceChangeListener(this);
    isInstalled= false;
  }
  //----------------------------------------------------------------------------
  /**
   * Notifies this listener that some resource changes 
   * are happening, or have already happened.
   *
   * @param event the resource change event
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  public final void resourceChanged(IResourceChangeEvent event) {
    IResourceDelta delta= event.getDelta();
    try {
      if(delta != null && isInstalled)
        delta.accept(this);
    } 
    catch(CoreException coreException) {
      ErrorDialog.openError(editorPart.getSite().getShell(), Messages.EditorFileSynchronizer_internal_error, coreException.getMessage(), 
          coreException.getStatus());
    }
  }
  //----------------------------------------------------------------------------
  /** 
   * Visits the given resource delta.
   * 
   * @return <code>true</code> if the resource delta's children should
   *    be visited; <code>false</code> if they should be skipped.
   *    
   * @throws CoreException if the visit fails for some reason.
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  public final boolean visit(IResourceDelta delta) throws CoreException {
    if(delta == null)
      return false;

    delta = delta.findMember(getFile().getFullPath());

    if(delta == null)
      return false;

    Runnable runnable = null;

    switch(delta.getKind()) {
      case IResourceDelta.CHANGED:
        EditorUIPlugin.getDefault().getWorkbench().getDisplay().syncExec(new Runnable() {
          public void run() {
            handleContentChange();
          }              
        });
        break;

      case IResourceDelta.REMOVED:
        if((IResourceDelta.MOVED_TO & delta.getFlags()) != 0) {
          final IPath path= delta.getMovedToPath();
          EditorUIPlugin.getDefault().getWorkbench().getDisplay().syncExec(new Runnable() {
            public void run() {
              IFile newFile = getFile().getWorkspace().getRoot().getFile(path);
              handleMoved(newFile);
            }              
          }); 
        } 
        else {
          EditorUIPlugin.getDefault().getWorkbench().getDisplay().syncExec(new Runnable() {
            public void run() {
              handleRemoved();
            }              
          });
        }
        break;
    }

    return false;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns the used file.
   *
   * @return the used file
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  protected IFile getFile() {
    return file;
  }
  //----------------------------------------------------------------------------
  /**
   * The method is called if the resource is remove. 
   * Default implementation closes the belonging editor.
   * Clients may overide this method.
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  protected void handleRemoved() {
    EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().
      getActivePage().closeEditor(editorPart, false);
  }
  //----------------------------------------------------------------------------
  /**
   * The method is called if the resource is moved. 
   * Default implementation closes the belonging editor.
   * Clients may overide this method.
   * 
   * @param newFile the new file resource, can be null
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  protected void handleMoved(IFile newFile) {
    EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().
      getActivePage().closeEditor(editorPart, false);
  }
  //----------------------------------------------------------------------------
  /**
   * The method is called whenever the resource's content changed. This
   * includes also a call of the "save" method of an editor.
   * This means, that if it is not intended to react when a "save" 
   * is done in an editor, then the editor has to un- and reinstall the synchronizer 
   * in its "save" method.
   * Default implementation closes the belonging editor.
   * Clients may overide this method.
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  protected void handleContentChange() {
    EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().
      getActivePage().closeEditor(editorPart, false);
  }
  //----------------------------------------------------------------------------
  
}