/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:09:58 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.internal.search.core;

import ag.ion.noa.document.ISearchService;
import ag.ion.noa.document.ISearchableDocument;

import ag.ion.noa.search.ISearchDescriptor;
import ag.ion.noa.search.ISearchResult;

import ag.ion.noa4e.internal.search.ui.OfficeSearchResult;

import ag.ion.noa4e.search.NOASearchPlugin;

import ag.ion.noa4e.ui.operations.LoadDocumentOperation;

import ag.ion.bion.officelayer.application.IOfficeApplication;

import ag.ion.bion.officelayer.document.DocumentDescriptor;
import ag.ion.bion.officelayer.document.IDocument;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceProxy;
import org.eclipse.core.resources.IResourceProxyVisitor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;

import org.eclipse.osgi.util.NLS;

import java.lang.reflect.InvocationTargetException;

/**
 * Visitor for the office search.
 * 
 * @author Andreas Brker
 * @version $Revision: 9195 $
 * @date 12.07.2006
 */ 
public class OfficeSearchVisitor implements IResourceProxyVisitor {

	private SearchScope 				searchScope 				= null;
	private IOfficeApplication 	officeApplication 	= null;
	private ISearchDescriptor 	searchDescriptor 		= null;
	private OfficeSearchResult 	officeSearchResult 	= null;
	private IProgressMonitor 		progressMonitor 		= null;
	private MultiStatus					multiStatus 				= null;
	
	private int numberOfScannedFiles 	= 0;
	private int numberOfFilesToScan 	= 0;
	
	private long fLastUpdateTime = 0;
	
  //----------------------------------------------------------------------------
	/**
	 * Constructs new OfficeSearchVisitor.
	 * 
	 * @param officeApplication office application to be used
	 * @param searchScope seach scope to be used
	 * @param searchDescriptor search descriptor to be used
	 * @param officeSearchResult office search result to be used
	 * @param multiStatus status to be used
	 * @param numberOfFilesToScan number of files to be scanned
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public OfficeSearchVisitor(IOfficeApplication officeApplication, SearchScope searchScope, 
			ISearchDescriptor searchDescriptor, OfficeSearchResult officeSearchResult, MultiStatus multiStatus, 
			int numberOfFilesToScan) {
		this.officeApplication = officeApplication;
		this.searchScope = searchScope;
		this.searchDescriptor = searchDescriptor;
		this.officeSearchResult = officeSearchResult;
		this.multiStatus = multiStatus;
		this.numberOfFilesToScan = numberOfFilesToScan;
	}
  //----------------------------------------------------------------------------
	/**
	 * Starts the visit process.
	 * 
	 * @param progressMonitor progress monitor to be used
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void process(IProgressMonitor progressMonitor) {
		assert progressMonitor != null;
		this.progressMonitor = progressMonitor;
		progressMonitor.beginTask(Messages.OfficeSearchVisitor_monitor_taskName, numberOfFilesToScan);
		IResource[] roots = searchScope.getRootElements();
		for (int i= 0; i < roots.length; i++) {
			try {
				roots[i].accept(this, 0);
			} 
			catch (CoreException coreException) {
				multiStatus.add(coreException.getStatus());
			}
		}
	}
  //----------------------------------------------------------------------------
	/** 
	 * Visits the given resource.
	 *
	 * @param proxy for requesting information about the resource being visited;
	 * this object is only valid for the duration of the invocation of this
	 * method, and must not be used after this method has completed
	 * 
	 * @return <code>true</code> if the resource's members should
	 *		be visited; <code>false</code> if they should be skipped
	 *
	 * @exception CoreException if the visit fails for some reason
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public boolean visit(IResourceProxy proxy) throws CoreException {		
		if (proxy.getType() != IResource.FILE) {
			return true;
		}
		
		if (!searchScope.matchesFileName(proxy.getName())) {
			return false;
		}
		
		IFile file= (IFile) proxy.requestResource();
		IDocument document = loadDoccument(file);
		if(document != null) {
			ISearchResult searchResult = search(document, searchDescriptor);
			if(!searchResult.isEmpty()) {
				progressMonitor.subTask(Messages.OfficeSearchVisitor_monitor_performinSearch);
				OfficeSearchMatch officeSearchMatch = new OfficeSearchMatch(file, searchResult.getTextRanges().length);
				officeSearchResult.addMatch(officeSearchMatch);
			}
		}
		document.close();
		updateProgressMonitor();
		return false;
	}
  //----------------------------------------------------------------------------
	/**
	 * Performs search on the submitted document.
	 * 
	 * @param document document to be used
	 * @param searchDescriptor search descriptor to be used
	 * 
	 * @return result of the search
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	protected ISearchResult search(IDocument document, ISearchDescriptor searchDescriptor) {
		if(document instanceof ISearchableDocument) {
			ISearchService searchService = ((ISearchableDocument)document).getSearchService();
			return searchService.findAll(searchDescriptor);
		}
		return null;
	}
  //----------------------------------------------------------------------------
	/**
	 * Loads document on the basis of the submitted file.
	 * 
	 * @param file file to be used
	 * 
	 * @return loaded document or null if a document is not available
	 * 
	 * @throws CoreException if the file can not be accessed
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	protected IDocument loadDoccument(IFile file) throws CoreException {
		DocumentDescriptor documentDescriptor = new DocumentDescriptor();
		documentDescriptor.setHidden(true);
		documentDescriptor.setAsTemplate(true);
		LoadDocumentOperation loadDocumentOperation = new LoadDocumentOperation(officeApplication, file.getContents(), documentDescriptor);
		loadDocumentOperation.setIsSubTask(true);
		try {
			loadDocumentOperation.run(progressMonitor);
		}
		catch(InvocationTargetException invocationTargetException) {
			throw new CoreException(new Status(IStatus.ERROR, NOASearchPlugin.PLUGIN_ID, 
					IStatus.ERROR, invocationTargetException.getCause().getMessage(), 
					invocationTargetException.getCause()));
		}
		catch (InterruptedException interruptedException) {
			//aborted
		}
		return loadDocumentOperation.getDocument();
	}
  //----------------------------------------------------------------------------
	/**
	 * Updates the progress monitor. 
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	protected void updateProgressMonitor() {
		numberOfScannedFiles++;
		if (numberOfScannedFiles < numberOfFilesToScan) {
			long currTime= System.currentTimeMillis();
			if (currTime - fLastUpdateTime > 1000) {
				Object[] args = { new Integer(numberOfScannedFiles + 1), new Integer(numberOfFilesToScan)};
				progressMonitor.setTaskName(NLS.bind(Messages.OfficeSearchVisitor_monitor_scanning, args)); 
				fLastUpdateTime=  currTime;
			}
		}
		progressMonitor.worked(1);
		if (progressMonitor.isCanceled())
			throw new OperationCanceledException(Messages.OfficeSearchVisitor_monitor_operationCanceled); 
	}
  //----------------------------------------------------------------------------
	
}