/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:09:58 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.internal.search.core;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;

import org.eclipse.jface.util.Assert;

import org.eclipse.ui.IWorkingSet;

import java.io.File;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Resource scope of a search. This class is nearly a mirror
 * of the class org.eclipse.search.internal.core.SearchScope.
 * 
 * @author Andreas Brker
 * @version $Revision: 9195 $
 * @date 10.07.2006
 */ 
public class SearchScope {
	
	private static final boolean IS_CASE_SENSITIVE_FILESYSTEM = !new File("Temp").equals(new File("temp")); //$NON-NLS-1$ //$NON-NLS-2$
	
	private String description = null;
	
	private IResource[] rootElements = null;
	
	private Set fileNamePatternsSet	= new HashSet(3);
	private Set fileExtensionsSet 	= null;
	
	private Matcher[] fileNameMatchers = null;
	
  //----------------------------------------------------------------------------
	/**
	 * Returns a workspace search scope.
	 * 
	 * @return a workspace search scope
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public static SearchScope newWorkspaceScope() {
		return new SearchScope(Messages.SearchScope_description_workspace, new IResource[] { ResourcesPlugin.getWorkspace().getRoot() }); 
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns a scope for the given resources.
	 * 
	 * @param description description of the scope
	 * @param resources the resources to be contained
	 * 
	 * @return a scope for the given resources
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public static SearchScope newSearchScope(String description, IResource[] resources) {
		return new SearchScope(description, removeRedundantEntries(resources));
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns a scope for the given working sets.
	 * 
	 * @param description description of the scope
	 * @param workingSets the working sets to be contained
	 * 
	 * @return a scope for the given working sets
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public static SearchScope newSearchScope(String description, IWorkingSet[] workingSets) {
		return new SearchScope(description, convertToResources(workingSets));
	}
  //----------------------------------------------------------------------------	
	/**
	 * Returns the description of the scope.
	 * 
	 * @return the description of the scope
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public String getDescription() {
		return description;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns the root elements of this scope.
	 * 
	 * @return the root elements of this scope
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public IResource[] getRootElements() {
		return rootElements;
	}
  //----------------------------------------------------------------------------
	/**
	 * Adds an file name pattern to the scope.
	 * 
	 * @param pattern file name pattern to be added
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public void addFileNamePattern(String pattern) {
		if (fileNamePatternsSet.add(pattern)) {
			fileNameMatchers= null; // clear cache
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Adds file name patterns.
	 * 
	 * @param fileNamePatterns file name patterns to be used
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public void addFileNamePatterns(String[] fileNamePatterns) {
		for(int i=0, n=fileNamePatterns.length; i<n; i++) {
			addFileNamePattern(fileNamePatterns[i]);
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Adds file extensions.
	 * 
	 * @param fileExtension file extensions to be used
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public void addFileExtension(String fileExtension) {
		if(fileExtension == null)
			return;
		
		if(fileExtensionsSet == null)
			fileExtensionsSet = new HashSet();
		
		if(fileExtensionsSet.add(fileExtension))
			fileNameMatchers = null;
	}
  //----------------------------------------------------------------------------
	/**
	 * Adds file extensions.
	 * 
	 * @param fileExtensions file extensions to be added
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public void addFileExtensions(String[] fileExtensions) {
		for(int i=0, n=fileExtensions.length; i<n; i++) {
			addFileExtension(fileExtensions[i]);
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Tests if a file name matches to the file name patterns contained in the scope.
	 * 
	 * @param fileName The file name to test
	 * 
	 * @return returns true if the file name is matching to a file name pattern
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public boolean matchesFileName(String fileName) {
 		Matcher[] matchers= getFileNameMatchers();
		for (int i= 0; i < matchers.length; i++) {
			if (matchers[i].reset(fileName).matches()) {
				return true;
			}
		}
		return matchers.length == 0;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns a description for the file name patterns in the scope.
	 * 
	 * @return description for the file name patterns in the scope
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public String getFileNamePatternDescription() {
		String[] fileNamePatterns = (String[]) fileNamePatternsSet.toArray(new String[fileNamePatternsSet.size()]);
		Arrays.sort(fileNamePatterns);
		StringBuffer buffer = new StringBuffer();
		for(int i= 0; i < fileNamePatterns.length; i++) {
			if(i > 0) {
				buffer.append(", "); //$NON-NLS-1$
			}
			buffer.append(fileNamePatterns[i]);
		}
		return buffer.toString();
	}
  //----------------------------------------------------------------------------	
	/**
	 * Returns matchers for file names.
	 * 
	 * @return matchers for file names
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private Matcher[] getFileNameMatchers() {
		if(fileNameMatchers == null) {
			List list = new ArrayList();
			if(fileNamePatternsSet.size() != 0) {
				for (Iterator iterator = fileNamePatternsSet.iterator(); iterator.hasNext();) {
					String fileNamePattern = (String)iterator.next();
					if(fileExtensionsSet != null) {
						for (Iterator innerIterator = fileExtensionsSet.iterator(); innerIterator.hasNext();) {
							String fileExtension = (String)innerIterator.next();
							String newFileNamePattern = merge(fileNamePattern, fileExtension);
							Pattern pattern = PatternConstructor.createPattern(newFileNamePattern, IS_CASE_SENSITIVE_FILESYSTEM, false); 
							list.add(pattern.matcher("")); //$NON-NLS-1$
						}
					}
					else {
						Pattern pattern = PatternConstructor.createPattern(fileNamePattern, IS_CASE_SENSITIVE_FILESYSTEM, false); 
						list.add(pattern.matcher("")); //$NON-NLS-1$
					}
				}			
			}
			else {
				for (Iterator iterator = fileExtensionsSet.iterator(); iterator.hasNext();) {
					String fileExtension = (String)iterator.next();
					Pattern pattern = PatternConstructor.createPattern("*." + fileExtension, IS_CASE_SENSITIVE_FILESYSTEM, false);  //$NON-NLS-1$
					list.add(pattern.matcher("")); //$NON-NLS-1$
				}
			}
			fileNameMatchers = (Matcher[])list.toArray(new Matcher[list.size()]);
		}
		return fileNameMatchers;
	}
  //----------------------------------------------------------------------------
	/**
	 * Merges a file name pattern with a file extension.
	 * 
	 * @param fileNamePattern file name pattern to be merged
	 * @param fileExtension file extension to be merged
	 * 
	 * @return merged file name pattern with a file extension
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private String merge(String fileNamePattern, String fileExtension) {
		int index = fileNamePattern.indexOf("."); //$NON-NLS-1$
		if(index != -1)
			fileNamePattern = fileNamePattern.substring(0, index -1);
		return fileNamePattern + "." + fileExtension; //$NON-NLS-1$
	}
  //----------------------------------------------------------------------------
	/**
	 * Removes redundant resource entries.
	 * 
	 * @param resources resources to be checked
	 * 
	 * @return cleaned resources
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private static IResource[] removeRedundantEntries(IResource[] resources) {
		ArrayList list = new ArrayList();
		for (int i= 0; i < resources.length; i++) {
			IResource resource = resources[i];
			addToList(list, resource);
		}
		return (IResource[])list.toArray(new IResource[list.size()]);
	}
  //----------------------------------------------------------------------------
	/**
	 * Converts the submitted working sets to resources.
	 * 
	 * @param workingSets working set to be converted
	 * 
	 * @return converted working sets as resources
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private static IResource[] convertToResources(IWorkingSet[] workingSets) {
		ArrayList list = new ArrayList();
		for(int i= 0; i < workingSets.length; i++) {
			IAdaptable[] elements = workingSets[i].getElements();
			for(int k = 0; k < elements.length; k++) {
				IResource resource = (IResource)elements[k].getAdapter(IResource.class);
				if (resource != null) {
					addToList(list, resource);
				}
			}
		}
		return (IResource[]) list.toArray(new IResource[list.size()]);
	}
  //----------------------------------------------------------------------------
	/**
	 * Adds the submitted resource to the submitted list.
	 * 
	 * @param list list to be used
	 * @param resource resource to be added
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private static void addToList(ArrayList list, IResource resource) {
		IPath path = resource.getFullPath();
		for(int k = list.size() - 1; k >= 0 ; k--) {
			IResource other = (IResource)list.get(k);
			IPath otherPath = other.getFullPath();
			if (otherPath.isPrefixOf(path)) {
				return;
			}
			if (path.isPrefixOf(otherPath)) {
				list.remove(k);
			}
		}
		list.add(resource);
	}
  //----------------------------------------------------------------------------
	/**
	 * Constructs new SearchScope.
	 * 
	 * @param description description to be used
	 * @param resources resources to be used
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private SearchScope(String description, IResource[] resources) {
		Assert.isNotNull(description);
		this. description = description;
		rootElements = resources;
	}
  //----------------------------------------------------------------------------
	
}