package operationgui;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.deft.artifacttypesupport.codefile.AstTransformerRegistry;
import org.deft.artifacttypesupport.codefile.LanguageSpecifics;
import org.deft.artifacttypesupport.codefile.PlainCodeFileArtifactType;
import org.deft.artifacttypesupport.codefile.XPathGenerator;
import org.deft.artifacttypesupport.codefile.transform.OutlineNode;
import org.deft.artifacttypesupport.codefile.transform.OutlineNodeTransformer;
import org.deft.configuration.OperationGui;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.astfilter.AstFilterConfiguration;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.TreeWalker;
import org.deft.repository.ast.jxpath.XPath;
import org.deft.repository.ast.jxpath.XPathCollection;
import org.deft.repository.ast.transform.ITreeNodeFilter;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.treenode.TreeNodeRepresentation;
import org.deft.util.ImageHelper;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

public class AstFilterOperationGui implements OperationGui {

	private AstFilterConfiguration astConfig = null;
	private TreeViewer treeView;
	private StoreSelectedXPathsListener storeSelectedXpathsListener;
	private boolean disableStoreXPathsListener = false;
	

	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);
		gui.setLayout(new FillLayout());

		treeView = new TreeViewer(gui);
		treeView.setContentProvider(new AstContentProvider());
		treeView.setLabelProvider(new AstLabelProvider());
		treeView.setAutoExpandLevel(1);
		return gui;
	}
	
	

	@Override
	public void initializeGui(OperationConfiguration config, Artifact artifact, ArtifactRepresentation artifactRepr) {
		astConfig = (AstFilterConfiguration) config;
		TreeNodeRoot ast = ((TreeNodeRepresentation)artifactRepr).getRoot();
		PlainCodeFileArtifactType type = (PlainCodeFileArtifactType)artifact.getArtifactType();		
		LanguageSpecifics ls = type.getLanguageSpecifics();
		AstTransformerRegistry transReg = ls.getTransformerRegistry();
		OutlineNodeTransformer transformer = transReg.getSimpleOutlineTransformer();
		OutlineNode newAst = transformer.transform(ast);

		XPathGenerator xpathGenerator = ls.getXPathGenerator();
		addStoreSelectedXPathsListenerIfNecessary(xpathGenerator);
		
		//setting the treeview input and selecting the right nodes calls the listener, which interferes;
		//so disable listener here
		disableStoreXPathsListener = true;
		treeView.setInput(newAst);
		loadAstConfig(newAst, ast);
		disableStoreXPathsListener = false;
	}
	
	private void addStoreSelectedXPathsListenerIfNecessary(XPathGenerator xpathGenerator) {
		if (storeSelectedXpathsListener == null) {
			storeSelectedXpathsListener = new StoreSelectedXPathsListener(xpathGenerator);
			treeView.addSelectionChangedListener(storeSelectedXpathsListener);
		}
	}
	
	
	private void loadAstConfig(OutlineNode outlineAst, TreeNodeRoot originalAstRoot) {
		TreeWalker tw = new TreeWalker(outlineAst, new OutlineNodeByXPathFilter(astConfig.getXPaths(), originalAstRoot));
		List<OutlineNode> selectedNodes = new LinkedList<OutlineNode>();
		while (tw.hasNext()) {
			OutlineNode on = (OutlineNode)tw.next();
			selectedNodes.add(on);
		}
		
		ISelection selection = new StructuredSelection(selectedNodes);
		treeView.setSelection(selection, true);
	}
	
	
	/**
	 * Filter for TreeWalker which accepts all OutlineNodes which are based on TreeNodes
	 * that are selected by certain XPaths.
	 */
	private class OutlineNodeByXPathFilter implements ITreeNodeFilter {

		private List<TreeNode> selectedTreeNodes;
		
		public OutlineNodeByXPathFilter(List<XPath> xPaths, TreeNodeRoot root) {
			this.selectedTreeNodes = root.executeXPathQueries(new XPathCollection(xPaths));
		}
		
		@Override
		public boolean accept(TreeNode node) {
			if (node instanceof OutlineNode) {
				OutlineNode on = (OutlineNode)node;
				TreeNode tn = on.getTreeNode();
				boolean isSelected = selectedTreeNodes.contains(tn);
				return isSelected;
			} else {
				throw new IllegalArgumentException("OutlineNodeByXPathFilter accepts only OutlineNodes.");
			}
			
		}
		
	}
	
	
	private class AstContentProvider implements ITreeContentProvider {
		private OutlineNode content;
		
		public Object[] getChildren(Object parentElement) {
			OutlineNode node = (OutlineNode) parentElement;
			if (node.hasChildren())
				return node.getChildren().toArray();
			return new Object[0];
		}

		public Object getParent(Object element) {
			OutlineNode node = (OutlineNode) element;
			return node.getParent();
		}

		public boolean hasChildren(Object element) {
			OutlineNode node = (OutlineNode) element;
			return node.hasChildren();
		}

		public Object[] getElements(Object inputElement) {
			if (content != null)
				return content.getChildren().toArray();

			return new Object[0];
		}

		public void dispose() {
			if (content != null) {
				content = null;
			}
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			if (newInput instanceof OutlineNode) {
				content = (OutlineNode)newInput;
			}
		}
	}
	
	private class AstLabelProvider extends LabelProvider {
		@Override
		public String getText(Object element) {
			if (element instanceof OutlineNode) {
				OutlineNode node = (OutlineNode) element;
				StringBuffer buffer = new StringBuffer(node.getLabel());
				return buffer.toString();
			}
			return "no label";
		}
		
		@Override
		public Image getImage(Object element) {
			if (element instanceof OutlineNode) {
				OutlineNode node = (OutlineNode) element;
				String imagePath = node.getImagePath();
				String pluginId = node.getPluginId();
				return ImageHelper.getImage(imagePath, pluginId);
			}
			return null;
		}
	}

	private class StoreSelectedXPathsListener implements ISelectionChangedListener {

		private XPathGenerator xpathGenerator;
		
		private StoreSelectedXPathsListener(XPathGenerator xpathGenerator) {
			this.xpathGenerator = xpathGenerator;
		}

		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			if (!disableStoreXPathsListener) {
				IStructuredSelection selection = (IStructuredSelection)event.getSelection();
				List<?> selectedObjects = selection.toList();
				List<XPath> paths = new ArrayList<XPath>();
				for (Object o : selectedObjects) {
					OutlineNode outlineNode = (OutlineNode)o;
					//the xpathGenerator does some XPath evaluations, which only work
					//if the tree has a TreeNodeRoot. The Outline-tree doesn't have a
					//TreeNodeRoot, so we work on the underlying TreeNode-tree instead.
					TreeNode treeNode = outlineNode.getTreeNode();
					XPath xpath = xpathGenerator.generateXPath(treeNode);
					paths.add(xpath);
				}
				astConfig.setXPaths(paths);
			}
		}
	}
}
