package org.deft;


import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.deft.gui.dialog.SelectRepositoryDialog;
import org.deft.preferences.PreferenceConstants;
import org.deft.repository.repositorymanager.OptionsService;
import org.deft.util.ImageHelper;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.application.IWorkbenchConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchAdvisor;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;


/**
 * Configures the workbench. Makes the DEFT perspective the initial perspective,
 * sets the DEFT specific WorkbenchWindowAdvisor which is responsible for configuring
 * the workbench window. Furthermore, initializes the repository.
 */
public class DeftWorkbenchAdvisor extends WorkbenchAdvisor {

	private Map<String, String> arguments = new HashMap<String, String>();
	
	public DeftWorkbenchAdvisor(Map<String, String> arguments) {
		if (arguments != null) {
			this.arguments = arguments;
		}
	}
	
	@Override
	public WorkbenchWindowAdvisor createWorkbenchWindowAdvisor(
			IWorkbenchWindowConfigurer configurer) {
		return new DeftWorkbenchWindowAdvisor(configurer);
	}
	


	@Override
	public String getInitialWindowPerspectiveId() {
		return "org.deft.DeftPerspective"; //$NON-NLS-1$
	}
	
	
	/**
	 * This method initializes the Repository and then opens the Windows.
	 * 
	 * We must do the initialization of the Repository here, because we need a
	 * point in time where there is no GUI yet (so that there are no requests to
	 * the Repository) and the PreferenceStore is already initialized.
	 * 
	 * @see {@link WorkbenchAdvisor#openWindows()}
	 */
	@Override
	public boolean openWindows() {
		initRepository();
		return super.openWindows();
	}
	
	/**
	 * Initializes the Repository with the correct path to a base directory.
	 */
	public void initRepository() {
		trySetRepositoryPathFromArguments();
		//If there is no valid repository location - ask the user
		if (!isValidRepositoryPathSet()) {
			askForRepositoryAndSave();
		}
		// If there's still no valid repository location - exit the application.
		if (!isValidRepositoryPathSet()) {
			PlatformUI.getWorkbench().close();
			System.exit(0);
		}
		
		//if we are here we might have found a valid path in the preference store, but
		//the actual repository does not know it yet. So tell it.
		IPreferenceStore store = DeftPlugin.getInstance().getPreferenceStore();
		String path = store.getString(PreferenceConstants.REPOSITORY_PATH);
		setRepositoryPath(new Path(path));		
	}
	
	/**
	 * Sets the path of the repository if a valid repositoryPath argument
	 * has been passed to DEFT
	 */
	private void trySetRepositoryPathFromArguments() {
		if (hasValidRepositoryPathArgumentBeenSpecified()) {
			String sPath = getArgumentValueFromArgumentsMap(PreferenceConstants.REPOSITORY_PATH);
			Path path = new Path(sPath);
			setRepositoryPath(path);
		} 
	}
	
	/**
	 * Returns true if during startup the repositoryPath parameter has been
	 * specified (and pointing to an existing directory). Otherwise false.
	 */
	private boolean hasValidRepositoryPathArgumentBeenSpecified() {
		String path = getArgumentValueFromArgumentsMap(PreferenceConstants.REPOSITORY_PATH);
		return isExistingDirectory(path);
	}
	
	private boolean isExistingDirectory(String pathString) {
		if (pathString != null && !pathString.equals("")) {
			IPath path = new Path(pathString);
			return isExistingDirectory(path);
		}
		return false;
	}
	
	private boolean isExistingDirectory(IPath path) {
		File dir = path.toFile();
		boolean exists = dir.exists();
		boolean isDir = dir.isDirectory();
		return exists && isDir;
	}
	
	/**
	 * Tries to retrieve the value for an argument. The argument
	 * is searched in the arguments map in both original case and lowercase,
	 */
	private String getArgumentValueFromArgumentsMap(String argumentName) {
		String value = arguments.get(argumentName);
		//no success, try lowercase
		if (value == null) {
			value = arguments.get(argumentName.toLowerCase());
		}
		//still on success, try with leading dash
		if (value == null) {
			value = arguments.get("-" + argumentName);
		}
		//still no success, try with leading dash and lowercase
		if (value == null) {
			value = arguments.get("-" + argumentName.toLowerCase());
		}
		return value;
	}
	
	
	private void setRepositoryPath(IPath dir) {
		//set path in preference store
		IPreferenceStore store = DeftPlugin.getInstance().getPreferenceStore();
		store.setValue(PreferenceConstants.REPOSITORY_PATH, dir.toOSString());
		//set path on repository
		OptionsService options = RepositoryFacade.instance().getRepositoryOptions(); 
		options.setRepositoryLocation(dir.toOSString());
	}
	
	/**
	 * Returns true if a valid location is stored in the preference store. 
	 * Returns false if no or no valid location is stored in the preference store.
	 */
	private boolean isValidRepositoryPathSet() {
		IPreferenceStore store = DeftPlugin.getInstance().getPreferenceStore();
		String path = store.getString(PreferenceConstants.REPOSITORY_PATH);
		boolean isValidDir = isExistingDirectory(path);
		return isValidDir;
	}
	
	/**
	 * Asks the user to enter a directory as the location for the repository. If
	 * the user cancels this action no directory will be saved.
	 */
	private void askForRepositoryAndSave() {
		SelectRepositoryDialog dia = createSelectRepositoryDialog();
		if (hasValidDirectoryBeenSelected(dia)) {
			IPath dir = dia.getDirectory();			
			setRepositoryPath(dir);
		}

		IPath dir = dia.getDirectory();

		// Only save if the user really selected something
		if (dir != null) {
			setRepositoryPath(dir);
		}
	}
	
	private SelectRepositoryDialog createSelectRepositoryDialog() {
		// Create shell, as there is no default shell yet
		Shell shell = new Shell(Display.getCurrent());

		String title = "Select repository location";
		String message = "Please enter the location for the repository. The repository "
				+ "is a folder where all your user created content will be stored.";
		SelectRepositoryDialog dia = new SelectRepositoryDialog(shell);
		dia.create();
		dia.setTitle(title);
		dia.setMessage(message);
		dia.setHelpAvailable(false);
		dia.setBlockOnOpen(true);
		return dia;
	}
	
	
	private boolean hasValidDirectoryBeenSelected(SelectRepositoryDialog dia) {
		//open dialog, let user interact, and save which button has been clicked
		int buttonPressed = dia.open();		
		if (buttonPressed == Dialog.OK) {
			IPath selectedDir = dia.getDirectory();
			if (isExistingDirectory(selectedDir)) {
				return true;
			}
		}
		return false;
	}
	
	
	
	
	/**
	 * Configures workbench to makes use of the memento mechanism to save and restore
	 * view and editor states, and overrides standard Eclipse icons with DEFT specific icons.
	 */
	@Override
	public void initialize(IWorkbenchConfigurer configurer) {
		configurer.setSaveAndRestore(true);
		overrideStandardIcons(configurer);
	}
	
	
	
	/**
	 * This method is used to change the appearance of standard Eclipse icons
	 * like save or save_all, copy, paste, etc...
	 */
	private void overrideStandardIcons(IWorkbenchConfigurer configurer) {

		//copy
		overrideIcon(configurer, ISharedImages.IMG_TOOL_COPY, "enabled/edit-copy.png");
		overrideIcon(configurer, ISharedImages.IMG_TOOL_COPY_DISABLED, "disabled/edit-copy.png");
			
		//paste
		overrideIcon(configurer, ISharedImages.IMG_TOOL_PASTE, "enabled/edit-paste.png");
		overrideIcon(configurer, ISharedImages.IMG_TOOL_PASTE_DISABLED, "disabled/edit-paste.png");

		//cut
		overrideIcon(configurer, ISharedImages.IMG_TOOL_CUT, "enabled/edit-cut.png");
		overrideIcon(configurer, ISharedImages.IMG_TOOL_CUT_DISABLED, "disabled/edit-cut.png");

		//undo
		overrideIcon(configurer, ISharedImages.IMG_TOOL_UNDO, "enabled/edit-undo.png");
		overrideIcon(configurer, ISharedImages.IMG_TOOL_UNDO_DISABLED, "disabled/edit-undo.png");

		//redo
		overrideIcon(configurer, ISharedImages.IMG_TOOL_REDO, "enabled/edit-redo.png");
		overrideIcon(configurer, ISharedImages.IMG_TOOL_REDO_DISABLED, "disabled/edit-redo.png");

		/*
		 * Save
		 */
		// desc = ImageHelper.getDeftIconDescriptor("enabled/file-save.png");
		// configurer.declareImage(IDEInternalWorkbenchImages.IMG_ETOOL_SAVE_EDIT,
		// desc, false);
		//
		// desc = ImageHelper.getDeftIconDescriptor("disabled/file-save.png");
		// configurer.declareImage(IDEInternalWorkbenchImages.IMG_ETOOL_SAVE_EDIT_DISABLED,
		// desc, false);
		/*
		 * Save all
		 */
		// desc =
		// ImageHelper.getDeftIconDescriptor("enabled/file-save-all.png");
		// configurer.declareImage(IDEInternalWorkbenchImages.IMG_ETOOL_SAVEALL_EDIT,
		// desc, false);
		//		
		// desc =
		// ImageHelper.getDeftIconDescriptor("disabled/file-save-all.png");
		// configurer.declareImage(IDEInternalWorkbenchImages.IMG_ETOOL_SAVEALL_EDIT,
		// desc, false);
	}
	
	private void overrideIcon(IWorkbenchConfigurer configurer, String iconId, String newIconPath) {
		ImageDescriptor desc = ImageHelper.getDeftIconDescriptor(newIconPath);
		configurer.declareImage(iconId, desc, false);
	}

	
	
	public void postStartup() {
		IWorkbenchWindow window = getWorkbenchConfigurer().getWorkbench().getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();
		EditorPartListener editorPartListener = new EditorPartListener();
		addArtifactIntegrationPartListener(page, editorPartListener);
		integrateArtifactsForOpenEditors(page, editorPartListener);
	}
	
	private void addArtifactIntegrationPartListener(IWorkbenchPage page, EditorPartListener editorPartListener) {
		page.addPartListener(editorPartListener);
	}
	
	private void integrateArtifactsForOpenEditors(IWorkbenchPage page, EditorPartListener editorPartListener) {
		
		for (IEditorReference editorReference : page.getEditorReferences()) {
			IEditorPart editor = editorReference.getEditor(false);
			editorPartListener.partOpened(editor);
		}
	}

}
