package org.deft;

import java.util.List;

import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.integrator.EditorChapterDocument;
import org.deft.integrator.ErrorMessageIntegrator;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;

/**
 * Listener at the workbench, listens for opening ChapterEditorParts. 
 * If a ChapterEditorPart has been opened, unused references are removed
 * from the repository (i.e. references which are stored in the repository
 * but not in the document), and a repository update listener is added
 * to the document.
 */
public class EditorPartListener implements IPartListener {

	
	public void partClosed(IWorkbenchPart part) {
		if (isChapterEditor(part)) {
			ChapterEditorPart editor = (ChapterEditorPart)part;
			EditorChapterDocument chapterDocument = editor.getChapterDocument();
			chapterDocument.removeChapterDocRepositoryListener();
		}
	}

	public void partOpened(IWorkbenchPart part) {
		if (isChapterEditor(part)) {
			ChapterEditorPart editor = (ChapterEditorPart)part;
			EditorChapterDocument chapterDocument = editor.getChapterDocument();
			chapterDocument.removeNonexistingReferencesFromRepository();
			chapterDocument.addChapterDocRepositoryListener();
			updateChangedRepresentations(chapterDocument);
		}
	}
	
	private boolean isChapterEditor(IWorkbenchPart part) {
		return part instanceof ChapterEditorPart;
	}
	
	
	private void updateChangedRepresentations(EditorChapterDocument chapterDocument) {
		Chapter chapter = chapterDocument.getChapter();
		List<ArtifactReference> references = chapter.getArtifactReferences();
		for (ArtifactReference ref : references) {
			if (ref.getUpdateType() == ArtifactReferenceUpdateType.MISSINGARTIFACT) {
				update(chapterDocument, ref);
			} else if (ref.getUpdateType() == ArtifactReferenceUpdateType.NORMALUPDATE) {
				update(chapterDocument, ref);
			} else if (ref.getUpdateType() == ArtifactReferenceUpdateType.UNCOMPUTABLEUPDATE) {
				ErrorMessageIntegrator emi = chapterDocument.getErrorMessageIntegrator();
				ArtifactReferencePosition pos = emi.getPositionForRepresentation(chapterDocument, ref.getId());
				boolean errorMessageAlreadyIntegrated = pos != null;
				if (!errorMessageAlreadyIntegrated) {
					update(chapterDocument, ref);
				}
			} else if (ref.getUpdateType() == ArtifactReferenceUpdateType.NOUPDATE){
				int artifactRev = ref.getArtifact().getLatestRevision();
				int referenceRev = ref.getLatestCheckedRevision();
				if (artifactRev > referenceRev) {
					//test added, because otherwise an artifact is always updated if its revision
					//is newer than the latest checked reference revision, even though the content is
					//the same
					if (!areEqual(ref, artifactRev, referenceRev)) {
						update(chapterDocument, ref);
					}
				}
			}
		}
	}

	private boolean areEqual(ArtifactReference ref, int revision1, int revision2) {
		Artifact artifact = ref.getArtifact();
		OperationChain chain = RepositoryFacade.instance().getOperationChain(ref);
		ArtifactRepresentation rep1 = chain.executeOperationsOnRevision(artifact, revision1);
		ArtifactRepresentation rep2 = chain.executeOperationsOnRevision(artifact, revision2);
		boolean equals = rep1 != null && rep2 != null && rep1.equals(rep2);
		return equals;
	}

	
	private void update(EditorChapterDocument chapterDocument, ArtifactReference ref) {
		ArtifactReferencePosition pos = chapterDocument.removeRepresentation(ref);
		chapterDocument.insertRepresentation(ref, pos);
	}
	
	
	public void partActivated(IWorkbenchPart part) {
	}

	public void partBroughtToTop(IWorkbenchPart part) {
	}

	public void partDeactivated(IWorkbenchPart part) {
	}

}
