package org.deft;

import java.util.List;

import org.deft.actions.InsertRepresentationAction;
import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.gui.dialog.ArtifactSelectionDialog;
import org.deft.integrator.EditorIntegrator;
import org.deft.operation.IdentifierOperationChain;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.service.InternalStateService;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

public class InsertIdentifierHandler extends AbstractHandler {

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		Chapter chapter = getActiveChapter();
		if (chapter == null) {
			printErrorMessage("No chapter editor opened.");
		} else {
			Project project = chapter.getProject();
			Artifact artifact = getArtifact(project);
			if (artifact != null) {
				OperationChain chain = getIdentifierOperationChain(artifact.getArtifactTypeId(), chapter.getTypeId());
				if (chain == null) {
					printErrorMessage("This artifact does either not contain identifiers or there is no appropriate operation chain/integrator registered.");
				} else {
					IWorkbench wb = PlatformUI.getWorkbench();
					IWorkbenchWindow window = wb.getActiveWorkbenchWindow();
					EditorIntegrator integrator = getIntegrator(chain, chapter.getTypeId());
					InsertRepresentationAction ira = new InsertRepresentationAction(window, chain, integrator);
					ira.insertRepresentation(artifact);
				}
			}
			
		}
		return null;
	}
	
	private Chapter getActiveChapter() {
		IEditorPart activeEditor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (activeEditor instanceof ChapterEditorPart) {
			ChapterEditorPart cep = (ChapterEditorPart)activeEditor;
			Chapter chapter = cep.getChapter();
			return chapter;
		}
		return null;
	}
	
	
	private Artifact getArtifact(Project project) {
		Artifact recentlyInsertedArtifact = getRecentlyInsertedArtifact();
		Artifact newlySelectedArtifact = null;
		if (isValidArtifact(recentlyInsertedArtifact, project)) {
			newlySelectedArtifact = getArtifactFromDialog(project, recentlyInsertedArtifact);
		} else {
			newlySelectedArtifact = getArtifactFromDialog(project, null);
		}
		return newlySelectedArtifact;
	}
	
	private boolean isValidArtifact(Artifact artifact, Project project) {
		return artifact != null && artifact.getProject() != null && artifact.getProject().equals(project);
	}
	
	/**
	 * Returns the artifact that has been inserted last.
	 * @return
	 */
	private Artifact getRecentlyInsertedArtifact() {
		InternalStateService iss = RepositoryFacade.instance().getInternalStateService(); 
		Artifact artifact = iss.getRecentlyInsertedArtifact();
		return artifact;
	}

	private Artifact getArtifactFromDialog(Project project, Artifact oldSelectedArtifact) {
		ArtifactSelectionDialog dia = new ArtifactSelectionDialog(Display.getCurrent().getActiveShell(), project);
		dia.setSelectedArtifact(oldSelectedArtifact);
		dia.setBlockOnOpen(true);
		
		// Show the dialog
		if (dia.open() == ArtifactSelectionDialog.OK) {
			Artifact newArtifact = dia.getSelectedArtifact();
			return newArtifact;
		}
		return null;

	}
	
	
	private OperationChain getIdentifierOperationChain(String artifactTypeId, String chapterTypeId) {
		ExtensionServiceExtension ese = RepositoryFacade.instance().getExtensionService();
		List<OperationChain> chains = ese.getCompatibleOperationChainsForEditorIntegration(artifactTypeId, chapterTypeId);
		for (OperationChain chain : chains) {
			if (chain instanceof IdentifierOperationChain) {
				//we suppose there is at most one IdentifierOperationChain
				return chain;
			}
		}
		return null;
	}
	
	private EditorIntegrator getIntegrator(OperationChain chain, String chapterTypeId) {
		ExtensionServiceExtension ese = RepositoryFacade.instance().getExtensionService();
		List<EditorIntegrator> integrators = ese.getEditorIntegratorsForOperationChain(chain.getId());
		for (EditorIntegrator integrator : integrators) {
			if (integrator.getChapterTypeIds().contains(chapterTypeId)) {
				return integrator;
			}
		}	
		return null;
	}

	
	private void printErrorMessage(String message) {
		System.out.println(message);
	}

}
