package org.deft;

import java.io.File;
import java.util.List;

import org.deft.artifactviewer.ArtifactViewerConnectorRegistry;
import org.deft.artifactviewer.EclipseArtifactViewerConnectorRegistry;
import org.deft.configuration.ChainableGuiRegistry;
import org.deft.configuration.EclipseChainableGuiRegistry;
import org.deft.configuration.EclipseProjectTypeGuiRegistry;
import org.deft.configuration.ProjectTypeGuiRegistry;
import org.deft.exporter.EclipseExporterRegistry;
import org.deft.exporter.ExporterRegistry;
import org.deft.integrator.EclipseEditorIntegratorRegistry;
import org.deft.integrator.EditorIntegrator;
import org.deft.integrator.EditorIntegratorRegistry;
import org.deft.operation.OperationChain;
import org.deft.repository.Repository;
import org.deft.repository.chaptertype.ChapterType2IntegratorIdMappingRegistry;
import org.deft.repository.chaptertype.EclipseChapterType2IntegratorIdMappingRegistry;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.event.RepositoryUpdateListener;
import org.deft.repository.exception.DeftIncompatibleNameException;
import org.deft.repository.exception.DeftInvalidArgumentException;
import org.deft.repository.exception.DeftNullArgumentException;
import org.deft.repository.exception.DeftProjectAlreadyExistsException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.integrator.FileIntegrator;
import org.deft.repository.integrator.Integrator;
import org.deft.repository.projecttype.ProjectType;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.deft.repository.repositorymanager.OptionsService;
import org.deft.repository.service.ExtensionService;
import org.deft.repository.service.InternalStateService;
import org.deft.representationrenderer.EclipseRepresentationRendererRegistry;
import org.deft.representationrenderer.RepresentationRendererRegistry;

public class RepositoryFacade {
	
	private static RepositoryFacade instance;
	
	private Repository repository = Repository.instance;
	private ExtensionServiceExtension extensionService;
	private ContentManagementService cms;
	
	public static RepositoryFacade instance() {
		if (instance == null) {
			instance = new ErrorMessageDisplayingRepositoryFacade();
			instance.cms = instance.repository.getContentManagementService();
		}
		return instance;
	}
	
	public OptionsService getRepositoryOptions() {
		return repository.getOptionsService();
	}
	
	public ExtensionServiceExtension getExtensionService() {
		if (extensionService == null) {
			ExtensionService basicExtensionService = repository.getExtensionService();
			ProjectTypeGuiRegistry projectTypeGuiReg = new EclipseProjectTypeGuiRegistry();
			ArtifactViewerConnectorRegistry artifactViewerConnectorReg = new EclipseArtifactViewerConnectorRegistry();
			RepresentationRendererRegistry rendererReg = new EclipseRepresentationRendererRegistry();
			ChainableGuiRegistry guiReg = new EclipseChainableGuiRegistry();
			ChapterType2IntegratorIdMappingRegistry mappingReg = new EclipseChapterType2IntegratorIdMappingRegistry();
			EditorIntegratorRegistry editorIntegratorReg = new EclipseEditorIntegratorRegistry();
			ExporterRegistry exporterReg = new EclipseExporterRegistry();

			extensionService = new ExtensionServiceExtensionImpl(basicExtensionService, 
					projectTypeGuiReg, artifactViewerConnectorReg, rendererReg, guiReg, 
					mappingReg, editorIntegratorReg, exporterReg);
			
		}
		return extensionService;
	}
	
	public InternalStateService getInternalStateService() {
		return repository.getInternalStateService();
	}
	
	public List<ProjectType> getProjectTypes() {
		return repository.getExtensionService().getProjectTypes();
	}
	
	public File getTempDirectory(Project project) {
		return repository.getContentManagementService().getTempDirectory(project);
	}
	
	public ProjectType getProjectType(String projectTypeId) {
		return repository.getExtensionService().getProjectType(projectTypeId);
	}
	

	public List<Project> getProjects() {
		return cms.getProjects();
	}
	
	public Project getProject(String name) {
		return cms.getProject(name);
	}
	
	public Project getProjectById(String id) {
		return cms.getProjectById(id);
	}
	
	public Fragment getFragment(String id) {
		return cms.getFragment(id);
	}
	
	public void createNewProject(String name, String chapterType) {
		cms.createProject(name, chapterType);
	}
	
	public Folder createFolder(Project project, FragmentType fragType, String name) {
		Folder folder = cms.createFolder(project, fragType, name);
		return folder;
	}
	
	public Folder createFolder(Folder parent, String name) {
		Folder folder = cms.createFolder(parent, name);
		return folder;
	}

	public Chapter createChapter(Project project, String name, String chapterType) {
		Chapter chapter = cms.createChapter(project, name, chapterType);
		return chapter;
	}
	
	public Chapter createChapter(Folder parent, String name, String chapterType) {
		Chapter chapter = cms.createChapter(parent, name, chapterType);
		return chapter;
	}

	
	public Tutorial createTutorial(Project project, String name) {
		Tutorial tutorial = cms.createTutorial(project, name);
		return tutorial;
	}
	
	public Tutorial createTutorial(Folder parent, String name) {
		Tutorial tutorial = cms.createTutorial(parent, name);
		return tutorial;
	}
	
	
	public Artifact importArtifact(Project project, Folder parent,  
			FragmentSource fragmentSource, OriginalFragmentLocation location) {
		Artifact artifact = null;
		if (parent == null) {
			artifact = cms.importArtifact(project, fragmentSource, location);
		} else {
			artifact = cms.importArtifact(parent, fragmentSource, location);	
		}
		return artifact;
	}
	
	
	public Artifact importArtifact(Project project, Folder parent, String artifactTypeId, 
			FragmentSource fragmentSource, OriginalFragmentLocation location, String name) {
		Artifact artifact = null;
		if (parent == null) {
			artifact = cms.importArtifact(project, artifactTypeId,
					fragmentSource, location, name);
		} else {
			artifact = cms.importArtifact(parent, artifactTypeId,
					fragmentSource, location, name);	
		}
		return artifact;
	}
	
	public Artifact importArtifact(Project project, Folder parent, String artifactTypeId, 
			FragmentSource fragmentSource, OriginalFragmentLocation location, String name, 
			int revision) {
		Artifact artifact = null;
		if (parent == null) {
			artifact = cms.importArtifact(project, artifactTypeId,
					fragmentSource, location, name, revision);
		} else {
			artifact = cms.importArtifact(parent, artifactTypeId,
					fragmentSource, location, name, revision);	
		}
		return artifact;
	}
	
	public void updateArtifact(Artifact artifact, int revision) {
		cms.updateArtifact(artifact, revision);
	}
	
	public void updateArtifactWith(Artifact artifact, FragmentSource fragmentSource, 
			OriginalFragmentLocation location, int revision) {
		cms.updateArtifactWith(artifact, fragmentSource, location, revision);
	}
	
	public void moveToFolder(Fragment fragment, Folder target) {
		cms.moveToFolder(fragment, target);
	}
	
	public void rename(Project project, String newName) throws DeftNullArgumentException, 
			DeftInvalidArgumentException, DeftProjectAlreadyExistsException {
		cms.rename(project, newName);
	}


	public void rename(Fragment fragment, String newName) throws DeftNullArgumentException, 
			DeftInvalidArgumentException, DeftIncompatibleNameException {
		cms.rename(fragment, newName);
	}
	
	
	public boolean canContainFragmentType(Folder parent, FragmentType fragmentType) {
		return cms.canContainFragmentType(parent, fragmentType);
	}

	public boolean canContainFragmentName(Folder parent, String childName) {
		return cms.canContainFragmentName(parent, childName);
	}
	

	public boolean canContainFragmentName(Project project, FragmentType fragmentType, 
			String childName) {
		return cms.canContainFragmentName(project, fragmentType, childName);
	}
	

	public boolean canContainFragment(Folder parent, Fragment child) {
		return cms.canContainFragment(parent, child);
	}
	
	
	public boolean canContainFragmentAtRoot(Project project, FragmentType fragmentType, Fragment child) {
		return cms.canContainFragmentAtRoot(project, fragmentType, child);
	}

	
	public boolean containsFragmentName(Folder parent, String childName) {
		return cms.containsFragmentName(parent, childName);
	}
	

	public boolean containsFragmentNameAtRoot(Project project, FragmentType fragmentType, 
			String childName) {
		return cms.containsFragmentNameAtRoot(project, fragmentType, childName);
	}


	public boolean canContainChapter(Tutorial tutorial, Chapter chapter) {
		return cms.canContainChapter(tutorial, chapter);
	}
	

	public boolean containsChapter(Tutorial tutorial, Chapter chapter) {
		return cms.containsChapter(tutorial, chapter);
	}
	
	
	public List<Fragment> getChildren(Project project, FragmentType fragmentType) {
		return cms.getChildren(project, fragmentType);
	}
	
	public List<Fragment> getChildren(Folder folder) {
		return cms.getChildren(folder);
	}
	
	
	public void addRepositoryUpdateListener(RepositoryUpdateListener listener) {
		cms.addRepositoryUpdateListener(listener);
	}
	
	public void removeRepositoryUpdateListener(RepositoryUpdateListener listener) {
		cms.removeRepositoryUpdateListener(listener);
	}
	
	
	public boolean projectExists(String name) {
		return cms.projectExists(name);
	}
	
	public boolean remove(Project project) {
		boolean success = cms.removeProject(project);
		return success;
	}
	
	public boolean remove(Fragment fragment) {
		boolean success = cms.removeFragment(fragment);
		return success;
	}
	
	public boolean remove(ArtifactReference reference) {
		boolean success = cms.removeArtifactReference(reference);
		return success;
	}

	public byte[] getChapterContent(Chapter chapter) {
		return cms.getChapterContent(chapter);
	}
	
	public void saveChapterContent(Chapter chapter, byte[] content) {
		cms.saveChapterContent(chapter, content);
	}
	
	public byte[] getArtifactContent(Artifact artifact) {
		return cms.getArtifactContent(artifact);
	}
	

	public ArtifactReference getArtifactReference(String refId) {
		return cms.getArtifactReference(refId);
	}
	
	public List<ArtifactReference> getArtifactReferences(Chapter chapter) {
		return cms.getArtifactReferences(chapter);
	}
	
	public List<ArtifactReference> getArtifactReferences(Artifact artifact) {
		return cms.getArtifactReferences(artifact);
	}
	
	public ArtifactReference createArtifactReference(Chapter chapter, Artifact artifact, OperationChain operationChain, Integrator integrator) {
		return cms.createArtifactReference(chapter, artifact, operationChain, integrator);
	}
	
	public void updateArtifactReference(ArtifactReference reference, OperationChain operationChain, Integrator integrator) {
		cms.updateArtifactReference(reference, operationChain, integrator);
	}
	
	public boolean removeArtifactReference(ArtifactReference reference) {
		return cms.removeArtifactReference(reference);
	}

	public void setArtifactReferenceChecked(ArtifactReference reference) {
		cms.setArtifactReferenceChecked(reference);
	}
	
	public void setArtifactReferenceUnchecked(ArtifactReference reference) {
		cms.setArtifactReferenceUnchecked(reference);
	}

	
	public OperationChain getOperationChain(ArtifactReference reference) {
		return cms.getOperationChain(reference);
	}
	
	public FileIntegrator getFileIntegrator(ArtifactReference reference) {
		return cms.getFileIntegrator(reference);
	}
	
	/**
	 * Returns the Integrator which belongs to the reference. If the Integrator
	 * had been configured (some parameters modified) by the user, this configuration
	 * will be restored.
	 *  
	 * @param ref the reference for which to return the Integrator
	 * @return the configured Integrator
	 */
	public EditorIntegrator getEditorIntegrator(ArtifactReference ref) {
		String integratorId = ref.getIntegratorId();
		EditorIntegrator integrator = extensionService.getEditorIntegrator(integratorId);
		cms.setConfiguration(integrator, ref);
		return integrator;
	}
	


	public List<Artifact> getArtifactsWithMissingOrigin(Project project) {
		return cms.getArtifactsWithMissingOrigin(project);
	}
	
	public List<Artifact> getArtifactsWithChangedOrigin(Project project) {
		return cms.getArtifactsWithChangedOrigin(project);
	}
}
