package org.deft.actions;

import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;


/**
 * Access to standard actions provided by DEFT. The design of this class
 * is inspired by the Eclipse ActionFactory class.
 */
public abstract class DeftActionFactory {

	/**
	 * Id of actions created by this action factory.
	 */
	private final String actionId;

	/**
	 * Creates a new DEFT action factory with the given id.
	 * 
	 * @param actionId
	 *            the id of actions created by this action factory
	 */
	protected DeftActionFactory(String actionId) {
		this.actionId = actionId;
	}

	/**
	 * Creates a new standard action for the given workbench window. The action
	 * has an id as specified by the particular factory.
	 * <p>
	 * Actions automatically register listeners against the workbench window so
	 * that they can keep their enablement state up to date. Ordinarily, the
	 * window's references to these listeners will be dropped automatically when
	 * the window closes. However, if the client needs to get rid of an action
	 * while the window is still open, the client must call
	 * {@link IWorkbenchAction#dispose dispose}to give the action an
	 * opportunity to unregister its listeners and to perform any other cleanup.
	 * </p>
	 * 
	 * @param window
	 *            the workbench window
	 * @return the workbench action
	 */
	public abstract IWorkbenchAction create(IWorkbenchWindow window);

	/**
	 * Returns the id of this action factory.
	 * 
	 * @return the id of actions created by this action factory
	 */
	public String getId() {
		return actionId;
	}

	
	/* All DEFT Actions */

	
	public static final ActionFactory NEW_PROJECT = new ActionFactory(
			"org.deft.newProjectAction") {
		
		public IWorkbenchAction create(IWorkbenchWindow window) {
			
			if (window == null) {
				throw new IllegalArgumentException();
			}

			IWorkbenchAction action = new NewProjectAction();
			action.setId(getId());
			return action;
		}
	};
	
	public static final ActionFactory NEW_DROP_DOWN = new ActionFactory(
			"org.deft.newWithMenuAction") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			IWorkbenchAction action = new NewDropDownAction(window);
			action.setId(getId());

			return action;
		}
	};
	
	/**
	 * This factory will return always the same instance of an
	 * ExportDropDownAction
	 */
	public static final ActionFactory EXPORT_DROP_DOWN = new ActionFactory(
			"org.deft.exportWithMenuAction") {

		private IWorkbenchAction cachedAction;

		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			if (cachedAction != null) {
				return cachedAction;
			}

			cachedAction = new ExportDropDownAction(window);
			cachedAction.setId(getId());

			return cachedAction;
		}
	};

	
	public static final ActionFactory IMPORT_DROP_DOWN = new ActionFactory(
			"org.deft.importWithMenuAction") {

		private IWorkbenchAction cachedAction;

		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			if (cachedAction != null) {
				return cachedAction;
			}

			cachedAction = new ImportDropDownAction(window);
			cachedAction.setId(getId());

			return cachedAction;
		}
	};
	
	
	
	public static final ActionFactory UPDATE_WITH_DROP_DOWN = new ActionFactory(
			"org.deft.updateWithMenuAction") {

		private IWorkbenchAction cachedAction;

		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			if (cachedAction != null) {
				return cachedAction;
			}

			cachedAction = new UpdateWithDropDownAction(window);
			cachedAction.setId(getId());

			return cachedAction;
		}
	};
	
	
	public static final ActionFactory INSERT_DROP_DOWN = new ActionFactory(
		"org.deft.insertWithMenuAction") {

		private IWorkbenchAction action;

		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			action = new InsertDropDownAction(window);
			action.setId(getId());

			return action;
		}
	};
	


	public static final ActionFactory DELETE = new ActionFactory("org.deft.deleteAction") {

		private IWorkbenchAction cachedAction;

		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException("window may not be null");
			}

			if (cachedAction != null) {
				return cachedAction;
			}

			cachedAction = new RemoveAction(window);
			cachedAction.setId(getId());
			return cachedAction;
		}
	};
	
		



	public static final ActionFactory NEW_CHAPTER = new ActionFactory("org.deft.newChapterAction") {

		public IWorkbenchAction create(IWorkbenchWindow window) {
	
			if (window == null) {
				throw new IllegalArgumentException("window may not be null");
			}

			IWorkbenchAction action = new NewChapterAction(window);
			action.setId(getId());
			return action;
		}
	};


	public static final ActionFactory NEW_TUTORIAL = new ActionFactory("org.deft.newTutorialAction") {
		
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException("window may not be null");
			}


			IWorkbenchAction action = new NewTutorialAction(window);
			action.setId(getId());
			return action;
		}
	};
	
	public static final ActionFactory NEW_FOLDER = new ActionFactory("org.deft.newFolderAction") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			IWorkbenchAction action = new NewFolderAction(window);
			action.setId(getId());
			return action;
		}
	};

	
	public static final ActionFactory OPEN = new ActionFactory("org.deft.openAction") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}
			IWorkbenchAction action = new OpenAction(window);
			action.setId(getId());
			return action;
		}
	};
	
	
	public static final ActionFactory GOTO = new ActionFactory("org.deft.gotoAction") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}
			IWorkbenchAction action = new GoToReferencedRepresentationAction(window);
			action.setId(getId());
			return action;
		}
	};
	
	
	public static final ActionFactory RENAME = new ActionFactory(
			"org.deft.renameAction") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			IWorkbenchAction action = new RenameAction(window);
			action.setId(getId());

			return action;
		}
	};

//	public static final ActionFactory UPDATE = new ActionFactory(
//			"org.deft.updateCodeFileAction") {
//		public IWorkbenchAction create(IWorkbenchWindow window) {
//			if (window == null) {
//				throw new IllegalArgumentException();
//			}
//
//			IWorkbenchAction action = new UpdateAction(window);
//			action.setId(getId());
//
//			return action;
//		}
//	};
//
	
	
	
	public static final ActionFactory SET_PERSPECTIVE = new ActionFactory(
			"org.deft.setPerspectiveAction") {
		
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}
			IWorkbenchAction action = new SetPerspectiveAction(window);
			action.setId(getId());
			return action;
		}
	};
	
	
	public static final ActionFactory SWITCH_REPOSITORY = new ActionFactory(
			"org.deft.switchRepositoryAction") {
		public IWorkbenchAction create(IWorkbenchWindow window) {
			if (window == null) {
				throw new IllegalArgumentException();
			}

			IWorkbenchAction action = new SwitchRepositoryAction(window);
			action.setId(getId());
			return action;
		}
	};



}
