package org.deft.actions;

import org.deft.RepositoryFacade;
import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.configuration.OperationChainGui;
import org.deft.gui.SelectionHelper;
import org.deft.integrator.EditorChapterDocument;
import org.deft.integrator.EditorIntegrator;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;

/**
 * Inserts a specific representation of an Artifact into the open Chapter document.
 * 
 * First, a representation is created from the artifact. Then an ArtifactReference is
 * created in the repository. Finally an integration is performed,
 * so the representation is inserted into the document.
 */
public class InsertRepresentationAction extends AbstractDeftAction {

	private RepositoryFacade repository = RepositoryFacade.instance();
	
	private OperationChain operationChain;
	private EditorIntegrator integrator;
	
	public InsertRepresentationAction(IWorkbenchWindow window, String caption, 
			OperationChain operationChain, EditorIntegrator integrator) {
		super(window);
		this.operationChain = operationChain;
		this.integrator = integrator;
		this.setText(caption);
	}
	
	public InsertRepresentationAction(IWorkbenchWindow window, 
			OperationChain operationChain, EditorIntegrator integrator) {
		this(window, operationChain.getDescription(), operationChain, integrator);
	}
	



	@Override
	protected void setEnabledState() {
		boolean canInsert = isChapterEditorCursorAtValidPosition();
		setEnabled(canInsert);
	}
	
	private boolean isChapterEditorCursorAtValidPosition() {
		return true; //TODO
	}
	
	
	@Override
	public void run() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleArtifactNodeSelected()) {
			Artifact artifact = selection.getSelectedArtifact();
			insertRepresentation(artifact);
		}
	}
	
	public void insertRepresentation(Artifact artifact) {
		//temporary insertion, because we need a reference object
		ArtifactReference reference = insertReferenceIntoRepository(artifact);
		operationChain.resetChainConfigurations();
		OperationChainGui ocg = new OperationChainGui(operationChain, integrator, reference);
		if (ocg.hasConfigurationTabs()) {
		    int result = ocg.openConfigurationGui();
		    if (result == IDialogConstants.OK_ID) {
		    	updateReference(reference);
		    	insertRepresentationIntoEditor(reference);
		    } else {
		    	removeReferenceFromRepository(reference);
		    }
		} else {
			insertRepresentationIntoEditor(reference);
		}
		getEditorHelper().setFocusToOpenEditor();
	}
	

	private ArtifactReference insertReferenceIntoRepository(Artifact artifact) {
		ChapterEditorPart editor = getActiveChapterEditorPart();
		if (editor != null) {
			EditorChapterDocument chapterDoc = editor.getChapterDocument();
			Chapter chapter = chapterDoc.getChapter();
			ArtifactReference reference = createAndStoreReferenceInRepository(chapter, artifact);
			return reference;
		}
		return null;
	}
	

	
	
	private void insertRepresentationIntoEditor(ArtifactReference reference) {
		ChapterEditorPart editor = getActiveChapterEditorPart();
		if (editor != null) {
			EditorChapterDocument chapterDoc = editor.getChapterDocument();
			ArtifactReferencePosition pos = chapterDoc.getCursorPosition();
			chapterDoc.insertRepresentation(reference, pos);
		}
	}
	
	private ChapterEditorPart getActiveChapterEditorPart() {
		IEditorPart activeEditor = getWindow().getActivePage().getActiveEditor();
		if (activeEditor instanceof ChapterEditorPart) {
			return (ChapterEditorPart)activeEditor;
		}
		return null;
	}
	
	
	private ArtifactReference createAndStoreReferenceInRepository(Chapter chapter, Artifact artifact) {
		ArtifactReference ref = repository.createArtifactReference(chapter, artifact, 
				operationChain, integrator);
		return ref;
	}
	
	private void updateReference(ArtifactReference reference) {
		repository.updateArtifactReference(reference, operationChain, integrator);
	}
	
	
	private void removeReferenceFromRepository(ArtifactReference reference) {
		repository.remove(reference);
	}
	
}
