package org.deft.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.deft.RepositoryFacade;
import org.deft.gui.EditorHelper;
import org.deft.gui.dialog.DeletionDialog;
import org.deft.gui.view.project.nodes.ArtifactNode;
import org.deft.gui.view.project.nodes.ArtifactReferenceNode;
import org.deft.gui.view.project.nodes.FragmentNode;
import org.deft.gui.view.project.nodes.ProjectNode;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Project;
import org.deft.util.ImageHelper;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;


/**
 * Action that removes one or more elements from the repository.
 * 
 * All selected nodes from the ProjectView will be removed, if possible.
 */
public class RemoveAction extends AbstractDeftAction {

	private RepositoryFacade repository = RepositoryFacade.instance();


	public RemoveAction(IWorkbenchWindow window) {
		super(window);
		setText("Delete");
		setImageDescriptor(ImageHelper.getDeftIconDescriptor("enabled/edit-delete.png"));
		setDisabledImageDescriptor(ImageHelper.getDeftIconDescriptor("disabled/edit-delete.png"));
		//makes action react like a standard eclipse delete action (i.e. react on DEL keypress) 
		setActionDefinitionId("org.eclipse.ui.edit.delete");
	}

	@Override
	public void run() {
		
		List<ProjectViewNode> removableNodes = getRemovableNodesFromSelectedNodes();
		
		DeletionDialog dia = new DeletionDialog(Display.getCurrent().getActiveShell(), removableNodes);
		dia.setBlockOnOpen(true);
		
		// Show the dialog
		if (dia.open() == DeletionDialog.OK) {
			closeEditorsForSelectedAndDescendantFragments(removableNodes);
			deleteInBackground(removableNodes);
		}
	}
	
	private List<ProjectViewNode> getRemovableNodesFromSelectedNodes() {
		List<ProjectViewNode> selectedNodes = getSelectionHelper().getSelectedProjectViewNodes();
		List<ProjectViewNode> removableNodes = getRemovableNodes(selectedNodes);
		removeSubsumedDescendants(removableNodes);
		return removableNodes;
	}

	
	
	private List<ProjectViewNode> getRemovableNodes(List<ProjectViewNode> nodes) {
		List<ProjectViewNode> removableNodes = new LinkedList<ProjectViewNode>();
		for (ProjectViewNode node : nodes) {
			if (node.deletionAllowed()) {
				removableNodes.add(node);
			}
		}
		return removableNodes;
	}
	
	/**
	 * Compares all nodes in the list with each other and removes the nodes that are
	 * descendants of another node from the list. This is used to filter out nodes
	 * from the list of to-be-removed nodes, which will be removed anyway because something
	 * at a higher stage of the hierarchy will be removed. For example, a chapter will be
	 * removed from the list, if its project is contained in the list.  
	 */
	private void removeSubsumedDescendants(List<ProjectViewNode> nodes) {
		Set<ProjectViewNode> subsumedNodes = getSubsumedNodes(nodes);
		for (ProjectViewNode node : subsumedNodes) {
			nodes.remove(node);
		}
	}
	
	/**
	 * Compares all nodes in the list with each other and returns a set of all nodes
	 * that are descendants of another node from the list. 
	 * A set is used to prevent double occurrences.
	 */
	private Set<ProjectViewNode> getSubsumedNodes(List<ProjectViewNode> nodes) {
		Set<ProjectViewNode> subsumedNodes = new HashSet<ProjectViewNode>();
		for (ProjectViewNode node : nodes) {
			List<ProjectViewNode> descendants = node.getDescendants();
			for (ProjectViewNode descendant : descendants) {
				if (nodes.contains(descendant)) {
					subsumedNodes.add(descendant);
				}
			}
		}
		return subsumedNodes;
	}
	
	
	
	private void delete(ProjectViewNode node) {
		if (node instanceof ProjectNode) {
			delete((ProjectNode)node);
		} else if (node instanceof ArtifactNode) {
			delete((ArtifactNode)node);
		} else if (node instanceof FragmentNode) {
			delete((FragmentNode)node);
		} else if (node instanceof ArtifactReferenceNode) {
			delete((ArtifactReferenceNode)node);
		} else {
			throw new IllegalStateException("Cannot delete unknown node type");
		}
	}
	

	private void delete(ProjectNode node) {
		Project project = node.getProject();
		repository.remove(project);
	}
	
	private void delete(FragmentNode node) {
		Fragment fragment = node.getFragment();
		repository.remove(fragment);
	}
	
	private void delete(ArtifactNode node) {
		Artifact artifact = (Artifact)node.getFragment();
		repository.remove(artifact);
		getEditorHelper().updateViewerIfOpen(artifact);
	}
	
	private void delete(ArtifactReferenceNode node) {
		ArtifactReference reference = node.getReference();
		repository.remove(reference);
	}
	
	
	private void closeEditorsForSelectedAndDescendantFragments(List<ProjectViewNode> removableTopNodes) {
		
		for (ProjectViewNode node : removableTopNodes) {
			for (ProjectViewNode desc : node.getDescendants()) {
				closeEditorForNode(desc);
			}
			closeEditorForNode(node);
		}
	}
	
	private void closeEditorForNode(ProjectViewNode node) {
		EditorHelper editorHelper = getEditorHelper();
		if (node instanceof FragmentNode) {
			Fragment fragment = ((FragmentNode)node).getFragment();
			editorHelper.closeEditor(fragment);
		}
	}


	/**
	 * Inside this method is the logic which Fragments should be deleted from
	 * the Repository and which Fragments only should be removed from the tree.
	 *
	 * The deleting process is then started inside a WorkspaceModifyOperation,
	 * so that in case of a long running operation the user will get feedback.
	 */
	private void deleteInBackground(final List<ProjectViewNode> toBeDeleted) {

		WorkspaceModifyOperation deletionOperation = createDeletionOperation(toBeDeleted);
		try {
			PlatformUI.getWorkbench().getProgressService().run(true, false, deletionOperation);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	
	private WorkspaceModifyOperation createDeletionOperation(final List<ProjectViewNode> toBeDeleted) {
		
		return new WorkspaceModifyOperation() {
		
			@Override
			protected void execute(final IProgressMonitor monitor)
					throws CoreException, InvocationTargetException,
					InterruptedException {
				//must run in ui thread, otherwise exception in latex editor
				Display.getDefault().syncExec(new Runnable() {

					@Override
					public void run() {
						monitor.beginTask("Deleting selected Fragments.", toBeDeleted.size());
						for (ProjectViewNode node : toBeDeleted) {
							delete(node);
							monitor.worked(1);
						}
						monitor.done();
					}
				});
			}
		};
	}



	@Override
	protected void setEnabledState() {
		if (isProjectViewActive()) {
			List<ProjectViewNode> selectedNodes = getSelectionHelper().getSelectedProjectViewNodes();
			boolean containsRemovableNodes = containsRemovableNodes(selectedNodes);
			setEnabled(containsRemovableNodes);
		} else {
			setEnabled(false);
		}
	}
	
	
	private boolean containsRemovableNodes(List<ProjectViewNode> nodes) {
		return !getRemovableNodes(nodes).isEmpty();
	}
	
}
