package org.deft.actions;

import org.deft.RepositoryFacade;
import org.deft.gui.SelectionHelper;
import org.deft.gui.input.FragmentNameInputValidator;
import org.deft.gui.input.ProjectNameInputValidator;
import org.deft.gui.view.project.nodes.FragmentNode;
import org.deft.gui.view.project.nodes.ProjectNode;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;



public class RenameAction extends AbstractDeftAction {
	
	private RepositoryFacade repository = RepositoryFacade.instance();
	
	public RenameAction(IWorkbenchWindow window) {
		super(window);
		this.setText("Rename");
		//makes action react like a standard eclipse rename action (i.e. react on F2 keypress) 
		this.setActionDefinitionId("org.eclipse.ui.edit.rename");
	}
	
	
	@Override
	protected void setEnabledState() {
		if (isProjectViewActive()) {
			boolean isSingleRenameableNodeSelected = isSingleRenameableNodeSelected();
			setEnabled(isSingleRenameableNodeSelected);
		} else {
			setEnabled(false);
		}
	}

	
	/**
	 * Returns true if there is one node selected in the project view and
	 * this node can be renamed. This is the case for projects, chapters,
	 * artifacts and tutorials. 
	 * For all other nodes, or if multiple nodes are selected, false is returned.
	 */
	private boolean isSingleRenameableNodeSelected() {
		SelectionHelper helper = getSelectionHelper();
		boolean renameable = helper.isOnlySingleProjectNodeSelected()
				|| helper.isOnlySingleArtifactNodeSelected()
				|| helper.isOnlySingleChapterNodeSelected()
				|| helper.isOnlySingleFolderNodeSelected()
				|| helper.isOnlySingleTutorialNodeSelected();
		return renameable;
	}

	

	public void run() {
		ProjectViewNode node = getSelectionHelper().getFirstSelectedProjectViewNode();
		if (node instanceof ProjectNode) {
			ProjectNode pn = (ProjectNode)node;
			renameProject(pn.getProject());
		} else if (node instanceof FragmentNode) {
			FragmentNode fn = (FragmentNode)node;
			renameFragment(fn.getFragment());
		}
	}
	

	public void renameProject(Project project) {
		ProjectNameInputValidator validator = new ProjectNameInputValidator();
		InputDialog dialog = openAndGetInputDialog(validator, project.getName());
		if (okPressed(dialog)) {
			String newName = dialog.getValue();
			//the new name is surely valid, the validator prevented 
			//invalid names
			repository.rename(project, newName);
		}		
	}
	
	
	public void renameFragment(Fragment fragment) {
		FragmentNameInputValidator validator = createFragmentNameValidator(fragment);
		InputDialog dialog = openAndGetInputDialog(validator, fragment.getName());
		if (okPressed(dialog)) {
			String newName = dialog.getValue();
			//the new name is surely valid, the validator prevented 
			//invalid names
			repository.rename(fragment, newName);
		}		
	}

	private FragmentNameInputValidator createFragmentNameValidator(Fragment fragment) {
		if (isFragmentAtRoot(fragment)) {
			return createFragmentNameValidatorForRootFragment(fragment);
		} else {
			return createFragmentNameValidatorForFragmentInFolder(fragment);
		}

	}
	
	
	private boolean isFragmentAtRoot(Fragment fragment) {
		Folder parentFolder = fragment.getParentFolder();
		return parentFolder == null;
	}
	
	private FragmentNameInputValidator createFragmentNameValidatorForRootFragment(Fragment fragment) {
		Project project = fragment.getProject();
		FragmentType fragType = getFragType(fragment);
		FragmentNameInputValidator validator = new FragmentNameInputValidator(project, fragType);
		return validator;
	}

	/**
	 * Returns the type of the fragment. If the fragment is a folder,
	 * the type of the fragments that the folder can contain is returned, 
	 * because the aim is to find out to which category the fragment belongs. 
	 */
	private FragmentType getFragType(Fragment fragment) {
		if (fragment instanceof Folder) {
			return ((Folder)fragment).getFragmentTypeAffiliation();
		} else {
			return fragment.getFragmentType();
		}
	}
	
	private FragmentNameInputValidator createFragmentNameValidatorForFragmentInFolder(Fragment fragment) {
		Folder folder = fragment.getParentFolder();
		FragmentNameInputValidator validator = new FragmentNameInputValidator(folder);
		return validator;
	}
	

//	@Override
//	public Folder createFragment(Folder parentFolder) {
//		FragmentNameInputValidator validator = new FragmentNameInputValidator(parentFolder);
//		InputDialog dialog = openAndGetInputDialog(validator);
//		if (okPressed(dialog)) {
//			String text = dialog.getValue();
//			Folder folder = repository.createFolder(parentFolder, text);
//			return folder;
//		}
//		return null;
//	}
	
	private InputDialog openAndGetInputDialog(IInputValidator validator, String initialValue) {

		String dialogText = "Please enter a new name.";
		String dialogTitle = "Enter name";

		InputDialog dialog = new InputDialog(Display.getDefault()
				.getActiveShell(), dialogTitle, dialogText, initialValue, validator);
		return dialog;
	}
	
	private boolean okPressed(InputDialog dialog) {
		return dialog.open() == Dialog.OK;
	}
	
//	
//		if (selection == null)
//			return;
//
//		if (selection.getFirstElement() instanceof Fragment) {
//			Fragment fragment = (Fragment) selection.getFirstElement();
//
//			String dialogTitle = "Enter new name";
//			String dialogMessage = "Please enter the new name for this element.";
//			String initialValue = fragment.getName();
//
//			// Create and open an InputDialog
//			Shell shell = Display.getDefault().getActiveShell();
//			InputDialog dialog = new InputDialog(shell, dialogTitle,
//					dialogMessage, initialValue,
//					new NewFragmentInputValidator());
//			if (dialog.open() == Dialog.CANCEL) {
//				return;
//			}
//
//			String newName = dialog.getValue();
//			// Rename the selected Fragment
//			if (newName != null && !newName.equals("")) {
//				try {
//					RepositoryFactory.getRepository().rename(fragment,
//							dialog.getValue());
//					updateFragment(fragment, newName);
//				} catch (DeftFragmentAlreadyExistsException e) {
//					MessageDialog
//							.openError(
//									shell,
//									"Duplicated name",
//									"The name you entered is already used in this project.\nPlease choose another one.");
//					// Try again
//					run();
//				}
//			}
//		}
//	}
//


	
	
	/**
	 * Updates the title element and the editor name if a chapter was renamed.

	 */
//	private void updateFragment(Fragment fragment, String newName) {
//		if (fragment instanceof Chapter) {
//			if (!updateOpenChapter(fragment, newName))
//				updateClosedChapter(fragment, newName);
//		}
//
//	}
	
//	/**
//	 * Updates the open editor if the corresponding chapter was renamed. Returns true if there was
//	 * a matching open editor else false.
//	 */
//	private boolean updateOpenChapter(Fragment fragment, String newName){
//		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
//		IEditorPart activeEditor = page.getActiveEditor();
//		
//		IEditorReference[] editorRefs = page.getEditorReferences();
//		for (IEditorReference reference : editorRefs) {
//			IEditorPart editor  = reference.getEditor(false);
//			if (editor instanceof VexEditor ){
//				Fragment input = ((FragmentEditorInput)editor.getEditorInput()).getFragment();
//				if (fragment.equals(input)){	
//					VexEditor vex = (VexEditor)editor;
//					
//					//update editor tab name
//					vex.setName(newName);
//					
//					//update title element
//					VexWidget widget = vex.getVexWidget();
//					net.sf.vex.dom.Element title = getTitleElement(widget.getDocument().getRootElement());
//					if (title != null){
//						net.sf.vex.dom.Document doc = title.getDocument();
//						int start = title.getStartOffset() + 1;
//						int end = title.getEndOffset();
//						doc.delete(start, end);
//						doc.insertText(start, newName);
//						widget.moveBy(newName.length() - (end - start));
//					}
//					
//
//					page.activate(activeEditor);
//					
//					return true;
//				}
//			}
//		}
//		
//		
//		page.activate(activeEditor);
//		return false;
//	}
//	
	
//	private net.sf.vex.dom.Element getTitleElement(net.sf.vex.dom.Element parent){
//		if (parent.getName().equals("title")){
//			return parent;
//		}
//
//		for (net.sf.vex.dom.Element element: parent.getChildElements()) {
//			net.sf.vex.dom.Element result = getTitleElement(element);
//			if (result != null)
//				return result;
//		}
//		
//		return null;
//	}
	
//	/**
//	 * Updates the title element of a closed chapter.
//	 * @param fragment
//	 * @param newName
//	 */
//	private void updateClosedChapter(Fragment fragment, String newName){
//		IRepository rep = RepositoryFactory.getRepository();
//		try {
//
//			// get document
//			DocumentBuilder docBuilder = OutputHelper.getDocumentBuilder();
//			Document doc = docBuilder.parse(rep.getInputStream(fragment));
//			Element element = doc.getDocumentElement();
//
//			// set title
//			Element title = (Element) element.getElementsByTagName("title")
//					.item(0);
//			title.setTextContent(newName);
//
//			// save document
//			OutputStream out = rep.getOutputStream(fragment);
//			DOMSource domSource = new DOMSource(doc);
//			StreamResult streamResult = new StreamResult(out);
//			TransformerFactory tf = TransformerFactory.newInstance();
//			Transformer serializer = tf.newTransformer();
//			IRepository repository = RepositoryFactory.getRepository();
//			Project project = repository.getProjectOf(fragment);
//			DeftProjectTypeRegistry deftProjectTypeRegistry = new DeftProjectTypeRegistry();			
//			serializer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM,	deftProjectTypeRegistry.findProjectType(project).getDtdId());
//			serializer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC,	deftProjectTypeRegistry.findProjectType(project).getDtdId());
//			serializer.transform(domSource, streamResult);
//			out.flush();
//			out.close();
//
//		} catch (Exception e) {
//			DeftLogger.log(Status.ERROR, "It was not possible to update a title.", e,
//					RenameFragmentAction.class);
//		}
//	}
//	



}
