package org.deft.actions;

import java.util.LinkedList;
import java.util.List;

import org.deft.RepositoryFacade;
import org.deft.gui.SelectionHelper;
import org.deft.gui.view.project.nodes.ArtifactNode;
import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.gui.view.project.nodes.FolderNode;
import org.deft.gui.view.project.nodes.ProjectNode;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.eclipse.ui.IWorkbenchWindow;

public class UpdateArtifactAction extends AbstractDeftAction {

	public UpdateArtifactAction(IWorkbenchWindow window) {
		super(window);
		setText("Update");
	}

	@Override
	protected void setEnabledState() {
		boolean onlyArtifactRelatedNodes = areOnlyArtifactRelatedNodesSelected();
		setEnabled(onlyArtifactRelatedNodes);
	}
	

	private boolean areOnlyArtifactRelatedNodesSelected() {
		SelectionHelper selection = getSelectionHelper();
		for (ProjectViewNode node : selection.getSelectedProjectViewNodes()) {
			if (!isOrContainsArtifact(node)) {
				return false;
			}
		}
		return true;
	}
	
	
	private boolean isOrContainsArtifact(ProjectViewNode node) {
		if (node instanceof ArtifactNode) {
			return true;
		} else if (isArtifactFolder(node)) {
			return true;
		} else if (isArtifactCategory(node)) {
			return true;
		}
		return false;
	}
	
	private boolean isArtifactFolder(ProjectViewNode node) {
		if (node instanceof FolderNode) {
			Folder f = ((FolderNode)node).getFolder();
			boolean isArtFolder = (f.getFragmentTypeAffiliation().equals(FragmentType.ARTIFACT));
			return isArtFolder;
		}
		return false;
	}
	
	
	private boolean isArtifactCategory(ProjectViewNode node) {
		if (node instanceof CategoryNode) {
			CategoryNode cn = (CategoryNode)node;
			boolean isArtCat = (cn.getType().equals(FragmentType.ARTIFACT));
			return isArtCat;
		}
		return false;
	}


	
    public void run() {
    	List<Artifact> selectedArtifacts = getDirectlyAndIndirectlySelectedArtifacts();

    	List<Project> projects = RepositoryFacade.instance().getProjects();
    	for (Project project : projects) {
    		int latestRevision = project.getLatestRevision();
    		int newRevision = latestRevision + 1;
        	List<Artifact> changedArtifacts = RepositoryFacade.instance().getArtifactsWithChangedOrigin(project);
        	for (Artifact changedArtifact : changedArtifacts) {
        		if (selectedArtifacts.contains(changedArtifact)) {
        			RepositoryFacade.instance().updateArtifact(changedArtifact, newRevision);
        			getEditorHelper().updateViewerIfOpen(changedArtifact);
        		}
        	}
    	}
    }
    
    
    public void updateArtifact(Artifact artifact) {
    	Project project = artifact.getProject();
    	List<Artifact> changedArtifacts = RepositoryFacade.instance().getArtifactsWithChangedOrigin(project);
    	if (changedArtifacts.contains(artifact)) {
    		int latestRevision = project.getLatestRevision();
    		int newRevision = latestRevision + 1;
    		RepositoryFacade.instance().updateArtifact(artifact, newRevision);
    		getEditorHelper().updateViewerIfOpen(artifact);
    	}
    }
    
    
    private List<Artifact> getDirectlyAndIndirectlySelectedArtifacts() {
    	List<Artifact> artifacts = new LinkedList<Artifact>();
    	SelectionHelper selection = getSelectionHelper();
    	for (ProjectViewNode node : selection.getSelectedProjectViewNodes()) {
    		if (node instanceof ArtifactNode) {
    			Artifact artifact = ((ArtifactNode)node).getArtifact(); 
    			artifacts.add(artifact);
    		} else if (isArtifactFolder(node)) {
    			Folder folder = ((FolderNode)node).getFolder();
    			Project project = folder.getProject();
    			ChildArtifactFilter filter = new ChildArtifactFilter(folder);
    			List<Fragment> folderDescendants = project.getFragments(filter);
    			addAll(artifacts, folderDescendants);
    		} else if (isArtifactCategory(node)) {
    			ProjectNode pn = ((CategoryNode)node).getParent();
    			Project project = pn.getProject();
    			List<Fragment> projectArtifacts = project.getFragments(FragmentFilter.ARTIFACTFILTER);
    			addAll(artifacts, projectArtifacts);
    		}
    	}
    	return artifacts;
    }
    	
    	
	private void addAll(List<Artifact> allArtifacts, List<Fragment> newArtifacts) {
    	for (Fragment f : newArtifacts) {
    		allArtifacts.add((Artifact)f);
    	}
    }
    	
    private class ChildArtifactFilter implements FragmentFilter {

    	private Folder folder;
    	
    	public ChildArtifactFilter(Folder folder) {
    		if (folder.getFragmentTypeAffiliation() == FragmentType.ARTIFACT) {
    			this.folder = folder;
    		} else {
    			throw new IllegalArgumentException("Fragment folder expected, but was of type " 
    					+ folder.getFragmentTypeAffiliation());
    		}
    	}
    	
		@Override
		public boolean accept(Fragment fragment) {
			if (fragment instanceof Artifact) {
				boolean doAccept = (folder != null && hasFolderAsAncestor((Artifact)fragment));
				return doAccept;
			}
			return false;
		}
		
		private boolean hasFolderAsAncestor(Artifact artifact) {
			Folder parent = artifact.getParentFolder();
			while (parent != null) {
				if (parent.equals(folder)) {
					return true;
				}
				parent = parent.getParentFolder();
			}
			return false;
		}
    	
    }

}
