package org.deft.actions.contributionitem;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ContributionItemFactory;

/**
 * Factory for various contribution items.
 * 
 * A contribution item represents a contribution to a shared UI resource such as a
 * menu or tool bar. More generally, contribution items are managed by a contribution
 * manager.
 * For instance, in a tool bar a contribution item is a tool bar button or a separator.
 * In a menu bar a contribution item is a menu, and in a menu a contribution item 
 * is a menu item or separator.
 */
public abstract class DeftContributionItemFactory {

	/**
	 * Id of contribution items created by this factory.
	 */
	private final String contributionItemId;

	/**
	 * Creates a new workbench contribution item factory with the given id.
	 * 
	 * @param contributionItemId
	 *            the id of contribution items created by this factory
	 */
	protected DeftContributionItemFactory(String contributionItemId) {
		this.contributionItemId = contributionItemId;
	}

	/**
	 * Creates a new standard contribution item for the given workbench window.
	 * <p>
	 * A typical contribution item automatically registers listeners against the
	 * workbench window so that it can keep its enabled state up to date.
	 * Ordinarily, the window's references to these listeners will be dropped
	 * automatically when the window closes. However, if the client needs to get
	 * rid of a contribution item while the window is still open, the client
	 * must call IContributionItem#dispose to give the item an opportunity to
	 * unregister its listeners and to perform any other cleanup.
	 * </p>
	 * 
	 * @param window
	 *            the workbench window
	 * @return the workbench contribution item
	 */
	public abstract IContributionItem create(IWorkbenchWindow window);

	/**
	 * Returns the id of this contribution item factory.
	 * 
	 * @return the id of contribution items created by this factory
	 */
	public String getId() {
		return contributionItemId;
	}

	
	
	/* Deft ContributionItems */
	
	/**
	 * The submenu which contains all ExportActions
	 */
	public static final ContributionItemFactory EXPORT_MENU = new ContributionItemFactory(
			"org.deft.exportMenu") {

		public IContributionItem create(IWorkbenchWindow window) {

			if (window == null) {
				throw new IllegalArgumentException();
			}

			return new ExportMenu(window, getId());
		}
	};

	/**
	 * The submenu which contains all ImportActions
	 */
	public static final ContributionItemFactory IMPORT_MENU = new ContributionItemFactory(
			"org.deft.importMenu") {

		public IContributionItem create(IWorkbenchWindow window) {

			if (window == null) {
				throw new IllegalArgumentException();
			}

			return new ImportMenu(window, getId());
		}
	};
	
	
	/**
	 * The submenu which contains all UpdateArtifactWithActions
	 */
	public static final ContributionItemFactory UPDATE_WITH_MENU = new ContributionItemFactory(
			"org.deft.updateWithMenu") {

		public IContributionItem create(IWorkbenchWindow window) {

			if (window == null) {
				throw new IllegalArgumentException();
			}

			return new UpdateWithMenu(window, getId());
		}
	};

	public static final ContributionItemFactory NEW_MENU = new ContributionItemFactory(
			"org.deft.newMenu") {

		public IContributionItem create(IWorkbenchWindow window) {

			if (window == null) {
				throw new IllegalArgumentException();
			}

			return new NewMenu(window, getId());
		}
	};

	/**
	 * The submenu which contains all InsertActions
	 */
	public static final ContributionItemFactory INSERT_MENU = new ContributionItemFactory(
			"org.deft.insertMenu") {

		public IContributionItem create(IWorkbenchWindow window) {

			if (window == null) {
				throw new IllegalArgumentException();
			}

			return new InsertMenu(window, getId());
		}
	};
	

	
	public static final ContributionItemFactory VIEW_MENU = new ContributionItemFactory(
			"org.deft.viewMenu") {

		public IContributionItem create(IWorkbenchWindow window) {

			if (window == null) {
				throw new IllegalArgumentException();
			}

			return new ViewMenu(window, getId());
		}
	};

}
