package org.deft.artifacttypesupport.codefile.java.parser;

import java.io.CharArrayReader;
import java.io.IOException;
import java.io.InputStream;
import java.nio.CharBuffer;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.ParserRuleReturnScope;
import org.antlr.runtime.RecognitionException;
import org.deft.artifacttypesupport.codefile.AbstractParser;
import org.deft.artifacttypesupport.codefile.PositionConverter;
import org.deft.artifacttypesupport.codefile.java.parser.antlradapter.DeftTreeAdaptor;
import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.ast.AstCombiner;
import org.deft.repository.ast.MergeConfiguration;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.exception.DeftParseException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.eclipse.jdt.core.compiler.IProblem;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.CompilationUnit;

public class JavaParser extends AbstractParser implements Validator {

	private static final String defaultSourceLevel = "1.5";
	
	private String sourceLevel = defaultSourceLevel;
	
	@Override
	public void validate(InputStream is) throws DeftParseException {
		CharBuffer buffer = getContent(is);
		getAst(buffer);
	}


	@Override
	public TreeNodeRoot getAst(CharBuffer content) throws DeftParseException {
		//make position converter
		PositionConverter pConv = getPositionConverter(content);		
		TreeNode javaTree = getJavaTree(content, pConv);
		TreeNode commentTree = getCommentTree(content, pConv);

        //merge Java and comment tree 
		MergeConfiguration config = new MergeConfiguration();
        AstCombiner combiner = new AstCombiner();
        combiner.combineTrees(javaTree, commentTree, config);    	


//		TreeNodeRoot root = visitor.getRoot();
		TreeNodeRoot root = new TreeNodeRoot(javaTree);
		return root;
	}
	
	
	private TreeNode getJavaTree(CharBuffer content, PositionConverter pConv) throws DeftParseException {
		ASTParser parser = ASTParser.newParser(AST.JLS3);
		parser.setKind(ASTParser.K_COMPILATION_UNIT);
		parser.setSource(content.array());
	
		//set options AFTER source, otherwise it is discarded
		Map<String, String> options = new HashMap<String, String>();
		//compiler source level is per default 1.3, change it
		options.put("org.eclipse.jdt.core.compiler.source", sourceLevel); 
		parser.setCompilerOptions(options);
		
		CompilationUnit node = (CompilationUnit) parser.createAST(null);
		AstConverterVisitor visitor = new AstConverterVisitor(content.array(), pConv);
		//if there is an exception during parsing 
		try {
			node.accept(visitor);
		}
		catch (Exception e) {
			DeftParseException dpe = new DeftParseException("Could not parse", e);
			throw dpe;
		}
		//if there were parse errors
		if (node.getProblems().length > 0) {
			IProblem firstProblem = node.getProblems()[0];
			String errorMsg = "Line [" + firstProblem.getSourceLineNumber() + "]: " + 
					firstProblem.getMessage();
			DeftParseException dpe = new DeftParseException(errorMsg);
			throw dpe;
		}
		TreeNode tree = visitor.getTree();
		return tree;
	}
	
	private TreeNode getCommentTree(CharBuffer content, PositionConverter pConv) {
        TreeNode commentTree = null;
		try {
			CStyleCommentsLexer commentLexer = 
				new CStyleCommentsLexer(new ANTLRReaderStream(new CharArrayReader(content.array())));
	        CommonTokenStream commentTokens = new CommonTokenStream(commentLexer);
	        CStyleCommentsParser commentParser = new CStyleCommentsParser(commentTokens);
	        commentParser.setTreeAdaptor(new DeftTreeAdaptor(new CStyleCommentsMapping(pConv)));
            ParserRuleReturnScope commentRet = commentParser.compilationunit();
            commentTree = (TreeNode)commentRet.getTree();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RecognitionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return commentTree;
	}
	
	


	public String getSourceLevel() {
		return sourceLevel;
	}

	/**
	 * Sets the source level to use. The default source level is "1.5".
	 * @param sourceLevel The source level to use.
	 */
	public void setSourceLevel(String sourceLevel) {
		this.sourceLevel = sourceLevel;
	}


	@Override
	public List<OriginalFragmentLocation> getDependencyLocations(FragmentSource source, OriginalFragmentLocation location) {
		return Collections.emptyList();
	}


	@Override
	public MultiArtifactDependencies getMultiArtifactDependencies(
			FragmentSource source, OriginalFragmentLocation location) {
		return new MultiArtifactDependencies(location);
	}
}
