package org.deft.artifacttypesupport.codefile.java.transformer;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.deft.artifacttypesupport.codefile.transform.AbstractOutlineNodeFactory;
import org.deft.artifacttypesupport.codefile.transform.OutlineNode;
import org.deft.repository.ast.TreeNode;

public class JavaDefaultNodeFactory extends AbstractOutlineNodeFactory {
	
	private static String SBLOCK = "{ ... }";
	//contains (after initialization) all node types, for which a Block in the
	//outline node should be ignored (see createBlock)
	private Set<String> ignoreBlockFormatting;
	
	public OutlineNode createFromLabel(TreeNode node) {
		OutlineNode oNode = new OutlineNode(getTrimmedLabel(node),
				getPosition(node), node);
		return oNode;		
	}
	
	public OutlineNode createStatement(TreeNode node) {
		return createFromLabel(node);
	}
	
	public OutlineNode createOutlineDefault(TreeNode node) {
		return createNode(node);
	}
	
	public OutlineNode createFormatDefault(TreeNode node) {
		if (isTokenNode(node)) {
			return createFromLabel(node);
		} else {
			String name = "[" + node.getName() + "]";
			OutlineNode oNode = new OutlineNode(name, getPosition(node), node);
			return oNode;
		}
	}
	
	private String getEmbeddedStatementString(TreeNode node) {
		String sStatementText = "...";
		if (node.getName().equals("Block")) {
			sStatementText = getBlockString(node);
		} else if (node.getName().equals("EmptyStatement")) {
			sStatementText = ";";
		}
		return sStatementText;
	}
	
	private String getBlockString(TreeNode node) {
		if (node.getChildren().size() == 2) { //only { }
			return "{ }";
		} else { 							//statements between { and }
			return SBLOCK;
		}
	}
	
	public OutlineNode createIfStatement(TreeNode node) {
		//node contains 5 or 7 child nodes:
		//0: the keyword 'if'
		//1: the '(' character
		//2: the BooleanExpression
		//3: the ')' character
		//4: the IfSection
		//5 (optional): the keyword 'else'
		//6 (if 5 exists): the ElseSection
		System.out.println(node.getChildren());
		TreeNode tBoolean = node.getChild(2);
		TreeNode tIfSection = node.getChild(4);
		String sBoolean = getTrimmedLabel(tBoolean);
		String sIfStatementText = getEmbeddedStatementString(tIfSection.getFirstChild());
		String sNodeText = "if (" + sBoolean + ") " + sIfStatementText;
		//if there is an else section
		if (node.getChildCount() == 7) {
			TreeNode tElseSection = node.getChild(6);
			String sElseStatementText = getEmbeddedStatementString(tElseSection.getFirstChild());
			sNodeText = sNodeText + " else " + sElseStatementText;
		}
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;
	}
	
	public OutlineNode createIfSection(TreeNode node) {
		//node contains 1 child node
		//0: the embeddedStatement
		//the parent IfStatement might contain an ElseSection
		TreeNode eParent = node.getParent();
		List<TreeNode> nl = eParent.getChildByName("ElseSection");
		//if there is an else section, we want both an 'if' and an 'else' node 
		//containing the respective statements. Here we create the if node
		if (!nl.isEmpty()) {
			TreeNode tnStatement = node.getFirstChild();			
			String sNodeText = "if " + getEmbeddedStatementString(tnStatement);
			return new OutlineNode(sNodeText, getPosition(node), node);
		}
		//if there is no else section, we need neither 'if' nor 'else' node
		return null;
	}
	
	public OutlineNode createElseSection(TreeNode node) {
		//node contains 1 child node
		//0: the embeddedStatement
		TreeNode eStatement = node.getFirstChild();			
		String sNodeText = "else " + getEmbeddedStatementString(eStatement);
		return new OutlineNode(sNodeText, getPosition(node), node);
	}	
	
	public OutlineNode createWhileStatement(TreeNode node) {
		//node contains 5 nodes:
		//0: the keyword 'while'
		//1: the '(' character
		//2: the BooleanExpression
		//3: the ')' character
		//4: a statement/block
		TreeNode tBoolean = node.getChild(2);
		String sBoolean = getTrimmedLabel(tBoolean);
		String sStatementText = getEmbeddedStatementString(node.getLastChild());
		String sNodeText = "while (" + sBoolean + ") " + sStatementText;
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;		
	}
	
	public OutlineNode createDoStatement(TreeNode node) {
		//node contains 7 nodes:
		//0: the keyword 'do'
		//1: a statement/block
		//2: the keyword 'while'
		//3: the '(' character
		//4: the BooleanExpression
		//5: the ')' character
		//6: the ';' character
		TreeNode tBlock = node.getChild(1);
		TreeNode tBoolean = node.getChild(4);
		String sBoolean = getTrimmedLabel(tBoolean);
		String sStatementText = getEmbeddedStatementString(tBlock);
		String sNodeText = "do " + sStatementText + " while (" + sBoolean + ");";
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;
	}
	
	public OutlineNode createForStatement(TreeNode node) {
		//node contains 6-9 nodes:
		//0: the keyword 'for'
		//1: the '(' character
		//2 (optional): the for initializer (e.g. VariableDeclarationExpression: int i = 0) 
		//3: the ';' character
		//4 (optional): the for condition (e.g. InfixExpression: i < 0)
		//5: the ';' character
		//6 (optional): the for iterator (e.g. PostfixExpression: i++)
		//7: the ')' character
		//8: a statement
		List<TreeNode> lSemicolons = node.executeXPathQuery("SEMICOLON");
		int firstSemiIdx = lSemicolons.get(0).getSiblingIndex();
		int secondSemiIdx = lSemicolons.get(1).getSiblingIndex();
		TreeNode tForInitializer = node.getChild(firstSemiIdx - 1);
		if (tForInitializer.getName().equals("LPAREN")) {
			tForInitializer = null;
		}
		
		TreeNode tForCondition = node.getChild(secondSemiIdx - 1);
		if (tForCondition.getName().equals("SEMICOLON")) {
			tForCondition = null;
		}
		
		TreeNode tForIterator = node.getChild(secondSemiIdx + 1);
		if (tForIterator.getName().equals("RPAREN")) {
			tForIterator = null;
		}
		
		String sInit = tForInitializer == null ? "" : getTrimmedLabel(tForInitializer);
		String sCond = tForCondition == null ? "" : getTrimmedLabel(tForCondition);
		String sIter = tForIterator == null ? "" : getTrimmedLabel(tForIterator);

		String sStatementText = getEmbeddedStatementString(node.getLastChild());
		String sNodeText = "for (" + sInit + "; " + sCond + "; " + sIter + ") " +
				sStatementText;
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;
	}
	
	public OutlineNode createEnhancedForStatement(TreeNode node) {
		//node contains 8 child nodes:
		//0: the keyword 'for'
		//1: the '(' character
		//2: the SingleVariableDeclaration
		//3: the ':' character
		//4: the expression
		//5: the ')' character
		//6: a statement
		TreeNode tVarDec = node.getChild(2); //contains type and name of var
		TreeNode tType = tVarDec.getChild(0);
		TreeNode tIdent = tVarDec.getChild(1);
		TreeNode tExpr = node.getChild(4);
		String sType = getTrimmedLabel(tType);
		String sIdent = getTrimmedLabel(tIdent);
		String sExpr = getTrimmedLabel(tExpr);
		String sStatementText = getEmbeddedStatementString(node.getLastChild());
		String sNodeText = "for (" + sType + " " + sIdent + " : " + sExpr + 
				") " + sStatementText;
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;		
	}
	
	public OutlineNode createTryStatement(TreeNode node) {
		//node contains 2-* child nodes:
		//1: the TryBlock
		//2 (0-n): a CatchClause
		//3 (optional, but mandatory if no CatchClause): FinallyClause
		TreeNode tBlock = node.getFirstChild().getLastChild(); //block in TryClause
		String sBlock = getBlockString(tBlock);
		String sNodeText = "try " + sBlock;
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;
	}
	
	public OutlineNode createTryBlock(TreeNode node) {
		//node contains 2 child node
		//0: the TRY keyword
		//0: the block
		TreeNode tBlock = node.getChild(1);
		String sNodeText = "try " + getBlockString(tBlock);
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;
	}
	
	public OutlineNode createCatchClause(TreeNode node) {
		//node contains 5 child nodes
		//0: the keyword 'catch'
		//1: the '(' character
		//2: the exception declaration
		//3: the ')' character
		//4: the block
		TreeNode tBlock = node.getLastChild();
		String sBlock = getBlockString(tBlock);
		String sNodeText = "catch " + sBlock;
		List<TreeNode> nl = node.getChildren();

		TreeNode tException = nl.get(2);
		String sException = getTrimmedLabel(tException);
		sNodeText = "catch (" + sException + ") " + sBlock;

		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;
	}
	
	public OutlineNode createFinallyClause(TreeNode node) {
		//node contains 2 child nodes
		//0: the keyword 'finally'
		//1: the block
		TreeNode tBlock = node.getLastChild();
		String sBlock = getBlockString(tBlock);
		String sNodeText = "finally " + sBlock;
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node),node);
		return oNode;
	}
	
	
	public OutlineNode createSwitchStatement(TreeNode node) {
		//node contains 5 child nodes
		//0: the keyword 'switch'
		//1: the '(' character
		//2: an expression
		//3: the ')' character
		//4: the switch block
		TreeNode tExpr = node.getChildren().get(2);
		String sExpr = getTrimmedLabel(tExpr);
		TreeNode tBlock = node.getLastChild();
		String sBlock = getBlockString(tBlock);
		String sNodeText = "switch (" + sExpr + ") " + sBlock;
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node),node);
		return oNode;
	}
	
	public OutlineNode createSwitchSection(TreeNode node) {
		//node contains 2-n child nodes
		//0: the switch labels (1 or more)
		//1: the statement list
		List<TreeNode> switchLabels = node.executeXPathQuery("SwitchCase");
		//for all empty labels (i.e. all labels except the last one)
		StringBuilder sbCompleteLabel = new StringBuilder();
		sbCompleteLabel.append("case ");
		boolean firstLoop = true;
		for (TreeNode tSwitchCase : switchLabels) {
			String sLabel = getSwitchLabel(tSwitchCase);
			if (!firstLoop) {
				sbCompleteLabel.append(",");
			}
			firstLoop = false;
			sbCompleteLabel.append(sLabel);

		}
		TreeNode tStatementList = node.getLastChild(); //the Block or statement
		String sStatementList = getEmbeddedStatementString(tStatementList);
		sbCompleteLabel.append(": ").append(sStatementList);
		OutlineNode oStatementNode 
				= new OutlineNode(sbCompleteLabel.toString(), getPosition(node),node);
		return oStatementNode;
	}
	
	/**
	 * Returns the label of a SwitchCase. Handles both normal switch cases
	 * and "default" cases.
	 * @param node
	 * @return
	 */
	private String getSwitchLabel(TreeNode node) {
		TreeNode fc = node.getFirstChild();
		if (fc.getName().equals("DEFAULT")) {
			return "default";
		} else if (fc.getName().equals("CASE")) {
			TreeNode tLabelIdent = node.getChild(1);
			String sLabel = getTrimmedLabel(tLabelIdent);
			return sLabel;
		} else {
			throw new IllegalArgumentException("Not a valid SwitchCase node");
		}
	}
	

	public OutlineNode createSynchronizedStatement(TreeNode node) {
		//node contains 5 child nodes
		//0: the Modifiers node with the keyword 'synchronized'
		//1: the '(' character
		//2: an identifier or such (the lock)
		//3: the ')' character
		//4: a block
		TreeNode tLock = node.getChild(2);
		TreeNode tBlock = node.getChild(4);
		String sLock = getTrimmedLabel(tLock);
		String sStatementText = getEmbeddedStatementString(tBlock);
		String sNodeText = "synchronized (" + sLock + ") " + sStatementText;
		OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node), node);
		return oNode;
	}
	
	
	public OutlineNode createBlock(TreeNode node) {
		//we only want this as an extra node if it appears within a statement
		//the main Blocks of methods and such should not get an extra node
		TreeNode tnParent = node.getParent();
		if (ignoreBlockFormatting == null) {
			initStatementsSet();
		}
		if (!ignoreBlockFormatting.contains(tnParent.getName())) {
			String sNodeText = getBlockString(node);
			OutlineNode oNode = new OutlineNode(sNodeText, getPosition(node),node);
			return oNode;
		}
		return null;
	}
	
	/**
	 * Configures which Block nodes should not be displayed.
	 * Blocks should not be displayed for method declarations and stuff.
	 * Neither should they appear for statements such as WhileStatement, 
	 * ForStatement and stuff, because the block is already visualized
	 * (e.g.: while (true) { ... }   -> we don't need another node with
	 * the caption { ... }, so leave it out.
	 * 
	 * The configuration created here is used by method createBlock(TreeNode)
	 */
	private void initStatementsSet() {		
		 ignoreBlockFormatting = new HashSet<String>();
		 ignoreBlockFormatting.add("MethodDeclaration");
		 ignoreBlockFormatting.add("ConstructorDeclaration");
		 ignoreBlockFormatting.add("EnumDeclaration");
		 ignoreBlockFormatting.add("TryBlock");
		 ignoreBlockFormatting.add("CatchClause");
		 ignoreBlockFormatting.add("FinallyClause");
		 ignoreBlockFormatting.add("ForStatement");
		 ignoreBlockFormatting.add("EnhancedForStatement");
		 ignoreBlockFormatting.add("WhileStatement");
		 ignoreBlockFormatting.add("DoStatement");
		 ignoreBlockFormatting.add("SynchronizedStatement");
	}
	
	
	//the following are for the Format tree
	
	
	public OutlineNode createLBrace(TreeNode node){
		OutlineNode oNode = new OutlineNode("{",getPosition(node),node);
		return oNode;
	}
	
	public OutlineNode createRBrace(TreeNode node){
		OutlineNode oNode = new OutlineNode("}",getPosition(node),node);
		return oNode;
	}
	

	
}
	

