package org.deft.artifacttypesupport.codefile.transform;

import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeWalker;
import org.deft.repository.ast.transform.ITreeNodeFilter;



public class TreeConverter {

	private INodeGrouper grouper;
	private NodeCreator factory;
	private boolean firstCall;
	private boolean dummyCreated;

	public TreeConverter() {
		// TODO Auto-generated constructor stub
	}

	/**
	 * Converts a xml node into a structure of TreeNodes. 
	 * @param node - the xml node that should be processed
	 * @param factory - describes how to create a TreeNode (or a list of TreeNodes) from a xml node. This
	 * argument is not allowed to be null.
	 * @param filter - describes which xml nodes should be processed. If this is null, all nodes will be converted
	 * @param grouper - describes if a special node should be grouped under TreeNode with the given name. This
	 * argument can be null.
	 * @return a NodeList<TreeNode>, which is a list of TreeNodes. This must be a list because you can create more than
	 * one TreeNode out of one xml node. The creation process is driven by the given INodeFactory.
	 */
	public NodeList<TreeNode> convert(TreeNode node, NodeCreator factory, 
			ITreeNodeFilter filter, INodeGrouper grouper) {

		if (factory == null) {
			throw new IllegalArgumentException("NodeFactory must not be null");
		}
		if (node == null) {
			throw new IllegalArgumentException("Node must not be null");
		}
		
		this.grouper = grouper;
		this.factory = factory;
		this.firstCall = true;
		this.dummyCreated = false;

		TreeWalker treeWalker = new TreeWalker(node, filter);
		NodeList<TreeNode> result = processNode(treeWalker);
		
		//remove the dummy root node that was created in getNodeList
		if (dummyCreated){
			TreeNode root = result.getParent();
			result = new NodeList<TreeNode>();
			for (TreeNode tn : root.getChildren()) {
				result.add(tn);
			}
		}
		firstCall = true;

		return result;
	}

	private NodeList<TreeNode> processNode(TreeWalker tw) {
		// get current node
		TreeNode n = tw.getCurrentNode();

		// get node list
		NodeList<TreeNode> nodeList = getNodeList(n);
		NodeList<TreeNode> result = nodeList;

		// add all children to parent node
		for (TreeNode child = tw.firstChild(); child != null; child = tw
				.nextSibling()) {

			NodeList<TreeNode> childNodeList = processNode(tw);

			if (nodeList == null) {
				if (result == null)
					result = new NodeList<TreeNode>();
				result.addAll(childNodeList);

			} else {
				// add all child nodes in the child node list to the parent of
				// the node list
				if (childNodeList != null) {
					for (TreeNode childNode : childNodeList.getElements()) {
						TreeNode parent = nodeList.getParent();

						// handle grouping
						TreeNode grouper = getGrouperNode(parent, child);
						if (grouper != null)
							parent = grouper;
						
						parent.addChild(childNode);
					}
				}
			}
		}

		// reset current node
		tw.setCurrentNode(n);

		return result;
	}



	private NodeList<TreeNode> getNodeList(TreeNode node) {
		// create nodes
		NodeList<TreeNode> result = factory.createNode(node);
		
		// add all information to all the nodes
		//addTemplateInformation(result, node);

		// this is a hack to realize grouping. this dummy root is deleted after the processing of
		// all nodes is finished.
		if (firstCall){
			firstCall = false;
			if (result == null){
				result = new NodeList<TreeNode>(new TreeNode("_dummy"));
				dummyCreated = true;
			}
		}
		
		
		return result;
	}

	
	private TreeNode getGrouperNode(TreeNode parent, TreeNode node){
		if (grouper == null)
			return null;
		
		String groupName = grouper.getGroupName(node);
		// this node should not be grouped
		if (groupName == null || groupName.length() == 0)
			return null;
		
		// the grouper node was already created
		for (TreeNode child : parent.getChildren()) {
			if (child.getName().equals(groupName)){
				return child;
			}
		}
		
		// create the grouper node
		TreeNode grouper = factory.createGroupingNode(groupName, node);
		parent.addChild(grouper);
		return grouper;
	}
}
