package org.deft.artifactviewer;

import java.util.regex.Pattern;

import org.deft.RepositoryFacade;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Fragment;
import org.deft.representationrenderer.RepresentationRenderer;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.ui.IElementFactory;
import org.eclipse.ui.IMemento;

/**
 * Factory creates an ArtifactViewerInput, so editors that have been open when DEFT was closed
 * can be restored when DEFT is started the next time.
 * 
 * This class must be registered at the org.eclipse.ui.elementFactories extension point.
 */
public class ArtifactViewerInputFactory implements IElementFactory {

	//the id used in the extension point
	public static final String ID = "org.deft.artifactviewer.ArtifactViewerInputFactory";
	
	
	/**
	 * This method tries to return a valid ArtifactViewerInput. It looks into
	 * the given memento for a previously stored value. This value should be the
	 * id of an Artifact. If it finds one, it tries to retrieve this Artifact
	 * from the Repository. If the Artifact is still in the Repository a new
	 * ArtifactViewerInput will be created out of it and returned. Otherwise null is returned.
	 */
	public IAdaptable createElement(IMemento memento) {
		String mementoString = memento.getString(ArtifactViewerInputImpl.KEY);
		String fragmentId = extractFragmentId(mementoString);
		String chainId = extractChainId(mementoString);
		String rendererId = extractRendererId(mementoString);
		
		// Try to retrieve the fragment with this id
		if (isValidFragment(fragmentId) && isValidChain(chainId) && isValidRenderer(rendererId)) {
			Fragment fragment = RepositoryFacade.instance().getFragment(fragmentId);
			OperationChain chain = RepositoryFacade.instance().getExtensionService().getOperationChain(chainId);
			RepresentationRenderer renderer = RepositoryFacade.instance().getExtensionService().getRepresentationRenderer(rendererId);
			if (isExistingArtifact(fragment) && isExistingChain(chain) && isExistingRenderer(renderer)) {
				Artifact artifact = (Artifact)fragment;
				return new ArtifactViewerInputImpl(artifact, chain, renderer);
			}
		}

		return null;
	}
	
	
	private String extractFragmentId(String mementoString) {
		return extractIdAtPos(mementoString, 0); 
	}
	
	private String extractChainId(String mementoString) {
		return extractIdAtPos(mementoString, 1); 
	}
	
	private String extractRendererId(String mementoString) {
		return extractIdAtPos(mementoString, 2); 
	}
	
	private String extractIdAtPos(String string, int pos) {
		Pattern p = Pattern.compile("::");
		String[] items = p.split(string);
		return items[pos];
	}
	
	
	
	private boolean isValidFragment(String fragmentId) {
		return fragmentId != null && !fragmentId.equals("");
	}
	
	private boolean isValidChain(String chainId) {
		return chainId != null && !chainId.equals("");
	}
	
	private boolean isValidRenderer(String rendererId) {
		return rendererId != null && !rendererId.equals("");
	}
	
	
	
	private boolean isExistingArtifact(Fragment fragment) {
		return fragment != null && fragment instanceof Artifact;
	}
	
	private boolean isExistingChain(OperationChain chain) {
		return chain != null;
	}
	
	private boolean isExistingRenderer(RepresentationRenderer renderer) {
		return renderer != null;
	}
	
}
