package org.deft.artifactviewer;

import java.util.LinkedList;
import java.util.List;

import org.deft.RepositoryFacade;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.representationrenderer.RepresentationRenderer;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistableEditor;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.MultiPageEditorPart;

public class MultiPageArtifactViewerPart extends MultiPageEditorPart implements ArtifactViewerPart, IPersistableEditor {

	public final static String ID = "org.deft.artifactviewer.multipage";
	private EclipseEditorRegistry editorReg = new EclipseEditorRegistry();
	
	/**
	 * A list that stores information about all pages except the first. These information are used
	 * for saving and restoring the editor state. By default only the first page is restored (that's why
	 * we don't need to save data about the first page). 
	 */
	private List<PageData> pageData = new LinkedList<PageData>();
	
	/**
	 * List to store information from the mementos when the editor is restored. With this information
	 * all editor pages will be restored.
	 */
	private List<PageData> pagesToBeAdded = new LinkedList<PageData>();
	private int activePageForRestore;

	
	private class PageData {
		private String viewerId, chainId, rendererId;
	}
	
	
	
	@Override
	public Artifact getArtifact() {
		ArtifactViewerInput input = (ArtifactViewerInput)getEditorInput();
		Artifact artifact = input.getArtifact();
		return artifact;
	}

	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		super.init(site, input);
		setPartName();
	}
	
	private void setPartName() {
		Artifact artifact = getArtifact();
		super.setPartName(artifact.getName());
	}
	
	
	@Override
	protected void createPages() {
		ArtifactViewerInput initialInput = (ArtifactViewerInput)getEditorInput();		
		RepresentationRenderer renderer = initialInput.getRenderer();
		OperationChain chain = initialInput.getChain();
		ArtifactViewerConnector connector = RepositoryFacade.instance().getExtensionService().getArtifactViewerConnectorForRendererId(renderer.getId());
		
		IEditorPart editor = editorReg.getEditor(connector.getIdOfEditorToBeOpened());

		try {
			addPage(editor, getEditorInput());
			setPageText(0, chain.getDescription());
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		
		//if the editor is being restored with mementos, there might be pages that need to be added
		//(see method restorePage(IMemento)). 
		while (!pagesToBeAdded.isEmpty()) {
			PageData nextPd = pagesToBeAdded.remove(0);
			addPage(nextPd.viewerId, nextPd.chainId, nextPd.rendererId);
		}
		setActivePage(activePageForRestore);
	}
	
	
	public void addPage(String artifactViewerId, String chainId, String rendererId) {
		Artifact artifact = getArtifact();
		OperationChain chain = RepositoryFacade.instance().getExtensionService().getOperationChain(chainId);
		RepresentationRenderer renderer = RepositoryFacade.instance().getExtensionService().getRepresentationRenderer(rendererId);
		ArtifactViewerInput input = new ArtifactViewerInputImpl(artifact, chain, renderer);

		IEditorPart editor = editorReg.getEditor(artifactViewerId);
		if (editor != null) {
			try {
				int pos = addPage(editor, input);
				setPageText(pos, chain.getDescription());
			} catch (PartInitException e) {
				e.printStackTrace();
			}
		}
		savePageData(artifactViewerId, chainId, rendererId);
	}
	
	private void savePageData(String viewerId, String chainId, String rendererId) {
		PageData pd = new PageData();
		pd.viewerId = viewerId;
		pd.chainId = chainId;
		pd.rendererId = rendererId;
		pageData.add(pd);
	}
	
	
	public void addPageDeferred(String artifactViewerId, String chainId, String rendererId) {
		
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
	}

	@Override
	public void doSaveAs() {
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void saveState(IMemento memento) {
		int pageCount = pageData.size(); //the number of pages to save: all pages except the first which is saved and restored by default
		for (int i = 0; i < pageCount; i++) {
			savePage(memento, i);
		}
		int activePage = getActivePage();
		memento.putInteger("activepage", activePage);
	}
	
	private void savePage(IMemento memento, int page) {
		PageData pd = pageData.get(page);
		IMemento mPage = memento.createChild("page");
		mPage.putString("viewerid", pd.viewerId);
		mPage.putString("chainid", pd.chainId);
		mPage.putString("rendererid", pd.rendererId);
	}
	

	@Override
	public void restoreState(IMemento memento) {
		IMemento[] mPages = memento.getChildren("page");
		for (IMemento m : mPages) {
			restorePage(m);
		}
		activePageForRestore = memento.getInteger("activepage");
	}

	/**
	 * Retrieves data to restore a single page from a memento. 
	 * The page is not restored immediately because the editor will
	 * not be ready when this method is called. Instead the data is saved
	 * in the local pagesToBeAdded list which will be processed when the
	 * time has come (see method createPages)
	 */
	private void restorePage(IMemento memento) {
		String viewerId = memento.getString("viewerid");
		String chainId = memento.getString("chainid");
		String rendererId = memento.getString("rendererid");
		PageData pd = new PageData();
		pd.viewerId = viewerId;
		pd.chainId = chainId;
		pd.rendererId = rendererId;
		
		pagesToBeAdded.add(pd);
	}

}
