package org.deft.editorintegrator.latex;

import java.util.List;

import org.deft.projecttype.texlipse.LatexEditorChapterDocument;
import org.deft.projecttype.texlipse.TexlipseArtifactReferencePosition;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.styleddocument.DocumentContentContainerRepresentation;
import org.deft.representation.styleddocument.conversion.LatexMapping;
import org.deft.representation.styleddocument.data.ConditionalCrossReference;
import org.deft.representation.styleddocument.data.ContentSequence;
import org.deft.representation.styleddocument.data.CrossReference;
import org.deft.representation.styleddocument.data.DefinitionListing;
import org.deft.representation.styleddocument.data.DocumentContent;
import org.deft.representation.styleddocument.data.DocumentContentContainer;
import org.deft.representation.styleddocument.data.SimpleListing;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.util.StyledDocumentUtil;

public class DocumentContentIntoLatexIntegrator extends AbstractLatexIntegrator {

	public static final String ID = "org.deft.editorintegrator.documentcontent4latex";

	private LatexMapping mapping = new LatexMapping();
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		DocumentContentContainer styledDoc = 
				((DocumentContentContainerRepresentation)representation).getDocumentContentContainer();
		List<DocumentContent> contents = styledDoc.getContents();
		StringBuilder sb = new StringBuilder();
		int indentLevel = 0;
		for (DocumentContent content : contents) {
			addContentToString(content, sb, indentLevel);
		}
		String string = removeLastLinebreakIfNecessary(sb.toString());
		TexlipseArtifactReferencePosition texPosition = (TexlipseArtifactReferencePosition)position;
		LatexEditorChapterDocument doc = texPosition.getChapterDocument();
		integrateDirective(doc, texPosition.getPosition(), string, referenceId);

	}
	
	
	private void addContentToString(DocumentContent content, StringBuilder sb, int indentLevel) {
		if (content instanceof ContentSequence) {
			ContentSequence sequence = (ContentSequence)content;
			for (DocumentContent dc : sequence) {
				addContentToString(dc, sb, indentLevel);
			}
		} else if (content instanceof TokenizedStyledDocument) {
			addContentToString((TokenizedStyledDocument)content, sb, indentLevel);
		} else if (content instanceof DefinitionListing) {
			addContentToString((DefinitionListing)content, sb, indentLevel);
		} else if (content instanceof SimpleListing) {
			addLinebreakIfNecessary(sb);
			addContentToString((SimpleListing)content, sb, indentLevel);
			addLinebreakIfNecessary(sb);
		} else if (content instanceof ConditionalCrossReference) {
			addContentToString((ConditionalCrossReference)content, sb, indentLevel);
		} else if (content instanceof CrossReference) {
			addContentToString((CrossReference)content, sb, indentLevel);
		}

	}
	
	
	private void addContentToString(TokenizedStyledDocument tsd, StringBuilder sb, int indentLevel) {
		String latexString = tsd.getMarkedUpText(mapping);
		sb.append(latexString);
	}
	
	
	
	private void addContentToString(DefinitionListing listing, StringBuilder sb, int indentLevel) {
		if (listing.isEmpty()) {
			TokenizedStyledDocument tsd = listing.getEmptyListingMessage();
			addContentToString(tsd, sb, indentLevel);
		} else {
			sb.append("\\begin{description}\n");
			List<TokenizedStyledDocument> docs = listing.getTerms();
			for (int i = 0; i < docs.size(); i++) {
				TokenizedStyledDocument tsd = docs.get(i);
				DocumentContent explanation = listing.getExplanation(tsd);
				sb.append("\\item[").append(StyledDocumentUtil.asString(tsd)).append("] ");
				addContentToString(explanation, sb, indentLevel);
				if (i + 1 < docs.size()) {
					sb.append("\n");
				}
			}
			sb.append("\\end{description}\n");
		}
	}
	
	private void addContentToString(SimpleListing listing, StringBuilder sb, int indentLevel) {
		addIndentSpaces(sb, indentLevel);
		sb.append("\\begin{itemize}\n");
		List<DocumentContent> items = listing.getListItems();
		for (int i = 0; i < items.size(); i++) {
			DocumentContent item = items.get(i);
			addIndentSpaces(sb, indentLevel + 1);
			sb.append("\\item ");
			addContentToString(item, sb, indentLevel + 1);
			if (!isStructuralContent(item)) {
				addLinebreakIfNecessary(sb);
			}
		}
		addIndentSpaces(sb, indentLevel);
		sb.append("\\end{itemize}");
	}
	

	private void addContentToString(ConditionalCrossReference reference, StringBuilder sb, int indentLevel) {
		String target = reference.getTarget();
		String type = reference.getTargetType();
		String alternativeText = reference.getAlternativeText();
		sb.append("\\eachlabelcase{{").append(target).append("}")								//check for label to exist
				.append("{(see ").append(type).append("~\\ref{").append(target).append("})}") 	//render if label exists
				.append("{").append(alternativeText).append("}}");								//render if label does not exist
	}
	
	private void addContentToString(CrossReference reference, StringBuilder sb, int indentLevel) {
		String target = reference.getTarget();
		String type = reference.getTargetType();
		sb.append("(see ").append(type).append("~\\ref{").append(target).append("})");
	}
	

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getInputRepresentationType() {
		return DocumentContentContainerRepresentation.TYPE;
	}
	
	@Override
	public String getDescription() {
		return "Text Content";
	}
	
	
	private void addIndentSpaces(StringBuilder sb, int indentLevel) {
		for (int i = 0; i < indentLevel; i++) {
			sb.append("    "); //4 spaces for each level
		}
	}
	
	
	private boolean isStructuralContent(DocumentContent content) {
		return content instanceof DefinitionListing
				|| content instanceof SimpleListing;
	}

	
	private void addLinebreakIfNecessary(StringBuilder sb) {
		if (sb.lastIndexOf("\n") + 1 != sb.length()) {
			sb.append("\n");
		}
	}
	
	
	private String removeLastLinebreakIfNecessary(String string) {
		char lastChar = string.charAt(string.length() - 1);
		if (lastChar == '\n') {
			String s = string.substring(0, string.length() - 1);
			return s;
		} else {
			return string;
		}
	}


}
