package org.deft.editorintegrator.latex;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.deft.integrator.EditorChapterDocument;
import org.deft.projecttype.texlipse.LatexEditorChapterDocument;
import org.deft.projecttype.texlipse.TexlipseArtifactReferencePosition;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.styleddocument.StyledTextRepresentation;
import org.deft.representation.styleddocument.data.StyledToken;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Position;

public class StyledRunningTextIntoLatexIntegrator extends AbstractLatexIntegrator {

	public static final String ID = "org.deft.editorintegrator.styledrunningtext4latex";

	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		TokenizedStyledDocument styledDoc = ((StyledTextRepresentation)representation).getStyledDocument();
		TexlipseArtifactReferencePosition texPosition = (TexlipseArtifactReferencePosition)position;
		LatexEditorChapterDocument latexDoc = texPosition.getChapterDocument();
		addIdentifierToText(latexDoc, texPosition.getPosition(), styledDoc, artifactType, referenceId);
	}
	
	
	private void addIdentifierToText(LatexEditorChapterDocument latexDoc, 
			Position position, TokenizedStyledDocument styledDoc, 
			ArtifactType artifactType, String referenceId) {
		StyledToken token = styledDoc.getLine(0).get(0);
		String text = token.getText();
		String directive = buildDeftReferenceDirective(referenceId, artifactType.getTypeId(), text);
		try {
			latexDoc.getDocument().replace(position.getOffset(), 0, directive);
			latexDoc.setPosition(position.getOffset() + directive.length());
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}
	
	private String buildDeftReferenceDirective(String referenceId, String artifactTypeId, String text) {
		String hypenizedText = addSoftHyphens(text);
		String artifactIdent = buildArtifactTypeIdentForDirective(artifactTypeId);
		String directive = "\\deftreference" + artifactIdent 
				+ "{" + referenceId + "}{" + hypenizedText + "}";
		return directive;
	}
	
	private String buildArtifactTypeIdentForDirective(String artifactTypeId) {
		String artifactType = artifactTypeId.replaceAll("_", "");
		String[] split = artifactType.split("\\.");
		int length = split.length;
		if (length != 0) {
			String ident = split[length - 1]; //last element of array
			return ident;
		} else {
			//can only happen if artifactTypeId consists only of dots.
			return "";
		}
	}

	
	private String addSoftHyphens(String text) {
		String hyphenRegex = "\\\\-";
		String hyphen = "\\-";
		String result = text.replaceAll("(?=\\p{Upper})", hyphenRegex);
		if (result.startsWith(hyphen)) {
			result = result.substring(hyphen.length());
		}
		return result;
	}
	
	
	@Override
	public TexlipseArtifactReferencePosition getPositionForRepresentation(
			EditorChapterDocument chapterDocument, String referenceId) {
		LatexEditorChapterDocument latexChapterDoc = (LatexEditorChapterDocument)chapterDocument;
		String text = latexChapterDoc.getDocument().get();
		String completeDirective = getCompleteDirective(text, referenceId);
		if (completeDirective != null) {
			int startOffset = text.indexOf(completeDirective);	//where the directive string begins
			int integratedTextLength = completeDirective.length();
			Position pos = new Position(startOffset, integratedTextLength);
			TexlipseArtifactReferencePosition artRefPos = new TexlipseArtifactReferencePosition(latexChapterDoc, pos);
			return artRefPos;
		} else {
			return null;
		}
	}

	private String getCompleteDirective(String text, String referenceId) {
		String anythingButBracesAndSpaces = "[^\\{\\}\\s]+?"; 
		String regex = "\\\\deftreference(" + anythingButBracesAndSpaces + ")\\{" + referenceId + "\\}\\{(.*?)\\}";
		Pattern pattern = Pattern.compile(regex);
		Matcher matcher = pattern.matcher(text);
		while (matcher.find()) {
			String type = matcher.group(1);
			String writtenText = matcher.group(2);
			return "\\deftreference" + type + "{" + referenceId + "}{" + writtenText +"}";
		}
		return null;
	}


	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getInputRepresentationType() {
		return StyledTextRepresentation.TYPE;
	}
	
	@Override
	public String getDescription() {
		return "Running Text";
	}
}
