package org.deft.editorintegrator.odftext;

import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import org.deft.integrator.AbstractEditorIntegrator;
import org.deft.integrator.EditorChapterDocument;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.projecttype.openoffice.OdfTextArtifactReferencePosition;
import org.deft.projecttype.openoffice.OdfTextChapterDocument;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.image.BufferedImageRepresentation;
import org.deft.util.ImageUtil;

import ag.ion.bion.officelayer.internal.text.TextRange;
import ag.ion.bion.officelayer.text.ITextContent;
import ag.ion.bion.officelayer.text.ITextContentService;
import ag.ion.bion.officelayer.text.ITextDocument;
import ag.ion.bion.officelayer.text.ITextDocumentImage;
import ag.ion.bion.officelayer.text.ITextRange;
import ag.ion.bion.officelayer.text.TextException;
import ag.ion.noa.NOAException;
import ag.ion.noa.graphic.GraphicInfo;

import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XNameAccess;
import com.sun.star.container.XNamed;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.text.HoriOrientation;
import com.sun.star.text.TextContentAnchorType;
import com.sun.star.text.VertOrientation;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextGraphicObjectsSupplier;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.Any;
import com.sun.star.uno.UnoRuntime;

/**
 * Integrates an image (BufferedImage) into an ODFText document using the OpenOffice API.
 */
public class ImageIntoODFTextIntegrator extends AbstractEditorIntegrator {

	public static final String ID = "org.deft.editorintegrator.image4odftext";
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		BufferedImageRepresentation imgRep = (BufferedImageRepresentation)representation;
		OdfTextArtifactReferencePosition odfPosition = (OdfTextArtifactReferencePosition)position;
		OdfTextChapterDocument doc = odfPosition.getChapterDocument();
		insertWithImageShape(doc, odfPosition.getTextRange(), imgRep, referenceId);
	}
	
	
	private void insertWithImageShape(OdfTextChapterDocument doc, 
			ITextRange textRange, BufferedImageRepresentation imgRep, String integrationId) {
		ITextDocument textDoc = (ITextDocument)doc.getDocument();
		BufferedImage img = imgRep.getImage();
		GraphicInfo graphicInfo = createGraphicInfo(textDoc, img, true);
		ITextDocumentImage docImage = insertImage(textDoc, graphicInfo, textRange);
		setFrameId(docImage.getXTextContent(), integrationId);
	}
	
	
	/**
	 * Sets the ID of the frame which contains the text. The ID is necessary to
	 * find the frame later. 
	 */
	private void setFrameId(XTextContent textContent, String id) {
		XNamed xNamed = (XNamed)UnoRuntime.queryInterface(XNamed.class, textContent);
		xNamed.setName(id);
	}
	
	private ITextDocumentImage insertImage(ITextDocument textDoc, 
			GraphicInfo graphicInfo, ITextRange textRange) {
		ITextContentService textContentService = textDoc.getTextService().getTextContentService();
		try {
			ITextDocumentImage textDocumentImage = textContentService.constructNewImage(graphicInfo);
			textContentService.insertTextContent(textRange, textDocumentImage);
			return textDocumentImage;
		} catch (TextException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	
	
	private GraphicInfo createGraphicInfo(ITextDocument textDoc, BufferedImage image, boolean shrinkToFit) {
		try {
			InputStream is = ImageUtil.getInputStream(image);
			GraphicInfo graphicInfo = new GraphicInfo(is, image.getWidth(), true,
					image.getHeight(), true, VertOrientation.TOP, HoriOrientation.LEFT, 
					TextContentAnchorType.AS_CHARACTER);
			if (shrinkToFit) {
				int availableImgWidth = getAvailableImageWidth(textDoc, graphicInfo);
				int availableImgHeight = getAvailableImageHeight(textDoc, graphicInfo);
				//if image fits on page, just return the original GraphicInfo
				if (availableImgWidth == graphicInfo.getWidth() && availableImgHeight == graphicInfo.getHeight()) {
					return graphicInfo;
				}

				double origRatio = (double)image.getWidth() / (double)image.getHeight();
				double newRatio = (double)availableImgWidth / (double)availableImgHeight;
				int newWidth = -1;
				int newHeight = -1;
				if (newRatio < origRatio) { //if image width has been shrunk, shrink also height
					newWidth = availableImgWidth;
					newHeight = (int)Math.round((double)graphicInfo.getHeight() * newRatio / origRatio);
				} else { 					//if image height has been shrunk, shrink also width
					newHeight = availableImgHeight;
					newWidth = (int)Math.round((double)graphicInfo.getWidth() * origRatio / newRatio);
				}
				is = ImageUtil.getInputStream(image);
				graphicInfo = new GraphicInfo(is, newWidth, false,
						newHeight, false, graphicInfo.getVerticalAlignment(), graphicInfo.getHorizontalAlignment(), 
						graphicInfo.getAnchor());
				return graphicInfo;
			}
		} catch (NOAException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	private int getAvailableImageWidth(ITextDocument textDoc, GraphicInfo graphicInfo) {
		try {
			XPropertySet set = textDoc.getPageService().getPage(0).getPageStyle().getProperties().getXPropertySet();
			//width is given in 1/10mm
			int pageWidth = (Integer)set.getPropertyValue("Width");
			int pageLeftBorderWidth = (Integer)set.getPropertyValue("LeftMargin");
			int pageRightBorderWidth = (Integer)set.getPropertyValue("RightMargin");
			int spaceForGraphic = pageWidth - pageLeftBorderWidth - pageRightBorderWidth;
			 
			//graphicInfo also returns width in 1/10mm
			int graphicWidth = graphicInfo.getWidth();
			
			int min = Math.min(spaceForGraphic, graphicWidth);
			return min;
			
		} catch (UnknownPropertyException e) {
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			e.printStackTrace();
		} catch (TextException e) {
			e.printStackTrace();
		}
		return -1;
	}
	
	private int getAvailableImageHeight(ITextDocument textDoc, GraphicInfo graphicInfo) {
		try {
			XPropertySet set = textDoc.getPageService().getPage(0).getPageStyle().getProperties().getXPropertySet();
			//height is given in 1/10mm
			int pageWidth = (Integer)set.getPropertyValue("Height");
			int pageTopBorderWidth = (Integer)set.getPropertyValue("TopMargin");
			int pageBottomBorderWidth = (Integer)set.getPropertyValue("BottomMargin");
			int spaceForGraphic = pageWidth - pageTopBorderWidth - pageBottomBorderWidth;
			 
			//graphicInfo also returns height in 1/10mm
			int graphicHeight = graphicInfo.getHeight();
			
			int min = Math.min(spaceForGraphic, graphicHeight);
			return min;
			
		} catch (UnknownPropertyException e) {
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			e.printStackTrace();
		} catch (TextException e) {
			e.printStackTrace();
		}
		return -1;
	}
	
	@Override
	public OdfTextArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
		OdfTextArtifactReferencePosition pos = getPositionForRepresentation(document, referenceId);
		OdfTextChapterDocument odfDoc = (OdfTextChapterDocument)document;
		ITextDocument textDocument = (ITextDocument)odfDoc.getDocument();
		XTextGraphicObjectsSupplier textGraphicObjectsSupplier = 
			(XTextGraphicObjectsSupplier) UnoRuntime.queryInterface(XTextGraphicObjectsSupplier.class, 
			textDocument.getXTextDocument());
		XNameAccess graphics = textGraphicObjectsSupplier.getGraphicObjects();

		if (graphics.hasByName(referenceId)) {
			try {
				Any graphic = (Any) graphics.getByName(referenceId);
				XTextContent textContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, graphic);
				textDocument.getTextService().getTextContentService()
						.removeTextContent(new TextContent(textContent, textDocument));
				return pos;
			} catch (NoSuchElementException e) {
				e.printStackTrace();
			} catch (WrappedTargetException e) {
				e.printStackTrace();
			} catch (TextException e) {
				e.printStackTrace();
			}
		}
		return null;
	}
		
	
	private class TextContent implements ITextContent {

		private XTextContent xTextContent;
		private ITextDocument textDocument;

		public TextContent(XTextContent textContent, ITextDocument textDocument) {
			super();
			xTextContent = textContent;
			this.textDocument = textDocument;
		}

		@Override
		public XTextContent getXTextContent() {
			return xTextContent;
		}

		@Override
		public ITextDocument getTextDocument() {
			return textDocument;
		}

	}
	
	

	@Override
	public String getInputRepresentationType() {
		return BufferedImageRepresentation.TYPE;
	}

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("odftext");
		return ids;
	}

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getDescription() {
		return "Image";
	}


	@Override
	public OdfTextArtifactReferencePosition getPositionForRepresentation(EditorChapterDocument chapterDocument, String referenceId) {
		OdfTextChapterDocument odfTextChapterDocument = (OdfTextChapterDocument)chapterDocument;
		ITextDocument textDocument = odfTextChapterDocument.getDocument();
		XTextGraphicObjectsSupplier textGraphicObjectsSupplier = 
			(XTextGraphicObjectsSupplier) UnoRuntime.queryInterface(XTextGraphicObjectsSupplier.class, 
			textDocument.getXTextDocument());
		XNameAccess graphics = textGraphicObjectsSupplier.getGraphicObjects();
		try {
			if (graphics.hasByName(referenceId)) {
				Any graphic = (Any)graphics.getByName(referenceId);
				XTextContent textContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, graphic);
				XTextRange xTextRange = textContent.getAnchor();
				ITextRange textRange = new TextRange(textDocument, xTextRange);
				OdfTextArtifactReferencePosition position = new OdfTextArtifactReferencePosition(odfTextChapterDocument, textRange);
				return position;
			}
		} catch (NoSuchElementException e) {
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			e.printStackTrace();
		}
		return null;
	}


	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}

}
