package org.deft.editorintegrator.odftext;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.integrator.AbstractEditorIntegrator;
import org.deft.integrator.EditorChapterDocument;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.projecttype.openoffice.OdfTextArtifactReferencePosition;
import org.deft.projecttype.openoffice.OdfTextChapterDocument;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.styleddocument.StyledTextRepresentation;
import org.deft.representation.styleddocument.data.StyledToken;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;

import ag.ion.bion.officelayer.internal.text.TextRange;
import ag.ion.bion.officelayer.text.IBookmark;
import ag.ion.bion.officelayer.text.IBookmarkService;
import ag.ion.bion.officelayer.text.ICharacterProperties;
import ag.ion.bion.officelayer.text.ITextContent;
import ag.ion.bion.officelayer.text.ITextCursor;
import ag.ion.bion.officelayer.text.ITextDocument;
import ag.ion.bion.officelayer.text.ITextRange;
import ag.ion.bion.officelayer.text.IViewCursor;

import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XNameAccess;
import com.sun.star.container.XNamed;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.text.XBookmarksSupplier;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;

/**
 * Integrates styled text (StyledDocument) into an ODFText document using the OpenOffice API.
 * 
 * The text is surrounded by a frame.
 */
public class StyledRunningTextIntoODFTextIntegrator extends AbstractEditorIntegrator {

	public static final String ID = "org.deft.editorintegrator.styledrunningtext4odftext";
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		
		StyledTextRepresentation textRep = (StyledTextRepresentation)representation;
		OdfTextArtifactReferencePosition odfPosition = (OdfTextArtifactReferencePosition)position;
		OdfTextChapterDocument doc = odfPosition.getChapterDocument();
		addStyledText(textRep.getStyledDocument(), doc.getDocument(), referenceId);
	}
	
	private void addStyledText(TokenizedStyledDocument doc, ITextDocument textDoc, String referenceId) {
		ITextRange range = insertStyledToken(doc, textDoc);
		saveRangeAsBookmark(range, referenceId, textDoc);
	}
	
	private ITextRange insertStyledToken(TokenizedStyledDocument doc, ITextDocument textDoc) {
		StyledToken token = doc.getLine(0).get(0);
		String font = token.getFontFamily();
		String text = token.getText();
		IViewCursor viewCursor = textDoc.getViewCursorService().getViewCursor();
		ITextRange range = viewCursor.getStartTextRange();
		//save char properties
		Map<String, Object> originalCharProperties = saveCharProperties(viewCursor);
		ITextCursor textCursor = viewCursor.getTextCursorFromStart();
		//set char properties according to StyledToken style
		try {
			textCursor.getCharacterProperties().getXPropertySet().setPropertyValue("CharFontName", font);
		} catch (Exception e) {
			e.printStackTrace();
		}
		range.setText(text);
		//restore original char properties so for text to by typed next by the user
		restoreCharProperties(viewCursor, originalCharProperties);
		return range;
	}

	
	private void saveRangeAsBookmark(ITextRange range, String bookmarkName, ITextDocument textDoc) {
		try {
			XMultiServiceFactory mxDocFactory = (XMultiServiceFactory) UnoRuntime.queryInterface(
					XMultiServiceFactory.class, textDoc.getXTextDocument()); 
			 Object bookmark = mxDocFactory.createInstance ("com.sun.star.text.Bookmark");
			 
			 // name the bookmark
			 XNamed xNamed = (XNamed) UnoRuntime.queryInterface(XNamed.class, bookmark);
			 xNamed.setName(bookmarkName);
			 XTextContent xTextContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, bookmark);

			 textDoc.getXTextDocument().getText().insertTextContent(range.getXTextRange(), xTextContent, true);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	private Map<String, Object> saveCharProperties(IViewCursor cursor) {
		Map<String, Object> map = new HashMap<String, Object>();
		ICharacterProperties props = cursor.getTextCursorFromStart().getCharacterProperties();
		XPropertySet xProp = props.getXPropertySet();
		copy(xProp, map, "CharWeight"); //reset bold
		copy(xProp, map, "CharPosture"); //reset italic
		copy(xProp, map, "CharUnderline");
		copy(xProp, map, "CharColor");
		copy(xProp, map, "CharFontName");
		return map;
	}
	
	private void restoreCharProperties(IViewCursor cursor, Map<String, Object> map) {
		ICharacterProperties props = cursor.getTextCursorFromStart().getCharacterProperties();
		XPropertySet newXPropertySet = props.getXPropertySet();
		copy(map, newXPropertySet, "CharWeight"); //reset bold
		copy(map, newXPropertySet, "CharPosture"); //reset italic
		copy(map, newXPropertySet, "CharUnderline");
		copy(map, newXPropertySet, "CharColor");
		copy(map, newXPropertySet, "CharFontName"); 
	}
	
	private void copy(Map<String, Object> map, XPropertySet xProp, String property) {
		try {
			Object oldValue = map.get(property);
			xProp.setPropertyValue(property, oldValue);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private void copy(XPropertySet xProp, Map<String, Object> map, String property) {
		try {
			Object value = xProp.getPropertyValue(property);
			map.put(property, value);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	

	
	@Override
	public OdfTextArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
		//save position for return
		OdfTextArtifactReferencePosition position = getPositionForRepresentation(document, referenceId);
		ITextDocument textDocument = ((OdfTextChapterDocument)document).getDocument();
		//use NOA API to get bookmark and delete content 
		IBookmarkService bookmarkService = textDocument.getTextService().getBookmarkService();
		IBookmark bookmark = bookmarkService.getBookmark(referenceId);
		bookmark.setText("");
		//use UNO API to remove actual bookmark
		XBookmarksSupplier xBookmarksSupplier = (XBookmarksSupplier)UnoRuntime.queryInterface(
		XBookmarksSupplier.class, textDocument.getXComponent());
		XNameAccess xNamedBookmarks = xBookmarksSupplier.getBookmarks();
		Object foundBookmark;
		try {
			foundBookmark = xNamedBookmarks.getByName(referenceId);
			XTextContent xFoundBookmark = (XTextContent)UnoRuntime.queryInterface(XTextContent.class, foundBookmark);
			TextContent tc = new TextContent(xFoundBookmark, textDocument);
			
			textDocument.getTextService().getTextContentService().removeTextContent(tc);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return position;
	}
	
	private class TextContent implements ITextContent {

		private XTextContent xTextContent;
		private ITextDocument textDocument;

		public TextContent(XTextContent textContent, ITextDocument textDocument) {
			super();
			xTextContent = textContent;
			this.textDocument = textDocument;
		}

		@Override
		public XTextContent getXTextContent() {
			return xTextContent;
		}

		@Override
		public ITextDocument getTextDocument() {
			return textDocument;
		}

	}
		
	

	@Override
	public String getInputRepresentationType() {
		return StyledTextRepresentation.TYPE;
	}

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("odftext");
		return ids;
	}

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getDescription() {
		return "Running Text";
	}


	@Override
	public OdfTextArtifactReferencePosition getPositionForRepresentation(EditorChapterDocument chapterDocument, String referenceId) {
		OdfTextChapterDocument odfTextChapterDocument = (OdfTextChapterDocument)chapterDocument;
		ITextDocument textDocument = odfTextChapterDocument.getDocument();
		IBookmarkService bookmarkService = textDocument.getTextService().getBookmarkService();
		IBookmark bookmark = bookmarkService.getBookmark(referenceId);
		if (bookmark != null) {
			XTextRange xTextRange = bookmark.getXTextRange();
			ITextRange textRange = new TextRange(textDocument, xTextRange);
			OdfTextArtifactReferencePosition position = new OdfTextArtifactReferencePosition(odfTextChapterDocument, textRange);
			return position;
		}
		return null;
	}



	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}
}
