package org.deft.editorintegrator.odftext;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.integrator.AbstractEditorIntegrator;
import org.deft.integrator.EditorChapterDocument;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.projecttype.openoffice.OdfTextArtifactReferencePosition;
import org.deft.projecttype.openoffice.OdfTextChapterDocument;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.styleddocument.StyledDefinitionListingRepresentation;
import org.deft.representation.styleddocument.conversion.RtfMapping;
import org.deft.representation.styleddocument.data.DefinitionListing;
import org.deft.representation.styleddocument.data.DocumentContent;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.util.StyledDocumentUtil;

import ag.ion.bion.officelayer.filter.RTFFilter;
import ag.ion.bion.officelayer.internal.text.TextRange;
import ag.ion.bion.officelayer.text.ICharacterProperties;
import ag.ion.bion.officelayer.text.ITextContent;
import ag.ion.bion.officelayer.text.ITextContentService;
import ag.ion.bion.officelayer.text.ITextDocument;
import ag.ion.bion.officelayer.text.ITextDocumentTextShape;
import ag.ion.bion.officelayer.text.ITextRange;
import ag.ion.bion.officelayer.text.IViewCursor;
import ag.ion.bion.officelayer.text.TextException;
import ag.ion.noa.NOAException;
import ag.ion.noa.graphic.TextInfo;

import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XIndexAccess;
import com.sun.star.container.XIndexReplace;
import com.sun.star.container.XNameAccess;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.style.NumberingType;
import com.sun.star.table.BorderLine;
import com.sun.star.text.ControlCharacter;
import com.sun.star.text.HoriOrientation;
import com.sun.star.text.TextContentAnchorType;
import com.sun.star.text.VertOrientation;
import com.sun.star.text.XText;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextFramesSupplier;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.Any;
import com.sun.star.uno.UnoRuntime;

public class StyledTextDefinitionListingIntoODFTextIntegrator extends AbstractEditorIntegrator {

	public static final String ID = "org.deft.editorintegrator.styledtextdefinitionlisting4odftext";
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		
		StyledDefinitionListingRepresentation listingRep = (StyledDefinitionListingRepresentation)representation;
		OdfTextArtifactReferencePosition odfPosition = (OdfTextArtifactReferencePosition)position;
		OdfTextChapterDocument doc = odfPosition.getChapterDocument();
		insertTextShapeWithListing(doc, odfPosition.getTextRange(), listingRep.getListing(), referenceId);
	}
	
	
	private void insertTextShapeWithListing(OdfTextChapterDocument doc, 
			ITextRange textRange, DefinitionListing listing, String integrationId) {
		ITextDocument textDoc = (ITextDocument)doc.getDocument();
	
		TextInfo textInfo = new TextInfo(integrationId, 
				VertOrientation.TOP, HoriOrientation.LEFT, 0, true, true, 
				0, true, true, -1, TextContentAnchorType.AS_CHARACTER);
		
		ITextDocumentTextShape textShape = insertEmptyTextShape(textDoc, textInfo, textRange);
		removeBorders(textShape);
		if (listing.isEmpty()) {
			TokenizedStyledDocument message = listing.getEmptyListingMessage();
			insertEmptyListingMessageIntoEmptyTextShape(textDoc, textShape, message);
		} else {
			List<TokenizedStyledDocument> terms = listing.getTerms();
			Map<TokenizedStyledDocument, DocumentContent> termExplanationMap = listing.getMapping();
			insertBulletListIntoEmptyTextShape(textDoc, textShape, terms, termExplanationMap);
		}
	}
	

	
	private ITextDocumentTextShape insertEmptyTextShape(ITextDocument textDoc, 
			TextInfo textInfo, ITextRange textRange) {
		ITextContentService textContentService = textDoc.getTextService().getTextContentService();
		try {
			ITextDocumentTextShape textDocumentTextShape = textContentService.constructNewTextShape(textInfo);
			textContentService.insertTextContent(textRange, textDocumentTextShape);
			return textDocumentTextShape;
		} catch (TextException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	
	private void removeBorders(ITextDocumentTextShape textShape) {
		try {
			XTextContent xTextShape = textShape.getXTextContent();
			XPropertySet xProps = (XPropertySet) UnoRuntime.queryInterface(XPropertySet.class, xTextShape);
			
			BorderLine blTop = new BorderLine();
			blTop.Color = -1;
			BorderLine blBottom = new BorderLine();
			blBottom.Color = -1;
			BorderLine blLeft = new BorderLine();
			blLeft.Color = -1;
			BorderLine blRight = new BorderLine();
			blRight.Color = -1;

			xProps.setPropertyValue("TopBorder", blTop);
			xProps.setPropertyValue("TopMargin", 0);
			xProps.setPropertyValue("BottomBorder", blBottom);
			xProps.setPropertyValue("BottomMargin", 0);
			xProps.setPropertyValue("LeftBorder", blLeft);
			xProps.setPropertyValue("LeftMargin", 0);
			xProps.setPropertyValue("RightBorder", blRight);
			xProps.setPropertyValue("RightMargin", 0);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	
	
	private void insertEmptyListingMessageIntoEmptyTextShape(ITextDocument textDoc, ITextDocumentTextShape textShape,
			TokenizedStyledDocument emptyListingMessage) { 
		setCursorIntoTextShape(textDoc, textShape);
		addStyledText(emptyListingMessage, textDoc);
	}
	
	
	private void insertBulletListIntoEmptyTextShape(ITextDocument textDoc, 
			ITextDocumentTextShape textShape, 
			List<TokenizedStyledDocument> terms, Map<TokenizedStyledDocument, DocumentContent> termExplanationMap){

		setCursorIntoTextShape(textDoc, textShape);
		convertToBulletParagraph(textDoc, textShape);

		TokenizedStyledDocument divider = createDivider();
		for (int i = 0; i < terms.size(); i++) {
			TokenizedStyledDocument term = terms.get(i);
			DocumentContent explanation = termExplanationMap.get(term);
			
			addStyledText(term, textDoc);
			
			addStyledText(divider, textDoc);
			if (explanation instanceof TokenizedStyledDocument) {
				addStyledText((TokenizedStyledDocument)explanation, textDoc);
			}
			if (!isLast(i, terms)) {
				insertParagraphBreak(textShape);
			}
		}			

	}
	
	private void setCursorIntoTextShape(ITextDocument textDoc, ITextDocumentTextShape textShape) {
		IViewCursor viewCursor = textDoc.getViewCursorService().getViewCursor();
		XText xShapeText = textShape.getXText();
		XTextCursor xTextCursor = xShapeText.createTextCursor();
		ITextContentService textContentService = textDoc.getTextService().getTextContentService();
		try {
			ITextRange textRange = textContentService.constructNewTextRange(textDoc, xTextCursor.getStart());
			viewCursor.goToRange(textRange, false);
		} catch (TextException e) {
			e.printStackTrace();
		}
	}
	
	private boolean isLast(int i, List<?> list) {
		return i == list.size() - 1;
	}
	
	private void insertParagraphBreak(ITextDocumentTextShape textShape) {
		try {
			XText xShapeText = textShape.getXText();
			xShapeText.insertControlCharacter(xShapeText.getEnd(), ControlCharacter.PARAGRAPH_BREAK, false);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
	}
	
	private TokenizedStyledDocument createDivider() {
		TokenizedStyledDocument doc = new TokenizedStyledDocument();
		StyledDocumentUtil.add(doc, " - ");
		return doc;
	}
	
	private void addStyledText(TokenizedStyledDocument doc, ITextDocument textDoc) {
		try {
			IViewCursor cursor = textDoc.getViewCursorService().getViewCursor();		
			//save current font properties so we can restore them later
			XPropertySet originalCharProperties = getCurrentCharProperties(cursor);
			//insert RTF text
			String rtfString = doc.getMarkedUpText(new RtfMapping());
			InputStream is = new ByteArrayInputStream(rtfString.getBytes());
			cursor.getTextCursorFromStart().insertDocument(is, RTFFilter.FILTER);
			//restore font properties
			restoreCharProperties(cursor, originalCharProperties);

		} catch (NOAException e) {
			e.printStackTrace();
		} 
	}
	
	
	/**
	 * Returns a property set that contains the font properties at the current
	 * cursor position.
	 */
	private XPropertySet getCurrentCharProperties(IViewCursor cursor) {
		ICharacterProperties props = cursor.getTextCursorFromStart().getCharacterProperties();
		XPropertySet currentPropertySet = props.getXPropertySet();
		return currentPropertySet;
	}
	
	
	private void restoreCharProperties(IViewCursor cursor, XPropertySet origXPropertySet) {
		ICharacterProperties props = cursor.getTextCursorFromStart().getCharacterProperties();
		XPropertySet newXPropertySet = props.getXPropertySet();
		copy(origXPropertySet, newXPropertySet, "CharWeight"); //reset bold
		copy(origXPropertySet, newXPropertySet, "CharPosture"); //reset italic
		copy(origXPropertySet, newXPropertySet, "CharUnderline");
		copy(origXPropertySet, newXPropertySet, "CharColor");
		copy(origXPropertySet, newXPropertySet, "CharFontName"); 
	}
	
	private void copy(XPropertySet oldSet, XPropertySet newSet, String property) {
		try {
			Object oldValue = oldSet.getPropertyValue(property);
			newSet.setPropertyValue(property, oldValue);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	

	private void convertToBulletParagraph(ITextDocument textDoc, ITextDocumentTextShape textShape) {
		
		try {
			XText xText = textShape.getXText();
			XTextCursor tc = xText.createTextCursor();

			XTextDocument mxDocText = textDoc.getXTextDocument();
			XMultiServiceFactory mxDocFactory = (XMultiServiceFactory) UnoRuntime.queryInterface( XMultiServiceFactory.class, mxDocText);
			 
			XIndexAccess xNum = (XIndexAccess) UnoRuntime.queryInterface(XIndexAccess.class, mxDocFactory .createInstance("com.sun.star.text.NumberingRules"));
			XIndexReplace xReplace = (XIndexReplace) UnoRuntime.queryInterface(XIndexReplace.class, xNum);
			XPropertySet xParas = (XPropertySet) UnoRuntime.queryInterface(XPropertySet.class, tc);
			xParas.setPropertyValue("NumberingRules", xNum);
			
			PropertyValue[] aProps = (PropertyValue []) xNum.getByIndex(0);
			
			for (int j = 0 ; j < aProps.length ; ++j) {
	             if (aProps[j].Name.equals ("NumberingType")) {
	            	 aProps[j].Value = new Short(NumberingType.CHAR_SPECIAL);
	                 xReplace.replaceByIndex (0, aProps); 
	             }
	             if (aProps[j].Name.equals ("NumberingLevel")) {
	            	 aProps[j].Value = 0;
	                 xReplace.replaceByIndex (0, aProps); 
	             }
	        }
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
	@Override
	public OdfTextArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
		OdfTextArtifactReferencePosition pos = getPositionForRepresentation(document, referenceId);
		OdfTextChapterDocument odfDoc = (OdfTextChapterDocument)document;
		ITextDocument textDocument = (ITextDocument)odfDoc.getDocument();
		
		XTextFramesSupplier textFramesSupplier = (XTextFramesSupplier) UnoRuntime
		.queryInterface(XTextFramesSupplier.class, textDocument.getXTextDocument());
		XNameAccess textFrames = textFramesSupplier.getTextFrames();

		if (textFrames.hasByName(referenceId)) {
			try {
				Any textFrame = (Any) textFrames.getByName(referenceId);
				XTextContent textContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, textFrame);
				textDocument.getTextService().getTextContentService().removeTextContent(new TextContent(textContent, textDocument));
				return pos;
			} catch (NoSuchElementException e) {
				e.printStackTrace();
			} catch (WrappedTargetException e) {
				e.printStackTrace();
			} catch (TextException e) {
				e.printStackTrace();
			}
		}
		return null;
	}
		
	
	private class TextContent implements ITextContent {

		private XTextContent xTextContent;
		private ITextDocument textDocument;

		public TextContent(XTextContent textContent, ITextDocument textDocument) {
			super();
			xTextContent = textContent;
			this.textDocument = textDocument;
		}

		@Override
		public XTextContent getXTextContent() {
			return xTextContent;
		}

		@Override
		public ITextDocument getTextDocument() {
			return textDocument;
		}

	}
	
	

	@Override
	public String getInputRepresentationType() {
		return StyledDefinitionListingRepresentation.TYPE;
	}

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("odftext");
		return ids;
	}

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getDescription() {
		return "Definition Listing";
	}


	@Override
	public OdfTextArtifactReferencePosition getPositionForRepresentation(EditorChapterDocument chapterDocument, String referenceId) {
		OdfTextChapterDocument odfTextChapterDocument = (OdfTextChapterDocument)chapterDocument;
		ITextDocument textDocument = odfTextChapterDocument.getDocument();
		XTextFramesSupplier textFramesSupplier = (XTextFramesSupplier) UnoRuntime.queryInterface(XTextFramesSupplier.class, textDocument.getXTextDocument());
		XNameAccess textFrames = textFramesSupplier.getTextFrames();
		try {
			if (textFrames.hasByName(referenceId)) {
				Any textFrame = (Any) textFrames.getByName(referenceId);
				XTextContent textContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, textFrame);
				XTextRange xTextRange = textContent.getAnchor();
				ITextRange textRange = new TextRange(textDocument, xTextRange);
				OdfTextArtifactReferencePosition position = new OdfTextArtifactReferencePosition(odfTextChapterDocument, textRange);
				return position;
			}
		} catch (NoSuchElementException e) {
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			e.printStackTrace();
		}
		return null;
	}



	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}
}
