package org.deft.editorintegrator.odftext;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import org.deft.integrator.AbstractEditorIntegrator;
import org.deft.integrator.EditorChapterDocument;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.projecttype.openoffice.OdfTextArtifactReferencePosition;
import org.deft.projecttype.openoffice.OdfTextChapterDocument;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.styleddocument.StyledTextRepresentation;
import org.deft.representation.styleddocument.conversion.RtfMapping;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.util.StyledDocumentUtil;

import ag.ion.bion.officelayer.filter.RTFFilter;
import ag.ion.bion.officelayer.internal.text.TextRange;
import ag.ion.bion.officelayer.text.ITextContent;
import ag.ion.bion.officelayer.text.ITextContentService;
import ag.ion.bion.officelayer.text.ITextDocument;
import ag.ion.bion.officelayer.text.ITextDocumentTextShape;
import ag.ion.bion.officelayer.text.ITextRange;
import ag.ion.bion.officelayer.text.IViewCursor;
import ag.ion.bion.officelayer.text.TextException;
import ag.ion.noa.NOAException;
import ag.ion.noa.graphic.TextInfo;

import com.sun.star.beans.XPropertySet;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XNameAccess;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.table.BorderLine;
import com.sun.star.text.HoriOrientation;
import com.sun.star.text.TextContentAnchorType;
import com.sun.star.text.VertOrientation;
import com.sun.star.text.XText;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextFramesSupplier;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.Any;
import com.sun.star.uno.UnoRuntime;

/**
 * Integrates styled text (StyledDocument) into an ODFText document using the OpenOffice API.
 * 
 * The text is surrounded by a frame.
 */
public class StyledTextIntoODFTextIntegrator extends AbstractEditorIntegrator {

	public static final String ID = "org.deft.editorintegrator.styledtext4odftext";
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		
		StyledTextRepresentation textRep = (StyledTextRepresentation)representation;
		OdfTextArtifactReferencePosition odfPosition = (OdfTextArtifactReferencePosition)position;
		OdfTextChapterDocument doc = odfPosition.getChapterDocument();
		insertWithFilledTextShape(doc, odfPosition.getTextRange(), textRep, referenceId);
	}
	
	
	private void insertWithFilledTextShape(OdfTextChapterDocument doc, 
			ITextRange textRange, StyledTextRepresentation textRep, String integrationId) {
		ITextDocument textDoc = (ITextDocument)doc.getDocument();
		
		TokenizedStyledDocument text = textRep.getStyledDocument();
		TextInfo textInfo = new TextInfo(integrationId, 
				VertOrientation.TOP, HoriOrientation.LEFT, 0, true, true, 
				0, true, true, -1, TextContentAnchorType.AS_CHARACTER);
		
		ITextDocumentTextShape textShape = insertEmptyTextShape(textDoc, textInfo, textRange);
		removeBorders(textShape);
		insertRenderedRTFIntoEmptyTextShape(textDoc, textShape, text);
	}
	

	
	private ITextDocumentTextShape insertEmptyTextShape(ITextDocument textDoc, 
			TextInfo textInfo, ITextRange textRange) {
		ITextContentService textContentService = textDoc.getTextService().getTextContentService();
		try {
			ITextDocumentTextShape textDocumentTextShape = textContentService.constructNewTextShape(textInfo);
			textContentService.insertTextContent(textRange, textDocumentTextShape);
			return textDocumentTextShape;
		} catch (TextException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	private void removeBorders(ITextDocumentTextShape textShape) {
		try {
			XTextContent xTextShape = textShape.getXTextContent();
			XPropertySet xProps = (XPropertySet) UnoRuntime.queryInterface(XPropertySet.class, xTextShape);
			
			BorderLine blTop = new BorderLine();
			blTop.Color = -1;
			BorderLine blBottom = new BorderLine();
			blBottom.Color = -1;
			BorderLine blLeft = new BorderLine();
			blLeft.Color = -1;
			BorderLine blRight = new BorderLine();
			blRight.Color = -1;

			xProps.setPropertyValue("TopBorder", blTop);
			xProps.setPropertyValue("TopMargin", 0);
			xProps.setPropertyValue("BottomBorder", blBottom);
			xProps.setPropertyValue("BottomMargin", 0);
			xProps.setPropertyValue("LeftBorder", blLeft);
			xProps.setPropertyValue("LeftMargin", 0);
			xProps.setPropertyValue("RightBorder", blRight);
			xProps.setPropertyValue("RightMargin", 0);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	private void insertRenderedRTFIntoEmptyTextShape(ITextDocument textDoc, 
			ITextDocumentTextShape textDocumentTextShape, TokenizedStyledDocument text){
		XText xText = textDocumentTextShape.getXText();
		if(xText != null){
			XTextCursor xTextCursor = xText.createTextCursor();
			xTextCursor.gotoStart(false);

			IViewCursor viewCursor = textDoc.getViewCursorService().getViewCursor();
			
			try {
				ITextContentService textContentService = textDoc.getTextService().getTextContentService();
				ITextRange textRange = textContentService.constructNewTextRange(textDoc, xTextCursor.getStart());
				viewCursor.goToRange(textRange, false);
				//we use RTF instead of HTML because the HTML filter sometimes causes OO to hang
				String rtfString = text.getMarkedUpText(new RtfMapping());
				System.out.println(StyledDocumentUtil.asString(text));
				System.out.println(rtfString);
				InputStream is = new ByteArrayInputStream(rtfString.getBytes());
				viewCursor.getTextCursorFromStart().insertDocument(is, RTFFilter.FILTER);

			} catch (TextException te) {
				te.printStackTrace();
			} catch (NOAException noae) {
				noae.printStackTrace();
			}

		}
	}
	

	
	@Override
	public OdfTextArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
		OdfTextArtifactReferencePosition pos = getPositionForRepresentation(document, referenceId);
		OdfTextChapterDocument odfDoc = (OdfTextChapterDocument)document;
		ITextDocument textDocument = (ITextDocument)odfDoc.getDocument();
		
		XTextFramesSupplier textFramesSupplier = (XTextFramesSupplier) UnoRuntime
		.queryInterface(XTextFramesSupplier.class, textDocument.getXTextDocument());
		XNameAccess textFrames = textFramesSupplier.getTextFrames();

		if (textFrames.hasByName(referenceId)) {
			try {
				Any textFrame = (Any) textFrames.getByName(referenceId);
				XTextContent textContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, textFrame);
				textDocument.getTextService().getTextContentService().removeTextContent(new TextContent(textContent, textDocument));
				return pos;
			} catch (NoSuchElementException e) {
				e.printStackTrace();
			} catch (WrappedTargetException e) {
				e.printStackTrace();
			} catch (TextException e) {
				e.printStackTrace();
			}
		}
		return null;
	}
		
	
	private class TextContent implements ITextContent {

		private XTextContent xTextContent;
		private ITextDocument textDocument;

		public TextContent(XTextContent textContent, ITextDocument textDocument) {
			super();
			xTextContent = textContent;
			this.textDocument = textDocument;
		}

		@Override
		public XTextContent getXTextContent() {
			return xTextContent;
		}

		@Override
		public ITextDocument getTextDocument() {
			return textDocument;
		}

	}
	
	

	@Override
	public String getInputRepresentationType() {
		return StyledTextRepresentation.TYPE;
	}

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("odftext");
		return ids;
	}

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getDescription() {
		return "Text";
	}


	@Override
	public OdfTextArtifactReferencePosition getPositionForRepresentation(EditorChapterDocument chapterDocument, String referenceId) {
		OdfTextChapterDocument odfTextChapterDocument = (OdfTextChapterDocument)chapterDocument;
		ITextDocument textDocument = odfTextChapterDocument.getDocument();
		XTextFramesSupplier textFramesSupplier = (XTextFramesSupplier) UnoRuntime.queryInterface(XTextFramesSupplier.class, textDocument.getXTextDocument());
		XNameAccess textFrames = textFramesSupplier.getTextFrames();
		try {
			if (textFrames.hasByName(referenceId)) {
				Any textFrame = (Any) textFrames.getByName(referenceId);
				XTextContent textContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, textFrame);
				XTextRange xTextRange = textContent.getAnchor();
				ITextRange textRange = new TextRange(textDocument, xTextRange);
				OdfTextArtifactReferencePosition position = new OdfTextArtifactReferencePosition(odfTextChapterDocument, textRange);
				return position;
			}
		} catch (NoSuchElementException e) {
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			e.printStackTrace();
		}
		return null;
	}



	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}
}
