package org.deft.editorintegrator.odftext;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.deft.integrator.AbstractEditorIntegrator;
import org.deft.integrator.EditorChapterDocument;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.projecttype.openoffice.OdfTextArtifactReferencePosition;
import org.deft.projecttype.openoffice.OdfTextChapterDocument;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.table.TableRepresentation;
import org.deft.representation.table.data.Table;
import org.deft.representation.table.data.TableCell;

import ag.ion.bion.officelayer.internal.text.TextRange;
import ag.ion.bion.officelayer.text.IText;
import ag.ion.bion.officelayer.text.ITextContentService;
import ag.ion.bion.officelayer.text.ITextCursor;
import ag.ion.bion.officelayer.text.ITextDocument;
import ag.ion.bion.officelayer.text.ITextRange;
import ag.ion.bion.officelayer.text.ITextTable;
import ag.ion.bion.officelayer.text.ITextTableCell;
import ag.ion.bion.officelayer.text.ITextTableCellRange;
import ag.ion.bion.officelayer.text.ITextTableService;
import ag.ion.bion.officelayer.text.IViewCursor;
import ag.ion.bion.officelayer.text.TextException;

import com.sun.star.container.XNamed;
import com.sun.star.text.XTextRange;
import com.sun.star.text.XTextViewCursorSupplier;
import com.sun.star.uno.UnoRuntime;

public class TableIntoODFTextIntegrator extends AbstractEditorIntegrator {

	public static final String ID = "org.deft.editorintegrator.table4odftext";
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		TableRepresentation tableRep = (TableRepresentation)representation;
		OdfTextArtifactReferencePosition odfPosition = (OdfTextArtifactReferencePosition)position;
		OdfTextChapterDocument doc = odfPosition.getChapterDocument();
		insertTable(doc, odfPosition.getTextRange(), tableRep, referenceId);
	}
	
	
	private void insertTable(OdfTextChapterDocument doc, 
			ITextRange textRange, TableRepresentation tableRep, String integrationId) {
		ITextDocument textDoc = doc.getDocument();
		Table table = tableRep.getTable();
		ITextTable textTable = computeEmptyTextTable(textDoc, table);
		insertTextTableAtPosition(textDoc, textTable, textRange, 
				table.hasColumnHeaders(), table.hasRowHeaders());
		fillEmptyTextTable(textTable, table);
		setTableName(textTable, integrationId);
	}
	

	private ITextTable computeEmptyTextTable(ITextDocument textDoc, Table table) {
		ITextTableService tableService = textDoc.getTextTableService();
		int columns = table.getColumnCount();
		if (table.hasRowHeaders()) {
			columns += 1;
		}
		int rows = table.getRowCount(); 	
		if (table.hasRowHeaders()) {
			rows += 1;
		}
		try {
			ITextTable textTable = tableService.constructTextTable(rows, columns);
			return textTable;
		} catch (TextException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	private void fillEmptyTextTable(ITextTable textTable, Table table) {
		try {
			if (table.hasColumnHeaders()) {
				insertColHeaders(textTable, table);
			}
			if (table.hasRowHeaders()) {
				insertRowHeaders(textTable, table);
			}
			insertContent(textTable, table);
		}
		catch (TextException te) {
			te.printStackTrace();
		}
	}

	
	private void insertColHeaders(ITextTable textTable, Table table) throws TextException {
		for (int i = 1; i < textTable.getColumnCount(); i++) {
			ITextTableCell cell = textTable.getCell(i, 0);
			TableCell headerCell = table.getColumnHeader(i);
			if (headerCell != null) {
				String headerText = headerCell.getText();
				setText(cell, headerText);
			}
		}
	}
	
	private void insertRowHeaders(ITextTable textTable, Table table) throws TextException {
		for (int i = 1; i < textTable.getRowCount(); i++) {
			ITextTableCell cell = textTable.getCell(0, i);
			TableCell headerCell = table.getRowHeader(i);
			if (headerCell != null) {
				String headerText = headerCell.getText();
				setText(cell, headerText);
			}
		}
	}
	
	
	
	private void insertContent(ITextTable textTable, Table table) throws TextException {
		int rowCount = table.getRowCount();
		int colCount = table.getColumnCount();
		int rowOffset = table.hasColumnHeaders() ? 1 : 0;
		int colOffset = table.hasRowHeaders() ? 1 : 0;
		Set<TableCell> alreadyProcessedCells = new HashSet<TableCell>();
		
		for (int row = 1; row <= rowCount; row++) {
			for (int col = 1; col <= colCount; col++) {
				TableCell cell = table.getTableCell(row, col);
				if (!alreadyProcessedCells.contains(cell)) {
					String text = cell.getText();
					int ttCol = table.getNumberOfColumnInRow(cell, row) - 1 + colOffset;
					int ttRow = table.getNumberOfRowInColumn(cell, col) - 1 + rowOffset;
					int ttColSpan = cell.getColSpan() - 1;
					int ttRowSpan = cell.getRowSpan() - 1;
					
					ITextTableCell ooTableCell = textTable.getCell(ttCol, ttRow);
					setText(ooTableCell, text);
					
					ITextTableCellRange range = textTable.getCellRange(ttCol, ttRow, ttCol + ttColSpan, ttRow + ttRowSpan);
					textTable.merge(range);
	
					alreadyProcessedCells.add(cell);
				}
			}
		}
	}
	
	private void setText(ITextTableCell cell, String text) {
		IText cellText = cell.getTextService().getText();
		cellText.setText(text);
	}
	
	
	private void insertTextTableAtPosition(ITextDocument textDoc, 
			ITextTable textTable, ITextRange textRange, boolean hasColumnHeaders, 
			boolean hasRowHeaders) {
		ITextContentService textContentService = textDoc.getTextService().getTextContentService();
		try {
			textContentService.insertTextContent(textRange, textTable);
			if (hasColumnHeaders) {
				textTable.setHeaderRows(1);
			} else {
				unsetColumnHeaderStyle(textTable);
			}
			if (hasRowHeaders) {
				setHeaderCol(textTable);
			}
		} catch (TextException e) {
			e.printStackTrace();
		}
	}
	
	
	public void setHeaderCol(ITextTable table) throws TextException {
		int rowCount = table.getRowCount();
		for (int i = 0; i < rowCount; i++) {
			ITextTableCell cell = table.getCell(0, i);
			cell.setCellParagraphStyle(ITextTableCell.STYLE_TABLE_HEADER);
		}
	}
	
	public void unsetColumnHeaderStyle(ITextTable table) throws TextException {
		int colCount = table.getColumnCount();
		for (int i = 0; i < colCount; i++) {
			ITextTableCell cell = table.getCell(i, 0);
			cell.setCellParagraphStyle(ITextTableCell.STYLE_TABLE_CONTENT);
		}
	}
	
	
	private void setTableName(ITextTable textTable, String name) {
	    XNamed xNamed = (XNamed) UnoRuntime.queryInterface(XNamed.class, textTable.getXTextTable());
	    xNamed.setName(name);
	}
		
	

	public OdfTextArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
		OdfTextChapterDocument odfDoc = (OdfTextChapterDocument)document;
		ITextDocument textDocument = (ITextDocument)odfDoc.getDocument();
		ITextTable textTable;
		try {
			textTable = textDocument.getTextTableService().getTextTable(referenceId);
			ITextRange range = textTable.getTextRange();
			textTable.remove();
		    ITextRange fixedRange = getRangeWithStartMovedOneLeft(textDocument, range);   
			fixedRange.setText("");
			return new OdfTextArtifactReferencePosition(odfDoc, fixedRange);
		} catch (TextException te) {
			//no such table
			te.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * The range of the text table is wrong (start is 1 too early). This error
	 * is fixed here.
	 */
	private ITextRange getRangeWithStartMovedOneLeft(ITextDocument textDocument, ITextRange originalRange) {
		try {
			//get cursor and go to start of range
			IViewCursor cursor = textDocument.getViewCursorService().getViewCursor();
			cursor.goToRange(originalRange, false); //this will throw an exception if the table is at the beginning of the document
			//move cursor 1 to the right
		    XTextViewCursorSupplier xTextViewCursorSupplier = (XTextViewCursorSupplier) UnoRuntime.queryInterface(XTextViewCursorSupplier.class,
		        textDocument.getXTextDocument().getCurrentController());
		    xTextViewCursorSupplier.getViewCursor().goRight((short) 1, false);
		    //create new range from cursor position
		    ITextRange range = textDocument.getViewCursorService().getViewCursor().getTextCursorFromEnd().getEnd();
			return range;
		}
		catch (Exception e) {
			//table was at beginning of document, the cursor is one off (NOA bug) -> exception
			//-> get cursor at beginning of document and return its range
			ITextCursor c = textDocument.getViewCursorService().getViewCursor().getTextCursorFromStart();
			return c.getStart();
		}
	}
	

	@Override
	public String getInputRepresentationType() {
		return TableRepresentation.TYPE;
	}

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("odftext");
		return ids;
	}

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getDescription() {
		return "Table";
	}


	@Override
	public OdfTextArtifactReferencePosition getPositionForRepresentation(EditorChapterDocument chapterDocument, String referenceId) {
		OdfTextChapterDocument odfDoc = (OdfTextChapterDocument)chapterDocument;
		ITextDocument textDocument = (ITextDocument)odfDoc.getDocument();

		try {
			ITextTable textTable = textDocument.getTextTableService().getTextTable(referenceId);
			XTextRange xRange = textTable.getXTextTable().getAnchor();
			ITextRange range = new TextRange(textDocument, xRange);
//			ITextRange fixedRange = getRangeWithStartMovedOneLeft(textDocument, range);
			OdfTextArtifactReferencePosition pos = new OdfTextArtifactReferencePosition(odfDoc, range);
			return pos;
		} catch (TextException te) {
			//no such table. This exception can be thrown when a table has been removed
			//manually but the repository still contains the reference. This is expected
			//behavior, so don't print stack trace here.
			return null;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		
	
//		OdfTextChapterDocument odfTextChapterDocument = (OdfTextChapterDocument)chapterDocument;
//		ITextDocument textDocument = odfTextChapterDocument.getDocument();
//		XTextFramesSupplier textFramesSupplier = (XTextFramesSupplier) UnoRuntime.queryInterface(XTextFramesSupplier.class, textDocument.getXTextDocument());
//		XNameAccess textFrames = textFramesSupplier.getTextFrames();
//		try {
//			if (textFrames.hasByName(referenceId)) {
//				Any textFrame = (Any) textFrames.getByName(referenceId);
//				XTextContent textContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, textFrame);
//				XTextRange xTextRange = textContent.getAnchor();
//				ITextRange textRange = new TextRange(textDocument, xTextRange);
//				OdfTextArtifactReferencePosition position = new OdfTextArtifactReferencePosition(odfTextChapterDocument, textRange);
//				return position;
//			}
//		} catch (NoSuchElementException e) {
//			e.printStackTrace();
//		} catch (WrappedTargetException e) {
//			e.printStackTrace();
//		}
//		return null;
	}



	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}
}
