package org.deft.exporter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.wizard.WizardPage;

public class EclipseExporterRegistry implements ExporterRegistry {
	
	private static final String EXPORTER_EXTENSION_POINT_NAME = "org.deft.exporter";
		
	private Map<String, ExporterConfig> registeredConfigs = new HashMap<String, ExporterConfig>(); 

	public EclipseExporterRegistry() {
		getAndCacheRegisteredEditors();
	}
	
	private void getAndCacheRegisteredEditors() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXPORTER_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			ExporterConfig config = new ExporterConfig(ce);
			registeredConfigs.put(config.getId(), config);
		}
	}
	
	@Override
	public Exporter getExporter(String exporterId) {
		ExporterConfig config = registeredConfigs.get(exporterId);
		if (config != null) {
			Exporter exporter = config.getExporter();
			return exporter;
		}
		return null;
	}
	
	@Override
	public List<String> getExporterIds() {
		List<String> exporterIds = new LinkedList<String>(registeredConfigs.keySet());
		return exporterIds;
	}
	
	public List<String> getExporterIds(String chapterTypId) {
		List<String> exporterIdsForChapter = new LinkedList<String>();
		List<ExporterConfig> exporterConfigs = new LinkedList<ExporterConfig>(registeredConfigs.values());
		for (ExporterConfig config : exporterConfigs) {
			if (config.getChapterTypeId().equals(chapterTypId)) {
				exporterIdsForChapter.add(config.getId());
			}
		}
		return exporterIdsForChapter;
	}
	
	@Override 
	public List<WizardPage> getWizardPages(String exporterId) {
		ExporterConfig config = registeredConfigs.get(exporterId);
		if (config != null) {
			List<WizardPage> pages = config.getWizardPages();
			return pages;
		}
		return new LinkedList<WizardPage>();
	}
	
	
	
	private class ExporterConfig {
	
		private IConfigurationElement ce;
		private Exporter exporter;
		private String chapterTypeId;
		private List<WizardPage> wizardPages;
		
		public ExporterConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public String getChapterTypeId() {
			if (chapterTypeId == null) {
				chapterTypeId = ce.getAttribute("chaptertypeid");
			}
			return chapterTypeId;
		}
		
		public Exporter getExporter() {
			if (exporter == null) {
				try {
					exporter = (Exporter)ce.createExecutableExtension("class");
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return exporter;
		}
		
		public List<WizardPage> getWizardPages() {
			if (wizardPages == null) {
				wizardPages = new LinkedList<WizardPage>();
				for (IConfigurationElement pageElement : ce.getChildren()) {
					try {
						WizardPage page = (WizardPage)pageElement.createExecutableExtension("class");
						wizardPages.add(page);
					} catch (CoreException e) {
						e.printStackTrace();
					}
				}
			}
			return wizardPages;
		}
		
		
	}


}