package org.deft.exporter.latex;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.Map;

import org.deft.exporter.AbstractExporter;
import org.deft.repository.chapterdocument.latex.BibtexChapterDocument;
import org.deft.repository.chapterdocument.latex.LatexChapterDocument;
import org.deft.repository.chaptertype.ChapterDocument;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.util.FileUtil;
import org.eclipse.core.runtime.IProgressMonitor;

public class LatexExporter extends AbstractExporter {

	@Override
	public String getName() {
		return "LaTeX";
	}

	@Override
	protected void export(Chapter chapter, Map<String, Object> options,
			IProgressMonitor monitor) {
		monitor.beginTask("Exporting LaTeX project", 5);
		monitor.worked(1);
		String outputDirPath = getOutputDirPath(options);
		String additionalFilesPath = getAdditionalFilesDirPath(options);
		if (additionalFilesPath != null) {
			copyAdditionalFiles(additionalFilesPath, outputDirPath);
		}
		copyTempDir(chapter.getProject(), outputDirPath);
		
		saveChapters(chapter, options);
		saveBibs(chapter, options);
		
		boolean compile = getCompile(options);
		if (compile) {
			runPdfLatexWithProgress(chapter, options, monitor);
			runBibtexWithProgress(chapter, options, monitor);
			runPdfLatexWithProgress(chapter, options, monitor);
			runPdfLatexWithProgress(chapter, options, monitor);
		}
	}
	
	
	private void saveChapters(Chapter chapter, Map<String, Object> options) {
		saveChapter(chapter, options);
		LatexChapterDocument chapterDoc = new LatexChapterDocument(chapter);
		List<LatexChapterDocument> referencedLatexDocs = chapterDoc.getDirectlyAndIndirectlyIncludedLatexChapters();
		for (ChapterDocument doc : referencedLatexDocs) {
			Chapter refChapter = doc.getChapter();
			saveChapter(refChapter, options);
		}
	}
	
	private String getFilePathOfChapter(Chapter chapter, Map<String, Object> options) {
		return getFilePathOfChapter(chapter, options, true);
	}
	
	private String getFilePathOfChapter(Chapter chapter, Map<String, Object> options, boolean addFileExtension) {
		String extension = "";
		if (addFileExtension) {
			if (chapter.getTypeId().equals("latex")) {
				extension = ".tex";
			} else if (chapter.getTypeId().equals("bibtex")) {
				extension = ".bib";
			}
		}
		String chapterFileName = chapter.getName() + extension;
		String dir = getOutputDirPath(options);
		String projectPath = getPathInProject(chapter);
		String chapterDir = new File(dir, projectPath).getAbsolutePath();
		String filePath = new File(chapterDir, chapterFileName).getAbsolutePath();
		return filePath;
	}
	
	private String getPathInProject(Chapter chapter) {
		if (chapter.getParentFolder() == null) {
			return "";
		} else {
			return chapter.getParentFolder().getProjectRelativePath();
		}
	}
	
	private void saveBibs(Chapter chapter, Map<String, Object> options) {
		LatexChapterDocument chapterDoc = new LatexChapterDocument(chapter);
		List<BibtexChapterDocument> referencedBibtexDocs = chapterDoc.getBibtexChapters();
		for (ChapterDocument doc : referencedBibtexDocs) {
			Chapter refChapter = doc.getChapter();
			saveChapter(refChapter, options);
		}
	}
	
	
	private String getOutputDirPath(Map<String, Object> options) {
		String path = (String)options.get(LatexExporterConstants.OUPUT_DIRECTORY);
		return path;
	}
	
	private String getAdditionalFilesDirPath(Map<String, Object> options) {
		String path = (String)options.get(LatexExporterConstants.ADDITIONAL_FILES_DIR);
		if (path == null ||  path.equals("")) {
			return null;
		}
		return path;
	}
	
	private boolean getCompile(Map<String, Object> options) {
		boolean compile = (Boolean)options.get(LatexExporterConstants.COMPILE_LATEX_FILES);
		return compile;
	}
	
	private void copyAdditionalFiles(String additionalFilesPath, String outputDirPath) {
		File af = new File(additionalFilesPath);
		File od = new File(outputDirPath);
		for (File fChild : af.listFiles()) {
			FileUtil.copy(fChild, od);
		}
	}
	
	
	private void copyTempDir(Project project, String outputDirPath) {
		File tempDir = project.getTempDirectory();
		File od = new File(outputDirPath);
		FileUtil.copy(tempDir, od);
	}
	
	private void saveChapter(Chapter chapter, Map<String, Object> options) {
		String chapterPath = getFilePathOfChapter(chapter, options);
		File chapterFile = new File(chapterPath);
		createFolderForChapter(chapterFile);

		try {
			chapterFile.createNewFile();
		} catch (IOException e) {
			e.printStackTrace();
		}
		FileUtil.saveToFile(chapter.getChapterContent(), chapterFile, true);
	}
	
	
	private void createFolderForChapter(File chapterFile) {
		File chapterDir = chapterFile.getParentFile();
		if (!chapterDir.exists()) {
			chapterDir.mkdirs();
		}
	}
	

	@Override
	protected void export(Tutorial tutorial, Map<String, Object> options,
			IProgressMonitor monitor) {
		throw new UnsupportedOperationException("No tutorial export for latex files");
	}
	
	
	private void runPdfLatexWithProgress(Chapter chapter, Map<String, Object> options, IProgressMonitor monitor) {
		monitor.setTaskName("Running pdflatex");
		doCompile(chapter, options, LatexExporterConstants.LATEX_EXECUTABLE);
		monitor.worked(1);
	}
	
	private void runBibtexWithProgress(Chapter chapter, Map<String, Object> options, IProgressMonitor monitor) {
		monitor.setTaskName("Running bibtex");
		doCompile(chapter, options, LatexExporterConstants.BIBTEX_EXECUTABLE);
		monitor.worked(1);
	}
	
	private void doCompile(Chapter chapter, Map<String, Object> options, String EXECUTABLE_KEY) {
		String latexPath = (String)options.get(EXECUTABLE_KEY);
		String mainLatexFile = getFilePathOfChapter(chapter, options, false);
		String outputDir = getOutputDirPath(options);
		String[] additionalArguments = getAdditionalArguments(EXECUTABLE_KEY);
		Runtime rt = Runtime.getRuntime();
		try {
			
			//create and fill array s: [pathToLatexExecutable, additionalArg1, additionalArg2 ..., mainLatexFile] 
			String[] s = new String[additionalArguments.length + 2];
			s[0] = latexPath;
			for (int i = 0; i < additionalArguments.length; i++) {
				s[i + 1] = additionalArguments[i];
			}
			s[s.length - 1] = mainLatexFile;

			Process process = rt.exec(s, null, new File(outputDir));
		    String line;
		    BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
		    while ((line = input.readLine()) != null) {
		        System.out.println(line);
		    }
		    input.close();

		} catch (IOException e) {
			e.printStackTrace();
		} 
	}
	
	private String[] getAdditionalArguments(String EXECUTABLE_KEY) {
		if (EXECUTABLE_KEY.equals(LatexExporterConstants.LATEX_EXECUTABLE)) {
			return new String[] {"-interaction=nonstopmode", "--src-specials"};
		} else {
			return new String[0];
		}
	}

}
