package org.deft.gui;

import java.util.LinkedList;
import java.util.List;

import org.deft.ExtensionServiceExtension;
import org.deft.RepositoryFacade;
import org.deft.artifactviewer.ArtifactViewerConnector;
import org.deft.artifactviewer.ArtifactViewerInput;
import org.deft.artifactviewer.ArtifactViewerInputImpl;
import org.deft.artifactviewer.ArtifactViewerPart;
import org.deft.artifactviewer.MultiPageArtifactViewerPart;
import org.deft.chaptereditor.ChapterEditorInputImpl;
import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.representationrenderer.RepresentationRenderer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

/**
 * Helper class to easily open and retrieve editors.
 */
public class EditorHelper {

	private IWorkbenchWindow window;
	private ExtensionServiceExtension extensionService = RepositoryFacade.instance().getExtensionService();
	
	public EditorHelper(IWorkbenchWindow window) {
		this.window = window;
	}
	
	public EditorHelper() {
	}
	
	
	private IWorkbenchWindow getWindow() {
		if (window == null) {
			IWorkbench wb = PlatformUI.getWorkbench();
			window = wb.getActiveWorkbenchWindow();
		}
		return window;
	}

	
	private IWorkbenchPage getActivePage() {
		return getWindow().getActivePage();
	}
	

	
	/**
	 * Returns the Chapter that is displayed by the currently
	 * active editor. Returns null if the editor does not
	 * display a chapter. (It might display an Artifact, for example.)
	 */
	public Chapter getChapterOfActiveEditor() {
		ChapterEditorPart editor = getActiveChapterEditor();
		if (editor != null) {
			Chapter chapter = editor.getChapter();
			return chapter;
		}
		return null;
	}

	/**
	 * Returns the currently active editor, if it is a
	 * ChapterEditor. If the currently active editor is
	 * not a ChapterEditor, or no editor is open, null
	 * is returned.
	 */
	public ChapterEditorPart getActiveChapterEditor() {
		IWorkbenchPage page = getActivePage();
		if (page != null) {
			IEditorPart editor = getActivePage().getActiveEditor();
			if (editor instanceof ChapterEditorPart) {
				return (ChapterEditorPart)editor;
			}
		}
		return null;
	}

	/**
	 * Returns a list of all open editors.
	 */
	public List<IEditorPart> getOpenEditors() {
		List<IEditorPart> editors = new LinkedList<IEditorPart>();
		IEditorReference[] editorsRefs = getActivePage().getEditorReferences();
		for (int i = 0; i < editorsRefs.length; i++) {
			IEditorPart editor = editorsRefs[i].getEditor(true);
			editors.add(editor);
		}
		return editors;
	}
	
	/**
	 * Returns a list of all open editors which display a chapter.
	 */
	public List<ChapterEditorPart> getOpenChapterEditors() {
		List<ChapterEditorPart> editors = new LinkedList<ChapterEditorPart>();
		IEditorReference[] editorsRefs = getActivePage().getEditorReferences();
		for (int i = 0; i < editorsRefs.length; i++) {
			IEditorPart editor = editorsRefs[i].getEditor(true);
			if (editor instanceof ChapterEditorPart) {
				editors.add((ChapterEditorPart)editor);
			}
		}
		return editors;
	}
	
	/**
	 * Returns the editor which displays a specific chapter. If
	 * there is no open editor which displays the chapter, a new
	 * editor for that chapter is opened, and then returned.
	 */
	public ChapterEditorPart getOrOpenEditorFor(Chapter chapter) {
		if (isEditorOpen(chapter)) {
			ChapterEditorPart editor = getOpenEditor(chapter);
			return editor;
		} else {
			ChapterEditorPart editor = openEditor(chapter);
			return editor;
		}
	}
	
	private boolean isEditorOpen(Chapter chapter) {
		boolean isOpen = getOpenEditor(chapter) != null;
		return isOpen;
	}
	
	private boolean isViewerOpen(Artifact artifact) {
		boolean isOpen = getOpenEditor(artifact) != null;
		return isOpen;
	}
	
	public ChapterEditorPart getOpenEditor(Chapter chapter) {
		if (chapter != null) {
			List<ChapterEditorPart> editors = getOpenChapterEditors();
			for (ChapterEditorPart editor : editors) {
				Chapter c = editor.getChapter();
				if (c.getId().equals(chapter.getId())) {
					return editor;
				}
			}
		}
		return null;
	}
	
	
	/**
	 * Returns a list of all open editors which display an artifact.
	 */
	public List<ArtifactViewerPart> getOpenArtifactViewers() {
		List<ArtifactViewerPart> editors = new LinkedList<ArtifactViewerPart>();
		IEditorReference[] editorsRefs = getActivePage().getEditorReferences();
		for (int i = 0; i < editorsRefs.length; i++) {
			IEditorPart editor = editorsRefs[i].getEditor(true);
			if (editor instanceof ArtifactViewerPart) {
				editors.add((ArtifactViewerPart)editor);
			}
		}
		return editors;
	}
	
	public ArtifactViewerPart getOpenEditor(Artifact artifact) {
		if (artifact != null) {
			List<ArtifactViewerPart> editors = getOpenArtifactViewers();
			for (ArtifactViewerPart editor : editors) {
				Artifact a = editor.getArtifact();
				if (a.getId().equals(artifact.getId())) {
					return editor;
				}
			}
		}
		return null;
	}
	
	
	/**
	 * Opens an editor to display a specific chapter.
	 */
	public ChapterEditorPart openEditor(Chapter chapter) {
		String projectTypeId = chapter.getProject().getProjectTypeId();
		String chapterTypeId = chapter.getTypeId();
		String chapterEditorId = extensionService.getChapterEditorId(projectTypeId, chapterTypeId);
		IWorkbenchPage page = getActivePage();
		ChapterEditorInputImpl input = new ChapterEditorInputImpl(chapter);
		try {
			IEditorPart editor = page.openEditor(input, chapterEditorId);
			return (ChapterEditorPart)editor;
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	public void setFocusToOpenEditor() {
		ChapterEditorPart editor = getActiveChapterEditor();
		editor.setFocus();
	}
	
	
	public boolean canBeDisplayedInViewer(Artifact artifact) {
		List<ChainRendererPair> pairs = findAllChainRendererPairs(artifact);
		boolean exist = !pairs.isEmpty();
		return exist;
	}

	
	public void openViewer(Artifact artifact) {
		
		MultiPageArtifactViewerPart editor = null;
		List<ChainRendererPair> chainRendererPairs = findAllChainRendererPairs(artifact);
		for (ChainRendererPair pair : chainRendererPairs) {
			if (hasEditorBeenCreated(editor)) {
				addNewPage(editor, pair);
			} else {
				editor = createEditor(artifact, pair);
			}
			
		}
	}
	
	private List<ChainRendererPair> findAllChainRendererPairs(Artifact artifact) {
		List<ChainRendererPair> pairs = new LinkedList<ChainRendererPair>();
		List<String> chainIds = extensionService.getOperationChainIdsForArtifactType(artifact.getArtifactTypeId());
		for (String chainId : chainIds) {
			OperationChain chain = extensionService.getOperationChain(chainId);
			chain.resetChainConfigurations();
			String repType = chain.getFinalRepresentationType();
			List<RepresentationRenderer> renderers = extensionService.getRenderersForRepresentationType(repType);
			for (RepresentationRenderer renderer : renderers) {
				ChainRendererPair pair = new ChainRendererPair(chain, renderer);
				pairs.add(pair);
			}
		}
		return pairs;
	}
	
	private boolean hasEditorBeenCreated(MultiPageArtifactViewerPart editor) {
		return editor != null;
	}
	
	private void addNewPage(MultiPageArtifactViewerPart editor, ChainRendererPair pair) {
		RepresentationRenderer renderer = pair.getRenderer();
		ArtifactViewerConnector connector = extensionService.getArtifactViewerConnectorForRendererId(renderer.getId());
		String editorId = connector.getIdOfEditorToBeOpened();
		editor.addPage(editorId, pair.getChain().getId(), pair.getRenderer().getId());
	}
	
	private MultiPageArtifactViewerPart createEditor(Artifact artifact, ChainRendererPair pair) {
		try {
			ArtifactViewerInput input = new ArtifactViewerInputImpl(artifact, pair.getChain(), pair.getRenderer());
			IWorkbenchPage page = getWindow().getActivePage();
			MultiPageArtifactViewerPart editor = (MultiPageArtifactViewerPart)page.openEditor(input, "org.deft.artifactviewer.multipage");
			return editor;
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	public void closeEditor(Fragment fragment) {
		if (fragment instanceof Chapter) {
			closeEditor((Chapter)fragment);
		} else if (fragment instanceof Artifact){
			closeEditor((Artifact)fragment);
		}
		//otherwise do nothing
	}
	
	
	public void closeEditor(Chapter chapter) {
		ChapterEditorPart editor = getOpenEditor(chapter);
		getActivePage().closeEditor(editor, true);
	}
	
	public void closeEditor(Artifact artifact) {
		ArtifactViewerPart editor = getOpenEditor(artifact);
		getActivePage().closeEditor(editor, true);
	}
	
	
	public void closeAllEditors() {
		getActivePage().closeAllEditors(true);
	}
	
	public void updateViewerIfOpen(Artifact artifact) {
		if (isViewerOpen(artifact)) {
			closeEditor(artifact);
			openViewer(artifact);
		}
	}
	
	
	
	private class ChainRendererPair {
		
		private OperationChain chain;
		private RepresentationRenderer renderer;
		
		public ChainRendererPair(OperationChain chain, RepresentationRenderer renderer) {
			this.chain = chain;
			this.renderer = renderer;
		}

		public OperationChain getChain() {
			return chain;
		}

		public RepresentationRenderer getRenderer() {
			return renderer;
		}
		
	}
	
}
