package org.deft.gui;

import java.util.LinkedList;
import java.util.List;

import org.deft.gui.view.project.ProjectExplorerView;
import org.deft.gui.view.project.nodes.ArtifactNode;
import org.deft.gui.view.project.nodes.ArtifactReferenceNode;
import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.gui.view.project.nodes.ChapterNode;
import org.deft.gui.view.project.nodes.FolderNode;
import org.deft.gui.view.project.nodes.FragmentNode;
import org.deft.gui.view.project.nodes.ProjectNode;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.deft.gui.view.project.nodes.TutorialNode;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

/**
 * Helper class to access selected items in the views.
 */
public class SelectionHelper {

	private IWorkbenchWindow window;
	
	public SelectionHelper(IWorkbenchWindow window) {
		this.window = window;
	}
	
	public SelectionHelper() {
	}
	
	
	private IWorkbenchWindow getWindow() {
		if (window == null) {
			IWorkbench wb = PlatformUI.getWorkbench();
			window = wb.getActiveWorkbenchWindow();
		}
		return window;
	}
	

	public boolean isExactlyOneProjectViewNodeSelected() {
		List<ProjectViewNode> selectedNodes = getSelectedProjectViewNodes();
		return selectedNodes.size() == 1; 
	}
	
	
	/**
	 * Returns all nodes from the project view that are currently selected.
	 */
	public List<ProjectViewNode> getSelectedProjectViewNodes() {
		List<ProjectViewNode> selectedNodes = new LinkedList<ProjectViewNode>();
		IStructuredSelection selection = getProjectViewSelections();
		for (Object o : selection.toList()) {
			selectedNodes.add((ProjectViewNode)o);
		}
		return selectedNodes;
	}

	
	private IStructuredSelection getProjectViewSelections() {
		ISelection selection = getWindow().getSelectionService().getSelection(ProjectExplorerView.ID);
		IStructuredSelection structuredSelection = (IStructuredSelection)selection;
		return structuredSelection;
	}
	
	
	/**
	 * Returns the first selected project view node.
	 * 
	 * What is considered first depends on the Eclipse internal implementation,
	 * no assertions are made which node will actually be returned.
	 * 
	 * If no node is selected, null is returned.
	 */
	public ProjectViewNode getFirstSelectedProjectViewNode() {
		List<ProjectViewNode> selectedNodes = getSelectedProjectViewNodes();
		if (!selectedNodes.isEmpty()) {
			ProjectViewNode node = selectedNodes.get(0);
			return node;
		}
		return null;
	}
	
	
	
	/**
	 * Returns true if only one project view nodeis selected and this
	 * node represents a Project.
	 */
	public boolean isOnlySingleProjectNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyProjectNode = firstNode instanceof ProjectNode;
			return isOnlyProjectNode;
		}
		return false;
	}
	
	/**
	 * Returns the project represented by the selected project view node.
	 * 
	 * If more or less than one node is selected, or the selected node does
	 * not represent a project, null is returned.
	 */
	public Project getSelectedProject() {
		if (isOnlySingleProjectNodeSelected()) {
			ProjectNode node = (ProjectNode)getFirstSelectedProjectViewNode();
			Project project = node.getProject();
			return project;
		}
		return null;
	}
	
	
	/**
	 * Returns true if only one project view node is selected and this
	 * node represents a fragment (i.e. an artifact, chapter, tutorial, 
	 * image or folder)
	 */
	public boolean isOnlySingleFragmentNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyFragmentNode = firstNode instanceof FragmentNode;
			return isOnlyFragmentNode;
		}
		return false;
	}

	

	/**
	 * Returns true if only one project view node is selected and this
	 * node represents an artifact.
	 */
	public boolean isOnlySingleArtifactNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyArtifactNode = firstNode instanceof ArtifactNode;
			return isOnlyArtifactNode;
		}
		return false;
	}
	
	/**
	 * Returns the artifact represented by the selected project view node.
	 * 
	 * If more or less than one node is selected, or the selected node does
	 * not represent an artifact, null is returned.
	 */
	public Artifact getSelectedArtifact() {
		if (isOnlySingleArtifactNodeSelected()) {
			ArtifactNode node = (ArtifactNode)getFirstSelectedProjectViewNode();
			Artifact artifact = node.getArtifact();
			return artifact;
		}
		return null;
	}
	
	
	/**
	 * Returns true if only one project view node is selected and this
	 * node represents a chapter.
	 */
	public boolean isOnlySingleChapterNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyChapterNode = firstNode instanceof ChapterNode;
			return isOnlyChapterNode;
		}
		return false;
	}
	
	/**
	 * Returns the chapter represented by the selected project view node.
	 * 
	 * If more or less than one node is selected, or the selected node does
	 * not represent a chapter, null is returned.
	 */
	public Chapter getSelectedChapter() {
		if (isOnlySingleChapterNodeSelected()) {
			ChapterNode node = (ChapterNode)getFirstSelectedProjectViewNode();
			Chapter chapter = node.getChapter();
			return chapter;
		}
		return null;
	}
	
	
	/**
	 * Returns true if only one project view node is selected and this
	 * node represents a category.
	 */
	public boolean isOnlySingleCategoryNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyCategoryNode = firstNode instanceof CategoryNode;
			return isOnlyCategoryNode;
		}
		return false;
	}
	


	/**
	 * Returns true if only one project view node is selected and this
	 * node represents a folder.
	 */
	public boolean isOnlySingleFolderNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyFolderNode = firstNode instanceof FolderNode;
			return isOnlyFolderNode;
		}
		return false;
	}
	
	

	
	/**
	 * Returns the folder represented by the selected project view node.
	 * 
	 * If more or less than one node is selected, or the selected node does
	 * not represent a folder, null is returned.
	 */
	public Folder getSelectedFolder() {
		if (isOnlySingleFolderNodeSelected()) {
			FolderNode node = (FolderNode)getFirstSelectedProjectViewNode();
			Folder folder = node.getFolder();
			return folder;
		}
		return null;
	}
	
	
	/**
	 * Returns true if only one project view node is selected and this
	 * node represents a tutorial.
	 */
	public boolean isOnlySingleTutorialNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyTutorialNode = firstNode instanceof TutorialNode;
			return isOnlyTutorialNode;
		}
		return false;
	}
	
	

	
	/**
	 * Returns the tutorial represented by the selected project view node.
	 * 
	 * If more or less than one node is selected, or the selected node does
	 * not represent a tutorial, null is returned.
	 */
	public Tutorial getSelectedTutorial() {
		if (isOnlySingleFolderNodeSelected()) {
			TutorialNode node = (TutorialNode)getFirstSelectedProjectViewNode();
			Tutorial tutorial = node.getTutorial();
			return tutorial;
		}
		return null;
	}
	
	
	public boolean isOnlySingleArtifactReferenceNodeSelected() {
		if (isExactlyOneProjectViewNodeSelected()) {
			ProjectViewNode firstNode = getFirstSelectedProjectViewNode();
			boolean isOnlyArtifactReferenceNode = firstNode instanceof ArtifactReferenceNode;
			return isOnlyArtifactReferenceNode;
		}
		return false;

	}
	
	public ArtifactReference getSelectedArtifactReference() {
		if (isOnlySingleArtifactReferenceNodeSelected()) {
			ArtifactReferenceNode node = (ArtifactReferenceNode)getFirstSelectedProjectViewNode();
			ArtifactReference reference = node.getReference();
			return reference;
		}
		return null;
	}
	
	
	
	public ISelection getActivePartSelection() {
		return null;
	}
	
	
}
