package org.deft.gui.dialog;

import java.util.List;

import org.deft.gui.input.FragmentNameInputValidator;
import org.deft.repository.chaptertype.ChapterType;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class NewChapterDialog extends TitleAreaDialog {
	private String title;
	private Text chapterNameText;
	private Combo chapterTypesCombo;
	
	private String chapterName;
	private ChapterType chapterType;
	private Project project;
	
	public NewChapterDialog(String title, Project project, Shell parentShell) {
		super(parentShell);	
		this.title = title;
		this.project = project;
	}

	@Override
	public void create() {
		// Create dialog
		super.create();
		//we have to call selectFirstChapterType here and not in createDialogArea,
		//otherwise we get an exception (combo listener tries to set status of ok button,
		//which has not yet been created)
		if (projectHasMultipleChapterTypes()) {
			selectFirstChapterType();
		}
		// Then disable ok button
		Button okButton = getButton(IDialogConstants.OK_ID);
		okButton.setEnabled(false);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = createAndSetupDialogComposite(parent);

		addChapterNameTextField(composite);
		
		if (projectHasMultipleChapterTypes()) {
			addChapterTypesCombo(composite);
			fillChapterTypesCombo();	
		} else {
			setOnlyAvailableChapterType();
		}
		return composite;
	}
	
	
	private boolean projectHasMultipleChapterTypes() {
		return project.getProjectType().getAllowedChapterTypes().size() > 1;
	}
	
	private Composite createAndSetupDialogComposite(Composite parent) {
		this.setTitle(title);
		this.setMessage("Enter chapter properties");
		
		// The root composite of our controls
		Composite composite = new Composite(parent, SWT.NONE);
		
		// Define how this composite should be layouted
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		composite.setLayoutData(gd);
		
		// Define how the children of this composite should be layouted
		GridLayout layout = new GridLayout(2, false);
		composite.setLayout(layout);
		return composite;
	}
	
	private void addChapterNameTextField(Composite composite) {
		// Label
		GridData labelGd = new GridData();
		Label label = new Label(composite, SWT.NONE);
		label.setText("Chapter name: ");
		label.setLayoutData(labelGd);
		
		// Text
		GridData textGd = new GridData(GridData.FILL_HORIZONTAL);
		textGd.grabExcessHorizontalSpace = true;
		chapterNameText = new Text(composite, SWT.BORDER);		
		chapterNameText.setLayoutData(textGd);
		chapterNameText.addModifyListener(new ModifyListener() {
		
			public void modifyText(ModifyEvent e) {				
				checkChapterNameInput();
				chapterName = getInternalChapterName();
			}
		});
	}
	
	
	
	private void addChapterTypesCombo(Composite composite) {
		// Label
		GridData labelGd = new GridData();
		Label label = new Label(composite, SWT.NONE);
		label.setText("Chapter type: ");
		label.setLayoutData(labelGd);
		// Combo box
		GridData textGd = new GridData(GridData.FILL_HORIZONTAL);
		textGd.grabExcessHorizontalSpace = true;
		chapterTypesCombo = new Combo(composite, SWT.DROP_DOWN);
		chapterTypesCombo.setLayoutData(textGd);
		chapterTypesCombo.addModifyListener(new ModifyListener() {
		
			public void modifyText(ModifyEvent e) {				
				checkChapterNameInput();
				chapterType = getChapterTypeInCombo();
			}
		});
	}
	
	
	private void selectFirstChapterType() {
		if (chapterTypesCombo.getItemCount() > 0)
			chapterTypesCombo.select(0);
	}
	
	private void setOnlyAvailableChapterType() {
		List<ChapterType> allowedChapterTypes = project.getProjectType().getAllowedChapterTypes();		
		chapterType = allowedChapterTypes.get(0);
	}

	private void fillChapterTypesCombo() {
		for (ChapterType type : project.getProjectType().getAllowedChapterTypes()) {
			chapterTypesCombo.add(type.getDescription());
		}
	}
	
	/**
	 * Returns the type of the project as selected by the user.
	 */
	private ChapterType getChapterTypeInCombo() {
		if (chapterTypesCombo.getSelectionIndex() == -1) {
			return null;
		}
		
		int index = 0;
		for (ChapterType type : project.getProjectType().getAllowedChapterTypes()) {
			if (index == chapterTypesCombo.getSelectionIndex()) {
				return type;
			}
			index++;
		}
		return null;
	}
	
	private String getInternalChapterName() {
		return chapterNameText.getText();
	}
	
	
	/**
	 * Checks if the user-specified project name is valid. If not, an error message
	 * is displayed and the OK button is disabled. 
	 */
	private void checkChapterNameInput() {
		Button okButton = getButton(IDialogConstants.OK_ID);
		okButton.setEnabled(false);
		
		if (getChapterType() == null)
			return;
		
		FragmentNameInputValidator validator = new FragmentNameInputValidator(project, FragmentType.CHAPTER);
		String errorMsg = validator.isValid(getInternalChapterName()); 
		if ((errorMsg != null) || (getInternalChapterName() == null) || ("".equals(getInternalChapterName()))) {
			setErrorMessage(errorMsg);
			return;
		}
						
		setErrorMessage(null);		
		okButton.setEnabled(true);
	}

	/**
	 * Returns the user-selected type for the chapter.
	 */
	public ChapterType getChapterType() {
		return chapterType;
	}

	/**
	 * Returns the name given to the chapter by the user.
	 */
	public String getChapterName() {
		return chapterName;
	}
	
}
