package org.deft.gui.dialog;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.deft.DeftPlugin;
import org.deft.gui.input.RepositoryLocationInputValidator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;


/**
 * A dialog which lets the user select the location of his repository. The last 5 entries
 * will be remembered and shown to the user as quick selection.
 */
public class SelectRepositoryDialog extends TitleAreaDialog {
	
	/**
	 * Constant for storing information to the dialog store via IDialogSettings
	 */
	private final String DIALOG_SETTINGS_SECTION = "SelectRepositoryDialog";
	
	/**
	 * Constant for storing information to the dialog store via IDialogSettings
	 */
	private final String DIALOG_SETTINGS_ENTRY = "LastUsedPaths";
	
	private String path;
	
	private Combo combo;

	/**
	 * Instantiates a new dialog - don't forget to call create() afterwards.
	 * 
	 * @param parentShell
	 */
	public SelectRepositoryDialog(Shell parentShell) {
		super(parentShell);
	}

	/**
	 * Creates the controls of this dialog. 
	 */
	@Override
	public void create() {
		// Create dialog
		super.create();

		// Then disable ok button
		Button okButton = getButton(IDialogConstants.OK_ID);
		okButton.setEnabled(false);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		
		// The root composite of our controls
		Composite composite = new Composite(parent, SWT.NONE);
		
		// Define how this composite should be layouted
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		composite.setLayoutData(gd);
		
		// Define how the children of this composite should be layouted
		GridLayout layout = new GridLayout(3, false);
		composite.setLayout(layout);
		
		// Label
		GridData labelGd = new GridData();
		Label label = new Label(composite, SWT.NONE);
		label.setText("Repository: ");
		label.setLayoutData(labelGd);
		
		// Combo box
		GridData textGd = new GridData(GridData.FILL_HORIZONTAL);
		textGd.grabExcessHorizontalSpace = true;
		combo = new Combo(composite, SWT.DROP_DOWN);
		// Fill combo box with data
		addPathsToCombo();
		combo.setLayoutData(textGd);
		combo.addModifyListener(new ModifyListener() {
		
			public void modifyText(ModifyEvent e) {
				
				path = combo.getText();
				IInputValidator validator = new RepositoryLocationInputValidator();
				String errorMsg = validator.isValid(path);
				
				if (isInvalidInput(errorMsg)) {
					handleInvalidInput(errorMsg);
				} else {
					handleValidInput();
				}
			}
		});
		
		// Button
		GridData buttonGd = new GridData();
		Button button = new Button(composite, SWT.PUSH);
		button.setText("Browse...");
		button.setLayoutData(buttonGd);
		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog dia = new DirectoryDialog(getShell());
				String pathInCombo = combo.getText();
				dia.setFilterPath(pathInCombo);
				String path = dia.open();
				if (path != null)
					combo.setText(path);
			}
		});
		
		return composite;
	}
	
	
	private boolean isInvalidInput(String errorMsg) {
		return errorMsg != null;
	}
	
	private void handleInvalidInput(String errorMsg) {
		Button okButton = getButton(IDialogConstants.OK_ID);
		okButton.setEnabled(false);
		setErrorMessage(errorMsg);
	}
	
	private void handleValidInput() {
		Button okButton = getButton(IDialogConstants.OK_ID);
		okButton.setEnabled(true);
		setErrorMessage(null);
	}
	
	
	@Override
	protected void okPressed() {
		
		// Save the entries of the combo box to the DialogSettings
		savePathFromCombo();
		
		// Exit this Dialog
		super.okPressed();
	}

	/**
	 * Returns selected path or null if nothing was selected.
	 * 
	 * @return
	 */
	public IPath getDirectory() {

		if (path == null || path.equals("")) {
			return null;
		}

		return new Path(path);
	}
	
	/**
	 * Adds all paths of previously used repositories to the combo box of this
	 * dialog. The paths are retrieved from the IDialogSettings of this plugin.
	 */
	private void addPathsToCombo() {
		
		// Get DialogSettings
		IDialogSettings settings = DeftPlugin.getInstance().getDialogSettings();
		
		// Get section of DialogSettings and add to stored path to the combo box
		IDialogSettings section = settings.getSection(DIALOG_SETTINGS_SECTION);
		if (section != null) {
			String[] paths = section.getArray(DIALOG_SETTINGS_ENTRY);
			combo.setItems(paths);
			combo.setText(paths[0]);
			combo.update();
		}
	}
	
	/**
	 * Saves all path from this dialogs combo box into the IDialogSettings of this
	 * plugin.
	 */
	private void savePathFromCombo() {
		
		// If combo box is empty, return and don't save anything
		String path = combo.getText();
		if (path == null || path.equals("")) {
			return;
		}
		
		// Get last used path from the combo box		
		Collection<String> storedPath = Arrays.asList(combo.getItems());
		
		// Convert this to a LinkedList for easy manipulation
		List<String> pathList = new LinkedList<String>(storedPath);
		
		// If this element is already present in the list, remove it
		pathList.remove(path);
		
		// Add the new item to the top of the list
		((LinkedList<String>)pathList).addFirst(path);
		
		// If now more then 5 items, remove the last one
		if (pathList.size() > 5) {
			pathList = pathList.subList(0, 5);
		}
		
		// Get dialog settings from plugin
		IDialogSettings settings = DeftPlugin.getInstance().getDialogSettings();
		
		// Get section
		IDialogSettings section = settings.getSection(DIALOG_SETTINGS_SECTION);
		
		// If section does not exist, create it
		if (section == null) {
			section = settings.addNewSection(DIALOG_SETTINGS_SECTION);
		}
		
		// Put the paths into the section
		section.put(DIALOG_SETTINGS_ENTRY, pathList.toArray(new String[0]));
		
	}

}
