package org.deft.gui.view.project;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.deft.ExtensionServiceExtension;
import org.deft.IDeftActionConstants;
import org.deft.RepositoryFacade;
import org.deft.actions.CreateScreenshotAction;
import org.deft.actions.DeftActionFactory;
import org.deft.actions.ExportAction;
import org.deft.actions.ForceRepresentationsUpdateAction;
import org.deft.actions.InsertRepresentationAction;
import org.deft.actions.SetArtifactReferenceCheckedAction;
import org.deft.actions.TestAction;
import org.deft.actions.UpdateArtifactAction;
import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.exporter.Exporter;
import org.deft.gui.EditorHelper;
import org.deft.gui.SelectionHelper;
import org.deft.integrator.EditorIntegrator;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Chapter;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.navigator.CommonActionProvider;
import org.eclipse.ui.navigator.ICommonActionConstants;
import org.eclipse.ui.navigator.ICommonActionExtensionSite;
import org.eclipse.ui.navigator.ICommonViewerSite;
import org.eclipse.ui.navigator.ICommonViewerWorkbenchSite;


/**
 * This class is used for the retargetable Action "Open" in the project view. Together with the
 * org.eclipse.ui.navigator.navigatorContent extension point, this class enables to open
 * fragments (chapters and artifacts) when the user double-clicks on them in the project explorer.
 * 
 * Also the menu entries for the context menu are defined here. We need to add them
 * dynamically depending on the plugins. We can't do that via plugin.xml
 */
public class ProjectActionProvider extends CommonActionProvider implements IDeftActionConstants {

	/**
	 * This is a cache for our created Actions
	 */
	private Map<String, IWorkbenchAction> actions = new HashMap<String, IWorkbenchAction>();
	private IWorkbenchWindow window;
	private ExtensionServiceExtension extService;
	private SelectionHelper selection;
	private EditorHelper editorHelper;

	public ProjectActionProvider() {
		this.extService = RepositoryFacade.instance().getExtensionService();
	}
	
	protected IWorkbenchWindow getWindow() {
		return window;
	}
	
	@Override
	public void init(ICommonActionExtensionSite aSite) {

		ICommonViewerSite viewSite = aSite.getViewSite();
		if (viewSite instanceof ICommonViewerWorkbenchSite) {
			
			ICommonViewerWorkbenchSite workbenchSite = (ICommonViewerWorkbenchSite) viewSite;
			window = workbenchSite.getWorkbenchWindow();
			//We only register actions which don't change (for reuse).
			//Actions like OPEN_DROP_DOWN or INSERT_DROP_DOWN may not be cached. They have to be recreated
			//every time the menu pops up, depending on the underlying artifact
			registerAction(DeftActionFactory.OPEN.create(window));
			registerAction(DeftActionFactory.GOTO.create(window));
			registerAction(DeftActionFactory.NEW_DROP_DOWN.create(window));
			registerAction(DeftActionFactory.IMPORT_DROP_DOWN.create(window));
			registerAction(DeftActionFactory.DELETE.create(window));
			registerAction(DeftActionFactory.RENAME.create(window));
			registerAction(DeftActionFactory.SET_PERSPECTIVE.create(window));
			registerAction(DeftActionFactory.UPDATE_WITH_DROP_DOWN.create(window));
		}
	}
	
	private void registerAction(IWorkbenchAction action) {
		String id = action.getId();
        Assert.isNotNull(id, "Action must not have null id"); //$NON-NLS-1$
        actions.put(id, action);
	}
	
	protected IAction getAction(String id) {
        return actions.get(id);
    }
	
	
	@Override
	public void dispose() {
		
		// Dispose all cached actions
		for (IWorkbenchAction action : actions.values()) {
			action.dispose();
		}
		
		super.dispose();
	}



	private SelectionHelper getSelectionHelper() {
		if (selection == null) {
			selection = new SelectionHelper(window);
		}
		return selection;
	}
	
	private EditorHelper getEditorHelper() {
		if (editorHelper == null) {
			editorHelper = new EditorHelper(window);
		}
		return editorHelper;
	}
	
	/**
	 * Defines that the Open-Action is to be executed upon double-click.
	 */
	@Override
	public void fillActionBars(IActionBars actionBars) {
		IAction openAction = getAction(DeftActionFactory.OPEN.getId());
		if (openAction.isEnabled()) {
			actionBars.setGlobalActionHandler(ICommonActionConstants.OPEN, openAction);
		}	
	}
	
	
	/**
	 * Setting the context menu of the Viewer.
	 * 
	 * Is called whenever a right click to the project view occurs
	 * 
	 */
	@Override
	public void fillContextMenu(IMenuManager menu) {
		menu.add(new Separator(GROUP_NEW));
		menu.add(new Separator(GROUP_OPEN));
		menu.add(new Separator(GROUP_EDIT));
		menu.add(new Separator(GROUP_VIEW));
		menu.add(new Separator());
		menu.add(new GroupMarker(GROUP_EXPORT));
		menu.add(new Separator(ADDITIONS));
		
		menu.appendToGroup(GROUP_NEW, getAction(DeftActionFactory.NEW_DROP_DOWN.getId()));
		menu.appendToGroup(GROUP_NEW, getAction(DeftActionFactory.IMPORT_DROP_DOWN.getId()));
		menu.appendToGroup(GROUP_OPEN, getAction(DeftActionFactory.OPEN.getId()));

		addInsertMenuIfApplicable(menu);
//		appendArtifactDropDownMenusIfApplicable(menu);

		menu.appendToGroup(GROUP_EDIT, getAction(DeftActionFactory.DELETE.getId()));
		menu.appendToGroup(GROUP_EDIT, getAction(DeftActionFactory.RENAME.getId()));
		menu.appendToGroup(GROUP_EDIT, new ForceRepresentationsUpdateAction(window));
		menu.appendToGroup(GROUP_EDIT, new CreateScreenshotAction(window));
		menu.appendToGroup(GROUP_EDIT, new TestAction(window));
		menu.appendToGroup(GROUP_EDIT, new UpdateArtifactAction(window));
		menu.appendToGroup(GROUP_EDIT, getAction(DeftActionFactory.UPDATE_WITH_DROP_DOWN.getId()));
		menu.appendToGroup(GROUP_EDIT, new SetArtifactReferenceCheckedAction(window));
		menu.appendToGroup(GROUP_VIEW, getAction(DeftActionFactory.SET_PERSPECTIVE.getId()));
//		menu.appendToGroup(GROUP_EDIT, getAction(DeftActionFactory.UPDATE.getId()));
		addExportMenuIfApplicable(menu);
	}
	

	
	/**
	 * Checks if only one node is selected and if this node is an ArtifactNode. 
	 */
	private boolean isSingleArtifactNodeSelected() {
		boolean singleArtifactNodeSelected = getSelectionHelper().isOnlySingleArtifactNodeSelected();
		return singleArtifactNodeSelected;
	}
	




	
	
	/**
	 * Adds an Insert menu item to the context menu if the selected fragment is an Artifact, the
	 * currently active editor is a chapter editor, and there exists at least one OperationChain+Integrator
	 * so that the artifact can be inserted into the chapter.
	 * 
	 * If there is only one way to insert the artifact, a normal "Insert" menu item is added. If there are
	 * multiple possibilities, an "Insert As" dropdown menu with the respective submenus is added instead.
	 */
	private void addInsertMenuIfApplicable(IMenuManager menu) {
		if (isSingleArtifactNodeSelected() && isChapterEditorActive()) {
			String artifactTypeId = getSelectionHelper().getSelectedArtifact().getArtifactTypeId();
			String chapterTypeId = getEditorHelper().getChapterOfActiveEditor().getTypeId();
			int insertCount = getNumberOfInsertableRepresentations(artifactTypeId, chapterTypeId);
			switch (insertCount) {
				case 0: {
					addDisabledInsertMenu(menu);
					break;
				}
				case 1: {
					addSimpleInsertMenuForArtifactNode(menu);
					break;
				}
				
				default: {
					addInsertDropdownMenuForArtifactNode(menu);
					break;
				}
			}
		} else {
			addDisabledInsertMenu(menu);
		}
	}
	
	
	private boolean isChapterEditorActive() {
		ChapterEditorPart chapterEditor = getEditorHelper().getActiveChapterEditor();
		return chapterEditor != null;
	}
	
	
	
	
	/**
	 * Returns how many OperationChain+Integrator pairs were found for a specific artifact type.
	 * The result is the same as the number of Open actions in the context menu. The result is used
	 * to determine if a simple or a dropdown open menu should be added to the context menu.
	 */
	private int getNumberOfInsertableRepresentations(String artifactTypeId, String chapterTypeId) {
		int count = 0;
		List<OperationChain> chains = extService.getCompatibleOperationChainsForEditorIntegration(artifactTypeId, chapterTypeId);
		for (OperationChain chain : chains) {
			List<EditorIntegrator> integrators = extService.getEditorIntegratorsForOperationChain(chain.getId());
			for (EditorIntegrator integrator : integrators) {
				if (integrator.getChapterTypeIds().contains(chapterTypeId)) {
					count++;
				}
			}
		}
		return count;
	}

	

	/**
	 * Creates and adds a dummy Insert menu, which will be disabled. This method is called
	 * if there is no way to insert an artifact into a chapter.
	 */
	private void addDisabledInsertMenu(IMenuManager menu) {
		IAction insertAction = new InsertRepresentationAction(window, "Insert", null, null);
		insertAction.setEnabled(false);
		menu.appendToGroup(GROUP_OPEN, insertAction);
	}
	
	
	/**
	 * Creates and adds a simple Insert action for artifacts. This method is called if
	 * there is only one way to display the artifact.  
	 */
	private void addSimpleInsertMenuForArtifactNode(IMenuManager menu) {
		String artifactTypeId = getSelectionHelper().getSelectedArtifact().getArtifactTypeId();
		String chapterTypeId = getEditorHelper().getChapterOfActiveEditor().getTypeId();
		IAction insertAction = createInsertActionForArtifact(artifactTypeId, chapterTypeId);
		menu.appendToGroup(GROUP_OPEN, insertAction);
	}
	
	
	/**
	 * Creates an Insert action for an artifact. This method assumes that there is only
	 * one OperationChain and only one Integrator available.
	 */
	private IAction createInsertActionForArtifact(String artifactTypeId, String chapterTypeId) {
		List<OperationChain> chains = extService.getCompatibleOperationChainsForEditorIntegration(artifactTypeId, chapterTypeId);
		OperationChain chain = chains.get(0);
		EditorIntegrator integrator = null;
		for (EditorIntegrator ei : extService.getEditorIntegratorsForOperationChain(chain.getId())) {
			if (ei.getChapterTypeIds().contains(chapterTypeId)) {
				integrator = ei;
			}
		}
		InsertRepresentationAction insertAction = new InsertRepresentationAction(window, "Insert", chain, integrator);
		return insertAction;
	}

	
	/**
	 * Adds an "Insert As" submenu to the context menu. This submenu will be filled with
	 * all possible OperationChain/Integrator combinations to display the artifact.
	 */
	private void addInsertDropdownMenuForArtifactNode(IMenuManager menu) {
		menu.appendToGroup(GROUP_OPEN, DeftActionFactory.INSERT_DROP_DOWN.create(window));	
	}


	
	/**
	 * Adds an Export menu item to the context menu if the selected fragment is a Chapter for which an exporter
	 * has been registered.
	 * 
	 * If there is only one exporter, a normal "Export" menu item is added. If there are
	 * multiple possibilities, an "Export As" dropdown menu with the respective submenus is added instead.
	 */
	private void addExportMenuIfApplicable(IMenuManager menu) {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleChapterNodeSelected()) {
			Chapter chapter = selection.getSelectedChapter();
			String chapterTypeId = chapter.getTypeId();
			List<String> exporterIds = extService.getExporterIds(chapterTypeId);
			int insertCount = exporterIds.size();
			switch (insertCount) {
				case 0: {
					addDisabledExportMenu(menu);
					break;
				}
				case 1: {
					addSimpleExportMenuForChapterNode(menu);
					break;
				}
				
				default: {
					addExportDropdownMenuForChapterNode(menu);
					break;
				}
			}
		} else {
			addDisabledExportMenu(menu);
		}
	}

	/**
	 * Creates and adds a dummy Insert menu, which will be disabled. This method is called
	 * if there is no way to insert an artifact into a chapter.
	 */
	private void addDisabledExportMenu(IMenuManager menu) {
		IAction exportAction = new ExportAction(getWindow(), "Export", null, null);
		exportAction.setEnabled(false);
		menu.appendToGroup(GROUP_EXPORT, exportAction);
	}
	
	
	/**
	 * Creates and adds a simple Export action for chapters. This method is called if
	 * there is only one exporter for the chapter type defined.  
	 */
	private void addSimpleExportMenuForChapterNode(IMenuManager menu) {
		Chapter chapter = selection.getSelectedChapter();
		String chapterTypeId = chapter.getTypeId();
		String exporterId = extService.getExporterIds(chapterTypeId).get(0);
		Exporter exporter = extService.getExporter(exporterId);
		List<WizardPage> pages = extService.getWizardPages(exporterId);
		IAction exportAction = new ExportAction(getWindow(), "Export", exporter, pages);
		menu.appendToGroup(GROUP_EXPORT, exportAction);
	}
	
	
	/**
	 * Adds an "Export As" submenu to the context menu. This submenu will be filled with
	 * all registered export actions for the chapter type.
	 */
	private void addExportDropdownMenuForChapterNode(IMenuManager menu) {
		menu.appendToGroup(GROUP_EXPORT, DeftActionFactory.EXPORT_DROP_DOWN.create(getWindow()));	
	}
	
}
