package org.deft.gui.view.project;

import java.util.ArrayList;
import java.util.List;

import org.deft.RepositoryFacade;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.deft.gui.view.project.nodes.ProjectViewNodeRoot;
import org.deft.repository.event.RepositoryUpdateListener;
import org.deft.repository.event.SimpleRepositoryUpdateListener;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.ICommonContentExtensionSite;
import org.eclipse.ui.navigator.ICommonContentProvider;
import org.eclipse.ui.progress.UIJob;

/**
 * The ProjectViewContentProvider defines how the tree in the project explorer looks like.
 * 
 * Please note that the ProjectExplorerView (a subclass of CommonNavigator), which uses this
 * class, must override method getInitialInput. The initial input should be the root of the
 * data to be displayed, but as we access our data directly from the repository, the initial
 * input can be any object (but not null). The initial input will be the parameter of
 * method getElements(Object).
 */
public class ProjectViewContentProvider  implements ICommonContentProvider {

	/**
	 * The XML tag name of the IMemento child in which all nodes are saved.
	 */
	private static final String EXPANDED_NODES = "org.deft.gui.view.project.expandednodes";
	
	/**
	 * The XML tag name of every node in the IMemento.
	 */
	private static final String NODE = "node";

	/**
	 * The XML attribute name of every node in the IMemento. In this attribute the actual id
	 * will be stored.
	 */
	private static final String NODE_ID = "id";

	private static final Object[] NO_CHILDREN = new Object[0];
	
	private TreeViewer treeViewer;
	private RepositoryFacade repository;
	private RepositoryUpdateListener updateListener = new UpdateListener(); 
	private ISelectionChangedListener selectionListener = new ActivateViewListener();
	private ProjectViewNodeRoot root = new ProjectViewNodeRoot();
	
	
	public ProjectViewContentProvider() {
		this.repository = RepositoryFacade.instance();
		this.repository.addRepositoryUpdateListener(updateListener);
		root.updateSubtree();
	}
	
	@Override
	public void init(ICommonContentExtensionSite aConfig) {
	}

	@Override
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof ProjectViewNode) {
			ProjectViewNode node = (ProjectViewNode)parentElement;
			Object[] children = node.getChildren().toArray();
			return children;
		}
		return NO_CHILDREN;
	}
	
	

	@Override
	public Object getParent(Object element) {
		if (element instanceof ProjectViewNode) {
			ProjectViewNode node = (ProjectViewNode)element;
			Object parent = node.getParent();
			return parent;
		}
		return null;
	}

	@Override
	public boolean hasChildren(Object element) {
		int childCount = getChildren(element).length;
		return (childCount > 0);
	}

	
	@Override
	public Object[] getElements(Object inputElement) {
		Object[] projects = getRootElements();
		return projects;
	}
	
	
	private Object[] getRootElements() {
		Object[] rootElements = root.getChildren().toArray();
		return rootElements;
	}

	@Override
	public void dispose() {
		repository.removeRepositoryUpdateListener(updateListener);
		repository = null;
		treeViewer = null;	
	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		this.treeViewer = (TreeViewer)viewer;
		//if this method is called more than once make sure we don't have the same
		//listener attached twice
		treeViewer.removeSelectionChangedListener(selectionListener);
		treeViewer.addSelectionChangedListener(selectionListener);

	}

	@Override
	public void restoreState(IMemento aMemento) {
		if (containsValidData(aMemento)) {
			List<Object> expandedNodes = getNodesToExpand(aMemento);

			// Expand the TreeViewer
			treeViewer.setExpandedElements(expandedNodes.toArray());
		}
	}
	
	private boolean containsValidData(IMemento memento) {
		boolean notNull = memento != null;
		boolean containsValidData = notNull && hasExpandedNodes(memento) && expandedNodesHaveChildren(memento);
		return containsValidData;
	}
	
	private boolean hasExpandedNodes(IMemento memento) {
		boolean hasExpandedNodes = memento.getChild(EXPANDED_NODES) != null;
		return hasExpandedNodes;
	}
	
	private boolean expandedNodesHaveChildren(IMemento memento) {
		boolean expandedNodesHaveChildren = memento.getChild(EXPANDED_NODES).getChildren(NODE) != null;
		return expandedNodesHaveChildren;
	}
	
	private List<Object> getNodesToExpand(IMemento memento) {
		List<Object> nodes = new ArrayList<Object>();
		memento = memento.getChild(EXPANDED_NODES);

		for (IMemento m : memento.getChildren(NODE)) {
			String id = m.getString(NODE_ID);
			ProjectViewNode node = root.getNodeWithId(id);
			if (node != null) {
				nodes.add(node);
			}
		}
		return nodes;
	}
	

	@Override
	public void saveState(IMemento aMemento) {
		IMemento memento = aMemento.createChild(EXPANDED_NODES);
		addMementoForAllExpandedNodes(memento);
	}
	
	private void addMementoForAllExpandedNodes(IMemento memento) {
		for (Object expandedElement : treeViewer.getExpandedElements()) {
			if (expandedElement != null && expandedElement instanceof ProjectViewNode) {
				IMemento child = memento.createChild(NODE);
				ProjectViewNode node = (ProjectViewNode) expandedElement;
				child.putString(NODE_ID, node.getId());
			}
		}
	}

	
	
	private void updateView() {
		new UIJob("Update Fragment Model in CommonViewer") {
			public IStatus runInUIThread(IProgressMonitor monitor) {
				if (treeViewer != null && !treeViewer.getControl().isDisposed()) {
					//the root contains ProjectViewNodes which may ultimately reference
					//repository objects which might not exist anymore after a repository
					//location change. Therefore make a new root and force a clean loading
					//of the repository content
//					root = new ProjectViewNodeRoot();
					root.updateSubtree();
					treeViewer.refresh();
				}
				return Status.OK_STATUS;
			}
		}.schedule();
	}
	
	
	private class UpdateListener extends SimpleRepositoryUpdateListener {

		@Override
		public void repositoryUpdated() {
			updateView();
		}
	}
	
	/**
	 * Activate Project View when one of the tree elements is selected. Usually this happens
	 * automatically. However, when the OO editor has been opened it is possible to select
	 * tree elements without the Project View being active. This listener fixes that behavior.
	 */
	private class ActivateViewListener implements ISelectionChangedListener {

		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			IViewPart projectView = page.findView(ProjectExplorerView.ID);
			page.activate(projectView);
		}
		
	}
	
}
