package org.deft.gui.view.project.nodes;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

/**
 * Superclass of all nodes that are displayed in the project explorer. ProjectViewNodes wrap elements from the
 * repository and provide information how to display them (label, icon). 
 * 
 * The nodes form a tree, which will be displayed 1:1 in the project explorer. Each node knows how to recursively 
 * produce its descendants. This happens usually by querying the repository for appropriate content. The tree is
 * rebuilt whenever the repository changes. The rebuild is triggered by the ProjectViewContentProvider. New child
 * nodes are only created if necessary, otherwise existing child nodes are reused. 
 */
public abstract class ProjectViewNode {
	
	private List<ProjectViewNode> children = new LinkedList<ProjectViewNode>();
	private ProjectViewNode parent;
	
	/**
	 * Returns the text to be displayed in the project explorer view 
	 */
	public abstract String getText();
	
	/**
	 * Returns the icon to be displayed in the project explorer view 
	 */
	public abstract Image getIcon();
	
	/**
	 * Returns the id of the node. In most cases this is the id of the wrapped content from
	 * the repository. 
	 */
	public abstract String getId();
	
	public abstract boolean deletionAllowed(); 
	
	/**
	 * Adds a node as a child of this node.
	 */
	public void addChild(ProjectViewNode child) {
		children.add(child);
		child.parent = this;
	}

	/**
	 * Removes a node as a child of this node. 
	 */
	public void removeChild(ProjectViewNode child) {
		if (children.contains(child)) {
			children.remove(child);
			child.parent = null;
		}
	}
	

	/**
	 * Returns all child nodes. 
	 */
	public List<ProjectViewNode> getChildren() {
		return children;
	}
	
	/**
	 * Returns the parent node. 
	 */
	public ProjectViewNode getParent() {
		return parent;
	}
	

	/**
	 * Searches over all descendant nodes a node with the given id. If it is found,
	 * it is returned. If no node with the id could be found, null is returned. 
	 */
	public ProjectViewNode getNodeWithId(String id) {
		if (getId().equals(id)) {
			return this;
		} else {
			for (ProjectViewNode node : getDescendants()) {
				if (node.getId().equals(id)) {
					return node;
				}
			}
		}
		return null;
	}
	
	
	/**
	 * Creates recursively a depth-first ordered list of the descendants of this node. The order
	 * of the nodes on one "hierarchy level" is unspecified.
	 */
	public List<ProjectViewNode> getDescendants() {
		List<ProjectViewNode> descendants = new LinkedList<ProjectViewNode>();
		for (ProjectViewNode child : getChildren()) {
			descendants.add(child);
			List<ProjectViewNode> childDescendants = child.getDescendants();
			descendants.addAll(childDescendants); 
		}
		return descendants;
		
	}
	

	

	/**
	 * Updates all descendants. After the update the tree reflects the contents of the
	 * repository. The update works as follows:
	 * 1 - The list of children is copied, then all children are removed.
	 * 2 - For each content to be displayed (usually taken from the repository) it is checked
	 *     if the child list copy contains already a node. If yes, the node is taken and added
	 *     as child. If not, a new node is created and added as child.
	 * 3 - This method is called recursively at the newly added child.
	 */
	public void updateSubtree() {
		List<ProjectViewNode> childrenCopy = new LinkedList<ProjectViewNode>(getChildren());
		removeAllChildren();
		for (Object content : getListOfChildContentToDisplay()) {
			ProjectViewNode node = getNodeFor(content, childrenCopy);
			if (node == null) {
				node = createNodeFor(content);
			}
			addChild(node);
			node.updateSubtree();
		}
	}

	/**
	 * Removes all children from this node.
	 */
	protected void removeAllChildren() {
		//copy of children list, to prevent ConcurrentModificationException
		List<ProjectViewNode> childrenCopy = new LinkedList<ProjectViewNode>(getChildren());
		for (ProjectViewNode child : childrenCopy) {
			removeChild(child);
		}
	}
	
	/**
	 * Returns a list of objects which should somehow be displayed in the project explorer tree.
	 * These objects will be used to create appropriate ProjectViewNodes. 
	 */
	protected abstract List<? extends Object> getListOfChildContentToDisplay();

	/**
	 * Checks a list of ProjectViewNodes if one contains a specific content object. If yes
	 * the node is returned. If not, null is returned. 
	 */
	protected ProjectViewNode getNodeFor(Object content, List<ProjectViewNode> nodes) {
		for (ProjectViewNode child : nodes) {
			if (child.getContent().equals(content)) {
				return child;
			}
		}
		return null;
	}
	
	/**
	 * Returns the content of the node. The content is the actual object (usually retrieved from the repository)
	 * which should be displayed in the ProjectExplorer. (after all, the ProjectViewNodes are only wrappers for
	 * a uniform handling)
	 */
	protected abstract Object getContent();
	
	/**
	 * Creates a specific ProjectViewNode for a content object. The created node
	 * will be a child of this node. So if this node does not have children, null
	 * can be returned here.
	 */
	protected abstract ProjectViewNode createNodeFor(Object childContent);

}

