package org.deft.gui.widgets;

import java.util.ArrayList;

import org.eclipse.jface.fieldassist.ContentProposal;
import org.eclipse.jface.fieldassist.IContentProposal;
import org.eclipse.jface.fieldassist.IContentProposalProvider;

/**
 * Provides an array of IContentProposals for hierarchical data.
 */
public class RegexContentProposalProvider implements IContentProposalProvider {

	/**
	 * The proposals provided.
	 */
	private String[] proposals;
	
	/**
	 * Character that serves as delimiter for the hierarchical data.
	 * For example: the . (dot) in org.deft.some.Class
	 * or the / (slash) in some/nested/Artifact 
	 */
	private char delimiter;

    /**
	 * @param proposals
	 *            the array of Strings to be returned whenever proposals are
	 *            requested.
	 */
	public RegexContentProposalProvider() {
		super();
		this.proposals = new String[] {};
		this.delimiter = '.';
	}

	/**
	 * Return an array of Objects representing the valid content proposals for a
	 * field. 
	 * 
	 * @param contents
	 *            the current contents of the field (only consulted if filtering
	 *            is set to <code>true</code>)
	 * @param position
	 *            the current cursor position within the field (ignored)
	 * @return the array of Objects that represent valid proposals for the field
	 *         given its current content.
	 */
	public IContentProposal[] getProposals(String contents, int position) {
		ArrayList<ContentProposal> list = new ArrayList<ContentProposal>();
		for (int i = 0; i < proposals.length; i++) {
			if (proposals[i].length() >= contents.length()
					&& fits(proposals[i], contents)) {
				list.add(new ContentProposal(proposals[i]));
			}
		}
		return (IContentProposal[]) list.toArray(new IContentProposal[list.size()]);
	}
	
	/**
	 * Checks if a concrete string fits the proposal (the user-entered string).
	 * 
	 *  For flexibility and a good user experience some magic has been added.
	 *  - if the user entered a * (asterisk), arbitrary characters are matched against it
	 *  - it is not necessary to add an asterisk in order to match an element of a
	 *    fully qualified name. Example: "someMe" will match "pckg.MyClass.someMethod(Object)".   
	 */
	private boolean fits(String proposal, String contents) {
		String ANYCHARS = ".*?";
		//escape dots and other regex specific chars
		String regexContents = contents.replaceAll("\\.", "\\\\.");
		regexContents = regexContents.replaceAll("\\(", "\\\\(");
		regexContents = regexContents.replaceAll("\\)", "\\\\)");
		regexContents = regexContents.replaceAll("\\[", "\\\\[");
		regexContents = regexContents.replaceAll("\\]", "\\\\]");
		//replace asterisks by the regex for "arbitrary number of arbitrary chars"
		regexContents = regexContents.replaceAll("\\*", ANYCHARS);
		//add regex for arbitrary matches to the end, so that the proposal also fits if it 
		//only matches the beginning of the contents
		regexContents += ANYCHARS;
		//if the proposal does not start with an asterisk (which has already been replaced
		//by the arbitrary chars regex) make it start with the arbitrary chars regex and an optional delimiter.
		//This matches the proposal with every element of a dot separated qualified name.
		//(see example in method comment)
		if (!regexContents.startsWith(ANYCHARS)) {
			regexContents = ANYCHARS + getDelimiterAsRegex() + "?" + regexContents;
		}
		//ignore case
		boolean fits = proposal.toLowerCase().matches(regexContents.toLowerCase());
		return fits;
	}

	/**
	 * Set the Strings to be used as content proposals.
	 */
	public void setProposals(String[] items) {
		this.proposals = items;
	}
	
	public void setDelimiter(char delimiter) {
		this.delimiter = delimiter;
	}

	//expand this as needed
	public String getDelimiterAsRegex() {
		if (delimiter == '.') {
			return "\\.";
		}
		return new Character(delimiter).toString();
	}


}
