package org.deft.guidance;

import java.util.LinkedList;
import java.util.List;

import org.deft.guidance.tasks.Task;
import org.deft.guidance.tasks.TaskList;
import org.deft.guidance.tasks.TaskListProvider;
import org.deft.repository.Repository;
import org.deft.repository.event.SimpleRepositoryUpdateListener;
import org.eclipse.swt.widgets.Display;
import org.semanticweb.owl.model.OWLOntology;

/**
 * The GuidanceTaskListManager creates and manages a list of guidance tasks according the state of
 * the DEFT repository.
 * 
 * The manager listens to changes in the repository and, if changes are fired, recomputes the guidance tasks.
 * The new guidance tasks are then propagated to listeners. 
 */
public class GuidanceTaskListManager {

	private static GuidanceTaskListManager manager;
	private OWLWriter owlWriter = new OWLWriter();
	private List<Task> tasks = new LinkedList<Task>();
	private TaskViewRepositoryUpdateListener repositoryListener;
	
	private List<TaskListUpdateListener> listeners = new LinkedList<TaskListUpdateListener>();
	
	private GuidanceTaskListManager() {
		this.repositoryListener = new TaskViewRepositoryUpdateListener();
		Repository.instance.getContentManagementService().addRepositoryUpdateListener(repositoryListener);
	}
	
	
	public static GuidanceTaskListManager instance() {
		if (manager == null) {
			manager = new GuidanceTaskListManager();
		}
		return manager;
	}
	
	public void addTaskListUpdateListener(TaskListUpdateListener listener) {
		listeners.add(listener);
	}
	
	public void removeTaskListUpdateListener(TaskListUpdateListener listener) {
		listeners.remove(listener);
	}
	
	
	
	public List<Task> getTasks() {
		return new LinkedList<Task>(tasks);
	}
	
	
	/**
	 * Updates the list of open guidance tasks and fires an update event.
	 */
	public void updateTaskList() {
		tasks.clear();

		OWLOntology ontology = owlWriter.createOWLObjectFromRepository();
		TaskListProvider provider = new TaskListProvider(ontology);
		TaskList taskList = provider.getTaskList();

		List<String> namesOfOpenCompulsoryTasks = taskList.getNamesOfOpenCompulsoryTasks();
		List<String> namesOfOpenOptionalTasks = taskList.getNamesOfOpenOptionalTasks();

		for (String ctask : namesOfOpenCompulsoryTasks) {
			List<Task> tasksOfThisName = taskList.getOpenCompulsoryTasks(ctask);
			for (Task t : tasksOfThisName) {
				tasks.add(t);
			}
		}

		for (String otask : namesOfOpenOptionalTasks) {
			List<Task> tasksOfThisName = taskList.getOpenOptionalTasks(otask);
			for (Task t : tasksOfThisName) {
				tasks.add(t);
			}
		}
		fireTaskListUpdated(tasks);
	}


	
	private void fireTaskListUpdated(List<Task> tasks) {
		for (TaskListUpdateListener listener : listeners) {
			try {
				listener.updated(tasks);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	
	
	private class TaskViewRepositoryUpdateListener extends SimpleRepositoryUpdateListener {

		@Override
		public void repositoryUpdated() {
			Display.getDefault().syncExec(new Runnable() {

				@Override
				public void run() {
					updateTaskList();
				}
			});
		}
	}	
}
