package org.deft.guidance;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.List;

import org.deft.repository.RepositoryImpl;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.semanticweb.owl.apibinding.OWLManager;
import org.semanticweb.owl.io.RDFXMLOntologyFormat;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLDataFactory;
import org.semanticweb.owl.model.OWLIndividual;
import org.semanticweb.owl.model.OWLIndividualAxiom;
import org.semanticweb.owl.model.OWLObjectProperty;
import org.semanticweb.owl.model.OWLObjectPropertyAssertionAxiom;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyChangeException;
import org.semanticweb.owl.model.OWLOntologyManager;
import org.semanticweb.owl.model.OWLOntologyStorageException;
import org.semanticweb.owl.model.UnknownOWLOntologyException;

public class OWLWriter {

	private OWLDataFactory odf;
	private OWLOntologyManager oom;
	private OWLOntology ontology;
	private ContentManagementService cms;
	private OntologyAccessor accessor;
	
	public OWLWriter() {
		oom = OWLManager.createOWLOntologyManager();
		odf = oom.getOWLDataFactory();
		this.accessor = new OntologyAccessor();
		this.cms = RepositoryImpl.instance().getContentManagementService();
	}
	
	/**
	 * This method creates the ontology
	 * 
	 * @return OWL Ontology
	 */
	public OWLOntology createOWLObjectFromRepository() {

		try {
			List<Project> projectList = cms.getProjects();
			URL ontologyUrl = OWLPlugin.getEmptyOntologyUrl();
			accessor.loadOntologyFromStream(ontologyUrl.openStream());
			ontology = accessor.getOntology();
			OWLIndividual workspaceIndividual = accessor.getIndividual("Workspace");
			for (Project p : projectList) {
				createProjectAxiom(p, workspaceIndividual);
				createChapterAxioms(p);
				createArtifactAxioms(p);
				createChangedArtifactAxioms(p);
				createUncheckedReferenceAxioms(p);
				createReferenceWithInvalidRepresentationAxioms(p);
				createReferenceToDeletedArtifactAxioms(p);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
//		saveOntologyInFile();
		return ontology;
	}

	@SuppressWarnings("unused")
	private void saveOntologyInFile() {
		File saveFile = new File("d:\\deft.owl");
		try {
			saveFile.createNewFile();
			oom.saveOntology(ontology, new RDFXMLOntologyFormat(), saveFile.toURI());
		} catch (UnknownOWLOntologyException e) {
			e.printStackTrace();
		} catch (OWLOntologyStorageException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	///////////////////////////////////////////////
	/////////////// Project ///////////////////////
	///////////////////////////////////////////////
	
	private void createProjectAxiom(Project p, OWLIndividual workspace) {
		try {
			OWLClass projectClass = accessor.getClass("Project");
			OWLIndividual project = accessor.getIndividual(p.getId());
			OWLIndividualAxiom projectAxiom = odf.getOWLClassAssertionAxiom(project, projectClass);
			oom.addAxiom(ontology, projectAxiom);
			createCreateProjectTaskForAlreadyCreatedProject(project, workspace);
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}

	}
	

	/**
	 * Creates an individual and object properties for the task of having created a certain project.
	 * This individual and the properties are necessary for the reasoner to realize that a 
	 * project has already been created in the workspace and that there should be no task for creating a project. 
	 */
	private void createCreateProjectTaskForAlreadyCreatedProject(OWLIndividual project, OWLIndividual workspace) {
		try {
			OWLClass createProjectTaskClass = accessor.getClass("CreateProjectTask");
			String projectUriString = project.getURI().toString();
			String projectId = accessor.getSimpleName(projectUriString);
			OWLIndividual createProjectTask = accessor.getIndividual("Task-Create" + projectId);
			
			OWLIndividualAxiom createProjectTaskAxiom = odf.getOWLClassAssertionAxiom(
					createProjectTask, createProjectTaskClass);

			OWLObjectProperty postProp = accessor.getObjectProperty("hasPostcondition");
			OWLObjectProperty createInWorkspaceProperty = accessor.getObjectProperty("createInWorkspace");
			OWLObjectPropertyAssertionAxiom createInWorkspaceProp = odf.getOWLObjectPropertyAssertionAxiom(
					workspace, createInWorkspaceProperty, createProjectTask);
			OWLObjectPropertyAssertionAxiom assertion = odf.getOWLObjectPropertyAssertionAxiom(
					createProjectTask, postProp, project);
			
			oom.addAxiom(ontology, createInWorkspaceProp);
			oom.addAxiom(ontology, createProjectTaskAxiom);
			oom.addAxiom(ontology, assertion);

		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}

	}

	
	
	
	
	
	
	
	///////////////////////////////////////////////
	/////////////// Chapter ///////////////////////
	///////////////////////////////////////////////
	
	
	/**
	 * Creates a Chapter individual for each Chapter in the repository and writes it into
	 * the ontology. Also creates a CreateChapterTask for each Chapter and writes it into
	 * the ontology. The CreateChaptertTask represents a task that has previously been executed.
	 * It is therefore connected via a property with its resulting chapter. 
	 */
	private void createChapterAxioms(Project p) {
		try {
			OWLClass chapterClass = accessor.getClass("Chapter");
			List<Fragment> chapters = p.getFragments(FragmentFilter.CHAPTERFILTER);
			for (Fragment f : chapters) {
				Chapter c = (Chapter)f;
				createChapterReferencesAxioms(c);
				OWLIndividual chapterInd = accessor.getIndividual(c.getId());
				OWLIndividual createChapterTask = createCreateChapterTaskForAlreadyCreatedChapter(chapterInd);
				createProjectPropertyForChapter(createChapterTask, p);
				OWLIndividualAxiom chapterAxiom = odf.getOWLClassAssertionAxiom(chapterInd, chapterClass);
				oom.addAxiom(ontology, chapterAxiom);
			}
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Adds a createInProject property from the project individual to the chapter individual.
	 * 
	 * This property states that a certain chapter has been created in a certain project. 
	 */
	private void createProjectPropertyForChapter(OWLIndividual createChapterTaskInd, Project p) {
		try {
			OWLIndividual projectInd = accessor.getIndividual(p.getId());
			OWLObjectProperty createInProjectProp = accessor.getObjectProperty("createInProject");
			OWLObjectPropertyAssertionAxiom assertion = odf.getOWLObjectPropertyAssertionAxiom(
					projectInd, createInProjectProp, createChapterTaskInd);
			oom.addAxiom(ontology, assertion);
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
	}


	
	/**
	 * Creates and returns (and stores in the ontology) an individual that represents the task which led
	 * to the creation of a chapter. As the chapter is in the DEFT repository, the creation task must have
	 * already been executed in the past.
	 * The CreateChapterTask individual and the corresponding properties are necessary for the reasoner to realize 
	 * that a chapter has already been created and that there should be no open task for creating a chapter.   
	 */
	private OWLIndividual createCreateChapterTaskForAlreadyCreatedChapter(OWLIndividual chapterInd) {
		try {
			OWLClass createChapterTaskClass = accessor.getClass("CreateChapterTask");
			String chapterUriString = chapterInd.getURI().toString();
			String chapterId = accessor.getSimpleName(chapterUriString);
			OWLIndividual createChapterTaskInd = accessor.getIndividual("Task-Create" + chapterId);
			OWLIndividualAxiom createChapterTaskAxiom = odf.getOWLClassAssertionAxiom(
					createChapterTaskInd, createChapterTaskClass);
			OWLObjectProperty postProp = accessor.getObjectProperty("hasPostcondition");
			OWLObjectPropertyAssertionAxiom assertion = odf.getOWLObjectPropertyAssertionAxiom(
					createChapterTaskInd, postProp, chapterInd);
			oom.addAxiom(ontology, createChapterTaskAxiom);
			oom.addAxiom(ontology, assertion);
			return createChapterTaskInd;
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
		return null;
	}


	
	
	
	
	
	
	
	
	
	///////////////////////////////////////////////
	/////////////// Artifact //////////////////////
	///////////////////////////////////////////////
	

	/**
	 * Creates an Artifact individual for each Artifact in the repository and writes it into
	 * the ontology. Also creates a CreateArtifactTask for each Artifact and writes it into
	 * the ontology. The CreateArtifactTask represents a task that has previously been executed.
	 * It is therefore connected via a property with its resulting artifact. 
	 */
	private void createArtifactAxioms(Project p) {
		try {
			OWLClass artifactClass = accessor.getClass("Artifact");
			List<Fragment> artifacts = p.getFragments(FragmentFilter.ARTIFACTFILTER);
			for (Fragment f : artifacts) {
				Artifact a = (Artifact)f;
				createArtifactReferencesAxioms(a);
				OWLIndividual artifactInd = accessor.getIndividual(a.getId());
				OWLIndividual createArtifactTask = createImportArtifactTaskForAlreadyImportedArtifact(artifactInd);
				createProjectPropertyForArtifact(createArtifactTask, p);
				OWLIndividualAxiom artifactAxiom = odf.getOWLClassAssertionAxiom(artifactInd, artifactClass);
				oom.addAxiom(ontology, artifactAxiom);
			}
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Adds a createInProject property from the project individual to the chapter individual.
	 * 
	 * This property states that a certain chapter has been created in a certain project. 
	 */
	private void createProjectPropertyForArtifact(OWLIndividual importArtifactTaskInd, Project p) {
		try {
			OWLIndividual projectInd = accessor.getIndividual(p.getId());
			OWLObjectProperty importIntoProjectProp = accessor.getObjectProperty("importIntoProject");
			OWLObjectPropertyAssertionAxiom assertion = odf.getOWLObjectPropertyAssertionAxiom(
					projectInd, importIntoProjectProp, importArtifactTaskInd);
			oom.addAxiom(ontology, assertion);
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
	}
	


	
	/**
	 * Creates and returns (and stores in the ontology) an individual that represents the task which led
	 * to the import of an artifact. As the artifact is in the DEFT repository, the import task must have
	 * been executed in the past.
	 * The ImportArtifactTask individual and the corresponding properties are necessary for the reasoner to realize 
	 * that an artifact has already been imported and that there should be no open task for importing an artifact.   
	 */
	private OWLIndividual createImportArtifactTaskForAlreadyImportedArtifact(OWLIndividual artifactInd) {
		try {
			OWLClass importArtifactTaskClass = accessor.getClass("ImportArtifactTask");
			String artifactUriString = artifactInd.getURI().toString();
			String artifactId = accessor.getSimpleName(artifactUriString);
			OWLIndividual importArtifactTaskInd = accessor.getIndividual("Task-Import" + artifactId);

			OWLIndividualAxiom importArtifactTaskAxiom = odf.getOWLClassAssertionAxiom(
					importArtifactTaskInd,importArtifactTaskClass);
			OWLObjectProperty postconditionProp = accessor.getObjectProperty("hasPostcondition");
			OWLObjectPropertyAssertionAxiom assertion = odf.getOWLObjectPropertyAssertionAxiom(
					importArtifactTaskInd, postconditionProp, artifactInd);
			oom.addAxiom(ontology, importArtifactTaskAxiom);
			oom.addAxiom(ontology, assertion);
			return importArtifactTaskInd;
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	
	
	
	
	
	
	
	
	
	///////////////////////////////////////////////
	/////////////// ChapterReference //////////////
	///////////////////////////////////////////////
	
	
	private void createChapterReferencesAxioms(Chapter c) {
		try {
			OWLClass referenceClass = accessor.getClass("Reference");
			
			List<ArtifactReference> artifactReferences = c.getArtifactReferences();
			for (ArtifactReference ar : artifactReferences) {
				OWLIndividual reference = accessor.getIndividual(ar.getId());
				OWLIndividualAxiom artifactRefAxiom = odf.getOWLClassAssertionAxiom(reference, referenceClass);
				oom.addAxiom(ontology, artifactRefAxiom);
			}
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
	}
	
	
	private void createArtifactReferencesAxioms(Artifact artifact) {
		try {
			OWLClass referenceClass = accessor.getClass("Reference");
			List<ArtifactReference> artifactReferences = artifact.getArtifactReferences();
			for (ArtifactReference ar : artifactReferences) {
				OWLIndividual reference = accessor.getIndividual(ar.getId());
				OWLIndividualAxiom artifactRefAxiom = odf.getOWLClassAssertionAxiom(reference, referenceClass);
				oom.addAxiom(ontology, artifactRefAxiom);
			}
		} catch (OWLOntologyChangeException e) {
			e.printStackTrace();
		}
	}
	
	
	
	private void createChangedArtifactAxioms(Project p) {
		OWLClass artifactOutOfSyncClass = accessor.getClass("ArtifactOutOfSync");
		
		List<Artifact> artifacts = cms.getArtifactsWithChangedOrigin(p);
		for (Artifact artifact : artifacts) {
			OWLIndividual artifactInd = accessor.getIndividual(artifact.getId());
			OWLIndividualAxiom axiom = odf.getOWLClassAssertionAxiom(artifactInd, artifactOutOfSyncClass);
			try {
				oom.addAxiom(ontology, axiom);
			} catch (OWLOntologyChangeException e) {
				e.printStackTrace();
			}
		}
	}

	
	
	
	private void createUncheckedReferenceAxioms(Project p) {
		createCheckReferenceAxioms(p, "RepresentationUpdatedButUnchecked", ArtifactReferenceUpdateType.NORMALUPDATE);
	}

	
	private void createReferenceWithInvalidRepresentationAxioms(Project p) {
		createCheckReferenceAxioms(p, "ReferenceWithInvalidRepresentation", ArtifactReferenceUpdateType.UNCOMPUTABLEUPDATE);
	}
	
	
	private void createReferenceToDeletedArtifactAxioms(Project p) {
		createCheckReferenceAxioms(p, "ReferenceToDeletedArtifact", ArtifactReferenceUpdateType.MISSINGARTIFACT);
	}
	
	private void createCheckReferenceAxioms(Project p, String ontologyClassName, 
			ArtifactReferenceUpdateType updateType) {
		OWLClass owlClass = accessor.getClass(ontologyClassName);
		List<ArtifactReference> references = p.getArtifactReferences();
		for (ArtifactReference reference : references) {
			if (reference.getUpdateType() == updateType) {
				OWLIndividual referenceInd = accessor.getIndividual(reference.getId());
				OWLIndividualAxiom axiom = odf.getOWLClassAssertionAxiom(referenceInd, owlClass);
				try {
					oom.addAxiom(ontology, axiom);
				} catch (OWLOntologyChangeException e) {
					e.printStackTrace();
				}
			}
		}
	}

	
	
}
