package org.deft.guidance;
import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.semanticweb.owl.apibinding.OWLManager;
import org.semanticweb.owl.io.StreamInputSource;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLConstant;
import org.semanticweb.owl.model.OWLDataFactory;
import org.semanticweb.owl.model.OWLDataProperty;
import org.semanticweb.owl.model.OWLDataPropertyExpression;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLIndividual;
import org.semanticweb.owl.model.OWLObjectProperty;
import org.semanticweb.owl.model.OWLObjectPropertyExpression;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyCreationException;
import org.semanticweb.owl.model.OWLOntologyManager;

public class OntologyAccessor
{
	private OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
	private OWLDataFactory factory = manager.getOWLDataFactory();
	private OWLOntology ontology;
	private File file;
	private String ontologyUriString;

	// loads the ontology from a specific file
	public void loadOntologyFromFile(String filename)
	{
		try
		{
			// load ontology from local file
			file = new File(filename);
			ontology = manager.loadOntologyFromPhysicalURI(file.toURI());
			ontologyUriString = ontology.getURI().toString();
		}
		catch (OWLOntologyCreationException e)
		{
			System.out.println("failed to load ontology from file " + file.toURI() + "\n" + e.getMessage());
		}
		// end try catch
		
	}
	// end loadOntology
	
	
	public void loadOntologyFromStream(InputStream stream) 
	{
		try
		{
			StreamInputSource source = new StreamInputSource(stream);
			ontology = manager.loadOntology(source);
			ontologyUriString = ontology.getURI().toString();
		}
		catch (OWLOntologyCreationException e)
		{
			System.out.println("failed to load ontology from stream" + "\n" + e.getMessage());
		}
		// end try catch
		
	}
	// end loadOntology
	
	
	// returns the URI of a specific class or property or individual
	public URI getUri(String simpleClassOrPropertyOrIndividualName)
	{
		URI uri = URI.create(this.ontologyUriString + "#" + simpleClassOrPropertyOrIndividualName);
		return uri;
	}
	// end getUri
	
	// returns the string of an URI of a specific class or property 
	public String getUriString(String simpleClassOrPropertyOrIndividualName)
	{
		URI uri = this.getUri(simpleClassOrPropertyOrIndividualName);
		return uri.toString();
	}
	// end getUriString
	
	// returns the simple name of an URI
	public String getSimpleName(String uriString)
	{
		int positionOfChar = uriString.indexOf('#');
		String simpleName = uriString.substring(positionOfChar + 1);
		return simpleName;
	}
	// end getSimpleName

	// returns an owl class
	public OWLClass getClass(String simpleClassName)
	{
		URI classUri = this.getUri(simpleClassName);
		OWLClass owlClass = factory.getOWLClass(classUri);
		return owlClass;
	}
	// end getOwlClass

	// returns an owl individual
	public OWLIndividual getIndividual(String simpleIndividualName)
	{
		URI individualUri = this.getUri(simpleIndividualName);
		OWLIndividual individual = factory.getOWLIndividual(individualUri);
		return individual;
	}
	// end getOwlIndividual
	
	// returns an owl object property
	public OWLObjectProperty getObjectProperty(String simpleObjectPropertyName)
	{
		URI objectPropertyUri = this.getUri(simpleObjectPropertyName);
		OWLObjectProperty objectProperty = factory.getOWLObjectProperty(objectPropertyUri);
		return objectProperty;
	}
	// end getOwlObjectProperty
	
	// returns an owl data property
	public OWLDataProperty getDataProperty(String simpleDataPropertyName)
	{
		URI dataPropertyUri = this.getUri(simpleDataPropertyName);
		OWLDataProperty dataProperty = factory.getOWLDataProperty(dataPropertyUri);
		return dataProperty;
	}
	// end getOwlObjectProperty

	// returns a set of all individuals to a given class name
	public Set<OWLIndividual> getAllIndividuals(String simpleClassName)
	{
		OWLClass owlClass = this.getClass(simpleClassName);
		Set<OWLIndividual> individualSet = owlClass.getIndividuals(ontology);
		return individualSet;
	}
	// end getAllOwlIndividuals
	
	// returns a map of all object properties to a given individual
	public Map<OWLObjectPropertyExpression,Set<OWLIndividual>> getAllObjectProperties(OWLIndividual individual)
	{
		Map<OWLObjectPropertyExpression,Set<OWLIndividual>> objectPropertyMap = individual.getObjectPropertyValues(ontology);
		return objectPropertyMap;
	}
	// end getAllObjectProperties
	
	// returns a map of all data properties to a given individual
	public Map<OWLDataPropertyExpression,Set<OWLConstant>> getAllDataProperties(OWLIndividual individual)
	{
		Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap = individual.getDataPropertyValues(ontology);
		return dataPropertyMap;
	}
	// end getAllDataProperties
	
	// returns the set of types of a given individual
	public Set<OWLDescription> getTypes(OWLIndividual individual)
	{
		Set<OWLDescription> typeSet = individual.getTypes(ontology);
		return typeSet;
	}
	// end getClassName
	
	// returns true if the given individual types the given class name
	public boolean isType(OWLIndividual individual, String simpleClassName)
	{
		boolean isType = false;
		Set<OWLDescription> typeSet = this.getTypes(individual);
		
		for (OWLDescription type : typeSet)
		{
			if (type.equals(simpleClassName) == true)
			{
				isType = true;
			}
			// end if
		}
		// end for
		
		return isType;
	}
	// end isType	
	
	// returns a set of all subclasses to a given class name (subsumption of all subclasses)
	public Set<OWLDescription> getAllSubclassNames(String simpleClassName)
	{
		Set<OWLDescription> subClassNameSet = this.getSubclassNames(simpleClassName);
		
		if (subClassNameSet.isEmpty() == false)
		{
			TreeSet<OWLDescription> tempSubclassNameSet = new TreeSet<OWLDescription>();
			
			for (OWLDescription classDescription : subClassNameSet)
			{
				tempSubclassNameSet.add(classDescription);
			}
			// end for
			
			for (OWLDescription classDescription : tempSubclassNameSet)
			{
				subClassNameSet.addAll(this.getAllSubclassNames(classDescription.toString()));
			}
			// end for
		}
		
		return subClassNameSet;
	}
	// end getAllSubclassNames

	// returns a set of the subclass names to a given class name
	// if there are no subclasses the returned set is going to be empty
	public Set<OWLDescription> getSubclassNames(String simpleClassName)
	{
		OWLClass owlClass = this.getClass(simpleClassName);
		Set<OWLDescription> subClassNameSet = owlClass.getSubClasses(this.ontology);
		return subClassNameSet;
	}
	// end getSubclassNames
	
	// returns a set of all superclasses to a given class name (subsumption of all superclasses)
	public Set<OWLDescription> getAllSuperclassNames(String simpleClassName)
	{
		Set<OWLDescription> superClassNameSet = this.getSuperclassNames(simpleClassName);
		
		if (superClassNameSet.isEmpty() == false)
		{
			TreeSet<OWLDescription> tempSuperclassNameSet = new TreeSet<OWLDescription>();
			
			for (OWLDescription classDescription : superClassNameSet)
			{
				tempSuperclassNameSet.add(classDescription);
			}
			// end for
			
			for (OWLDescription classDescription : tempSuperclassNameSet)
			{
				if (classDescription.getClass().getSimpleName().equals("OWLClassImpl") == true)
				{
					superClassNameSet.addAll(this.getAllSuperclassNames(classDescription.toString()));
				}
				// end if
			}
			// end for
		}
		// end if
		
		return superClassNameSet;
	}
	// end getSuperclassNames

	// returns a set of the superclass names to a given class name
	// if there are no superclasses the returned set is going to be empty
	public Set<OWLDescription> getSuperclassNames(String simpleClassName)
	{
		OWLClass owlClass = this.getClass(simpleClassName);
		Set<OWLDescription> superClassNameSet = owlClass.getSuperClasses(this.ontology);
		return superClassNameSet;
	}
	// end getSuperclassNames
	
	// returns a set of the some-restrictions to a given class name
	// if there are no some-restrictions the returned set is going to be empty
	public Set<OWLDescription> getSomeRestrictions(String simpleClassName)
	{
		Set<OWLDescription> someRestrictionSet = this.getAllSuperclassNames(simpleClassName);
		
		if (someRestrictionSet.isEmpty() == false)
		{
			TreeSet<OWLDescription> tempObjectSomeRestrictionSet = new TreeSet<OWLDescription>();
			
			for (OWLDescription classDescription : someRestrictionSet)
			{
				tempObjectSomeRestrictionSet.add(classDescription);
			}
			// end for
			
			for (OWLDescription classDescription : tempObjectSomeRestrictionSet)
			{
				if ((classDescription instanceof uk.ac.manchester.cs.owl.OWLObjectSomeRestrictionImpl) == false &&
					(classDescription instanceof uk.ac.manchester.cs.owl.OWLDataSomeRestrictionImpl) == false	)
				{
					someRestrictionSet.remove(classDescription);
				}
				// end if
			}
			// end if
		}
		// end if
		
		return someRestrictionSet;
	}
	// end getSomeRestrictions
	
	// returns a set of all some-restrictions to a given class name (subsumption of all some-restrictions of all leaf classes)
	public Set<OWLDescription> getAllSomeRestrictions(String simpleClassName)
	{
		Set<OWLDescription> subclassNameSet = this.getAllSubclassNames(simpleClassName);
		Set<OWLDescription> someRestrictionSet = new TreeSet<OWLDescription>();
		
		if (subclassNameSet.isEmpty() == true)
		{
			someRestrictionSet = this.getSomeRestrictions(simpleClassName);
		}
		else
		{
			for (OWLDescription classDescription : subclassNameSet)
			{
				someRestrictionSet.addAll(this.getSomeRestrictions(classDescription.toString()));
			}
			// end for
		}
		// end if else

		return someRestrictionSet;
	}
	// end getAllSomeRestrictions
	
	// returns the loaded ontology
	public OWLOntology getOntology()
	{
		return ontology;
	}
	// end getOntology
}
// end class OntologyAccessor
