package org.deft.guidance.tasks;

import java.net.URI;
import java.util.List;
import java.util.Set;

import org.semanticweb.owl.apibinding.OWLManager;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyManager;

import uk.ac.abdn.csd.REL.CQA.model.Solution;


/**
 * Example reasoner for documentation use case.
 *
 * Reasoning works as follows:
 *  - retrieve all possible tasks
 *  - delete from those all tasks that have already been performed
 *  
 * See the Model Repository and Deliverable D3.3 for details.
 * 
 * Please note: DO NOT USE underscores to name your individuals. 
 */
public class DocumentationProcessGuidanceExample {

	private GuidanceOntologyAccessor accessor;
	
	public static void main(String[] args) {
		DocumentationProcessGuidanceExample example = new DocumentationProcessGuidanceExample("file:d://deft.owl");
		example.run();
	}
	
	
	public DocumentationProcessGuidanceExample(String pathToOntology) {
		try {
			OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
			OWLOntology ontology = manager.loadOntologyFromPhysicalURI(URI.create(pathToOntology));
			this.accessor = new GuidanceOntologyAccessor(ontology);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	

	
	public void run() {
		DocumentationTaskFactory factory = new DocumentationTaskFactory(accessor);
		TaskQueryExecutor executor = new TaskQueryExecutor(accessor);
		TaskList taskList = new TaskList();
		
		addAllCompulsoryTasks(taskList, executor, factory);
		removeAllFulfilledCompulsoryTasks(taskList, executor, factory);
		addAllOptionalTasks(taskList, executor, factory);
		removeAllFulfilledOptionalTasks(taskList, executor, factory);
		
		System.out.println(taskList);
		
	}
	
	
	
	private void addAllCompulsoryTasks(TaskList taskList, TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor.queryForCompulsoryTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addCompulsoryTasks(tasks);
		}
	}
	
	private void addAllOptionalTasks(TaskList taskList, TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor.queryForOptionalTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addOptionalTasks(tasks);
		}
	}
	


	private void removeAllFulfilledCompulsoryTasks(TaskList taskList, TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor.queryForFulfilledCompulsoryTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addFulfilledCompulsoryTasks(tasks);
		}
	}
	
	private void removeAllFulfilledOptionalTasks(TaskList taskList, TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor.queryForFulfilledOptionalTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addFulfilledOptionalTasks(tasks);
		}
	}

    
	
}
