package org.deft.guidance.tasks;

import java.net.URI;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.semanticweb.owl.apibinding.OWLManager;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLDataFactory;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyChangeException;
import org.semanticweb.owl.model.OWLOntologyCreationException;
import org.semanticweb.owl.model.OWLOntologyManager;

import uk.ac.abdn.csd.REL.reasoner.RELReasoner;

/**
 * Helper class for easy access to the guidance ontology and the reasoner. 
 * Reasoning itself is not performed in this class, however. See TaskQueryExecutor
 * for task reasoning.  
 */
public class GuidanceOntologyAccessor {
	
	private OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
	private OWLDataFactory factory = manager.getOWLDataFactory();
	private RELReasoner reasoner = new RELReasoner(manager);
	private OWLOntology ontology;
	
	
	public GuidanceOntologyAccessor(OWLOntology ontology) {
		this.ontology = ontology;
		init();
	}
	
	private void init() {
		try {
			reasoner.loadOntology(ontology);
			reasoner.TBoxClassification();
			reasoner.generatePseudoModel(ontology);
		} catch (OWLOntologyChangeException oche) {
			oche.printStackTrace();
		} catch (OWLOntologyCreationException ocre) {
			ocre.printStackTrace();
		}

	}

	/**
	 * Returns the REL-Reasoner that is used for EL-Reasoning on the guidance ontology
	 */
	public RELReasoner getReasoner() {
		return reasoner;
	}

	/**
	 * Returns (creates) an OWLClass depending on its name. The name must be provided
	 * without the uri prefix. The returned OWLClass has the same prefix as the 
	 * guidance ontology.
	 */
	public OWLClass getOwlClass(String name) {
		URI uri = getUri(name);
        OWLClass owlClass = factory.getOWLClass(uri);
        return owlClass;
	}
	
	/**
	 * Returns an URI for a class or property, depending on its name.
	 * This methods concatenates the ontology URI, the '#' symbol and the name
	 * passed via the parameter. 
	 */
	public URI getUri(String classOrProperty) {
		String ontologyUriString = ontology.getURI().toString();
		URI classOrPropertyUri = URI.create(ontologyUriString + "#" + classOrProperty);
		return classOrPropertyUri;
	}
	
	/**
	 * Same as getUri(String), but the return type is a String
	 */
	public String getUriString(String classOrProperty) {
		URI uri = getUri(classOrProperty);
		return uri.toString();
	}
	
	/**
	 * Returns the name of an OWLClass, without the prefix 
	 */
	public String getSimpleName(OWLClass owlClass) {
		String classUriString = owlClass.getURI().toString();
		String name = getSimpleName(classUriString);
		return name;
	}

	/**
	 * Returns the name part of an URI string (everything after the '#' character) 
	 */
	public String getSimpleName(String uriString) {
		int posOfHashChar = uriString.indexOf('#');
		String name = uriString.substring(posOfHashChar + 1);
		return name;
	}

	
	/**
	 * Returns a list of OWLClasses which are directly (not transitively!) subsumed 
	 * by a specific OWLClass. The subsumption relation is computed by the REL reasoner.
	 */
	public List<OWLClass> getDirectlySubsumedClasses(OWLClass subsumingClass) {

	    List<OWLClass> subsumedClasses = new LinkedList<OWLClass>();
        // get all the concrete task types
        for (OWLClass concept : ontology.getReferencedClasses()) {
            if (!concept.isOWLNothing() && !concept.equals(subsumingClass)) { // omit "nothing" and the TaskType class
	            boolean toAdd = false;
	            
	            Set<OWLClass> subsumers = getSubsumers(reasoner, concept);
	            
	            if (subsumers.contains(subsumingClass)) { // if the concept is a subclass of "TaskType"
	                toAdd = true;
	                subsumers.remove(concept);
	                subsumers.remove(subsumingClass);
	                // and it does not have another proper subclass of "TaskType" as
	                // its proper superclass
	                for (OWLClass subsumer : subsumers) {
	                    if (reasoner.getsubsumer(subsumer).contains(subsumingClass)) {
	                        toAdd = false;
	                        break;
	                    }
	                }
	            }
	            // then this concept is a concrete task type
	            if (toAdd) {
	                subsumedClasses.add(concept);
	            }
            }
        }
        return subsumedClasses;

    }
	
	/**
	 * Returns a list of OWLClasses which are directly or transitively subsumed 
	 * by a specific OWLClass. The subsumption relation is computed by the REL reasoner.
	 */
	public List<OWLClass> getAllSubsumedClasses(OWLClass subsumingClass) {

	    List<OWLClass> subsumedClasses = new LinkedList<OWLClass>();
        // get all the concrete task types
        for (OWLClass concept : ontology.getReferencedClasses()) {
            if (!concept.isOWLNothing() && !concept.equals(subsumingClass)) { // omit "nothing" and the TaskType class
	            boolean toAdd = false;
	            
	            Set<OWLClass> subsumers = getSubsumers(reasoner, concept);
	            
	            if (subsumers.contains(subsumingClass)) { // if the concept is a subclass of "TaskType"
	                toAdd = true;
	                subsumers.remove(concept);
	                subsumers.remove(subsumingClass);


	            }
	            // then this concept is a concrete task type
	            if (toAdd) {
	                subsumedClasses.add(concept);
	            }
            }
        }
        return subsumedClasses;

    }
	
	private Set<OWLClass> getSubsumers(RELReasoner reasoner, OWLClass concept) {
		Set<OWLClass> subsumers = new HashSet<OWLClass>();
        try {
        	subsumers = reasoner.getsubsumer(concept);
        }
        catch (Exception e) {
        }
        return subsumers;
	}
    
    

	
	/**
	 * Returns a list of OWLClasses which are subtypes of TaskType.
	 */
	public List<OWLClass> getTaskTypes() {
		URI taskTypeUri = getUri("TaskType");
        OWLClass taskTypeClass = factory.getOWLClass(taskTypeUri);
	    
	    List<OWLClass> taskTypes = new LinkedList<OWLClass>();
        // get all the concrete task types
        for (OWLClass concept : ontology.getReferencedClasses()) {
            if (concept.isOWLNothing() || concept.equals(taskTypeClass)) { // omit "nothing" and the TaskType class
                continue;
            }
            boolean toAdd = false;
            
            Set<OWLClass> subsumers = getSubsumers(reasoner, concept);
            
            if (subsumers.contains(taskTypeClass)) { // if the concept is a subclass of "TaskType"
                toAdd = true;
                subsumers.remove(concept);
                subsumers.remove(taskTypeClass);
                // and it does not have another proper subclass of "TaskType" as
                // its proper superclass
                for (OWLClass subsumer : subsumers) {
                    if (reasoner.getsubsumer(subsumer).contains(taskTypeClass)) {
                        toAdd = false;
                        break;
                    }
                }
            }
            // then this concept is a concrete task type
            if (toAdd) {
                taskTypes.add(concept);
            }
        }
        return taskTypes;

    }

}
