package org.deft.guidance.views;

import java.util.Arrays;
import java.util.Comparator;

import org.deft.guidance.tasks.CheckUpdatedRepresentationTask;
import org.deft.guidance.tasks.ReferenceSpecificTask;
import org.deft.guidance.tasks.SetReferenceCheckedTask;
import org.deft.guidance.tasks.Task;
import org.deft.repository.datamodel.ArtifactReference;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;

/**
 * Comparator for sorting the tasks in the task list
 */
public class TaskListComparator extends ViewerComparator {

	
    public void sort(final Viewer viewer, Object[] elements) {
        Arrays.sort(elements, new InternalComparator());
    }
    
	
	/**
	 * The actual comparator
	 */
	private class InternalComparator implements Comparator<Object> {

		/**
		 * Compares 2 Tasks.
		 * 
		 * Currently only tasks which deal with references are considered.
		 */
		public int compare(Object o1, Object o2) {
			Task t1 = (Task)o1;
			Task t2 = (Task)o2;
			if (areReferenceSpecificTasks(t1, t2)) {
				int result = getReferenceSpecificTasksComparison((ReferenceSpecificTask)t1, (ReferenceSpecificTask)t2);
				return result;
			}
			return 0;
		}
		
		
		private boolean areReferenceSpecificTasks(Task t1, Task t2) {
			return t1 instanceof ReferenceSpecificTask && t2 instanceof ReferenceSpecificTask;
		}
		
		
		private int getReferenceSpecificTasksComparison(ReferenceSpecificTask t1, ReferenceSpecificTask t2) {
			if (areCheckAndAcceptTasks(t1, t2)) {
				if (arePairwiseCheckAndAcceptTasks(t1, t2)) {
					int result = getPairwiseCheckAndAcceptComparison(t1, t2);
					return result;
				}
			}
			return getSortByChapterAndArtifactComparison(t1, t2);
		}
		

	
		private boolean areCheckAndAcceptTasks(ReferenceSpecificTask t1, ReferenceSpecificTask t2) {
			if (t1 instanceof CheckUpdatedRepresentationTask && t2 instanceof SetReferenceCheckedTask
					|| t2 instanceof CheckUpdatedRepresentationTask && t1 instanceof SetReferenceCheckedTask) {
				return true;
			}
			return false;
		}
		
		private boolean arePairwiseCheckAndAcceptTasks(ReferenceSpecificTask t1, ReferenceSpecificTask t2) {
			return t1.getReferenceId().equals(t2.getReferenceId());
		}
		

		
		private int getPairwiseCheckAndAcceptComparison(ReferenceSpecificTask t1, ReferenceSpecificTask t2) {
			if (t1 instanceof CheckUpdatedRepresentationTask && t2 instanceof SetReferenceCheckedTask) {
				return -1;
			} else {
				return 1;
			}
		}
		
		
		private int getSortByChapterAndArtifactComparison(ReferenceSpecificTask t1, ReferenceSpecificTask t2) {
			ArtifactReference ref1 = t1.getDeftArtifactReference();
			ArtifactReference ref2 = t2.getDeftArtifactReference();
			
			String chapName1 = ref1.getChapter().getName();
			String chapName2 = ref2.getChapter().getName();
			if (chapName1.equals(chapName2)) {//same chaptername? compare by artifact
				String artName1 = ref1.getArtifact().getName();
				String artName2 = ref2.getArtifact().getName();
				if (artName1.equals(artName2)) {//same artifact name? compare by reference id
					String refId1 = ref1.getId();
					String refId2 = ref2.getId();
					return refId1.compareTo(refId2);
				} else {
					return artName1.compareTo(artName2);
				}
			} else {
				return chapName1.compareTo(chapName2);
			}

		}
	}

}
