package org.deft.integrator;

import java.util.List;

import org.deft.RepositoryFacade;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.integrator.ArtifactReferencePosition;

public abstract class AbstractEditorChapterDocument implements EditorChapterDocument {
	
	protected RepositoryFacade repository = RepositoryFacade.instance();
	
	private Chapter chapter;
	
	public AbstractEditorChapterDocument(Chapter chapter) {
		this.chapter = chapter;
	}

	@Override
	public void removeNonexistingReferencesFromRepository() {
		List<ArtifactReference> references = repository.getArtifactReferences(chapter);
		for (ArtifactReference reference : references) {
			EditorIntegrator integrator = getIntegrator(reference);
			String referenceId = reference.getId();
			if (integrator != null) {
				ArtifactReferencePosition position = integrator.getPositionForRepresentation(this, referenceId);
				if (position == null) {
					integrator.cleanup(referenceId);
					repository.remove(reference);
				}
			}
		}
	}
	
	protected EditorIntegrator getIntegrator(ArtifactReference reference) {
		if (isErrorReference(reference)) {
			EditorIntegrator integrator = getErrorMessageIntegrator();
			ArtifactReferencePosition pos = integrator.getPositionForRepresentation(this, reference.getId());
			if (pos != null) {
				return integrator;
			}
		} 
		EditorIntegrator integrator = repository.getEditorIntegrator(reference);
		return integrator;
	}
	
	
	private boolean isErrorReference(ArtifactReference reference) {
		ArtifactReferenceUpdateType type = reference.getUpdateType();
		return type == ArtifactReferenceUpdateType.MISSINGARTIFACT 
				|| type == ArtifactReferenceUpdateType.UNCOMPUTABLEUPDATE;
	}

	
	
	@Override
	public void insertRepresentation(ArtifactReference reference, ArtifactReferencePosition pos) {
		if (hasMissingArtifact(reference)) {
			integrateErrorMessage(reference.getId(), "Artifact is missing", pos);
			return;
		}
		String referenceId = reference.getId();
		Artifact artifact = reference.getArtifact();
		OperationChain chain = repository.getOperationChain(reference);
		if (chain == null) {
			integrateErrorMessage(reference.getId(), "Operation chain \"" + reference.getOperationChainId() + "\" not found.", pos);
			return;
		}
		EditorIntegrator integrator = (EditorIntegrator)repository.getEditorIntegrator(reference);
		if (integrator == null) {
			integrateErrorMessage(reference.getId(), "Integrator \"" + reference.getIntegratorId() + "\" not found.", pos);
			return;
		}
		ArtifactRepresentation rep = chain.executeOperations(artifact);
		if (rep == null) {
			integrateErrorMessage(referenceId, "Could not compute representation", pos);
			return;
		}
		integrator.integrate(referenceId, rep, artifact.getArtifactType(), pos);
	}
	
	private boolean hasMissingArtifact(ArtifactReference reference) {
		return reference.getArtifact() == null;
	}

	private void integrateErrorMessage(String referenceId, String message, ArtifactReferencePosition pos) {
		ErrorMessageIntegrator integrator = getErrorMessageIntegrator();
		integrator.setErrorMessage(message);
		integrator.integrate(referenceId, null, null, pos);
	}
	
	@Override
	public ArtifactReferencePosition removeRepresentation(ArtifactReference reference) {
		if (reference.getUpdateType() == ArtifactReferenceUpdateType.UNCOMPUTABLEUPDATE) {
			ErrorMessageIntegrator integrator = getErrorMessageIntegrator();
			ArtifactReferencePosition pos = integrator.undoIntegration(this, reference.getId());
			boolean successfullyUndone = pos != null;
			if (successfullyUndone) {
				return pos;
			}
			//if not successfully undone, then there is probably an old representation in
			//the document. Try the original integrator to remove it.
		}
		EditorIntegrator integrator = (EditorIntegrator)repository.getEditorIntegrator(reference);
		ArtifactReferencePosition pos = integrator.undoIntegration(this, reference.getId());
		return pos;
	}


	@Override
	public Chapter getChapter() {
		return chapter;
	}

	
	
	public boolean isValidReference(ArtifactReference reference) {
		return reference != null;
	}


}
