package org.deft.openoffice;

import org.deft.dependencyactivator.activation.IActivationStartup;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import ag.ion.bion.officelayer.application.IApplicationAssistant;
import ag.ion.bion.officelayer.application.ILazyApplicationInfo;
import ag.ion.bion.officelayer.application.IOfficeApplication;
import ag.ion.bion.officelayer.application.OfficeApplicationException;
import ag.ion.bion.officelayer.application.OfficeApplicationRuntime;
import ag.ion.bion.workbench.office.editor.core.EditorCorePlugin;
import ag.ion.noa.NOAException;
import ag.ion.noa4e.ui.NOAUIPlugin;

/**
 * Accessor class to get and stop the OpenOffice instance. 
 */
public class OpenOfficeManager implements IActivationStartup {

	private static IOfficeApplication officeApplication;

	/**
	 * Returns a local instance of OpenOffice.
	 * 
	 * The returned instance is the same that the NOA framework uses.
	 * It is a singleton. By using this instance we can later terminate
	 * all running OpenOffice processes (soffice.bin in Windows taskmanager).
	 * 
	 * Had we created another instance here, the one created and used by
	 * the NOA framework would still be active after DEFT has been closed.
	 */
	public IOfficeApplication getOfficeApplication() {
		officeApplication = EditorCorePlugin.getDefault().getManagedLocalOfficeApplication();
		return officeApplication;
	}
	
	
	public void activate() {
		//We need the active workbench window for activation (see #doActivate()), but this can only
		//be accessed from the UI thread, so start activation in UI thread
		//
		//The code will also work if the current thread is already the UI thread
		Display.getDefault().syncExec(new Runnable() {

			@Override
			public void run() {
				doActivate();
			}
			
		});
	}
	
	private void doActivate() {
		IOfficeApplication app = getOfficeApplication();
	    if(!app.isActive()) {
			IWorkbench wb = PlatformUI.getWorkbench();
			IWorkbenchWindow window = wb.getActiveWorkbenchWindow(); //will be null if called from non-UI thread
			Shell shell = window.getShell();
			NOAUIPlugin.startLocalOfficeApplication(shell, app);
	    }
	}

	
	
	public void deactivateOpenOfficeIfPossible() {
		if (officeApplication == null) {
			getOfficeApplication();
		}
		if (officeApplication != null) {
			deactivateOpenOffice(officeApplication);
		}
	}
	
	private void deactivateOpenOffice(IOfficeApplication office) {
		try {
			System.out.println("deactivating OpenOffice");
			if (!office.isActive()) {
				//we can't terminate if office is not active
				office.activate();
			}
			office.getDesktopService().terminate();
			System.out.println("deactivated OpenOffice");
		} catch (NOAException e) {
			e.printStackTrace();
		} catch (OfficeApplicationException e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Returns a path to an installed OpenOffice. If multiple versions are installed,
	 * one of them is returned.
	 */
	public String getPathToInstalledOpenOffice() {
		try {
			String noaLibrariesLocation = EditorCorePlugin.getDefault().getLibrariesLocation();
			IApplicationAssistant applicationAssistant 
					= OfficeApplicationRuntime.getApplicationAssistant(noaLibrariesLocation);
			ILazyApplicationInfo[] appInfos = applicationAssistant.getLocalApplications();
			if(appInfos.length > 0) {
				String firstAppInfoPath = appInfos[0].getHome();
				return firstAppInfoPath;
			}
		} catch (OfficeApplicationException e) {
			e.printStackTrace();
		}
		return null;
	}


	/**
	 * Method for the dependency activator, to activate this plugin 
	 * (org.deft.openoffice) when the NOA framework has been activated. 
	 * It is necessary that this plugin is activated, otherwise the 
	 * deactivateOpenOffice() method would not be executed when DEFT 
	 * closes (it is called by {@link Activator#stop(org.osgi.framework.BundleContext)}).
	 */
	@Override
	public void startup() {
		//nothing to be done, we just need to run this empty method to cause the plugin
		//to be activated
	}


}
