package org.deft.operation;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.repository.artifacttype.ArtifactType;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseOperationChainRegistry implements OperationChainRegistry {
	
	private static final String OPERATIONCHAIN_EXTENSION_POINT_NAME = "org.deft.operation.operationchain";
		
	private Map<String, OperationChainConfig> registeredOperationChains = new HashMap<String, OperationChainConfig>(); 

	public EclipseOperationChainRegistry() {
		getAndCacheRegisteredSources();
	}
	
	private void getAndCacheRegisteredSources() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(OPERATIONCHAIN_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			OperationChainConfig config = new OperationChainConfig(ce);
			registeredOperationChains.put(config.getId(), config);
		}
	}


	@Override
	public OperationChain getOperationChain(String operationChainId) {
		OperationChainConfig config = registeredOperationChains.get(operationChainId);
		if (config == null) {
			return null;
		}
		OperationChain chain = config.getOperationChain();
		chain.resetChainConfigurations();
		return chain;
	}

	
	
	@Override
	public List<String> getOperationChainIds() {
		List<String> chainIds = new LinkedList<String>(registeredOperationChains.keySet());
		return chainIds;
	}
	
	
	@Override
	public List<OperationChain> getOperationChains() {
		List<OperationChain> chains = new LinkedList<OperationChain>();
		for (String id : getOperationChainIds()) {
			OperationChain chain = getOperationChain(id);
			chains.add(chain);
		}
		return chains;
	}

	
	@Override
	public List<OperationChain> getOperationChainsWithFinalRepresentationType(
			String representationType) {
		List<OperationChain> chains = new LinkedList<OperationChain>();
		for (OperationChain chain : getOperationChains()) {
			String finalType = chain.getFinalRepresentationType();
			if (finalType.equals(representationType)) {
				chains.add(chain);
			}
		}
		return chains;
	}
	
	
	@Override
	public List<OperationChain> getOperationChainsForArtifactType(ArtifactType artifactType) {
		List<OperationChain> chains = new LinkedList<OperationChain>();
		for (OperationChain chain : getOperationChains()) {
			if (chain.isValidInputArtifactType(artifactType)) {
				chains.add(chain);
			}
		}
		return chains;

	}

	@Override
	public List<OperationChain> getOperationChains(ArtifactType artifactType, String finalRepresentationType) {
		List<OperationChain> chains = new LinkedList<OperationChain>();
		for (OperationChain chain : getOperationChains()) {
			String finalType = chain.getFinalRepresentationType();
			if (chain.isValidInputArtifactType(artifactType)
					&& finalType.equals(finalRepresentationType)) {
				chains.add(chain);
			}
		}
		return chains;
	}


	
	

	
	
	
	private class OperationChainConfig {
	
		private IConfigurationElement ce;
		private OperationChain operationChain;
		
		public OperationChainConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public OperationChain getOperationChain() {
			if (operationChain == null) {
				try {
					String operationChainPropertyInExtensionPoint = "class";
					operationChain = (OperationChain)ce.createExecutableExtension(
							operationChainPropertyInExtensionPoint);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return operationChain;
		}	
	}








	
}
