package org.deft.operation;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.semanticweb.owl.model.OWLOntology;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class Ontology2MatrixConfiguration extends AbstractOperationConfiguration {

	public static String ID = "org.deft.operation.ontology2matrix";
	
	private boolean isUnmodified;
	private List<String> sourceIndividuals = new LinkedList<String>();
	private List<String> targetIndividuals = new LinkedList<String>();
	
	
	public Ontology2MatrixConfiguration() {
		super("org.deft.requirementsontology"); //pass bundle id in constructor!
		this.isUnmodified = true; //no individuals have been set or unset
	}
	
	@Override
	public String getId() {
		return ID;
	}
	
	
	/**
	 * By default the config is empty, so an empty table will be generated. By filling
	 * the config (adding all individuals from the ontology as source and target) a complete
	 * table will be generated instead.
	 */
	public void initIfIsUnmodified(OWLOntology ontology) {
		if (isUnmodified) {
			OntologyAccessor oa = new OntologyAccessor(ontology);
			ROModel roModel = new ROModel(oa);
			List<String> sourceIndividuals = new LinkedList<String>(roModel.getOntologyIndividualNames());
			Collections.sort(sourceIndividuals);
			setSourceIndividuals(sourceIndividuals);
			List<String> targetIndividuals = new LinkedList<String>(roModel.getOntologyIndividualNames());
			Collections.sort(targetIndividuals);
			setTargetIndividuals(targetIndividuals);
		}
		isUnmodified = false;
	}
	
	
	
	public void setSourceIndividuals(List<String> sourceIndividuals) {
		this.sourceIndividuals.clear();
		this.sourceIndividuals.addAll(sourceIndividuals);
		isUnmodified = false;
	}
	
	
	public void setTargetIndividuals(List<String> targetIndividuals) {
		this.targetIndividuals.clear();
		this.targetIndividuals.addAll(targetIndividuals);
		isUnmodified = false;
	}
	
	public boolean isEmpty() {
		return sourceIndividuals.isEmpty() || targetIndividuals.isEmpty();
	}
	
	public List<String> getSourceIndividuals() {
		return new LinkedList<String>(sourceIndividuals);
	}
	
	
	public List<String> getTargetIndividuals() {
		return new LinkedList<String>(targetIndividuals);
	}
	

	@Override
	public void loadFromXml(Element element) {
		if (isValid(element)) {
			loadIndividuals(element, "source", sourceIndividuals);
			loadIndividuals(element, "target", targetIndividuals);
		}
	}
	
	private void loadIndividuals(Element element, String groupingNodeName, List<String> individuals) {
		individuals.clear();
		Element eGroupingNode = (Element)element.getElementsByTagName(groupingNodeName).item(0);
		NodeList nl = eGroupingNode.getElementsByTagName("individual");
		for (int i = 0; i < nl.getLength(); i++) {
			Element e = (Element)nl.item(i);
			String individualName = e.getAttribute("name");
			individuals.add(individualName);
		}
		isUnmodified = false;
	}


	@Override
	public void addToXml(Element root) {
		addIndividualsToXml(root, "source", getSourceIndividuals());
		addIndividualsToXml(root, "target", getTargetIndividuals());
	}
	
	private void addIndividualsToXml(Element root, String groupingNodeName, List<String> individuals) {
		Document doc = root.getOwnerDocument();
		Element eGroupingNode = doc.createElement(groupingNodeName);
		for (String individualName : individuals) {
			Element eIndividual = doc.createElement("individual");
			eIndividual.setAttribute("name", individualName);
			eGroupingNode.appendChild(eIndividual);
		}
		root.appendChild(eGroupingNode);
	}
	
	

	@Override
	protected String getLocalSchemaFileLocation() {
		return "resources/o2mschema.xsd";
	}
	


}
