package org.deft.operation;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.deft.requirementsontology.IndividualVertex;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.semanticweb.owl.model.OWLOntology;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class Ontology2RequirementsGraphConfiguration extends AbstractOperationConfiguration {

	public static String ID = "org.deft.operation.ontology2requirementsgraph";
	
	private List<String> individuals = new LinkedList<String>();
	private boolean isUnmodified;
	
	public Ontology2RequirementsGraphConfiguration() {
		super("org.deft.requirementsontology"); //pass bundle id in constructor!
		this.isUnmodified = true; //no individuals have been set or unset
	}
	
	
	@Override
	public String getId() {
		return ID;
	}
	
	public void setIndividuals(List<String> individuals) {
		this.individuals.clear();
		this.individuals.addAll(individuals);
		isUnmodified = false;
	}
	
	
	/**
	 * By default the config is empty, so an empty image will be generated. By filling
	 * the config (adding RequirementArtifact individuals from the ontology) a requirement artifact
	 * graph will be generated instead
	 */
	public void initIfIsUnmodified(OWLOntology ontology) {
		if (isUnmodified) {
			OntologyAccessor oa = new OntologyAccessor(ontology);
			//have only RequirementArtifacts with at least one link (property) in the roModel graph
			ROModel roModel = new ROModel(oa);
			removeNonRequirementsArtifacts(roModel);
			roModel.removeUnconnectedIndividuals();

			//make a list of names from all individuals which are now in the roModel graph
			Collection<IndividualVertex> individuals = roModel.getIndividuals();
			List<String> individualNames = new LinkedList<String>();
			for (IndividualVertex iv : individuals) {
				individualNames.add(iv.getSimpleName());
			}
			Collections.sort(individualNames);
			setIndividuals(individualNames);
		}
		isUnmodified = false;
	}
	
	private void removeNonRequirementsArtifacts(ROModel roModel) {
		List<IndividualVertex> individuals = new LinkedList<IndividualVertex>(roModel.getIndividuals());
		Collection<IndividualVertex> reqArts = roModel.getIndividuals("RequirementArtifact");
		for (IndividualVertex iv : individuals) {
			if (!reqArts.contains(iv)) {
				roModel.removeIndividual(iv.getSimpleName());
			}
		}
	}
	
	public boolean isEmpty() {
		return individuals.isEmpty();
	}
	
	public List<String> getIndividuals() {
		return new LinkedList<String>(individuals);
	}
	
	

	@Override
	public void loadFromXml(Element element) {
		if (isValid(element)) {
			individuals.clear();
			Element eIndividuals = (Element)element.getElementsByTagName("individuals").item(0);
			NodeList nl = eIndividuals.getElementsByTagName("individual");
			for (int i = 0; i < nl.getLength(); i++) {
				Element e = (Element)nl.item(i);
				String individualName = e.getAttribute("name");
				individuals.add(individualName);
			}
			isUnmodified = false;
		}
	}

	@Override
	public void addToXml(Element root) {
		Document doc = root.getOwnerDocument();
		Element eIndividuals = doc.createElement("individuals");
		for (String individualName : individuals) {
			Element eIndividual = doc.createElement("individual");
			eIndividual.setAttribute("name", individualName);
			eIndividuals.appendChild(eIndividual);
		}
		root.appendChild(eIndividuals);
	}
	
	

	@Override
	protected String getLocalSchemaFileLocation() {
		return "resources/o2graphschema.xsd";
	}

}
