package org.deft.operation;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Chapter;
import org.deft.util.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Contains the concrete OperationConfigurations for all operations of the
 * corresponding OperationChain and the Integrator
 */
public class OperationChainConfiguration {
	
	/**
	 * Stores mapping from operations to their configurations and vice versa.
	 */
	private Map<Operation, OperationConfiguration> op2configMap = new HashMap<Operation, OperationConfiguration>();
	private Map<OperationConfiguration, Operation> config2opMap = new HashMap<OperationConfiguration, Operation>();
	private List<OperationConfiguration> configList = new LinkedList<OperationConfiguration>();
	
	public void addInitialConfiguration(Operation operation) {
		OperationConfiguration config = operation.createConfiguration();
		op2configMap.put(operation, config);
		config2opMap.put(config, operation);
		configList.add(config);
	}
	
	public OperationConfiguration getConfiguration(Operation operation) {
		OperationConfiguration config = op2configMap.get(operation);
		return config;
	}
	
	public Operation getOperation(OperationConfiguration config) {
		Operation operation = config2opMap.get(config);
		return operation;
	}
	

	public OperationConfiguration getConfiguration(int index) {
		return configList.get(index);
	}

	public int getConfigCount() {
		return configList.size();
	}
	
	
	public void reset() {
		List<Operation> operations = getOperations();
		op2configMap.clear();
		config2opMap.clear();
		configList.clear();
		for (Operation operation : operations) {
			addInitialConfiguration(operation);
		}
	}
	
	private List<Operation> getOperations() {
		List<Operation> operations = new LinkedList<Operation>(); 
		for (OperationConfiguration config : configList) {
			Operation operation = config2opMap.get(config);
			operations.add(operation);
		}
		return operations;
	}
	
	
	
	
	
	public Document getConfigurationAsXml(String artifactRefId, Artifact artifact, Chapter chapter) {
		Document document = XmlUtil.makeDocument();
		Element root = document.createElement("reference");
		document.appendChild(root);
		root.setAttribute("artifact", artifact.getId());
		root.setAttribute("chapter", chapter.getId());
		root.setAttribute("id", artifactRefId);
		Element eOperationsRoot = createAndAddOperationsRoot(root); 
		for (OperationConfiguration config : configList) {
			Operation operation = getOperation(config);
			addOperationConfigXml(eOperationsRoot, operation, config);
		}		
		return document;
	}
	
	private Element createAndAddOperationsRoot(Element element) {
		Document doc = element.getOwnerDocument();
		Element configRoot = doc.createElement("operations");
		element.appendChild(configRoot);
		return configRoot;
	}
	
	private void addOperationConfigXml(Element element, Operation operation, OperationConfiguration config) {
		Document doc = element.getOwnerDocument();
		Element eOperation = doc.createElement("operation");
		eOperation.setAttribute("type", operation.getClass().getCanonicalName());
		
		element.appendChild(eOperation);
		config.addToXml(eOperation);
	}


	
	public void loadConfigurationForOperationChain(Document doc) {
		List<Element> elements = getOperationConfigurationXmlElements(doc);
		for (int i = 0; i < elements.size(); i++) {
			OperationConfiguration config = getConfiguration(i);
			Element xmlConfigContent = elements.get(i);
			loadConfigurationForOperation(config, xmlConfigContent);
		}
	}

	private void loadConfigurationForOperation(OperationConfiguration config, Element configContent) {
		try {
			config.loadFromXml(configContent);
		}
		catch (Exception ex) {
			System.err.println("Exception loading configuration " + config.getId());
			ex.printStackTrace();
			System.err.println("Using default configuration instead");
			exchangeFaultyConfiguration(config);
		}
	}
	
	private void exchangeFaultyConfiguration(OperationConfiguration oldConfig) {
		Operation op = getOperation(oldConfig);
		OperationConfiguration newConfig = op.createConfiguration();
		replaceConfiguraton(oldConfig, newConfig);

	}

	
	private void replaceConfiguraton(OperationConfiguration oldConfig, OperationConfiguration newConfig) {
		Operation operation = getOperation(oldConfig);
		op2configMap.put(operation, newConfig);
		config2opMap.put(newConfig, operation);
		config2opMap.remove(oldConfig);
		int index = configList.indexOf(oldConfig);
		configList.remove(oldConfig);
		configList.add(index, newConfig);
	}
	
	
	private List<Element> getOperationConfigurationXmlElements(Document doc) {
		Element root = doc.getDocumentElement();
		Element eOperations = (Element)root.getElementsByTagName("operations").item(0);
		List<Element> elements = new LinkedList<Element>();
		NodeList nl = eOperations.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			if (n instanceof Element) {
				Element e = (Element)nl.item(i);
				elements.add(e);
			}
		}
		return elements;
	}
	

	
}
