package org.deft.operation.ast2styledtext;

import java.awt.Color;
import java.util.List;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.decoration.Templates;
import org.deft.repository.ast.decoration.tokentype.TokenTypeInformation;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.styleddocument.StyledCodeRepresentation;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.representation.treenode.TreeNodeRepresentation;

public class TreeNodeToStyledCodeOperation implements Operation {


	
	@Override
	public StyledCodeRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		TreeNodeRoot root = checkAndUnwrap(representation);
		TokenizedStyledDocument doc = createStyledDocument(root);
		StyledCodeRepresentation sdr = new StyledCodeRepresentation(doc);
		return sdr;
	}
	
	
	private TreeNodeRoot checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(TreeNodeRepresentation.TYPE)) {
			TreeNodeRepresentation tnr = (TreeNodeRepresentation)rep;
			return tnr.getRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	

	
	private TokenizedStyledDocument createStyledDocument(TreeNodeRoot root) {
		StyleContext context = setupStyleContext();
		TokenizedStyledDocument document = new TokenizedStyledDocument(context);

		List<TokenNode> nodeList = root.serialize();
        TokenNode tn = nodeList.get(0);
        int startCol = tn.getStartCol();
        addStyledToken(tn, document, context);

        int i = 1;
        while (i < nodeList.size()) {
        	TokenNode prevNode = tn;
            tn = nodeList.get(i);
            String spaces = prevNode.computeSpacesBetweenNodes(tn);
            addStyledSpaces(spaces, document, context);
            addStyledToken(tn, document, context);
            i++;
        }

        TokenizedStyledDocument trimmedDocument = document.multilineTrim(startCol - 1);
		return trimmedDocument;
	}
	
	
	
	private StyleContext setupStyleContext() {
		//TODO load it from external data source according to artifact type
		StyleContext context = new StyleContext();
		Style defaultStyle = context.getStyle(StyleContext.DEFAULT_STYLE);

		Style normal = context.addStyle("normal", defaultStyle);
		StyleConstants.setFontFamily(normal, "Courier New");
		
		Style builtintype = context.addStyle("builtintype", normal);
		StyleConstants.setForeground(builtintype, new Color(127,0,85));
		StyleConstants.setBold(builtintype, true);
		
		Style keyword = context.addStyle("keyword", normal);
		StyleConstants.setForeground(keyword, new Color(127,0,85));
		StyleConstants.setBold(keyword, true);
		
		Style number = context.addStyle("number", normal);
		StyleConstants.setForeground(number, Color.BLUE);

		Style string = context.addStyle("string", normal);
		StyleConstants.setForeground(string, Color.BLUE);

		Style character = context.addStyle("char", normal);
		StyleConstants.setForeground(character, Color.BLUE);
		
		Style bool = context.addStyle("boolean", normal);
		StyleConstants.setForeground(bool, new Color(127,0,85));
		StyleConstants.setBold(bool, true);
		
		Style nulltype = context.addStyle("null", normal);
		StyleConstants.setForeground(nulltype, new Color(127,0,85));
		StyleConstants.setBold(nulltype, true);
		
		Style comment = context.addStyle("comment", normal);
		StyleConstants.setForeground(comment, new Color(0, 127, 0));
		return context;
	}
	
	
	private void addStyledToken(TokenNode tn, StyledDocument document, StyleContext context) {
		try {
			if (tn.hasInformation(Templates.TOKENTYPE)) {
				TokenTypeInformation ni = (TokenTypeInformation)tn.getInformation(Templates.TOKENTYPE);
				Style style = context.getStyle(ni.getType());
				document.insertString(document.getLength(), tn.getTextContent(), style);
			} else {
				Style normal = context.getStyle("normal");
				document.insertString(document.getLength(), tn.getTextContent(), normal);
			}
		}
		catch (BadLocationException ble) {
			ble.printStackTrace();
		}
	}
	
	private void addStyledSpaces(String spaces, StyledDocument document, StyleContext context) {
		try {
			Style normal = context.getStyle("normal");
			document.insertString(document.getLength(), spaces, normal);
		}
		catch (BadLocationException ble) {
			ble.printStackTrace();
		}
	}
	

	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return StyledCodeRepresentation.TYPE;
	}
	
	
}
