package org.deft.operation.astfilter;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeNodeComparator;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.TreeWalker;
import org.deft.repository.ast.decoration.Templates;
import org.deft.repository.ast.decoration.selected.SelectedInformation;
import org.deft.repository.ast.jxpath.XPath;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.treenode.TreeNodeRepresentation;

/**
 * An operation that takes an AST in form of a TreeNodeRoot, applies a number of XPaths on it
 * and returns a TreeNodeRoot which only contains nodes (and their descendants) which were the
 * target of one or more XPaths. The selected nodes are decorated with a SelectedInformation.
 */
public class AstFilterOperation implements Operation {


	
	@Override
	public TreeNodeRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		TreeNodeRoot root = checkAndUnwrap(representation);
		AstFilterConfiguration afConfig = checkAndUnwrap(config);
		List<XPath> xpaths = afConfig.getXPaths();
		String referenceId = afConfig.getRefId();
		List<TreeNode> queriedNodes = evaluateXPathsAndGetDecoratedNodes(root, xpaths, referenceId);
		Collections.sort(queriedNodes, new TreeNodeComparator());
		removeTreeNodesWithoutSelectedDescendants(root);
		TreeNodeRoot newRoot = new TreeNodeRoot(queriedNodes);
		TreeNodeRepresentation tnRep = new TreeNodeRepresentation(newRoot);
//		TreeNodeRepresentation tnRep = new TreeNodeRepresentation(root);
		return tnRep;
	}
	
	
	private TreeNodeRoot checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(TreeNodeRepresentation.TYPE)) {
			TreeNodeRepresentation tnr = (TreeNodeRepresentation)rep;
			return tnr.getRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	private AstFilterConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof AstFilterConfiguration) {
			AstFilterConfiguration afConfig = (AstFilterConfiguration)config;
			return afConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	private List<TreeNode> evaluateXPathsAndGetDecoratedNodes(
			TreeNodeRoot root, Collection<XPath> xpaths, String referenceId) {
		List<TreeNode> queriedNodes = new LinkedList<TreeNode>();
		for (XPath xpath : xpaths) {
			List<TreeNode> nodes = evaluateSingleXPathAndGetDecoratedNodes(root, xpath, referenceId);
			queriedNodes.addAll(nodes);
		}
		return queriedNodes;
	}
	
	
	private List<TreeNode> evaluateSingleXPathAndGetDecoratedNodes(
			TreeNodeRoot root, XPath xpath, String referenceId) {
		List<TreeNode> queriedNodes = new LinkedList<TreeNode>();
       	List<TreeNode> nodes = root.executeXPathQuery(xpath);
    	for (TreeNode node : nodes) {
    		SelectedInformation info = new SelectedInformation();
    		info.addSelectedData(xpath, referenceId);
    		node.addInformation(info);
    		queriedNodes.add(node);
		}
		return queriedNodes;
	}
	
	
	
	
	/**
	 * Removes all TreeNodes that do neither have a selected node as an ancestor or
	 * a descendant
	 */
    private TreeNode removeTreeNodesWithoutSelectedDescendants(TreeNodeRoot root) {
    	Set<TreeNode> sToRemove = new LinkedHashSet<TreeNode>();
    	Set<TreeNode> ancestorsAndDescendantsOfSelectedNodes = new HashSet<TreeNode>();
    	List<TreeNode> selectedNodes = root.getDescendants(Templates.SELECTED);
    	for (TreeNode node : selectedNodes) {
    		ancestorsAndDescendantsOfSelectedNodes.addAll(node.getAncestors());
    		ancestorsAndDescendantsOfSelectedNodes.addAll(node.getDescendants());
    	}
    	
    	TreeWalker tw = new TreeWalker(root);
    	while (tw.hasNext()) {
    		TreeNode tn = tw.next();
    		if (!ancestorsAndDescendantsOfSelectedNodes.contains(tn) && !selectedNodes.contains(tn)) {
    			sToRemove.add(tn);
    		}
    	}
    	for (TreeNode tn : sToRemove) {
    		if (tn.getParent() != null)
    		tn.getParent().removeChild(tn);
    	}
    	return root;
    }


	@Override
	public AstFilterConfiguration createConfiguration() {
		return new AstFilterConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}

}



