package org.deft.operation.astmodifier;

import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.ast.Token;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeNodeComparator;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.decoration.ReplaceGroup;
import org.deft.repository.ast.decoration.Templates;
import org.deft.repository.ast.decoration.replace.ReplaceInformation;
import org.deft.repository.ast.jxpath.XPath;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.treenode.TreeNodeRepresentation;

public class AstModifierOperation implements Operation {


	
	@Override
	public TreeNodeRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		TreeNodeRoot root = checkAndUnwrap(representation);
		AstModifierConfiguration amConfig = checkAndUnwrap(config);
		List<ReplaceGroup> replaceGroups = amConfig.getReplaceGroups();
		attachReplaceInformationToNodes(root, replaceGroups);
		replaceHiddenNodes(root);
		TreeNodeRepresentation tnRep = new TreeNodeRepresentation(root);
        return tnRep;
	}
	
	
	private void attachReplaceInformationToNodes(TreeNodeRoot root, List<ReplaceGroup> replaceGroups) {
		for (ReplaceGroup rg : replaceGroups) {
			//mark nodes to be hidden:
			//first mark the nodes identified with the ReplaceGroup's "hide"  
			for (XPath xpathForHide : rg.getXPathsHide()) {
				List<TreeNode> nodes = root.executeXPathQuery(xpathForHide);
				//there should only be one result node
				if (!nodes.isEmpty()) {
					TreeNode node = nodes.get(0);
					ReplaceInformation info = new ReplaceInformation(rg);
					node.addInformation(info);
					List<TreeNode> descendants = node.getDescendants();
					for (TreeNode descendant : descendants) {
						descendant.addInformation(info);
					}
				}
			}
		}
	}
	
	
	private List<TreeNode> replaceHiddenNodes(TreeNode root) {
		List<TreeNode> lReplace = new LinkedList<TreeNode>();
		List<TreeNode> hiddenNodes = root.getDescendants(Templates.REPLACE);
		Set<ReplaceGroup> lAppliedGroups = new HashSet<ReplaceGroup>();
		//for all nodes that should be hidden
		for (TreeNode tn : hiddenNodes) { 
			ReplaceInformation ri =	(ReplaceInformation)tn.getInformation(Templates.REPLACE);
			ReplaceGroup replaceGroup = ri.getReplaceGroup();

			//if no node from this group had the replacement applied
			if (!lAppliedGroups.contains(replaceGroup)) {
				//compute the new token node
				TokenNode tnRep = getTokenNode(tn, replaceGroup.getReplaceString());	
				if (tnRep != null) {
					//if a replace was defined, replace old token node with new token node
					tn.getParent().replaceChild(tn, tnRep);	
				} else {
					//if no replace was defined, just remove old token node
					tn.getParent().removeChild(tn);
				}
				//save that we used the replaceGroup
				lAppliedGroups.add(replaceGroup);
				//save replaced node in return list
				lReplace.add(tn);
			} else {
				//if replacement has already been applied: just remove old node
				tn.getParent().removeChild(tn);
				//save removed node in return list
				lReplace.add(tn);
			}

		}
		return lReplace;
	}
	
	private TokenNode getTokenNode(TreeNode del, String newToken) {
		if (newToken == null || newToken.length() == 0) {
			return null;
		}	
		List<TokenNode> lTokens = new LinkedList<TokenNode>();
		del.serialize(lTokens);

		//if the node to be replaced did not exist, return null.
		//We want no replacement for nodes that don't exist.
		//This can happen if for example the ClassBody node of
		//an empty class should be hidden - the empty class has no
		//ClassBody node to hide		
		if (lTokens.size() == 0) {
			return null;
		}
		Collections.sort(lTokens, new TreeNodeComparator());
		Token firstToken = lTokens.get(0).getToken();
		return new TokenNode("UNKNOWN", new Token(firstToken.getLine(),
				firstToken.getCol(), firstToken.getOffset(), newToken));

	}	

	
	
	
	
	private TreeNodeRoot checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(TreeNodeRepresentation.TYPE)) {
			TreeNodeRepresentation tnr = (TreeNodeRepresentation)rep;
			return tnr.getRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	private AstModifierConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof AstModifierConfiguration) {
			AstModifierConfiguration afConfig = (AstModifierConfiguration)config;
			return afConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}



	@Override
	public AstModifierConfiguration createConfiguration() {
		return new AstModifierConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}

}



