package org.deft.operation.coolsoftware;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.coolsoftware.textcompute.AssociationsBuilder;
import org.deft.operation.coolsoftware.textcompute.AttributesBuilder;
import org.deft.operation.coolsoftware.textcompute.MainCharacteristicsBuilder;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreRepresentation;
import org.deft.representation.styleddocument.DocumentContentContainerRepresentation;
import org.deft.representation.styleddocument.data.DocumentContentContainer;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.ResourceSet;

public class Ecore2CoolSoftwareTextOperation implements Operation {

	public static String ID = "org.deft.operation.ecore2coolsoftwaretext";
	

	@Override
	public ArtifactRepresentation executeOperation(
			ArtifactRepresentation representation, ArtifactType type,
			OperationConfiguration config) {
		EObject modelRoot = checkAndUnwrap(representation);
		Ecore2CoolSoftwareTextConfiguration csConfig = checkAndUnwrap(config);
		String qualifiedClassName = csConfig.getClassName();
		EClass eClassToDocument = getEClass(modelRoot, qualifiedClassName);

		DocumentContentContainer doc = createDocument(eClassToDocument);
		DocumentContentContainerRepresentation rep = new DocumentContentContainerRepresentation(doc);
		return rep;
	}

	
	
	private DocumentContentContainer createDocument(EClass eClass) {
		DocumentContentContainer doc = new DocumentContentContainer();
		MainCharacteristicsBuilder mcb = new MainCharacteristicsBuilder();
		mcb.addMainCharacteristics(eClass, doc);
		AttributesBuilder ab = new AttributesBuilder();
		ab.addAttributes(eClass, doc);
		AssociationsBuilder asb = new AssociationsBuilder();
		asb.addReferences(eClass, doc);
		return doc;
	}
	
	
	
	private EClass getEClass(EObject modelRoot, String qualifiedClassName) {
		ResourceSet rs = modelRoot.eResource().getResourceSet();
		for (Iterator<?> it = rs.getAllContents(); it.hasNext();) {
			Object o = it.next();
			if (o instanceof EClass) {
				EClass eClass = (EClass)o;
				if (representsClass(qualifiedClassName, eClass)) {
					return eClass;
				}
			}
		}
		return null;
	}


	private boolean representsClass(String qualifiedClassName, EClass eClass) {
		String[] classNameElementsArray = qualifiedClassName.split("\\.");
		List<String> classNameElements = Arrays.asList(classNameElementsArray);
		List<String> eClassPathElements = getPathElements(eClass); 
		boolean equals = classNameElements.equals(eClassPathElements);
		return equals;
	}
	
	/**
	 * Converts the absolute name of an eClass into a list.
	 * 
	 * For example, the class my.pck.MyClass will be converted to the list
	 * ["my", "pck", "MyClass"].
	 */
	private List<String> getPathElements(EClass eClass) {
		List<String> list = new LinkedList<String>();
		list.add(eClass.getName());
		EObject container = eClass.eContainer();
		while (container instanceof EPackage) {
			EPackage ePackage = (EPackage)container;
			list.add(0, ePackage.getName());
			container = ePackage.eContainer();
		}
		return list;
	}

	
	
	private EObject checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreRepresentation.TYPE)) {
			EcoreRepresentation er = (EcoreRepresentation)rep;
			return er.getModelRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private Ecore2CoolSoftwareTextConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof Ecore2CoolSoftwareTextConfiguration) {
			Ecore2CoolSoftwareTextConfiguration e2cConfig = (Ecore2CoolSoftwareTextConfiguration)config;
			return e2cConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	@Override
	public OperationConfiguration createConfiguration() {
		return new Ecore2CoolSoftwareTextConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return DocumentContentContainerRepresentation.TYPE;
	}

}
