package org.deft.operation.coolsoftware.textcompute;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.deft.representation.styleddocument.data.ConditionalCrossReference;
import org.deft.representation.styleddocument.data.ContentSequence;
import org.deft.representation.styleddocument.data.DocumentContentContainer;
import org.deft.representation.styleddocument.data.SimpleListing;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.util.StyledDocumentUtil;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.ResourceSet;

/**
 * Builds the Meta class characteristics part of Metaclass description for the Cool Software
 * Component specification. The result looks like this:
 * 
 * Meta class characteristics
 * - Abstract class: true
 * - Inherited from: SomeClass (see Sect. xxx)
 * - Known subclasses:
 * 		- SC1 (see Sect. yyy)
 * 		- SC2 (see Sect. zzz)
 */
public class MainCharacteristicsBuilder extends LatexTextBuilder {
	
	public void addMainCharacteristics(EClass eClass, DocumentContentContainer container) {
		ContentSequence contentSequence =  new ContentSequence();
		container.addDocumentContent(contentSequence);
		addHeading(contentSequence, "Meta class characteristics");
		addLineBreak(contentSequence);
		addCharacteristicsListing(eClass, contentSequence);
	}
	

	
	private void addCharacteristicsListing(EClass eClass, ContentSequence contentSequence) {
		SimpleListing listing = new SimpleListing();
		addToListing(listing, "Abstract class: ", new Boolean(eClass.isAbstract()).toString());
		addToListing(listing, "Inherited from: ", eClass.getESuperTypes());
		addToListing(listing, "Known subclasses: ", getDirectSubclasses(eClass));
		contentSequence.addDocumentContent(listing);
	}

	
	private void addToListing(SimpleListing listing, String propertyString, String valueString) {
		TokenizedStyledDocument propertyDoc = new TokenizedStyledDocument();
		StyledDocumentUtil.add(propertyDoc, propertyString);
		TokenizedStyledDocument valueDoc = new TokenizedStyledDocument();
		StyledDocumentUtil.add(valueDoc, valueString);
		ContentSequence sequence = new ContentSequence();
		sequence.addDocumentContent(propertyDoc);
		sequence.addDocumentContent(valueDoc);
		listing.addListItem(sequence);
	}
	

	private void addToListing(SimpleListing listing, String propertyString, List<EClass> eClasses) {
		TokenizedStyledDocument propertyDoc = new TokenizedStyledDocument();
		StyledDocumentUtil.add(propertyDoc, propertyString);
		ContentSequence sequence = new ContentSequence();
		listing.addListItem(sequence);
		sequence.addDocumentContent(propertyDoc);
		if (eClasses.isEmpty()) {
			TokenizedStyledDocument valueDoc = new TokenizedStyledDocument();
			StyledDocumentUtil.add(valueDoc, "---");
			sequence.addDocumentContent(valueDoc);
		} else if (eClasses.size() == 1) {
			EClass superType = eClasses.get(0);
			ContentSequence explanation = makeSequenceWithNameAndReference(superType);
			sequence.addDocumentContent(explanation);
		} else {//has multiple supertypes
			SimpleListing classListing = makeSimpleListing(eClasses);
			sequence.addDocumentContent(classListing);			
		}
	}
	
	private SimpleListing makeSimpleListing(List<EClass> eClasses) {
		SimpleListing listing = new SimpleListing();
		for (EClass ec : eClasses) {
			ContentSequence cs = makeSequenceWithNameAndReference(ec);
			listing.addListItem(cs);
		}
		return listing;
	}
	
	private ContentSequence makeSequenceWithNameAndReference(EClass eClass) {
		TokenizedStyledDocument tsd = new TokenizedStyledDocument();
		StyledDocumentUtil.add(tsd, eClass.getName());
		TokenizedStyledDocument space = new TokenizedStyledDocument();
		StyledDocumentUtil.add(space, " ");
		ConditionalCrossReference ref = new ConditionalCrossReference(computeRefString(eClass), "Sect.", "");
		ContentSequence cs = new ContentSequence();
		cs.addDocumentContent(tsd);
		cs.addDocumentContent(space);
		cs.addDocumentContent(ref);
		return cs;
	}
	
	private String computeRefString(EClass eClass) {
		String classString = eClass.getName();
		String packageString = "";
		EPackage ep = eClass.getEPackage();
		while (ep != null) {
			packageString = ep.getName() + ":" + packageString;
			ep = ep.getESuperPackage();
		}
		return "sec:ccmspec:" + packageString + classString;
	}
	
	/**
	 * Returns the direct subclasses of a class. 
	 * @param eClass the EClass for which to get the subclass
	 * @param modelRoots the model roots (should be packages) in which to search for subclasses
	 */
	private List<EClass> getDirectSubclasses(EClass eClass) {
		ResourceSet rs = eClass.eResource().getResourceSet();
		
		List<EClass> subclasses = new LinkedList<EClass>();

		for (Iterator<Notifier> it = rs.getAllContents(); it.hasNext();) {
			Object o = it.next();
			if (o instanceof EClass) {
				EClass ec = (EClass)o;
				List<EClass> superTypes = ec.getESuperTypes();
				if (superTypes.contains(eClass)) {
					subclasses.add(ec);
				}
			}
		}
		Collections.sort(subclasses, new ClassNameComparator());
		return subclasses;
	}
	
	private class ClassNameComparator implements Comparator<EClass> {

		@Override
		public int compare(EClass class1, EClass class2) {
			String name1 = class1.getName();
			String name2 = class2.getName();
			int result = name1.compareToIgnoreCase(name2);
			return result;
		}
		
	}
	
}
