package org.deft.operation.ecorediag2image;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.imageio.ImageIO;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecorediag.EcoreDiagRepresentation;
import org.deft.representation.image.BufferedImageRepresentation;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.gmf.runtime.diagram.ui.OffscreenEditPartFactory;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.image.ImageFileFormat;
import org.eclipse.gmf.runtime.diagram.ui.render.util.CopyToImageUtil;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;



public class EcoreDiag2ImageOperation implements Operation {


	
	@Override
	public BufferedImageRepresentation executeOperation(ArtifactRepresentation representation,
			ArtifactType type, OperationConfiguration config) {
		List<Diagram> diagrams = checkAndUnwrap(representation);
		EcoreDiag2ImageConfiguration diag2ImageConfig = checkAndUnwrap(config);
		String diagramId = diag2ImageConfig.getDiagramId();
		Diagram diagram = findDiagram(diagrams, diagramId);
		BufferedImage image = createImage(diagram);
		BufferedImageRepresentation ir = new BufferedImageRepresentation(image);
		return ir;
	}
	
	
	private List<Diagram> checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(getInputRepresentationType())) {
			EcoreDiagRepresentation edr = (EcoreDiagRepresentation)rep;
			return edr.getDiagrams();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private EcoreDiag2ImageConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof EcoreDiag2ImageConfiguration) {
			EcoreDiag2ImageConfiguration diag2ImageConfig = (EcoreDiag2ImageConfiguration)config;
			return diag2ImageConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	/**
	 * Returns the diagram with the matching id.
	 * If no matching id could be found:
	 * - If the id is null or empty, the first diagram is returned (backward compatibility).
	 * Otherwise null is returned.
	 */
	private Diagram findDiagram(List<Diagram> diagrams, String diagramId) {
		for (Diagram diagram : diagrams) {
			XMLResource resource = (XMLResource)diagram.eResource();
			String id = resource.getID(diagram);
			if (id.equals(diagramId)) {
				return diagram;
			}
		}
		if (!diagrams.isEmpty() && diagramId == null || diagramId.equals("")) {
			Diagram diagram = diagrams.get(0);
			return diagram;
		}
		return null;
	}
	
	
	private File imageFile = null;
	
	private BufferedImage createImage(final Diagram diagram) {
		Display display = Display.getDefault();
		display.syncExec(new Runnable() {
			@Override
			public void run() {
				imageFile = exportToImage(diagram, ImageFileFormat.PNG);
			}
		});

		BufferedImage image = null;
		try {
			image = ImageIO.read(imageFile);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return image;

		
	}
	
	private File exportToImage(Diagram diagram, ImageFileFormat imageFileFormat) {
		DiagramEditPart diagramEditPart = OffscreenEditPartFactory
				.getInstance().createDiagramEditPart(diagram, new Shell());
		
		File destinationFile = BufferedImageRepresentation.allocateTempImage(imageFileFormat.toString());
		IPath destination = new Path(destinationFile.getAbsolutePath());

		CopyToImageUtil copyToImageUtil = new CopyToImageUtil();
		try {
			copyToImageUtil.copyToImage(diagramEditPart, destination,
					imageFileFormat, new NullProgressMonitor());
		} catch (CoreException e) {
			e.printStackTrace();
			return null;
		}
		return destination.toFile();
	}
	
	


	@Override
	public OperationConfiguration createConfiguration() {
		return new EcoreDiag2ImageConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreDiagRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return BufferedImageRepresentation.TYPE;
	}

}



