package org.deft.operation.ecoreinstance2image;

import java.util.List;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.representation.image.BufferedImageRepresentation;
import org.deft.util.EcoreUtil;
import org.deft.util.ScreenshotUtil;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;

/**
 * Operation that takes an Ecore Instance representation and produces an
 * Image Representation from it. The image contains the Model tree as shown in
 * the outline view in Eclipse. The image is created by instantiating an invisible
 * TreeViewer and taking a screenshot. 
 */
public class EcoreInstance2ImageOperation implements Operation {


	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation,
			ArtifactType type, OperationConfiguration config) {
		EObject modelRoot = checkAndUnwrap(representation);
		EcoreInstance2ImageConfiguration ecore2ImageConfig = checkAndUnwrap(config);
		Image image = createImage(modelRoot, ecore2ImageConfig);
		BufferedImageRepresentation bir = new BufferedImageRepresentation(image);
		return bir;
	}
	
	
	private EObject checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreInstanceRepresentation.TYPE)) {
			EcoreInstanceRepresentation eir = (EcoreInstanceRepresentation)rep;
			return eir.getModelRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private EcoreInstance2ImageConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof EcoreInstance2ImageConfiguration) {
			EcoreInstance2ImageConfiguration instance2ImageConfig = (EcoreInstance2ImageConfiguration)config;
			return instance2ImageConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	private Image createImage(EObject modelRoot, EcoreInstance2ImageConfiguration config) {
		Composite parent = ScreenshotUtil.createInvisibleComposite();
		TreeViewer treeViewer = new TreeViewer(parent, SWT.NONE);
		EcoreUtil.setupEcoreTreeViewer(modelRoot, treeViewer);
		List<EObject> expandedNodes = config.getEObjects(modelRoot.eResource());
		treeViewer.collapseAll();
		if (!expandedNodes.isEmpty()) {
			for (EObject eo : expandedNodes) {
				if (areAllAncestorsExpanded(eo, expandedNodes)) {
					treeViewer.expandToLevel(eo, 1);
				}
			}
		}
		Image image = ScreenshotUtil.makeScreenshot(treeViewer.getControl());
		return image;
	}
	

	/**
	 * Checks if all ancestors of an EObject in a tree viewer are expanded.
	 * 
	 * This check is necessary because we want to expand an EObject ONLY IF all its ancestors
	 * are expanded. 
	 * Expanding something in a tree viewer automatically causes all its ancestors
	 * to be expanded. However, if we have an expanded element with a collapsed ancestor we DON'T
	 * want the ancestor to be expanded along with the element.
	 */
	private boolean areAllAncestorsExpanded(EObject eo, List<EObject> expandedNodes) {
		EObject parent = eo.eContainer();
		//while we still have a parent element and this parent element is not the
		//root element (which will not be displayed in the tree viewer)
		while (parent != null && parent.eContainer() != null) {
			//check if this parent is expanded, and if so, prepare the next iteration
			if (expandedNodes.contains(parent)) {
				parent = parent.eContainer();	
			} else {
				//if the parent is not expanded we don't want to expand our child
				return false;
			}
		}
		//all ancestors are expanded, we can expand the child
		return true;
	}
	
	@Override
	public OperationConfiguration createConfiguration() {
		return new EcoreInstance2ImageConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreInstanceRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return BufferedImageRepresentation.TYPE;
	}

}



