package org.deft.operation.emftext_text2styledcode;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;

import org.deft.emftext.EmfTextArtifactType;
import org.deft.emftext.EmfTextManager;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.styleddocument.StyledCodeRepresentation;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.representation.text.TextRepresentation;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.ITokenScanner;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;

/**
 * This operation transforms an EMFText-DSL (which exists as pure text/String) into
 * styled text. 
 * 
 * The operation queries the EMFText registry for the ITokenScanner (JFace) 
 * which is actually responsible to style the tokens in the EMF editor.
 * 
 * The ITokenScanner knows for each token how to render it. This information is
 * used to produce styled text.
 */
public class EmfText2StyledCodeOperation implements Operation {

	private ArtifactType artifactType;
	
	@Override
	public StyledCodeRepresentation executeOperation(ArtifactRepresentation representation,
			ArtifactType type, OperationConfiguration config) {
		this.artifactType = type;
		String text = checkAndUnwrap(representation);
		TextRepresentation textRep = (TextRepresentation)representation;
		int startOffset = textRep.getStartOffset();
		int endOffset = textRep.getEndOffset();

		TokenizedStyledDocument doc = createStyledDocument(text, startOffset, endOffset);
		StyledCodeRepresentation scr = new StyledCodeRepresentation(doc);
		return scr;
	}
	
	
	private String checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(getInputRepresentationType())) {
			TextRepresentation codeRep = (TextRepresentation)rep;
//			return codeRep.getText();
			return codeRep.getTextFromOffsets(true);
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	private TokenizedStyledDocument createStyledDocument(String text, int startOffset, int endOffset) {
		StyleContext context = setupBasicStyleContext();
		TokenizedStyledDocument document = new TokenizedStyledDocument(context);

		EmfTextManager etm = new EmfTextManager();
		String emfTextLanguageId = getEmfTextLanguageId();
    	ITokenScanner scanner = etm.getStyledTokenScanner(emfTextLanguageId);
    	IDocument doc = new Document(text);
    	scanner.setRange(doc, 0, text.length());
    	processAllTokens(text, scanner, document, context);
		
		return document;
	}
	
	private StyleContext setupBasicStyleContext() {
		StyleContext context = new StyleContext();
		Style defaultStyle = context.getStyle(StyleContext.DEFAULT_STYLE);

		Style normal = context.addStyle("normal", defaultStyle);
		StyleConstants.setFontFamily(normal, "Courier New");
		
		return context;
	}
	
	
	private void processAllTokens(String text, ITokenScanner scanner, 
			TokenizedStyledDocument document, StyleContext context) {
		IToken token = scanner.nextToken();
		while (token != null && !token.isEOF()) {
			
			int start = scanner.getTokenOffset();
			int end = start + scanner.getTokenLength();
			String tokenText = text.substring(start, end);
			
			Style style = getStyle(context, token, start);
	
			try {
				document.insertString(document.getLength(), tokenText, style);
			}
			catch (BadLocationException ble) {
				ble.printStackTrace();
			}

			token = scanner.nextToken();
		}
	}
	
	
	private Style getStyle(StyleContext context, IToken token, int offset) {
		Style normal = context.getStyle("normal");
		Object o = token.getData();
		if (o != null && o instanceof TextAttribute) {
			TextAttribute attr = (TextAttribute)o;
			Color color = attr.getForeground();
			String styleName = "styleAtPos" + offset;
			Style specialStyle = context.addStyle(styleName, normal);
			if (color != null) {
				int r = color.getRed();
				int g = color.getGreen();
				int b = color.getBlue();
				StyleConstants.setForeground(specialStyle, 
						new java.awt.Color(r, g, b));
			}
			int style = attr.getStyle();
			if (SWT.BOLD == (style & SWT.BOLD)) {
				StyleConstants.setBold(specialStyle, true);
			}
			if (SWT.ITALIC == (style & SWT.ITALIC)) {
				StyleConstants.setItalic(specialStyle, true);
			}
			if (TextAttribute.STRIKETHROUGH  == (style & TextAttribute.STRIKETHROUGH)) {
				StyleConstants.setStrikeThrough(specialStyle, true);
			}
			if (TextAttribute.UNDERLINE == (style & TextAttribute.UNDERLINE)) {
				StyleConstants.setUnderline(specialStyle, true);
			}
			return specialStyle;
		} else {
			return normal;
		}
	}
	

	protected String getEmfTextLanguageId() {
		EmfTextArtifactType emfTextType = (EmfTextArtifactType)artifactType;
		String emfTextLanguageId = emfTextType.getEmfTextLanguageId();
		return emfTextLanguageId;
	}

	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}
	

	@Override
	public String getInputRepresentationType() {
		return TextRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return StyledCodeRepresentation.TYPE;
	}
	
}
