package org.deft.operation.gui;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.operation.Ontology2MatrixConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.OwlRepresentation;
import org.deft.requirementsontology.IndividualVertex;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.semanticweb.owl.model.OWLOntology;

public class Ontology2MatrixOperationGui implements OperationGui {

	private Ontology2MatrixConfiguration matrixConfig;
	private CheckboxTableViewer tv1, tv2;
	private Combo combo1, combo2;
	private enum Dimension {
		SOURCE,
		TARGET
	}
	
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gui.setLayout(gridLayout);
		
		tv1 = insertTableViewer(gui, Dimension.SOURCE);
		tv2 = insertTableViewer(gui, Dimension.TARGET);
		combo1 = new Combo(gui, SWT.DROP_DOWN | SWT.BORDER);
		combo2 = new Combo(gui, SWT.DROP_DOWN | SWT.BORDER);
		return gui;
	}
	
	
	
	
	private CheckboxTableViewer insertTableViewer(Composite parent, final Dimension dimension) {
		String header = getHeaderText(dimension);
		Table table = createTable(parent, header);

		GridData gd = new GridData(GridData.FILL_BOTH);
		table.setLayoutData(gd);

		CheckboxTableViewer tableViewer = new CheckboxTableViewer(table);
		tableViewer.setUseHashlookup(true);
		tableViewer.setContentProvider(new ArrayContentProvider());
		return tableViewer;
	}
	
	
	private String getHeaderText(Dimension dimension) {
		if (dimension == Dimension.SOURCE) {
			return "Source";
		} else {
			return "Target";
		}
	}
	
	
	private Table createTable(Composite parent, String header) {
		
		int style = SWT.BORDER | SWT.CHECK;
		Table table = new Table(parent, style);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		table.setLayoutData(gridData);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableColumn column = new TableColumn(table, SWT.CENTER, 0);
		column.setWidth(500);
		column.setText(header);
		
		return table;
	}
	
	
	
	
	
	@Override
	public void initializeGui(OperationConfiguration config,
			Artifact artifact, ArtifactRepresentation artifactRepr) {
		
		this.matrixConfig = (Ontology2MatrixConfiguration)config;
		OWLOntology ontology = ((OwlRepresentation)artifactRepr).getOntology();
		OntologyAccessor oa = new OntologyAccessor(ontology);
		ROModel roModel = new ROModel(oa);
		roModel.getOntologyIndividualNames();
		List<String> ontologyIndividuals = new LinkedList<String>(roModel.getOntologyIndividualNames());
		Collections.sort(ontologyIndividuals);
		
		initializeTableViewer(tv1, ontologyIndividuals, Dimension.SOURCE);
		initializeTableViewer(tv2, ontologyIndividuals, Dimension.TARGET);
		initializeDropdownFilter(tv1, combo1, roModel, Dimension.SOURCE);
		initializeDropdownFilter(tv2, combo2, roModel, Dimension.TARGET);
	}

	

	
	private void initializeTableViewer(CheckboxTableViewer tv, List<String> allOntologyIndividualNames, final Dimension dimension) {
		tv.setInput(allOntologyIndividualNames.toArray());
		tv.addCheckStateListener(new IndividualSelectionListener(dimension));
	}
	
	private void initializeDropdownFilter(CheckboxTableViewer tableViewer, Combo combo, ROModel roModel,
			Dimension dimension) {
		List<String> categoryNames = new LinkedList<String>(roModel.getOntologyMainTypeNames());
		Collections.sort(categoryNames);

		combo.add("All Entries");
		for (String category : categoryNames) {
			combo.add(category);
		}
		combo.select(0);
		combo.addSelectionListener(new ComboBoxListener(tableViewer, roModel, dimension));
	}
	
	
	
	
	/**
	 * Updates the OperationConfiguration whenever an individual is checked or unchecked
	 */
	private class IndividualSelectionListener implements ICheckStateListener {
		
		private List<String> selectedIndividualNames = new LinkedList<String>();
		private Dimension dimension;
		
		public IndividualSelectionListener(Dimension dimension) {
			this.dimension = dimension;
		}

		
		@Override
		public void checkStateChanged(CheckStateChangedEvent event) {
			String indName = (String)event.getElement();
			if (event.getChecked()) {
				selectedIndividualNames.add(indName);
			} else {
				selectedIndividualNames.remove(indName);
			}
			Collections.sort(selectedIndividualNames);
			if (dimension == Dimension.SOURCE) {
				matrixConfig.setSourceIndividuals(selectedIndividualNames);
			} else {
				matrixConfig.setTargetIndividuals(selectedIndividualNames);
			}
		}
	}

	
	private class ComboBoxListener implements SelectionListener {

		private CheckboxTableViewer tableViewer;
		private ROModel roModel;
		private Dimension dimension;
		
		public ComboBoxListener(CheckboxTableViewer tableViewer, ROModel roModel, Dimension dimension) {
			this.tableViewer = tableViewer;
			this.roModel = roModel;
			this.dimension = dimension;
		}
		
		@Override
		public void widgetSelected(SelectionEvent e) {
			Combo combo = (Combo)e.getSource();
			int selectedIndex = combo.getSelectionIndex();
			if (selectedIndex == 0) {
				setAllIndividuals();
			} else {
				String category = combo.getItem(selectedIndex);
				setIndividuals(category);
			}
			checkSelectedItemsIfDisplayed();
		}
		
		private void setAllIndividuals() {
			List<String> ontologyIndividuals = new LinkedList<String>(roModel.getOntologyIndividualNames());
			Collections.sort(ontologyIndividuals);
			tableViewer.setInput(ontologyIndividuals.toArray());
		}
		
		private void setIndividuals(String category) {
			Collection<IndividualVertex> vertexes = roModel.getIndividuals(category);
			List<String> individualNames = new LinkedList<String>();
			for (IndividualVertex vertex : vertexes) {
				individualNames.add(vertex.getSimpleName());
			}
			Collections.sort(individualNames);
			tableViewer.setInput(individualNames.toArray());
		}

		@Override
		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
		
		private void checkSelectedItemsIfDisplayed() {
			List<String> individualNames = getIndividualNames();
			for (String indName : individualNames) {
				tableViewer.setChecked(indName, true);
			}
		}
		
		List<String> getIndividualNames() {
			if (dimension == Dimension.SOURCE) {
				return matrixConfig.getSourceIndividuals();
			} else {
				return matrixConfig.getTargetIndividuals();
			}
		}
		
	}


	



	
}
