package org.deft.operation.linebreak;

import java.io.IOException;
import java.net.URL;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.operation.linebreak.wrapper.LineWrapper;
import org.deft.operation.linebreak.wrapper.exceptions.LineWrapperOptionsReaderException;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptions;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptionsReader;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.treenode.TreeNodeRepresentation;
import org.deft.resource.ResourceHandler;
import org.deft.resource.impl.EclipsePluginResourceHandler;


/**
 * An operation that takes an AST in form of a TreeNodeRoot and the number of available space (in characters),
 * and inserts logical linebreaks if the AST's tokens exceed the available space.
 */
public class LineBreakOperation implements Operation {


	
	@Override
	public TreeNodeRepresentation executeOperation(ArtifactRepresentation representation,
			ArtifactType type, OperationConfiguration config) {
		TreeNodeRoot root = checkAndUnwrap(representation);
		LineBreakConfiguration lbConfig = checkAndUnwrap(config);
		int characterLimit = lbConfig.getCharacterLimit();
		applyLineBreaks(root, characterLimit);
		TreeNodeRepresentation tnRep = new TreeNodeRepresentation(root);
        return tnRep;
	}
	
	
	private TreeNodeRoot checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(TreeNodeRepresentation.TYPE)) {
			TreeNodeRepresentation tnr = (TreeNodeRepresentation)rep;
			return tnr.getRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	private LineBreakConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof LineBreakConfiguration) {
			LineBreakConfiguration afConfig = (LineBreakConfiguration)config;
			return afConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	
	private void applyLineBreaks(TreeNodeRoot root, int characterLimit) {
		LineWrapper wrapper = new LineWrapper();
		LineWrapperOptionsReader reader = new LineWrapperOptionsReader();
		ResourceHandler resourceHandler = new EclipsePluginResourceHandler("org.deft.operation.linebreak");
		URL optionsUrl = resourceHandler.getResourceUrl("options/Options1.xml");
		try {
			LineWrapperOptions options = reader.read(optionsUrl.openStream());
			wrapper.wrapLines(root, options, characterLimit);
		} catch (LineWrapperOptionsReaderException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	
	

	@Override
	public LineBreakConfiguration createConfiguration() {
		return new LineBreakConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}

}



