package org.deft.operation.linebreak.exporter.html.writers;

import org.deft.operation.linebreak.debug.AbstractDebuggable;

/**
 * Abstract base class for all content writers, such as HtmlWirter or JsonWriter.
 * Provides abstractions for including whitespace and the possibility to remove 
 * excess whitespace to reduce file size at the cost of readability.
 * 
 * @author Christoph Seidl
 */
public abstract class ContentWriter extends AbstractDebuggable
{	
	private static final String INDENT = "\t";
	private static final String ENDL = "\n";
	
	private boolean contractOutput;
	
	/**
	 * Creates a new <code>ContentWriter</code>.
	 * 
	 * @param contractOutput Determines whether excess whitespace should be reduced to reduce file size.
	 */
	public ContentWriter(boolean contractOutput)
	{
		this.contractOutput = contractOutput;
	}
	
	/**
	 * Indents by the specified amount.
	 * 
	 * @param amount The number of indents.
	 * 
	 * @return A string that is indented by the requested amount.
	 */
	protected String indent(int amount)
	{
		if (contractOutput)
		{
			return "";
		}
		
		String s = "";
		
		for (int i = 0; i < amount; ++i)
		{
			s += INDENT;
		}
		
		return s;
	}
	
	/**
	 * Creates a string containing a line feed.
	 * 
	 * @return A string containing a line feed.
	 */
	protected String endl()
	{
		if (contractOutput)
		{
			return "";
		}
		
		return ENDL;
	}
	
	/**
	 * Creates a string containing a white space.
	 * 
	 * @return A string containing a white space.
	 */
	protected String space()
	{
		if (contractOutput)
		{
			return "";
		}
		
		return " ";
	}
}
