package org.deft.operation.linebreak.exporter.html.writers;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.deft.operation.linebreak.exporter.text.PlainTextExporter;
import org.deft.operation.linebreak.wrapper.util.HtmlEncoder;
import org.deft.repository.ast.TreeNode;

/**
 * Assembles the contents of an HTML document in a string that can be used to write 
 * the according file to disc.
 * 
 * @author Christoph Seidl
 */
public class HtmlWriter extends ContentWriter
{
	private PlainTextExporter plainTextExporter;
	
	/**
	 * Creates a new <code>HtmlWriter</code>.
	 * 
	 * @param contractOutput Determines whether excess whitespace should be reduced to reduce file size.
	 */
	public HtmlWriter(boolean contractOutput)
	{
		super(contractOutput);
		
		plainTextExporter = new PlainTextExporter();	
	}
	
	/**
	 * Writes the contents of an HTML file to a string.
	 * 
	 * @param jsonPathsAndFilenames A list of the paths and filenames of the JSON files used for data exchange.
	 * @param snippets The snippets that are to be included.
	 * @param snippetIDs The IDs of the snippets.
	 * @param cssFiles A list of all the CSS style sheets used for this HTML document.
	 * @param javaScriptFiles A list of all JavaScript files used for this HTML document.
	 * 
	 * @return A string containing the content of the HTML document.
	 */
	public String writeHtml(List<String> jsonPathsAndFilenames, List<TreeNode> snippets, Map<TreeNode, String> snippetIDs, List<String> cssFiles, List<String> javaScriptFiles)
	{
		String output = "";
		
		output += "<html>" + endl();
		
		output += writeHead(cssFiles, javaScriptFiles);
		output += writeBody(jsonPathsAndFilenames, snippets, snippetIDs);
		
		output += "</html>" + endl();
		
		return output;
	}
	
	/**
	 * Writes the head of the HTML document.
	 * 
	 * @param cssFiles A list of all the CSS style sheets used for this HTML document.
	 * @param javaScriptFiles A list of all JavaScript files used for this HTML document.
	 * 
	 * @return A string containing the content of the head element of the HTML document
	 */
	private String writeHead(List<String> cssFiles, List<String> javaScriptFiles)
	{
		String output = "";
		
		output += indent(1) + "<head>" + endl();
		output += indent(2) + "<title>Exported Snippets</title>" + endl();

		//import css files
		Iterator<String> iterator = cssFiles.iterator();
		
		while(iterator.hasNext())
		{
			String cssFile = iterator.next().replace("\\", "/");
			
			output += indent(2) + "<LINK REL=StyleSheet HREF=\"" + cssFile + "\" TYPE=\"text/css\">" + endl();
		}

		output += endl();
		
		//import js files
		Iterator<String> iterator2 = javaScriptFiles.iterator();
		
		while(iterator2.hasNext())
		{
			String javaScriptFile = iterator2.next().replace("\\", "/");
			
			output += indent(2) + "<script type=\"text/javascript\" src=\"" + javaScriptFile + "\"></script>" + endl();
		}
		
		
		output += indent(1) + "</head>" + endl();
		
		return output;
	}
	
	/**
	 * Writes the body of the HTML document.
	 * 
	 * @param jsonPathsAndFilenames A list of the paths and filenames of the JSON files used for data exchange.
	 * @param snippets The snippets that are to be included.
	 * @param snippetIDs The IDs of the snippets.
	 * 
	 * @return A string containing the content of the body element of the HTML document.
	 */
	private String writeBody(List<String> jsonPathsAndFilenames, List<TreeNode> snippets, Map<TreeNode, String> snippetIDs)
	{
		String output = "";
		
		//Write multiple JSON files to a JAvaSCript array.
		Iterator<String> iterator2 = jsonPathsAndFilenames.iterator();
		String jsonFilenamesArray = "new Array(";
		boolean isFirstFilename = true;
		
		while (iterator2.hasNext())
		{
			if (!isFirstFilename)
			{
				jsonFilenamesArray += ", ";
			}
			
			String jsonFilename = iterator2.next().replace("\\", "/");
			
			jsonFilenamesArray += "'" + jsonFilename + "'";
			
			isFirstFilename = false;
		}
		
		jsonFilenamesArray += ")";
		
		output += indent(1) + "<body onload=\"wrapAllSnippets(" + jsonFilenamesArray + ");\">" + endl();
		
		Iterator<TreeNode> iterator = snippets.iterator();
		
		//Needed to measure the width of a character with the current formatting and
		//thus to determine the character limit for a code box.
		output += indent(2) + "<pre id=\"sourcecodeWidthTest\" class=\"sourcecode sourcecodeWidthTest\"></pre>" + endl();
		
		//Write snippets.
		while(iterator.hasNext())
		{
			TreeNode snippet = iterator.next();
			String snippetID = snippetIDs.get(snippet);
			
			assert(snippetID != null);
			
			if (snippetID == null)
			{
				logError("Snippet ID is missing!");
				return "";
			}
			
			output += writeSnippet(snippet, snippetID, 2);
		}
		
		output += indent(1) + "</body>" + endl();
		
		return output;
	}
	
	/**
	 * Writes a snippet to plain text surrounded by a pre tag. This is used as
	 * a fall-back if the user has JavaScript disabled and dynamic wrapping cannot
	 * be performed. This way, the user can at least see the code in its original
	 * formatting.
	 *  
	 * @param snippet The snippet to be written.
	 * @param snippetID the ID of that snippet.
	 * @param level The level of indentation used for pretty printing.
	 * 
	 * @return A string containing the contents of a pre tag with the code from the specified snippet.
	 */
	private String writeSnippet(TreeNode snippet, String snippetID, int level)
	{
		String output = "";
		
		output += indent(level) + "<pre id=\"" + snippetID + "\" class=\"sourcecode\">";
		
		//Write plain text snippet in case java script is disabled at the client side
		//and html encode characters.
		output += HtmlEncoder.encodeHtml(plainTextExporter.export(snippet, true));
			
		output += "</pre>" + endl();
		
		return output;
	}
}
