package org.deft.operation.linebreak.exporter.html.writers;

import java.util.Iterator;

import org.deft.operation.linebreak.wrapper.binding.TokenBindingContainer;
import org.deft.operation.linebreak.wrapper.binding.TokenBindingKey;
import org.deft.operation.linebreak.wrapper.core.SharedLineWrapperFunctionality;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptions;
import org.deft.operation.linebreak.wrapper.util.Caret;
import org.deft.operation.linebreak.wrapper.util.CodeLine;
import org.deft.operation.linebreak.wrapper.util.HtmlEncoder;
import org.deft.repository.ast.Token;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;

/**
 * Writes snippets (parts of an abstract syntax tree) to JSON keeping line and column
 * information and the relevant part of line wrapper options to be used for data exchange
 * with the dynamic line wrapper in JavaScript.
 * 
 * @author Christoph Seidl
 */
public class JsonWriter extends ContentWriter
{
	private SharedLineWrapperFunctionality sharedFunctionality;
	private Caret caret;
		
	/**
	 * Creates a new <code>JsonWriter</code>.
	 * 
	 * @param contractOutput Determines whether excess whitespace should be reduced to reduce file size.
	 */
	public JsonWriter(boolean contractOutput)
	{
		super(contractOutput);
		
		sharedFunctionality = new SharedLineWrapperFunctionality();
	}
	
	/**
	 * Converts a line to the format of the JSON file.
	 * 
	 * @param line The line to be converted.
	 * 
	 * @return The converted line.
	 */
	private int convertLine(int line)
	{
		return line - caret.getStartLine() + Caret.FIRST_LINE;
	}
	
	/**
	 * Converts a column to the format of the JSON file.
	 * 
	 * @param col The column to be converted.
	 * 
	 * @return The converted column.
	 */
	private int convertCol(int col)
	{
		return col - caret.getStartCol() + Caret.FIRST_COL;
	}
		
	/**
	 * Exports the part of the line wrapper options stating whether to consume blank lines.
	 * 
	 * @param lineWrapperOptions The line wrapper options to be used.
	 * @param level The level of indentation used for pretty printing.
	 * 
	 * @return A block of JSON code.
	 */
	private String exportConsumeBlankLines(LineWrapperOptions lineWrapperOptions, int level)
	{
		String s = "";
		
		s += indent(level) + "\"consumeBlankLines\"" + space() + ":" + space() + (lineWrapperOptions.getConsumeExistingBlankLines() ? "true" : "false") + "," + endl();
		s += indent(level) + "\"preserveCount\"" + space() + ":" + space() + lineWrapperOptions.getPreserveBlankLinesCount();
		
		return s;
	}
	
	/**
	 * Exports a token node and all its relevant associated information.
	 * 
	 * @param tokenNode The token node to be exported.
	 * @param oldTokenNode The previously encountered token node used to determine binding strength and alignment column.
	 * @param lineWrapperOptions The line wrapper options to be used.
	 * @param level The level of indentation used for pretty printing.
	 * 
	 * @return A block of JSON code.
	 */
	private String exportTokenNode(TokenNode tokenNode, TokenNode oldTokenNode, LineWrapperOptions lineWrapperOptions, int level)
	{
		String s = "";
		
		Token token = tokenNode.getToken();
		
		//HTML encode critical characters
		String tokenText = HtmlEncoder.encodeHtml(token.getText());
		
		s += indent(level) + "{" + endl();
		s += indent(level + 1) + "\"text\"" + space() + ":" + space() + "\"" + tokenText + "\"," + endl();
		s += indent(level + 1) + "\"startLine\"" + space() + ":" + space() + convertLine(tokenNode.getStartLine()) + "," + endl();
		s += indent(level + 1) + "\"startCol\"" + space() + ":" + space() + convertCol(tokenNode.getStartCol()) + "," + endl();
		s += indent(level + 1) + "\"endLine\"" + space() + ":" + space() + convertLine(tokenNode.getEndLine()) + "," + endl();
		s += indent(level + 1) + "\"endCol\"" + space() + ":" + space() + convertCol(tokenNode.getEndCol()) + "," + endl();
				
		TokenBindingContainer tbc = sharedFunctionality.getTbc();
		
		int bindingStrength = 1;
		int alignmentCol = Caret.FIRST_COL;
		
		if (oldTokenNode != null)
		{
			TokenBindingKey key = new TokenBindingKey(oldTokenNode, tokenNode);
			
			bindingStrength = tbc.getBindingStrength(key);
			alignmentCol = tbc.getAlignmentCol(key);
		}
		
		s += indent(level + 1) + "\"bindingStrength\"" + space() + ":" + space() + bindingStrength + "," + endl();
		s += indent(level + 1) + "\"alignmentCol\"" + space() + ":" + space() + alignmentCol + endl();
		s += indent(level) + "}";
		
		return s;
	}
		
	/**
	 * Exports an entire physical line.
	 * 
	 * @param physicalLine The physical line to be exported.
	 * @param lineWrapperOptions The line wrapper options to be used.
	 * @param level The level of indentation used for pretty printing.
	 * 
	 * @return A block of JSON code.
	 */
	private String exportPhysicalLine(CodeLine physicalLine, LineWrapperOptions lineWrapperOptions, int level)
	{
		String s = "";
		
		s += indent(level) + "[" + endl();
		
		TokenNode oldTokenNode = null;
		Iterator<TokenNode> iterator = physicalLine.iterator();
		
		
		while(iterator.hasNext())
		{
			if (oldTokenNode != null)
			{
				s += "," + endl();
			}
			
			TokenNode currentTokenNode = iterator.next();
			
			s += exportTokenNode(currentTokenNode, oldTokenNode, lineWrapperOptions, level + 1);
						
			oldTokenNode = currentTokenNode;
		}
		
		s += endl();
		s += indent(level) + "]";
		
		return s;
	}
	
	/**
	 * Exports all physical lines of a given snippet.
	 * 
	 * @param snippet The snippet containing the physical lines to be exported.
	 * @param lineWrapperOptions The line wrapper options to be used.
	 * @param level The level of indentation used for pretty printing.
	 * 
	 * @return A block of JSON code.
	 */
	private String exportPhysicalLines(TreeNode snippet, LineWrapperOptions lineWrapperOptions, int level)
	{
		String s = "";
		
		s += indent(level) + "\"physicalLines\"" + space() + ":" + space() + "[" + endl();				
		Iterator<CodeLine> iterator = sharedFunctionality.getPhysicalLines().iterator();
		
		boolean isFirstLine = true;
		
		while(iterator.hasNext())
		{
			if (isFirstLine)
			{
				isFirstLine = false;
			}
			else
			{
				s += "," + endl();
			}
			
			CodeLine currentLine = iterator.next();
				
			s += exportPhysicalLine(currentLine, lineWrapperOptions, level + 1);
		}
		
		s += endl();
		s += indent(level) + "]" + endl();
		
		return s;
	}
	
	/**
	 * Exports a snippet with a given unique ID.
	 * 
	 * @param snippet The snippet to be exported.
	 * @param snippetID The unique ID of the snippet that is exported.
	 * @param lineWrapperOptions The line wrapper options to be used.
	 * @param level The level of indentation used for pretty printing.
	 * 
	 * @return A block of JSON code.
	 */
	private String exportSnippet(TreeNode snippet, String snippetID, LineWrapperOptions lineWrapperOptions, int level)
	{	
		sharedFunctionality.process(snippet, lineWrapperOptions);
		
		caret = sharedFunctionality.getCaret();
				
		String s = "";
		
		s += indent(level) + "{" + endl();
		s += indent(level + 1) + "\"snippetID\"" + space() + ":" + space() + "\"" + snippetID + "\"," + endl();
		s += exportConsumeBlankLines(lineWrapperOptions, level + 1) + "," + endl();
		s += exportPhysicalLines(snippet, lineWrapperOptions, level + 1);	
		s += indent(level) + "}";
				
		return s;
	}
	
	/**
	 * Exports the specified snippet to JSON.
	 * 
	 * @param snippet The snippet that is to be exported.
	 * @param snippetID The ID of the snippet. Has to be unique within the target document.
	 * @param lineWrapperOptions The line wrapper options to be used.
	 * 
	 * @return A block of JSON code.
	 */
	public String writeJson(TreeNode snippet, String snippetID, LineWrapperOptions lineWrapperOptions)
	{
		return exportSnippet(snippet, snippetID, lineWrapperOptions, 0);
	}
}
