package org.deft.operation.linebreak.exporter.text;

import java.util.Iterator;
import java.util.List;

import org.deft.operation.linebreak.debug.AbstractDebuggable;
import org.deft.operation.linebreak.wrapper.util.Caret;
import org.deft.operation.linebreak.wrapper.util.CodeLine;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.annotation.linewrapping.LineWrappingInformation;

/**
 * Exports a snippet of code to plain text. If present, the <code>LineWrappingInformation</code>
 * of a <code>TokenNode</code> is used to reflect the changes made by the line wrapper. If this
 * information cannot be found, the physical position is used as a fall-back. Note that partial
 * line wrapping information might lead to undesirable results.
 * 
 * @author Christoph Seidl
 */
public class PlainTextExporter extends AbstractDebuggable
{	
	private int actualLine;
	private int actualCol;
	
	private String exported;
	
	/**
	 * Exports a snippet of code to plain text, optionally disregarding any line attached
	 * line wrapping information
	 * 
	 * @param snippet The snippet of code that is to be exported.
	 * @param useOriginalFormatting If set to <code>true</code> physical coordinates are used even if line wrapping information might be present.
	 * @return A string containing the plain text version of the code snippet.
	 */
	public String export(TreeNode snippet, boolean useOriginalFormatting)
	{
		actualLine = Caret.FIRST_LINE;
		actualCol = Caret.FIRST_COL;
		
		exported = "";
		
		List<TokenNode> tokens = snippet.serialize();
		
		Iterator<TokenNode> iterator = tokens.iterator();
		
		while(iterator.hasNext())
		{
			TokenNode tokenNode = iterator.next();

			LineWrappingInformation lwi = null;
			
			if (tokenNode.hasInformation(LineWrappingInformation.IDENT))
			{
				lwi = (LineWrappingInformation) tokenNode.getInformation(LineWrappingInformation.IDENT);
			}
			
			advanceToStartPosition(tokenNode, lwi, useOriginalFormatting);
			writeToken(tokenNode, lwi, useOriginalFormatting);
		}
		
		return exported;
	}
	
	/**
	 * Convenience method that is equivalent to a call of <code>export(snippet, false)</code>.
	 * 
	 * @param snippet The snippet of code that is to be exported.
	 * @return A string containing the plain text version of the code snippet.
	 */
	public String export(TreeNode snippet)
	{
		return export(snippet, false);
	}
	
	/**
	 * Adds a specified number of new lines to the output string.
	 * 
	 * @param number The number  of new lines to add.
	 */
	private void addNewLines(int number)
	{		
		if (number < 0)
		{
			debug("WARNING: Trying to add negative new lines (" + number + ")");
			return;
		}
		
		for (int i = 0; i < number; ++i)
		{
			exported += "\n";
		}
		
		actualLine += number;
		
		//advancing to next line means starting at column 1
		if (number > 0)
		{
			actualCol = Caret.FIRST_COL;
		}
	}
	
	/**
	 * Adds a number of whitespace characters to the output string.
	 * 
	 * @param number the number of whitespace characters to add.
	 */
	private void addBlanks(int number)
	{		
		if (number < 0)
		{
			debug("WARNING: Trying to add negative blanks (" + number + ")");
			return;
		}
		
		for (int i = 0; i < number; ++i)
		{
			exported += " ";
		}
		
		actualCol += number;
	}
	
	/**
	 * Fills the output string with whitespace from the current position up to the
	 * requested position.
	 * 
	 * @param requestedLine The line the exported string is at after filling with whitespace.
	 * @param requestedCol The column the exported string is at after filling with whitespace.
	 */
	private void fillWithWhitespaceToRequestedPosition(int requestedLine, int requestedCol)
	{		
		//add required white space
		int linesToAdd = requestedLine - actualLine;
		addNewLines(linesToAdd);

		//NOTE: has to be calculated, after new lines were added
		int colsToAdd = requestedCol - actualCol;
		addBlanks(colsToAdd);
	}
	
	/**
	 * Advances the exported string to the start position of the specified <code>TokenNode</code>
	 * (physical or virtual).
	 * 
	 * @param currentTokenNode The <code>TokenNode</code>, which's start position is to be used.
	 * @param lwi The associated line wrapping information, if it exists (was predetermined).
	 * @param useOriginalFormatting Whether to disregard potentially present line wrapping information.
	 */
	private void advanceToStartPosition(TokenNode currentTokenNode, LineWrappingInformation lwi, boolean useOriginalFormatting)
	{
		int startLine;
		int startCol;
		
		if (lwi != null && !useOriginalFormatting)
		{
			startLine = lwi.getLine();
			startCol = lwi.getCol();
		}
		else
		{
			startLine = currentTokenNode.getStartLine();
			startCol = currentTokenNode.getStartCol();
		}
		
		fillWithWhitespaceToRequestedPosition(startLine, startCol);
	}
	
	/**
	 * Writes the text of a token and advances to its end position.
	 * 
	 * @param currentTokenNode The <code>TokenNode<code>, which's <code>Token</code> is to be written to the exported string.
	 * @param lwi The associated line wrapping information, if it exists (was predetermined).
	 * @param useOriginalFormatting Whether to disregard potentially present line wrapping information.
	 */
	private void writeToken(TokenNode currentTokenNode, LineWrappingInformation lwi, boolean useOriginalFormatting)
	{
		//add actual content
		exported += currentTokenNode.getToken().getText();
		
		advanceToEndPosition(currentTokenNode, lwi, useOriginalFormatting);
	}
	
	/**
	 * Advances the exported string to the end position of the specified <code>TokenNode</code>
	 * (physical or virtual).
	 * 
	 * @param currentTokenNode The <code>TokenNode</code>, which's end position is to be used.
	 * @param lwi The associated line wrapping information, if it exists (was predetermined).
	 * @param useOriginalFormatting Whether to disregard potentially present line wrapping information.
	 */
	private void advanceToEndPosition(TokenNode currentTokenNode, LineWrappingInformation lwi, boolean useOriginalFormatting)
	{		
		int endLine;
		int endCol;
		
		if (lwi != null && !useOriginalFormatting)
		{
			endLine = lwi.getEndLine();
			endCol = lwi.getEndCol();
		}
		else
		{
			endLine = currentTokenNode.getEndLine();
			endCol = currentTokenNode.getEndCol();
		}

		//in accordance with written token
		
		//debug(currentToken.getText() + ": " + actualCol + " + " + currentToken.getText().length() + " ?= " + endCol);
		//int supposedEndCol = actualCol + currentToken.getText().length();
		
		actualLine = endLine;
		actualCol = endCol;
		
		//debug("Ending col is=" + actualCol + ", should be=" + supposedEndCol);
	}
	
	//TODO: remove DEBUG
	public String exportCodeLine(CodeLine codeLine)
	{		
		exported = "";
		
		Iterator<TokenNode> iterator = codeLine.iterator();
		boolean isFirstToken = true;
		
		while(iterator.hasNext())
		{
			TokenNode tokenNode = iterator.next();
			
			if (isFirstToken)
			{
//				indentationAdjustment = currentToken.getCol() - 1;
				actualLine = tokenNode.getStartLine();
				actualCol = tokenNode.getStartCol();
				isFirstToken = false;
			}
			
			advanceToStartPosition(tokenNode, null, false);
			writeToken(tokenNode, null, false);
		}
		
		return exported;
	}
}
