package org.deft.operation.linebreak.wrapper;

import java.util.Iterator;

import org.deft.operation.linebreak.debug.AbstractDebuggable;
import org.deft.operation.linebreak.wrapper.binding.TokenBindingContainer;
import org.deft.operation.linebreak.wrapper.core.SharedLineWrapperFunctionality;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptions;
import org.deft.operation.linebreak.wrapper.strategies.DefaultLineWrappingStrategy;
import org.deft.operation.linebreak.wrapper.strategies.LineWrappingStrategy;
import org.deft.operation.linebreak.wrapper.util.Caret;
import org.deft.operation.linebreak.wrapper.util.CodeLine;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;

/**
 * Central class for performing a line wrap of a snippet of code in a width bounded
 * environment obeying a set of rules.
 * 
 * @author Christoph Seidl
 */
public class LineWrapper extends AbstractDebuggable
{			
	private SharedLineWrapperFunctionality sharedFunctionality;
	
	private LineWrapperOptions lineWrapperOptions;
	private LineWrappingStrategy lineWrappingStrategy;
		
	/**
	 * Creates a new <code>LineWrapper</code>.
	 */
	public LineWrapper()
	{
		//NOTE: Choose appropriate strategy here.
		//lineWrappingStrategy = new FastLineWrappingStrategy();
		lineWrappingStrategy = new DefaultLineWrappingStrategy();
		sharedFunctionality = new SharedLineWrapperFunctionality();
	}
	
	/**
	 * Takes a fragment of a code file and wraps all lines that exceed the given maximum number
	 * of characters. The wrapping procedure is guided by a specified set of options and rules.
	 * 
	 * @param snippet A fragment of a code file that is to be wrapped to fit the width bound. Must not be <code>null</code>.
	 * @param lineWrapperOptions A set of options and rules that are used to wrap the input fragment. Must not be <code>null</code>.
	 * @param characterLimit The maximum number of characters that can fit on a single line. Has to be a non negative integer.
	 */
	public void wrapLines(TreeNode snippet, LineWrapperOptions lineWrapperOptions, int characterLimit)
	{
		assert(snippet != null);
		assert(lineWrapperOptions != null);
		assert(characterLimit >= 0);
		
		if (snippet == null)
		{
			logError("Unable to perform line wrap due to null value of snippet.");
			return;
		}
		
		if (lineWrapperOptions == null)
		{
			logError("Unable to perform line wrap due to null value of LineWrapperOptions object.");
			return;
		}
		
		if (characterLimit < 0)
		{
			logError("Unable to perform line wrap due to negative value of characterLimit.");
			return;
		}
		
		if (characterLimit == 0)
		{
			//No use to perform a line wrap.
			return;
		}
		
		this.lineWrapperOptions = lineWrapperOptions;

		sharedFunctionality.process(snippet, lineWrapperOptions);

		performLineWrap(characterLimit);
	}
	
	/**
	 * Fast re-wrapping procedure of an already wrapped fragment of code. Can be used in dynamic-width
	 * environments for maximum performance.This procedure assumes that all internal preparation has
	 * already been done by a prior call to <code>wrapLines(...)</code>. Calling this method without the
	 * internal preparation will yield unpredictable results or even crash. 
	 * 
	 * @param characterLimit The maximum number of characters that can fit on a single line.
	 */
	public void rewrapLines(int characterLimit)
	{
		performLineWrap(characterLimit);
	}

	/**
	 * Worker method that performs the line wrap after all relevant internal data structures have been
	 * initialized.
	 * 
	 * This algorithm assumes that each token is located on a single line. Note that logic dictates that
	 * if there is a multi-line token on a physical line, it is necessarily the last one as all following
	 * tokens would not be on that physical line. Hence, this has no effect on the correctness of the
	 * algorithm.
	 */
	private void performLineWrap(int characterLimit)
	{
		Caret caret = sharedFunctionality.getCaret();
		int numberOfWraps = 0;

		CodeLine oldPhysicalLine = null;
		
		Iterator<CodeLine> iterator = sharedFunctionality.getPhysicalLines().iterator();
		
		//Processing might have changed the caret, even though it shouldn't.
		//Better reset it to be on the safe side.
		caret.reset();
		
		//The iterator is one element ahead of currentPhysicalLine in order to realize
		//a one line look-ahead mechanism.
		while(iterator.hasNext())
		{
			CodeLine currentPhysicalLine = iterator.next();
			
			tryToConsumeBlankLines(currentPhysicalLine, oldPhysicalLine, caret, numberOfWraps);
			
			//With a new physical line, the wrap-dependent parameters are obsolete.
			caret.returnToFirstColumn();
			
			TokenBindingContainer tbc = sharedFunctionality.getTbc();
			
			numberOfWraps = lineWrappingStrategy.wrapSinglePhysicalLine(lineWrapperOptions, tbc, currentPhysicalLine, caret, characterLimit);
			
			oldPhysicalLine = currentPhysicalLine;
		}
	}
	
	/**
	 * Tries to consume as many blank lines of the input as possible to use them for wrapped lines
	 * in the output. Provided that consuming blank lines is enabled in the options, the maximum number
	 * of blank lines that can be consumed is influenced by a multitude of factors:
	 * 
	 * 1. The number of actually occurring blank lines between two lines of code (calculated from
	 * <code>currentPhysicalLine</code> and <code>oldPhysicalLine</code>).
	 * 
	 * 2. The number of wraps that occurred in the last wrapped line (as blank lines may only be used
	 * for the wrapped part of a line of code).
	 * 
	 * 3. The maximum number of permissible numbers of consumptions specified in the options used with
	 * the call to <code>wrapLines(...)</code>. 
	 * 
	 * If physical lines are consumed, the caret is adjusted to convert accordingly. 
	 * 
	 * @param currentPhysicalLine The first physical line that has not yet been wrapped.
	 * @param oldPhysicalLine The last physical line that was wrapped.
	 * @param caret The caret that reflects the current conversion of physical to virtual coordinates.
	 * @param numberOfWraps The number of wraps that occurred on the last physical line.
	 */
	private void tryToConsumeBlankLines(CodeLine currentPhysicalLine, CodeLine oldPhysicalLine, Caret caret, int numberOfWraps)
	{
		if (oldPhysicalLine == null || !lineWrapperOptions.getConsumeExistingBlankLines() || numberOfWraps == 0)
		{
			return;
		}
		
		//difference in lines of the first token of the current line and the
		//last token of the old line is the number of originally blank lines
		TokenNode lastTokenNodeOfOldLine = oldPhysicalLine.get(oldPhysicalLine.numberOfTokenNodes() - 1);
		TokenNode firstTokenNodeOfCurrentLine = currentPhysicalLine.get(0);
		
		int originalBlankLines = firstTokenNodeOfCurrentLine.getStartLine() - lastTokenNodeOfOldLine.getEndLine() - 1;
		int preserveBlankLinesCount = lineWrapperOptions.getPreserveBlankLinesCount();
		
		//Can only consume number of lines equal to the number of wraps (by policy) and no more
		//than there originally were. Additionally a set amount of blank lines has to be preserved.
		int numberOfBlankLinesToConsume = Math.min(numberOfWraps, originalBlankLines - preserveBlankLinesCount);
		
		if (numberOfBlankLinesToConsume > 0)
		{
			caret.consumeLines(numberOfBlankLinesToConsume);
		}
	}
}
