package org.deft.operation.linebreak.wrapper.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.deft.operation.linebreak.exporter.html.writers.JsonWriter;
import org.deft.operation.linebreak.wrapper.binding.TokenBindingContainer;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptions;
import org.deft.operation.linebreak.wrapper.preprocessing.PreprocessingStep;
import org.deft.operation.linebreak.wrapper.preprocessing.StructuralInformationPreprocessingStep;
import org.deft.operation.linebreak.wrapper.preprocessing.WrappingRulesPreprocessingStep;
import org.deft.operation.linebreak.wrapper.util.Caret;
import org.deft.operation.linebreak.wrapper.util.CodeLine;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;

/**
 * Line wrapping functionality that is shared between the <code>LineWrapper</code>
 * and part of the export procedure.
 * 
 * @author Christoph Seidl
 *  
 * @see wrapper.LineWrapper
 * @see JsonWriter
 */
public class SharedLineWrapperFunctionality
{	
	List<PreprocessingStep> preprocessingSteps;
	
	private TokenBindingContainer tbc;
	private List<CodeLine> physicalLines;
	private Caret caret;
	
	private int minimumLine;
	private int minimumCol;
	
	/**
	 * Create a new <code>SharedLineWrapperFunctionality</code>.
	 */
	public SharedLineWrapperFunctionality()
	{
		preprocessingSteps = new LinkedList<PreprocessingStep>();
		
		preprocessingSteps.add(new StructuralInformationPreprocessingStep());
		preprocessingSteps.add(new WrappingRulesPreprocessingStep());
		
		tbc = null;
		physicalLines = null;
		caret = null;
	}
	
	/**
	 * Reads a snippet of code an creates a list of physical code line from it.
	 * A physical code line is a line of code whose tokens all have the same start
	 * line. Note that only lines that contain tokens will be contained in the list.
	 * Hence, blank lines do not have a representation in the list.
	 * 
	 * @param selectedTreeNode The fragment of code that is to be read.
	 * 
	 * @return A list of physical code lines.
	 */
	public List<CodeLine> readPhysicalLines(TreeNode selectedTreeNode)
	{
		minimumLine = -1;
		minimumCol = -1;
		List<CodeLine> physicalLines = new ArrayList<CodeLine>();
		
		Iterator<TokenNode> iterator = selectedTreeNode.serialize().iterator();
		
		CodeLine tempPhysicalLine = null;
		
		int lastLine = 0;
		
		while(iterator.hasNext())
		{
			TokenNode tokenNode = iterator.next();
			int currentLine = tokenNode.getStartLine();
			int currentCol = tokenNode.getStartCol();

			if (currentLine < minimumLine || minimumLine == -1)
			{
				minimumLine = currentLine;
			}
			
			if (currentCol < minimumCol || minimumCol == -1)
			{
				minimumCol = currentCol;
			}
			
			if (currentLine > lastLine)
			{
				if (tempPhysicalLine != null)
				{
					physicalLines.add(tempPhysicalLine);
				}
				
				tempPhysicalLine = new CodeLine();
				lastLine = currentLine;
			}

			tempPhysicalLine.add(tokenNode);
		}
		
		//add last line
		if (tempPhysicalLine != null)
		{
			physicalLines.add(tempPhysicalLine);
		}
		
		return physicalLines;
	}
	
	/**
	 * Performs all the pre-processing steps registered in the constructor on the snippet.
	 * 
	 * @param snippet A code fragment that is to be processed.
	 * @param lineWrapperOptions The options for the line wrapper used during the processing.
	 */
	public void process(TreeNode snippet, LineWrapperOptions lineWrapperOptions)
	{
		tbc = new TokenBindingContainer();
		caret = new Caret();		
		physicalLines = readPhysicalLines(snippet);
		
		caret.setStartLine(minimumLine);
		caret.setStartCol(minimumCol);
		
		Iterator<PreprocessingStep> iterator = preprocessingSteps.iterator();
		
		while(iterator.hasNext())
		{
			PreprocessingStep step = iterator.next();
			
			step.preprocess(lineWrapperOptions, physicalLines, tbc, caret);
		}
	}

	/**
	 * Retrieves the token binding container created during processing.
	 * 
	 * @return The token binding container with the pre-processed binding strength and alignment column information.
	 */
	public TokenBindingContainer getTbc()
	{
		return tbc;
	}

	/**
	 * Retrieves the physical lines created during processing.
	 * 
	 * @return The physical lines created during processing.
	 */
	public List<CodeLine> getPhysicalLines()
	{
		return physicalLines;
	}

	/**
	 * Retrieves the caret created during processing.
	 * 
	 * @return The caret created during processing.
	 */
	public Caret getCaret()
	{
		return caret;
	}
}
