package org.deft.operation.linebreak.wrapper.options;

import org.deft.operation.linebreak.debug.AbstractDebuggable;
import org.deft.operation.linebreak.wrapper.rules.alignment.AlignmentRule;
import org.deft.operation.linebreak.wrapper.rules.alignment.AlignmentRulesContainer;
import org.deft.operation.linebreak.wrapper.rules.wrapping.WrappingRule;
import org.deft.operation.linebreak.wrapper.rules.wrapping.WrappingRulesContainer;
import org.deft.repository.ast.TokenNode;

/**
 * Container class for all options associated with the LineWrapper. Use LineWrapperOptionsReader
 * to load a configuration file.
 * 
 * @author Christoph Seidl
 * 
 * @see wrapper.LineWrapper
 * @see LineWrapperOptionsReader
 */
public class LineWrapperOptions extends AbstractDebuggable
{	
	private boolean consumeExistingBlankLines;
	private int preserveBlankLinesCount;
			
	private WrappingRulesContainer wrappingRulesContainer;
	private AlignmentRulesContainer alignmentRulesContainer;
	
	/**
	 * Creates new <code>LineWrapperOptions</code>.
	 */
	public LineWrapperOptions()
	{
		//Assume defaults until told otherwise.
		consumeExistingBlankLines = LineWrapperOptionsDefaults.CONSUME_EXISTING_BLANK_LINES;
		preserveBlankLinesCount = LineWrapperOptionsDefaults.PRESERVE_BLANK_LINES_COUNT;
		
		wrappingRulesContainer = new WrappingRulesContainer();
		alignmentRulesContainer = new AlignmentRulesContainer();
	}
		
	/**
	 * Get whether blank lines should be filled up with wrapped lines.
	 * 
	 * @return Whether blank lines should be filled up with wrapped lines.
	 */
	public boolean getConsumeExistingBlankLines()
	{
		return consumeExistingBlankLines;
	}

	/**
	 * Set whether blank lines should be filled up with wrapped lines.
	 * 
	 * @param consumeExistingBlankLines Whether blank lines should be filled up with wrapped lines.
	 */
	public void setConsumeExistingBlankLines(boolean consumeExistingBlankLines)
	{
		this.consumeExistingBlankLines = consumeExistingBlankLines;
	}
	
	/**
	 * Get the number of blank lines that should be preserved even if in general they
	 * are filled up with wrapped lines.
	 *  
	 * @return The number of blank lines that should be preserved.
	 */
	public int getPreserveBlankLinesCount()
	{
		return preserveBlankLinesCount;
	}

	/**
	 * Set the number of blank lines that should be preserved even if in general they
	 * are filled up with wrapped lines.
	 *  
	 * @param preserveBlankLinesCount The number of blank lines that should be preserved.
	 */
	public void setPreserveBlankLinesCount(int preserveBlankLinesCount)
	{
		this.preserveBlankLinesCount = preserveBlankLinesCount;
	}
	
	/**
	 * Returns the default wrapping rule.
	 * 
	 * @return The default wrapping rule.
	 */
	public WrappingRule getDefaultWrappingRule()
	{
		return wrappingRulesContainer.getDefaultRule();
	}

	/**
	 * Sets the default wrapping rule.
	 * 
	 * @param defaultWrappingRule The default wrapping rule.
	 */
	public void setDefaultWrappingRule(WrappingRule defaultWrappingRule)
	{
		wrappingRulesContainer.setDefaultRule(defaultWrappingRule);
	}
	
	/**
	 * Adds a wrapping rule.
	 * 
	 * @param wrappingRule The wrapping rule to add.
	 */
	public void addWrappingRule(WrappingRule wrappingRule)
	{
		wrappingRulesContainer.addRule(wrappingRule);
	}
	
	/**
	 * Retrieves the wrapping rule that best matches the two specified token nodes.
	 * 
	 * @param firstTokenNode The first token node.
	 * @param secondTokenNode The second token node.
	 * 
	 * @return The wrapping rule that best matches the two specified token nodes.
	 */
	public WrappingRule findMatchingWrappingRule(TokenNode firstTokenNode, TokenNode secondTokenNode)
	{
		return wrappingRulesContainer.findMatchingWrappingRule(firstTokenNode, secondTokenNode);
	}
	
	/**
	 * Returns the default alignment rule.
	 * 
	 * @return The default alignment rule.
	 */
	public AlignmentRule getDefaultAlignmentRule()
	{
		return alignmentRulesContainer.getDefaultRule();
	}

	/**
	 * Sets the default alignment rule.
	 * 
	 * @param defaultAlignmentRule The default alignment rule.
	 */
	public void setDefaultAlignmentRule(AlignmentRule defaultAlignmentRule)
	{
		alignmentRulesContainer.setDefaultRule(defaultAlignmentRule);
	}
	
	/**
	 * Adds an alignment rule.
	 * 
	 * @param alignmentRule The alignment rule to add.
	 */
	public void addAlignmentRule(AlignmentRule alignmentRule)
	{
		alignmentRulesContainer.addRule(alignmentRule);
	}
	
	/**
	 * Retrieves the alignment rule that best matches the two specified token nodes.
	 * 
	 * @param firstTokenNode The first token node.
	 * @param secondTokenNode The second token node.
	 * 
	 * @return The alignment rule that best matches the two specified token nodes.
	 */
	public AlignmentRule findMatchingAlignmentRule(TokenNode firstTokenNode, TokenNode secondTokenNode)
	{
		return alignmentRulesContainer.findMatchingWrappingRule(firstTokenNode, secondTokenNode);
	}
}
