package org.deft.operation.linebreak.wrapper.options;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.deft.operation.linebreak.debug.AbstractDebuggable;
import org.deft.operation.linebreak.wrapper.exceptions.LineWrapperOptionsReaderException;
import org.deft.operation.linebreak.wrapper.rules.RuleIdentifier;
import org.deft.operation.linebreak.wrapper.rules.alignment.Alignment;
import org.deft.operation.linebreak.wrapper.rules.alignment.AlignmentRule;
import org.deft.operation.linebreak.wrapper.rules.wrapping.WrappingRule;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Reads XML based configuration files for the LineWrapper and creates a LineWrapperOptions object.
 * 
 * @author Christoph Seidl
 * 
 * @see LineWrapperOptions
 */
public class LineWrapperOptionsReader extends AbstractDebuggable
{	
	private LineWrapperOptions options;
	private Document doc;
		
	private Map<String, List<TokenIdentifier>> tokenGroups;
	
	/**
	 * Internal helper class to hold the details for one type of token.
	 * 
	 * @author Christoph Seidl
	 */
	private class TokenIdentifier
	{
		private String name;
		private String context;
		
		public TokenIdentifier(String name, String context)
		{
			this.name = name;
			this.context = context;
		}

		public String getName()
		{
			return name;
		}

		public String getContext()
		{
			return context;
		}
	}
	
	/**
	 * Creates a new <code>LineWrapperOptionsReader</code>.
	 */
	public LineWrapperOptionsReader()
	{
		options = null;
		doc = null;
		
		tokenGroups = new HashMap<String, List<TokenIdentifier>>();
	}
		
	/**
	 * Helper method to parse a boolean XML node with a default value.
	 * 
	 * @param booleanNode The node to parse.
	 * @param defaultValue The default value that is returned, if the node does not exist.
	 * 
	 * @return The value of the node or its default value.
	 */
	private static boolean parseBooleanNode(Node booleanNode, boolean defaultValue)
	{
		if (booleanNode == null)
		{
			return defaultValue;
		}
		
		String booleanValue = booleanNode.getNodeValue().toLowerCase();
		
		if (booleanValue.equals("true"))
		{
			return true;
		}
		
		if (booleanValue.equals("false"))
		{
			return false;
		}
		
		return defaultValue;
	}
	
	/**
	 * Helper method to parse an integer XML node with a default value.
	 * 
	 * @param intNode The node to parse.
	 * @param defaultValue The default value that is returned, if the node does not exist.
	 * 
	 * @return The value of the node or its default value.
	 */
	private static int parseIntNode(Node intNode, int defaultValue)
	{
		if (intNode == null)
		{
			return defaultValue;
		}
		
		String intValue = intNode.getNodeValue().toLowerCase();
		
		try
		{
			return Integer.parseInt(intValue);
		}
		catch(NumberFormatException e)
		{
			return defaultValue;
		}
	}
	
	/**
	 * Helper method to parse a string XML node with a default value.
	 * 
	 * @param stringNode The node to parse.
	 * @param defaultValue The default value that is returned, if the node does not exist.
	 * 
	 * @return The value of the node or its default value.
	 */
	private static String parseStringNode(Node stringNode, String defaultValue)
	{
		if (stringNode == null)
		{
			return defaultValue;
		}
		
		return stringNode.getNodeValue();
	}
	
	/**
	 * Reads the alignments of an alignment rule.
	 * 
	 * @param alignmentRuleNode The XML node containing the alignment rule.
	 * 
	 * @return A list of all detected alignments or <code>null</code> if no alignment was found.
	 */
	private List<Alignment> readAlignments(Node alignmentRuleNode)
	{
		List<Alignment> resultingAlignments = new LinkedList<Alignment>();
				
		if (alignmentRuleNode.hasChildNodes())
		{
			NodeList childNodes = alignmentRuleNode.getChildNodes();

			for (int i = 0; i < childNodes.getLength(); ++i)
			{
				Node childNode = childNodes.item(i);
				String childNodeName = childNode.getNodeName();
								
				if (childNodeName.equals("alignment"))
				{
					Alignment alignment = readAlignmentTag(childNode);
					
					if (alignment != null)
					{
						resultingAlignments.add(alignment);
					}
				}
			}
		}
		
		if (!resultingAlignments.isEmpty())
		{
			return resultingAlignments;
		}
		
		return null;
	}
	
	/**
	 * Read a single alignment node.
	 * 
	 * @param alignmentNode The node to be read.
	 * 
	 * @return The according alignment.
	 */
	private Alignment readAlignmentTag(Node alignmentNode)
	{		
		NamedNodeMap attributes = alignmentNode.getAttributes();
		
		String anchorName = parseStringNode(attributes.getNamedItem("anchor"), LineWrapperOptionsDefaults.ANCHOR_NAME);
		int anchor = Alignment.createAnchor(anchorName);
		
		if (anchor == Alignment.UNKNOWN_ANCHOR)
		{
			logWarning("Anchor \"" + anchorName + "\" could not be created.");
			return null;
		}
		
		int indentation = parseIntNode(attributes.getNamedItem("indentation"), LineWrapperOptionsDefaults.INDENTATION);	
		
		return new Alignment(anchor, indentation);
	}
	
	/**
	 * Parses and resolves either the first or the second token of a rule (wrapping/alignment).
	 * The first way to specify a token is by its name (firstToken/secondToken) and optionally its context (firstContext/secondContext).
	 * The second way is to use the name of a token group as substitute for a number of tokens (firstGroup/secondGroup).
	 * 
	 * @param ruleNode the node of the rule that is to be parsed,
	 * @param firstToken Whether the first or the second token of the rule should be resolved.
	 * 
	 * @return A list of <code>TokenIdentifier</code>s that reflect the resolved tokens.
	 */
	private List<TokenIdentifier> parseToken(Node ruleNode, boolean firstToken)
	{
		NamedNodeMap attributes = ruleNode.getAttributes();
		
		List<TokenIdentifier> resultingIdentifiers = null;
		
		final String tokenAttributeName = (firstToken ? "first" : "second") + "Token";
		final String contextAttributeName = (firstToken ? "first" : "second") + "Context";
		final String groupAttributeName = (firstToken ? "first" : "second") + "Group";
		
		String group = parseStringNode(attributes.getNamedItem(groupAttributeName), "");
		
		//Try to determine if a token group was specified.
		if (tokenGroups.containsKey(group))
		{
			resultingIdentifiers = tokenGroups.get(group);
		}
		
		//If no group was found or if it was illegal, use direct token specification as
		//fall-back.
		if (resultingIdentifiers == null)
		{
			String token = parseStringNode(attributes.getNamedItem(tokenAttributeName), LineWrapperOptionsDefaults.TOKEN);
			String context = parseStringNode(attributes.getNamedItem(contextAttributeName), LineWrapperOptionsDefaults.CONTEXT);
			
			TokenIdentifier ti = new TokenIdentifier(token, context);
			
			resultingIdentifiers = new LinkedList<TokenIdentifier>();
			resultingIdentifiers.add(ti);
		}
		
		return resultingIdentifiers;
	}
	
	/**
	 * Read and add a single wrapping rule.
	 * 
	 * @param wrappingRuleNode The XML node to read.
	 */
	private void parseWrappingRuleNode(Node wrappingRuleNode)
	{			
		NamedNodeMap attributes = wrappingRuleNode.getAttributes();
		WrappingRule defaultRule = options.getDefaultWrappingRule();
		int defaultBindingStrength = defaultRule != null ? defaultRule.getBindingStrength() : LineWrapperOptionsDefaults.BINDING_STRENGTH;
		int bindingStrength = parseIntNode(attributes.getNamedItem("bindingStrength"), defaultBindingStrength);
		
		List<TokenIdentifier> firstTokens = parseToken(wrappingRuleNode, true);
		List<TokenIdentifier> secondTokens = parseToken(wrappingRuleNode, false);
		
		//Create the cross product of the first and second tokens as rules.
		Iterator<TokenIdentifier> firstIterator = firstTokens.iterator();	
		
		while(firstIterator.hasNext())
		{
			TokenIdentifier currentFirstToken = firstIterator.next();
			
			Iterator<TokenIdentifier> secondIterator = secondTokens.iterator();
			
			while(secondIterator.hasNext())
			{
				TokenIdentifier currentSecondToken = secondIterator.next();
				
				RuleIdentifier ri = new RuleIdentifier(currentFirstToken.getName(), currentSecondToken.getName());
				
				options.addWrappingRule(new WrappingRule(ri, currentFirstToken.getContext(), currentSecondToken.getContext(), bindingStrength));
			}
		}
	}
	
	/**
	 * Read and add a single alignment rule.
	 * 
	 * @param alignmentRuleNode The XML node to read.
	 */
	private void parseAlignmentRuleNode(Node alignmentRuleNode)
	{
		List<Alignment> alignments = readAlignments(alignmentRuleNode);
		
		if (alignments == null)
		{
			logWarning("Empty alignment rule was ignored.");
			return;
		}
		
		List<TokenIdentifier> firstTokens = parseToken(alignmentRuleNode, true);
		List<TokenIdentifier> secondTokens = parseToken(alignmentRuleNode, false);
		
		//Create the cross product of the first and second tokens as rules.
		Iterator<TokenIdentifier> firstIterator = firstTokens.iterator();	
		
		while(firstIterator.hasNext())
		{
			TokenIdentifier currentFirstToken = firstIterator.next();
			
			Iterator<TokenIdentifier> secondIterator = secondTokens.iterator();
			
			while(secondIterator.hasNext())
			{
				TokenIdentifier currentSecondToken = secondIterator.next();
				
				RuleIdentifier ri = new RuleIdentifier(currentFirstToken.getName(), currentSecondToken.getName());				
				AlignmentRule ar = new AlignmentRule(ri, currentFirstToken.getContext(), currentSecondToken.getContext());
				
				ar.addAlignments(alignments);
				options.addAlignmentRule(ar);
			}
		}
	}
	
	/**
	 * Parses all token groups in the specified node.
	 * 
	 * @param tokenGroupsNode The node that is to be parsed.
	 */
	private void parseTokenGroupsNode(Node tokenGroupsNode)
	{
		if (tokenGroupsNode.hasChildNodes())
		{
			NodeList childNodes = tokenGroupsNode.getChildNodes();

			for (int i = 0; i < childNodes.getLength(); ++i)
			{
				Node childNode = childNodes.item(i);
				String childNodeName = childNode.getNodeName();
								
				if (childNodeName.equals("tokenGroup"))
				{
					NamedNodeMap attributes = childNode.getAttributes();
					
					String name = parseStringNode(attributes.getNamedItem("name"), "");
					
					//Blank name is not permissible for token groups.
					if (!name.equals(""))
					{
						List<TokenIdentifier> tokens = parseTokensOfTokenGroup(childNode);
						
						if (tokens != null)
						{
							tokenGroups.put(name, tokens);
						}
						else
						{
							logWarning("Empty token group \"" + name + "\" was ignored.");
						}
					}
					else
					{
						logWarning("Token group with missing/blank name was ignored.");
					}
				}
			}
		}
	}
	
	/**
	 * Parses all tokens of the specified token group node and returns them.
	 * 
	 * @param tokenGroupNode The node holding the token group that is to be parsed.
	 * 
	 * @return The list of <code>TokenIdentifier</code>s reflecting the tokens of the token group.
	 */
	private List<TokenIdentifier> parseTokensOfTokenGroup(Node tokenGroupNode)
	{
		List<TokenIdentifier> resultingIdentifiers = new LinkedList<TokenIdentifier>();
		
		if (tokenGroupNode.hasChildNodes())
		{
			NodeList childNodes = tokenGroupNode.getChildNodes();

			for (int i = 0; i < childNodes.getLength(); ++i)
			{
				Node childNode = childNodes.item(i);
				String childNodeName = childNode.getNodeName();
								
				if (childNodeName.equals("token"))
				{
					NamedNodeMap attributes = childNode.getAttributes();
					
					String name = parseStringNode(attributes.getNamedItem("name"), LineWrapperOptionsDefaults.TOKEN);					
					String context = parseStringNode(attributes.getNamedItem("context"), LineWrapperOptionsDefaults.CONTEXT);
					TokenIdentifier ti = new TokenIdentifier(name, context);
					
					resultingIdentifiers.add(ti);
				}
			}
		}
		
		if (!resultingIdentifiers.isEmpty())
		{
			return resultingIdentifiers;
		}
		
		return null;
	}
	
	/**
	 * Reads the tag determining how to deal with existing blank lines.
	 */
	private void readExistingBlankLinesTag()
	{
		NodeList nodeList = doc.getElementsByTagName("existingBlankLines");
		
		if (nodeList != null)
		{
			Node node = nodeList.item(0);
			NamedNodeMap attributes = node.getAttributes();
			
			boolean consumeExistingBlankLines = parseBooleanNode(attributes.getNamedItem("consume"), LineWrapperOptionsDefaults.CONSUME_EXISTING_BLANK_LINES);
			int preserveBlankLinesCount = parseIntNode(attributes.getNamedItem("preserveCount"), LineWrapperOptionsDefaults.PRESERVE_BLANK_LINES_COUNT);
			
			options.setConsumeExistingBlankLines(consumeExistingBlankLines);
			options.setPreserveBlankLinesCount(preserveBlankLinesCount);
		}
	}
		
	/**
	 * Reads the default wrapping rule.
	 */
	private void readDefaultWrappingRuleTag()
	{
		NodeList nodeList = doc.getElementsByTagName("defaultWrappingRule");
		
		if (nodeList != null)
		{
			Node node = nodeList.item(0);
			
			if (node != null)
			{			
				NamedNodeMap attributes = node.getAttributes();

				int bindingStrength = parseIntNode(attributes.getNamedItem("bindingStrength"), LineWrapperOptionsDefaults.BINDING_STRENGTH);
				
				RuleIdentifier wri = new RuleIdentifier(LineWrapperOptionsDefaults.TOKEN, LineWrapperOptionsDefaults.TOKEN);				
				WrappingRule wr = new WrappingRule(wri, LineWrapperOptionsDefaults.CONTEXT, LineWrapperOptionsDefaults.CONTEXT, bindingStrength);
				
				options.setDefaultWrappingRule(wr);
			}
		}
	}
		
	/**
	 * Reads the default alignment rule.
	 */
	private void readDefaultAlignmentRuleTag()
	{
		NodeList nodeList = doc.getElementsByTagName("defaultAlignmentRule");
		
		if (nodeList != null)
		{
			Node node = nodeList.item(0);
					
			if (node == null)
			{
				logError("Mandatory default alignment rule was omitted.");
				return;
			}
			
			RuleIdentifier ri = new RuleIdentifier(LineWrapperOptionsDefaults.TOKEN, LineWrapperOptionsDefaults.TOKEN);
			AlignmentRule ar = new AlignmentRule(ri, LineWrapperOptionsDefaults.CONTEXT, LineWrapperOptionsDefaults.CONTEXT);

			List<Alignment> alignments = readAlignments(node);
			
			if (alignments == null)
			{
				logError("Default alignment rule has to contain at least one alignment.");
				return;
			}
			
			ar.addAlignments(alignments);
			
			options.setDefaultAlignmentRule(ar);
		}
	}
	
	/**
	 * Reads the token groups.
	 */
	private void readTokenGroupsTag()
	{
		NodeList nodeList = doc.getElementsByTagName("tokenGroups");
		
		if (nodeList != null)
		{
			Node node = nodeList.item(0);
			
			if (node != null)
			{
				parseTokenGroupsNode(node);
			}
		}
	}
	
	/**
	 * Reads the wrapping rules.
	 */
	private void readWrappingRulesTag()
	{
		NodeList nodeList = doc.getElementsByTagName("wrappingRule");
				
		if (nodeList != null)
		{
			for (int i = 0; i < nodeList.getLength(); ++ i)
			{
				Node node = nodeList.item(i);

				parseWrappingRuleNode(node);
			}
		}
	}
	
	/**
	 * Reads the alignment rules.
	 */
	private void readAlignmentRulesTag()
	{
		NodeList nodeList = doc.getElementsByTagName("alignmentRule");
				
		if (nodeList != null)
		{
			for (int i = 0; i < nodeList.getLength(); ++ i)
			{
				Node node = nodeList.item(i);

				parseAlignmentRuleNode(node);
			}
		}
	}
	
	/**
	 * Reads the specified line wrapper options file.
	 * 
	 * @param pathAndFilename Path and filename of the line wrapper options file.
	 * 
	 * @return The read line wrapper options.
	 * 
	 * @throws LineWrapperOptionsReaderException Throws an exception if the options could not be read.
	 */
	public LineWrapperOptions read(String pathAndFilename) throws LineWrapperOptionsReaderException
	{
		return read(new File(pathAndFilename));
	}
	
	/**
	 * Reads the specified line wrapper options file.
	 * 
	 * @param file The line wrapper options file.
	 * 
	 * @return The read line wrapper options.
	 * 
	 * @throws LineWrapperOptionsReaderException Throws an exception if the options could not be read.
	 */
	public LineWrapperOptions read(File file) throws LineWrapperOptionsReaderException {
		FileInputStream fis;
		try {
			fis = new FileInputStream(file);
			LineWrapperOptions options = read(fis);
			return options;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	public LineWrapperOptions read(InputStream is) throws LineWrapperOptionsReaderException
	{		
		options = new LineWrapperOptions();
		tokenGroups.clear();
		
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
	        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
	        doc = docBuilder.parse(is);
	        
	        readExistingBlankLinesTag();
	        readDefaultWrappingRuleTag();
	        readDefaultAlignmentRuleTag();
	        
	        readTokenGroupsTag();
	        
	        readWrappingRulesTag();
	        readAlignmentRulesTag();
	        is.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new LineWrapperOptionsReaderException("Internal error.", e);
		}

		//Discard reference of reader so that garbage collector can
		//dispose of the created options object if it is no longer used
		//outside of the reader.
		LineWrapperOptions optionsToReturn = options;
        options = null;
        
		return optionsToReturn;
	}
}
