package org.deft.operation.linebreak.wrapper.preprocessing;

import java.util.Iterator;
import java.util.List;

import org.deft.operation.linebreak.debug.AbstractDebuggable;
import org.deft.operation.linebreak.wrapper.binding.TokenBindingContainer;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptions;
import org.deft.operation.linebreak.wrapper.util.Caret;
import org.deft.operation.linebreak.wrapper.util.CodeLine;

/**
 * Class representing a step in the pre-processing chain that determines
 * biding strength and alignment columns for all relevant token combinations.
 * 
 * @author Christoph Seidl
 * 
 * @see wrapper.core.SharedLineWrapperFunctionality
 */
public abstract class PreprocessingStep extends AbstractDebuggable
{
	/**
	 * Worker method to determine binding strength and alignment column for all relevant 
	 * token combinations.
	 * 
	 * @param lineWrapperOptions The line wrapper options to use.
	 * @param physicalLine The physical line to process.
	 * @param nextPhysicalLine The next physical line.
	 * @param tbc The token binding container holding the binding strength and alignment column for the relevant token combinations.
	 * @param caret The caret used to reduce indentation and lines when merely a partial fragment of a class is processed.
	 */
	protected abstract void doPreprocess(LineWrapperOptions lineWrapperOptions, CodeLine physicalLine, CodeLine nextPhysicalLine, TokenBindingContainer tbc, Caret caret);
	
	/**
	 * Iterates over all physical lines and determined binding strength and/or alignment column
	 * for the relevant token combinations.
	 * 
	 * @param lineWrapperOptions The line wrapper options to use.
	 * @param physicalLines The list of physical lines to process.
	 * @param tbc The token binding container holding the binding strength and alignment column for the relevant token combinations.
	 * @param caret The caret used to reduce indentation and lines when merely a partial fragment of a class is processed.
	 */
	public void preprocess(LineWrapperOptions lineWrapperOptions, List<CodeLine> physicalLines, TokenBindingContainer tbc, Caret caret)
	{
		CodeLine currentPhysicalLine = null;
		CodeLine nextPhysicalLine = null;
		
		Iterator<CodeLine> iterator = physicalLines.iterator();
		
		//The iterator is one element ahead of currentPhysicalLine in order to realize
		//a one line look-ahead mechanism.
		while(iterator.hasNext())
		{
			nextPhysicalLine = iterator.next();
						
			if (currentPhysicalLine != null)
			{
				doPreprocess(lineWrapperOptions, currentPhysicalLine, nextPhysicalLine, tbc, caret);

			}

			currentPhysicalLine = nextPhysicalLine;
		}
		
		//Do not forget the last line.
		doPreprocess(lineWrapperOptions, currentPhysicalLine, null, tbc, caret);
	}
}
