package org.deft.operation.linebreak.wrapper.preprocessing;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.deft.operation.linebreak.wrapper.binding.TokenBindingContainer;
import org.deft.operation.linebreak.wrapper.binding.TokenBindingKey;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptions;
import org.deft.operation.linebreak.wrapper.util.Caret;
import org.deft.operation.linebreak.wrapper.util.CodeLine;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;

/**
 * A pre-processing step that analyzes the structure of the AST and
 * derives a basic binding strength for the contained tokens from it.
 * 
 * @author Christoph Seidl
 */
public class StructuralInformationPreprocessingStep extends PreprocessingStep
{
	private static final int DEFAULT_VALUE = 100;
	private static final int DECREMENT = 10;
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void doPreprocess(LineWrapperOptions lineWrapperOptions, CodeLine physicalLine, CodeLine nextPhysicalLine, TokenBindingContainer tbc, Caret caret)
	{
		Iterator<TokenNode> iterator = physicalLine.iterator();
		
		List<TreeNode> firstAncestors = null;
		List<TreeNode> secondAncestors = null;
		
		TokenNode oldTokenNode = null;
		
		while(iterator.hasNext())
		{
			TokenNode currentTokenNode = iterator.next();
			secondAncestors = determineAncestors(currentTokenNode);
			
			//Not in the first turn
			if (firstAncestors != null)
			{
				int normalizedTotalDistance = getNormalizedTotalDistance(firstAncestors, secondAncestors);
				int bindingStrength = calculateBindingStrengthForNormalizedTotalDistance(normalizedTotalDistance);
				
				TokenBindingKey key = new TokenBindingKey(oldTokenNode, currentTokenNode);
								
				tbc.setBindingStrength(key, bindingStrength);
			}
			
			firstAncestors = secondAncestors;
			oldTokenNode = currentTokenNode;
		}
	}
	
	/**
	 * Converts a normalized total distance to an appropriate binding strength.
	 * 
	 * @param normalizedTotalDistance The normalized total distance of two tokens.
	 * 
	 * @return The equivalent binding strength of those two tokens.
	 */
	private int calculateBindingStrengthForNormalizedTotalDistance(int normalizedTotalDistance)
	{
		return Math.max(1, DEFAULT_VALUE - normalizedTotalDistance * DECREMENT);
	}
	
	/**
	 * Searches the most specific common ancestor in the two lists of ancestors and calculates
	 * the normalized total distance to it. The normalized total distance is the sum of the distances
	 * of each of the two tokens to the most specific common ancestor minus two (as each of the tokens
	 * has a minimum distance of 1 that should be disregarded).
	 * 
	 * @param firstAncestors The ancestors of the first token.
	 * @param secondAncestors The ancestors of the second token.
	 * 
	 * @return The normalized total distance of the two input tokens.
	 */
	private int getNormalizedTotalDistance(List<TreeNode> firstAncestors, List<TreeNode> secondAncestors)
	{
		final int firstNodeLevel = firstAncestors.size() + 1;
		final int secondNodeLevel = secondAncestors.size() + 1;
		
		final int startI = Math.max(firstNodeLevel - secondNodeLevel, 0);
		final int startJ = Math.max(secondNodeLevel - firstNodeLevel, 0);

		int currentLevel = firstNodeLevel - startI;
		
		Iterator<TreeNode> firstIterator = firstAncestors.listIterator(startI);
		Iterator<TreeNode> secondIterator = secondAncestors.listIterator(startJ);
		
		while(firstIterator.hasNext() && secondIterator.hasNext())
		{
			TreeNode firstAncestor = firstIterator.next();
			TreeNode secondAncestor = secondIterator.next();
			
			if (firstAncestor.equals(secondAncestor))
			{
				//level of most specific common ancestor was found
				int ntd = (firstNodeLevel - currentLevel - 1) + (secondNodeLevel - currentLevel - 1);
				
				return ntd;
			}
			
			--currentLevel;
		}
				
		assert(false);
		debug("Should not be here");
		
		return 0;
	}
	
	/**
	 * Determines all ancestors of the specified token node.
	 * 
	 * @param tn The token node which's ancestors are to be determined.
	 * 
	 * @return A list of all ancestors of the specified token node.
	 */
	private static List<TreeNode> determineAncestors(TokenNode tn)
	{
		List<TreeNode> ancestors = new LinkedList<TreeNode>();
		TreeNode parent = tn.getParent();
		
		while (parent != null)
		{
			ancestors.add(parent);
			
			parent = parent.getParent();
		}
		
		return ancestors;
	}
}
