package org.deft.operation.linebreak.wrapper.rules;

import org.deft.operation.linebreak.debug.AbstractDebuggable;

/**
 * The abstract base class for the rules used with the line wrapper.
 * In particular it provides standard support for the context a token node
 * can appear in. The context of a token node is a relative XPath expression
 * disambiguating special cases of token combinations from their more specific
 * counterparts. E.g., it might make a difference if a comma separates a list of
 * parameters or appears within a generic (Map<Integer, String>). For details on
 * how the context is assembled see the documentation for completeContexts().
 * 
 * @author Christoph Seidl
 * 
 * @see wrapper.rules.alignment.AlignmentRule
 * @see wrapper.rules.wrapping.WrappingRule
 */
public abstract class Rule extends AbstractDebuggable
{
	private RuleIdentifier ruleIdentifier;
	
	private String firstContext;
	private String secondContext;
	
	private int logicalLengthOfFirstContext;
	private int logicalLengthOfSecondContext;
	
	/**
	 * Creates a new <code>Rule</code>.
	 * 
	 * @param ruleIdentifier The combination of types of token nodes.
	 * @param firstContext The context of the first token node.
	 * @param secondContext The context of the second token node.
	 */
	public Rule(RuleIdentifier ruleIdentifier, String firstContext, String secondContext)
	{
		this.ruleIdentifier = ruleIdentifier;
		this.firstContext = firstContext;
		this.secondContext = secondContext;
		
		logicalLengthOfFirstContext = getLogicalLengthOfContext(firstContext);
		logicalLengthOfSecondContext = getLogicalLengthOfContext(secondContext);
	
		completeContexts();
	}
	
	/**
	 * Returns the rule identifier of this rule.
	 * 
	 * @return The rule identifier of this rule.
	 */
	public RuleIdentifier getRuleIdentifier()
	{
		return ruleIdentifier;
	}

	/**
	 * Returns the first context of this rule.
	 * 
	 * @return The fist context of this rule.
	 */
	public String getFirstContext()
	{
		return firstContext;
	}

	/**
	 * Returns the second context of this rule.
	 * 
	 * @return The second context of this rule.
	 */
	public String getSecondContext()
	{
		return secondContext;
	}

	/**
	 * Returns the logical length of the first context of this rule.
	 * 
	 * @return The logical length of the first context of this rule.
	 */
	public int getLogicalLengthOfFirstContext()
	{
		return logicalLengthOfFirstContext;
	}
	
	/**
	 * Returns the logical length of the second context of this rule.
	 * 
	 * @return The logical length of the second context of this rule.
	 */
	public int getLogicalLengthOfSecondContext()
	{
		return logicalLengthOfSecondContext;
	}
	
	/**
	 * Determines the logical length of a specified context. The logical
	 * length of an XPath expression is equal to the number of slashes
	 * contained in that expression.
	 * 
	 * @param context The context to analyze.
	 * 
	 * @return The logical length of the context.
	 */
	private int getLogicalLengthOfContext(String context)
	{
		int logicalLength = 0;
		
		for (int i = 0; i < context.length(); ++i)
		{
			if (context.charAt(i) == '/')
			{
				++logicalLength;
			}
		}
		
		return logicalLength;
	}
	
	/**
	 * Completes the relative contexts used as input to complete XPath expressions
	 * that can be used for queries on the AST. The relative input is an XPath expression
	 * that is formulated as if an unknown node would lead to the specified node. E.g.,
	 * context "SingleVariableDeclaration/SimpleType" with token type "Identifier" would mean
	 * an identifier that is part of a variable declaration. Internally the context is
	 * transformed to become a valid XPath expression by determining its logical length
	 * (counting the slashes) and then adding an equal number of "../" in front of it. In 
	 * addition to that another "../" is added in front of the expression because of the (implicit)
	 * slash between the expression so far and the token type. Furthermore, another "../" for 
	 * the token type itself is prepended and the type of the token is appended. In the above example
	 * this would lead to "../../../SingleVariableDeclaration/SimpleType/Identifier" as XPath expression. 
	 */
	private void completeContexts()
	{		
		if (!firstContext.equals(""))
		{
			for (int i = 0; i < logicalLengthOfFirstContext + 1; ++i)
			{
				firstContext = "../" + firstContext;
			}
			
			String tokenIdentifier = ruleIdentifier.getFirstTokenName();
			
			//Substitute the wild card of the context with its equivalent in XPath notation.
			if (tokenIdentifier.isEmpty())
			{
				tokenIdentifier = "*";
			}
			
			firstContext = "../" + firstContext + "/" + tokenIdentifier;
		}
		
		if (!secondContext.equals(""))
		{
			for (int i = 0; i < logicalLengthOfSecondContext + 1; ++i)
			{
				secondContext = "../" + secondContext;
			}
			
			String tokenIdentifier = ruleIdentifier.getSecondTokenName();
			
			//Substitute the wild card of the context with its equivalent in XPath notation.
			if (tokenIdentifier.isEmpty())
			{
				tokenIdentifier = "*";
			}
			
			secondContext = "../" + secondContext + "/" + tokenIdentifier;
		}
	}
}
