package org.deft.operation.linebreak.wrapper.rules;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.deft.operation.linebreak.debug.AbstractDebuggable;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;

/**
 * Abstract base class for all containers containing rules.
 * 
 * @author Christoph Seidl
 */
public abstract class RulesContainer<T extends Rule> extends AbstractDebuggable
{
	private T defaultRule;
	private Map<RuleIdentifier, List<T>> rules;
	
	/**
	 * Creates a new <code>RulesContainer</code>.
	 */
	public RulesContainer()
	{
		defaultRule = null;
		rules = new HashMap<RuleIdentifier, List<T>>();
	}
	
	/**
	 * Determines the index of the specified rule in the list of matching rules.
	 * 
	 * @param rule The rule to find the index for.
	 * @param listOfMatchingRules The list of matching rules containing the specified rule.
	 * 
	 * @return The index of the rule in the list of matching rules.
	 */
	private int findIndexForRule(T rule, List<T> listOfMatchingRules)
	{
		Iterator<T> iterator = listOfMatchingRules.iterator();
		int i = 0;
		
		int firstContextLength = rule.getLogicalLengthOfFirstContext();
		int secondContextLength = rule.getLogicalLengthOfSecondContext();
		
		while(iterator.hasNext())
		{
			T nextRule = iterator.next();
			
			int nextRuleFirstContextLength = nextRule.getLogicalLengthOfFirstContext();
			int nextRuleSecondContextLength = nextRule.getLogicalLengthOfSecondContext();
			
			boolean firstContextIsLonger = firstContextLength > nextRuleFirstContextLength;
			boolean firstContextIsEqualAndSecondContextIsLonger = firstContextLength == nextRuleFirstContextLength && secondContextLength > nextRuleSecondContextLength;
			
			if (firstContextIsLonger || firstContextIsEqualAndSecondContextIsLonger)
			{
				return i;
			}
				
			++i;
		}
		
		//Insert as last item if nothing else matched.
		return listOfMatchingRules.size();
	}

	/**
	 * Adds a rule to this container.
	 * 
	 * @param rule The rule to add.
	 */
	public void addRule(T rule)
	{
		if (rule == null)
		{
			return;
		}
		
		RuleIdentifier ri = rule.getRuleIdentifier();
		List<T> listOfMatchingRules = rules.get(ri);
				
		if (listOfMatchingRules == null)
		{
			//No prior list there. Add a new one with only one entry.
			listOfMatchingRules = new ArrayList<T>();
			listOfMatchingRules.add(rule);
			rules.put(ri, listOfMatchingRules);
		}
		else
		{
			//Ensure that the WrappingRule is at the proper position to be retrieved easily
			//when it is needed. The proper position means being sorted in descending order 
			//first by the logical length of firstContext, then by secondContext.			
			int index = findIndexForRule(rule, listOfMatchingRules);
			listOfMatchingRules.add(index, rule);
		}
	}
		
	/**
	 * Determines the rule with the matching context (if any) from the list of rules
	 * that matched the two tokens.
	 * 
	 * @param matchingRules The list of rules matching the two tokens.
	 * @param firstTokenNode The concrete instance of the first token node.
	 * @param secondTokenNode The concrete instance of the second token node.
	 * 
	 * @return The rule with matching context for the first and second token node if it exists. Otherwise <code>null</code>.
	 */
	protected T findRuleWithMatchingContext(List<T> matchingRules, TokenNode firstTokenNode, TokenNode secondTokenNode)
	{
		if (matchingRules == null)
		{
			return null;
		}
		
		Iterator<T> iterator = matchingRules.iterator();
		
		while(iterator.hasNext())
		{
			T rule = iterator.next();
			
			String firstContext = rule.getFirstContext();
			String secondContext = rule.getSecondContext();
			
			//check context
			if (contextMatches(firstContext, firstTokenNode) && contextMatches(secondContext, secondTokenNode))
			{
				return rule;
			}
		}
		
		return null;
	}
	
	/**
	 * Determines whether a concrete instance of a token node matches the
	 * specified context.
	 * 
	 * @param context The context to match.
	 * @param tokenNode The concrete instance of the token node to check.
	 * 
	 * @return <code>true</code> if the concrete instance of the token node matches the specified context. Otherwise <code>false</code>.
	 */
	protected boolean contextMatches(String context, TokenNode tokenNode)
	{
		if (context.equals(""))
		{
			return true;
		}
		
		List<TreeNode> treeNodes = tokenNode.executeXPathQuery(context);

		if (treeNodes != null && !treeNodes.isEmpty())
		{
			return true;
		}

		return false;
	}
	
	/**
	 * Determines the rule that best matches the two specified concrete instances of
	 * token nodes.
	 * 
	 * First the group of applicable rules is determined:
	 * 1. the rules that match both token types exactly
	 * 2. the rules that match the first token type via wildcard and the second token type exactly
	 * 3. the rules that match the first token type exactly and the second token type via wildcard
	 * 4. the rules that match both token types via wildcard
	 * 
	 * Once the first group with at least one applicable rule was selected, the concrete rule in that
	 * group is determined. In general, a rule is considered better the longer (in logical terms) its
	 * matching context is. If the maximum first context is of equal length, the second context decides.   
	 * 
	 * @param firstTokenNode The concrete instance of the first token node.
	 * @param secondTokenNode The concrete instance of the second token node.
	 * 
	 * @return The rule that best matched the specified token nodes.
	 */
	public T findMatchingWrappingRule(TokenNode firstTokenNode, TokenNode secondTokenNode)
	{
		T matchingRule = null;
				
		//NOTE: Token _name_ is the name of the containing token node.
		//The text of the token is its value (e.g., DOT VS. ".").
		String firstTokenName = firstTokenNode.getName();
		String secondTokenName = secondTokenNode.getName();
		
		//direct match
		matchingRule = doFindMatchingRule(firstTokenName, firstTokenNode, secondTokenName, secondTokenNode);
		
		if (matchingRule != null)
		{
			return matchingRule;
		}

		//first as wild-card, second exact
		matchingRule = doFindMatchingRule("", firstTokenNode, secondTokenName, secondTokenNode);
		
		if (matchingRule != null)
		{
			return matchingRule;
		}
		
		//first exact, second as wild-card
		matchingRule = doFindMatchingRule(firstTokenName, firstTokenNode, "", secondTokenNode);

		if (matchingRule != null)
		{
			return matchingRule;
		}
		
		//it is possible that there are rules that have two wild-cards but a context
		matchingRule = doFindMatchingRule("", firstTokenNode, "", secondTokenNode);

		if (matchingRule != null)
		{
			return matchingRule;
		}
		
		//default wrapping rule
		return defaultRule;
	}
	
	/**
	 * Worker method to determine the rule that best matches the two specified token nodes. The names
	 * of the token nodes may be wildcards (empty strings).
	 * 
	 * @param firstTokenName The name of the first token node. May be a wildcard (an empty string).
	 * @param firstTokenNode The concrete instance of the first token node.
	 * @param secondTokenName The name of the second token node. May be a wildcard (an empty string).
	 * @param secondTokenNode The concrete instance of the second token node.
	 * 
	 * @return The rule that best matches the two specified token nodes.
	 */
	private T doFindMatchingRule(String firstTokenName, TokenNode firstTokenNode, String secondTokenName, TokenNode secondTokenNode)
	{
		RuleIdentifier ri = new RuleIdentifier(firstTokenName, secondTokenName);
		List<T> matchingRules = rules.get(ri);
		
		return findRuleWithMatchingContext(matchingRules, firstTokenNode, secondTokenNode);
	}
		
	/**
	 * Returns the default rule of this container.
	 * 
	 * @return The default rule of this container.
	 */
	public T getDefaultRule()
	{		
		return defaultRule;
	}
	
	/**
	 * Sets the default rule of this container.
	 * 
	 * @param defaultRule The default rule to be set.
	 */
	public void setDefaultRule(T defaultRule)
	{
		this.defaultRule = defaultRule;
	}
}
