package org.deft.operation.linebreak.wrapper.rules.alignment;

import org.deft.operation.linebreak.wrapper.exceptions.AlignmentImpossibleException;
import org.deft.operation.linebreak.wrapper.util.Caret;

/**
 * Helper class to determine the column of a line of code after
 * alignment. Alignment takes a reference point and adds a certain
 * indentation to it to determine the alignment column.
 * 
 * @author Christoph Seidl
 */
public class Alignment
{
	/**
	 * Marker if the anchor is unknown. Used be the file reader
	 * if an unknown value is encountered.
	 */
	public static final int UNKNOWN_ANCHOR = -1;
	
	/**
	 * Anchors at the first column.
	 */
	public static final int FIRST_COLUMN_ANCHOR = 0;
	
	/**
	 * Anchors at the start column of the original (unwrapped) line.
	 */
	public static final int ORIGINAL_LINE_ANCHOR = 1;
	
	/**
	 * Anchors at the start column of the next line if it is indented.
	 * Otherwise throws an exception.
	 */
	public static final int NEXT_LINE_IF_INDENTED_ANCHOR = 2;
	
	private int anchor;
	private int indentation;
	
	/**
	 * Create a new <code>Alignment</code>.
	 * 
	 * @param anchor One of the anchor constants.
	 * @param indentation The number of blanks used for indentation.
	 */
	public Alignment(int anchor, int indentation)
	{		
		this.anchor = anchor;
		this.indentation = indentation;
	}
	
	/**
	 * Determines whether this alignment is applicable for the input
	 * values.
	 * 
	 * @param colOfOriginalLine The converted column of the original line.
	 * @param colOfNextLine The converted column of the next line.
	 * 
	 * @return <code>true</code> if this alignment is applicable.
	 */
	public boolean matches(int colOfOriginalLine, int colOfNextLine)
	{
		try
		{
			getColForAnchor(colOfOriginalLine, colOfNextLine);
		}
		catch(AlignmentImpossibleException e)
		{
			return false;
		}
		
		return true;
	}
	
	/**
	 * Determines the column after the anchor was calculated and the indentation
	 * was applied.
	 * 
	 * @param colOfOriginalLine The converted column of the original line.
	 * @param colOfNextLine The converted column of the next line.
	 * 
	 * @return The converted aligned column.
	 * 
	 * @throws AlignmentImpossibleException If the alignment is impossible (e.g., trying to align to the next line if it was indented, when it was not indented).
	 */
	public int getAlignedCol(int colOfOriginalLine, int colOfNextLine) throws AlignmentImpossibleException
	{
		int colForAnchor = getColForAnchor(colOfOriginalLine, colOfNextLine);

		return colForAnchor + indentation;
	}
	
	/**
	 * Determines the column of the anchor of this alignment.
	 * 
	 * @param colOfOriginalLine The converted column of the original line.
	 * @param colOfNextLine The converted column of the next line.
	 * 
	 * @return The converted column of the anchor.
	 * 
	 * @throws AlignmentImpossibleException If the alignment is impossible (e.g., trying to align to the next line if it was indented, when it was not indented).
	 */
	private int getColForAnchor(int colOfOriginalLine, int colOfNextLine) throws AlignmentImpossibleException
	{			
		switch (anchor)
		{
			case FIRST_COLUMN_ANCHOR:
				return Caret.FIRST_COL;

			case ORIGINAL_LINE_ANCHOR:
				return colOfOriginalLine;
			
			case NEXT_LINE_IF_INDENTED_ANCHOR:
				if (colOfNextLine > colOfOriginalLine)
				{
					return colOfNextLine;	
				}
		}
		
		throw new AlignmentImpossibleException();
	}
	
	/**
	 * Creates an anchor for its symbolic name.
	 * 
	 * @param name The symbolic name of the anchor to be created.
	 * 
	 * @return The newly created anchor.
	 */
	public static int createAnchor(String name)
	{
		if (name.equals("firstColumn"))
		{
			return FIRST_COLUMN_ANCHOR;
		}
		
		if (name.equals("originalLine"))
		{
			return ORIGINAL_LINE_ANCHOR;
		}
						
		if (name.equals("nextLineIfIndented"))
		{
			return NEXT_LINE_IF_INDENTED_ANCHOR;
		}
		
		//Anchor could not be created.
		return UNKNOWN_ANCHOR;
	}
}
