package org.deft.operation.linebreak.wrapper.rules.alignment;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.deft.operation.linebreak.wrapper.exceptions.AlignmentImpossibleException;
import org.deft.operation.linebreak.wrapper.rules.Rule;
import org.deft.operation.linebreak.wrapper.rules.RuleIdentifier;
import org.deft.operation.linebreak.wrapper.util.Caret;

/**
 * A class to determine a matching alignment for a specified
 * token combination. 
 * 
 * @author Christoph Seidl
 */
public class AlignmentRule extends Rule
{
	private List<Alignment> alignments;
	
	/**
	 * Creates a new <code>AlignmentRule</code>.
	 * 
	 * @param ruleIdentifier The identifier of this rule.
	 * @param firstContext The context the first token has to appear in. Pass an empty string as wildcard.
	 * @param secondContext The context the second token has to appear in. Pass an empty string as wildcard.
	 */
	public AlignmentRule(RuleIdentifier ruleIdentifier, String firstContext, String secondContext)
	{
		super(ruleIdentifier, firstContext, secondContext);
		
		alignments = new ArrayList<Alignment>();
	}
		
	/**
	 * Adds an alignment to this alignment rule. A valid alignment rule
	 * must have at least one alignment.
	 * 
	 * @param alignment The alignment to add.
	 */
	public void addAlignment(Alignment alignment)
	{
		alignments.add(alignment);
	}

	/**
	 * Adds all alignments to this alignment rule. A valid alignment rule
	 * must have at least one alignment.
	 * 
	 * @param alignments The alignments to add.
	 */
	public void addAlignments(List<Alignment> alignments)
	{
		this.alignments.addAll(alignments);
	}
	
	/**
	 * Determines the column to align to after a wrap.
	 * 
	 * @param colOfOriginalLine The converted column of the original line.
	 * @param colOfNextLine The converted column of the next line.
	 * 
	 * @return The column to align to after a wrap.
	 */
	public int determineColAfterWrap(int colOfOriginalLine, int colOfNextLine)
	{		
		Alignment alignment = findMatchingAlignment(colOfOriginalLine, colOfNextLine);
		
		if (alignment != null)
		{
			try
			{
				return alignment.getAlignedCol(colOfOriginalLine, colOfNextLine);
			}
			catch(AlignmentImpossibleException e)
			{
			}
		}
		else
		{
			logError("Alignment impossible!");
		}
		
		return Caret.FIRST_COL;
	}
	
	/**
	 * Finds the matching alignment for the situation described by the 
	 * parameters if there is any. 
	 * @param colOfOriginalLine The converted column of the original line.
	 * @param colOfNextLine The converted column of the next line.
	 * 
	 * @return Returns the matching <code>Alignment</code> if it exists, otherwise <code>null</code>.
	 */
	public Alignment findMatchingAlignment(int colOfOriginalLine, int colOfNextLine)
	{
		//Find the first matching alignment rule.
		Iterator<Alignment> iterator = alignments.iterator();
		
		while(iterator.hasNext())
		{
			Alignment alignment = iterator.next();

			if (alignment.matches(colOfOriginalLine, colOfNextLine))
			{
				return alignment;
			}
		}
		
		return null;
	}
}
