package org.deft.operation.linebreak.wrapper.strategies;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import org.deft.operation.linebreak.wrapper.util.CodeLine;
import org.deft.repository.ast.TokenNode;

/**
 * A strategy to perform an optimal line wrap. An optimal line wrap is defined as the wrap
 * that fits a line of code in the given character limit (if possible) and minimizes the 
 * total cost for wrapping. The total cost for wrapping is defined as the sum of the squares
 * of all binding strengths that are broken up.
 * 
 * @author Christoph Seidl
 */
public class DefaultLineWrappingStrategy extends LineWrappingStrategy
{	
	private List<Integer> bestSolution;
	private int costOfBestSolution;
	
	private int indexOfLastToken;
	private int lastColOfPhysicalLine;
	
	/**
	 * Creates a new <code>DefaultLineWrappingStrategy</code>.
	 */
	public DefaultLineWrappingStrategy()
	{
		bestSolution = new LinkedList<Integer>();
	}
	
	/**
	 * Initializes the strategy for the wrapping of one line.
	 * 
	 * @param physicalLine The physical line that is to be wrapped.
	 */
	private void initialize(CodeLine physicalLine)
	{
		bestSolution.clear();
		costOfBestSolution = -1;
		
		indexOfLastToken = physicalLine.numberOfTokenNodes() - 1;
		lastColOfPhysicalLine = physicalLine.get(indexOfLastToken).getEndCol() - 1;
	}
	
	/**
	 * Resets the best solution and its cost.
	 * 
	 * @param costSoFar The cost for the best solution.
	 */
	private void initializeBestSolution(int costSoFar)
	{
		costOfBestSolution = costSoFar;
		bestSolution.clear();
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void doWrapSinglePhysicalLine(CodeLine physicalLine)
	{				
		//Build the state tree of all possible formattings.
		initialize(physicalLine);
		
		//System.out.println("Wrapping \"" + physicalLine + "\"");
		
		findBestSolution(physicalLine, 0, 0, 0);
		
		wrapAccordingToBestSolution(physicalLine);
	}
			
	/**
	 * Determines the best possible solution for wrapping the specified physical line
	 * by building the tree of all possible solutions and then traversing it.
	 * This step checks whether the current state already is a solution or whether it
	 * can be disregarded as a better solution than this one can ever be has already been
	 * found.
	 * 
	 * @param physicalLine The physical line to be wrapped.
	 * @param startIndex The index of the first token on the potentially wrapped line.
	 * @param colOffset The offset that was applied if the physical line was wrapped at the positions determined on prior levels of this method.
	 * @param costSoFar The cost up to this point.
	 * 
	 * @return <code>true</code> if the run lead to the currently best solution.
	 */
	private boolean findBestSolution(CodeLine physicalLine, int startIndex, int colOffset, int costSoFar)
	{		
		//Check if a better solution already exists.
		if (costOfBestSolution != -1 && costSoFar >= costOfBestSolution)
		{
			return false;
		}
		
		//startIndex is first token on new line and cannot be wrapped
		//virtually align wrapped token
		if (startIndex > 0)
		{
			colOffset = getCaret().convertCol(physicalLine.get(startIndex).getStartCol()) - getAlignmentCol(startIndex);
		}
		
		final int maximumCol = getCharacterLimit() + colOffset;
		
		//Check if it already is a solution
		if (maximumCol >= lastColOfPhysicalLine)
		{
			//If this solution costs less than the previously best one it
			//now is the best one.
			if (costSoFar <= costOfBestSolution || costOfBestSolution == -1)
			{
				initializeBestSolution(costSoFar);
				
				return true;
			}
		}
		else
		{
			//If the last token is longer than the character limit, it still is a solution.
			if (startIndex == indexOfLastToken)
			{
				initializeBestSolution(costSoFar);
				
				return true;
			}
		}
		
		return tryToAssembleBestSolution(physicalLine, startIndex, maximumCol, colOffset, costSoFar);
	}
	
	/**
	 * Determines the best possible solution for wrapping the specified physical line
	 * by building the tree of all possible solutions and then traversing it.
	 * This step introduces a new potential wrap and determines all its resulting
	 * follow-up wraps.
	 * 
	 * @param physicalLine The physical line to be wrapped.
	 * @param startIndex The index of the first token on the potentially wrapped line.
	 * @param maximumCol The maximum permissible column.
	 * @param colOffset The offset that was applied if the physical line was wrapped at the positions determined on prior levels of this method.
	 * @param costSoFar The cost up to this point.
	 * 
	 * @return <code>true</code> if the run lead to the currently best solution.
	 */
	private boolean tryToAssembleBestSolution(CodeLine physicalLine, int startIndex, int maximumCol, int colOffset, int costSoFar)
	{
		int firstPotentialWrappingIndex = startIndex + 1;
		int lastPotentialWrappingIndex = findLastPotentialWrappingIndex(physicalLine, startIndex, maximumCol);
		int bestIndex = -1;
		
		//Iterate backwards
		for (int i = lastPotentialWrappingIndex; i >= firstPotentialWrappingIndex; --i)
		{			
			int bindingStrength = getBindingStrength(i);
			int newCost = costSoFar + (bindingStrength * bindingStrength);
			
			boolean partOfBestSolution = findBestSolution(physicalLine, i, colOffset, newCost);
			
			if (partOfBestSolution)
			{
				bestIndex = i;
			}
		}
		
		if (bestIndex != -1)
		{
			bestSolution.add(bestIndex);
			
			return true;
		}
		
		return false;
	}
	
	/**
	 * Determines the last potential index a line wrap can occur at after the start index.
	 * 
	 * @param physicalLine The physical line to be analyzed.
	 * @param startIndex The index of the first token to consider. The earliest a wrap can occur is startIndex + 1.
	 * @param maximumCol The maximum permissible column.
	 * 
	 * @return The index of the first token after the start index which's end column exceeds the maximum column.
	 */
	private int findLastPotentialWrappingIndex(CodeLine physicalLine, int startIndex, int maximumCol)
	{
		int i = startIndex + 1;
		Iterator<TokenNode> iterator = physicalLine.iterator(i);
	
		while(iterator.hasNext() && i <= indexOfLastToken)
		{			
			if (iterator.next().getEndCol() - 1 > maximumCol)
			{
				return i;
			}
			
			++i;
		}
		
		//Create solution even if there is a single token that exceeds the character limit.
		if (startIndex < indexOfLastToken && costOfBestSolution == -1)
		{
			//keep at least one token on the line
			return startIndex + 1;
		}
		
		return startIndex;
	}
	
	/**
	 * Takes the wrap indices determined in the best solution and wraps the physical line
	 * at these positions.
	 * 
	 * @param physicalLine the physical line to wrap.
	 */
	private void wrapAccordingToBestSolution(CodeLine physicalLine)
	{
		//Backwards iteration to make equivalent JavaScript implementation faster.
		ListIterator<Integer> iterator = bestSolution.listIterator(bestSolution.size());
		
		int startIndex = 0;
				
		while(iterator.hasPrevious())
		{
			int wrapIndex = iterator.previous();
			int endIndex = wrapIndex - 1;
			
			TokenNode wrappedNode = physicalLine.get(wrapIndex);
			int alignmentCol = getAlignmentCol(wrapIndex);
			
			physicalLine.placeTokenNodes(getCaret(), startIndex, endIndex);
			updateCaretForLineWrap(wrappedNode, alignmentCol);
			
			startIndex = wrapIndex;
		}
		
		physicalLine.placeTokenNodes(getCaret(), startIndex, indexOfLastToken);
	}
}
