package org.deft.operation.linebreak.wrapper.util;

import org.deft.repository.ast.TokenNode;

/**
 * Helper class that translates physical coordinates of tokens to virtual coordinates
 * after performing one or more line wraps.
 *  
 * @author Christoph Seidl
 */
public class Caret
{
	/**
	 * The number of the first input line.
	 */
	public static final int FIRST_LINE = 1;
	
	/**
	 * The number of the first input column.
	 */
	public static final int FIRST_COL = 1;
	
	
	private int startLine;
	private int startCol;
		
	private int lineOffset;
	private int colOffset;
	
	
	/**
	 * Creates a new <code>Caret</code>.
	 */
	public Caret()
	{
		this(0, 0);
	}
	
	//for test & copy
	/**
	 * Creates a new <code>Caret</code>.
	 * 
	 * This constructor is only for internal and test use.
	 */
	public Caret(int lineOffset, int colOffset)
	{
		this.lineOffset = lineOffset;
		this.colOffset = colOffset;
		
		startLine = FIRST_LINE;
		startCol = FIRST_COL;
	}

	/**
	 * Resets the caret to the <code>startLine</code> and <code>startCol</code>.
	 */
	public void reset()
	{
		lineOffset = 0;
		
		returnToFirstColumn();
	}
	
	/**
	 * Returns to the <code>startCol</code>. This is the equivalent to a carriage return.
	 */
	public void returnToFirstColumn()
	{
		colOffset = 0;
	}
	
	/**
	 * Equivalent to a line feed and indentation to the starting column of the first <code>TokenNode</code>
	 * on a newly wrapped line.
	 * 
	 * @param firstWrappedNode The first <code>TokenNode</code> on the newly wrapped line.
	 * @param colAfterWrap The column the <code>firstWrappedNode</code> is aligned to after the wrap.
	 */
	public void advanceForLineWrap(TokenNode firstWrappedNode, int colAfterWrap)
	{
		assert(firstWrappedNode != null);
		assert(colAfterWrap >= FIRST_COL);
		
		if (firstWrappedNode == null || colAfterWrap < FIRST_COL)
		{
			return;
		}
		
		//Advance caret to first column of next line and 
		//align it to the specified column.		
		--lineOffset;
		colOffset = firstWrappedNode.getStartCol() - colAfterWrap;		
	}
	
	/**
	 * Reduces the distance of virtual to physical line by the given number, effectively
	 * consuming (potentially) blank lines of the input code.
	 * 
	 * @param lines The number of lines that are to be consumed.
	 */
	public void consumeLines(int lines)
	{
		assert(lines >= 0);
		
		if (lines < 0)
		{
			return;
		}
		
		lineOffset += lines;
	}
	
	/**
	 * Converts a physical line to a virtual line that can be used to place a token in the output.
	 * 
	 * @param line The physical line that is to be converted.
	 * 
	 * @return The according virtual line.
	 */
	public int convertLine(int line)
	{
		assert(line >= FIRST_LINE);
		
		if (line < FIRST_LINE)
		{
			return FIRST_LINE;
		}
				
		return line - startLine + FIRST_LINE - lineOffset;
	}
	
	/**
	 * Converts a physical column to a virtual column that can be used to place a token in the output.
	 * 
	 * @param col The physical column that is to be converted.
	 * 
	 * @return The according virtual column.
	 */
	public int convertCol(int col)
	{
		assert(col >= FIRST_COL);
		
		if (col < FIRST_COL)
		{
			return FIRST_COL;
		}
		
		return col - startCol + FIRST_COL - colOffset; 
	}
	
	/**
	 * Sets the virtual start line. This is used to reduce blank lines in the output when merely a fragment
	 * of code is used as input. For example, a piece of code might start on line 5 in the input but will
	 * start on line 1 in the output. In this case the <code>startLine</code> is set to 5.
	 * 
	 * @param startLine The line the virtual output should start with.
	 */
	public void setStartLine(int startLine)
	{
		assert(startLine >= FIRST_LINE);
		
		if (startLine < FIRST_LINE)
		{
			return;
		}
		
		this.startLine = startLine;
	}

	/**
	 * Sets the virtual start column. This is used to reduce indentation in the output when merely a fragment
	 * of code is used as input. For example, a piece of code might start on column 8 in the input but will
	 * start on column 1 in the output. In this case the <code>startCol</code> is set to 8.
	 * 
	 * @param startCol The column the virtual output should start with.
	 */
	public void setStartCol(int startCol)
	{
		assert(startCol >= FIRST_COL);
		
		if (startCol < FIRST_COL)
		{
			return;
		}
		
		this.startCol = startCol;
	}
	
	/**
	 * Returns the <code>startLine</code>.
	 * 
	 * @return The line the virtual output should start with.
	 */
	public int getStartLine()
	{
		return startLine;
	}

	/**
	 * Returns the <code>startCol</code>.
	 * 
	 * @return The column the virtual output should start with.
	 */
	public int getStartCol()
	{
		return startCol;
	}
	
	/**
	 * Creates an exact clone of this caret by deep copying the original.
	 * 
	 * @return An exact clone of this caret.
	 */
	public Caret copy()
	{
		Caret caret = new Caret(lineOffset, colOffset);
		
		caret.setStartLine(startLine);
		caret.setStartCol(startCol);
		
		return caret;
	}
	
	//for test
	public int getLineOffset()
	{
		return lineOffset;
	}
	
	public int getColOffset()
	{
		return colOffset;
	}
}
