package org.deft.operation.linebreak.wrapper.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.deft.operation.linebreak.debug.AbstractDebuggable;
import org.deft.operation.linebreak.exporter.text.PlainTextExporter;
import org.deft.repository.ast.Token;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.annotation.linewrapping.LineWrappingInformation;

/**
 * Helper class that represents a series of <code>TokenNode</code>s as a line of code. All <code>TokenNode</code>s
 * of a code line have to share an equal value for the <code>startLine</code>. With the exception
 * of the last <code>TokenNode</code>, all <code>TokenNode</code>s have to share the value for <code>endLine</code>
 * as well. Note that the value of the <code>endLine</code> of the last <code>TokenNode</code> may be greater when
 * it contains a multi-line <code>Token</code> (such as multi-line comments). These constraints have to be enforced
 * by the user of this class and are not checked for the sake of efficiency.
 * 
 * @author Christoph Seidl
 */
public class CodeLine extends AbstractDebuggable
{
	private List<TokenNode> tokens;
	
	/**
	 * Creates a new line of code.
	 */
	public CodeLine()
	{
		tokens = new ArrayList<TokenNode>();
	}
	
	/**
	 * Places part of this line of code using the virtual position provided by <code>caret</code>. Placing a 
	 * <code>TokenNode</code> means appending a <code>LineWrappingInformation</code> object to it that contains
	 * the virtual coordinates of the <code>Token</code> after the line wrapping was performed.
	 * 
	 * Note that the element with <code>firstIndex</code> as well as the element with <code>lastIndex</code>
	 * are included in the range and thus are affected by the procedure.
	 * 
	 * Also note that old <code>LineWrappingInformation</code> objects that might have been appended to some
	 * of the <code>TokenNode</code>s in question are replaced. Thus, <code>LineWrappingInformation</code> is
	 * to be considered volatile and is to be used (e.g., by an exporter) before placing an affected 
	 * <code>TokenNode</code> again in another context.
	 * 
	 * @param caret A caret at the appropriate position to convert physical positions to appropriate virtual positions. 
	 * @param firstIndex Index of the first <code>TokenNode</code> in this line that is to be placed.
	 * @param lastIndex Index of the last <code>TokenNode</code> in this line that is to be placed.
	 */
	public void placeTokenNodes(final Caret caret, int firstIndex, int lastIndex)
	{
		Iterator<TokenNode> iterator = tokens.listIterator(firstIndex);
		int i = firstIndex;
		
		while(iterator.hasNext() && i <= lastIndex)
		{
			TokenNode tokenNode = iterator.next();
			Token token = tokenNode.getToken();
			
			int actualLine = caret.convertLine(token.getLine());
			int actualCol = caret.convertCol(token.getCol());
			int actualEndLine = caret.convertLine(token.getEndLine());
			int actualEndCol = caret.convertCol(token.getEndCol());

			//add LineWrappingInformation to node for further processing
			LineWrappingInformation lwi = new LineWrappingInformation(actualLine, actualCol, actualEndLine, actualEndCol);

			tokenNode.addInformation(lwi);
			++i;
		}
	}	
	
	//Wrapped List interface below.
	/**
	 * Returns an iterator over the <code>TokenNode</code>s in this <code>CodeLine</code>
	 * in proper sequence.
	 */
	public Iterator<TokenNode> iterator()
	{
		return tokens.iterator();
	}
	
	/**
	 * Returns an iterator over the <code>TokenNode</code>s in this <code>CodeLine</code>
	 * in proper sequence having the element with the specified index as return value of an
	 * initial call to <code>next()</code> if it exists.
	 */
	public ListIterator<TokenNode> iterator(int index)
	{
		return tokens.listIterator(index);
	}
	
	/**
	 * Appends a <code>TokenNode</code> to the end of this line of code.
	 * 
	 * @param tokenNode
	 */
	public void add(TokenNode tokenNode)
	{
		tokens.add(tokenNode);
	}
	
	/**
	 * Returns the <code>TokenNode</code> at the specified index if it exists.
	 * 
	 * @param index The index of the <code>TokenNode</code> to return.
	 * @return The <code>TokenNode</code> at the specified index if it exists.
	 */
	public TokenNode get(int index)
	{
		return tokens.get(index);
	}
	
	
	/**
	 * Returns the number of <code>TokenNode</code>s on this line of code.
	 * 
	 * @return The number of <code>TokenNode</code>s on this line of code.
	 */
	public int numberOfTokenNodes()
	{
		return tokens.size();
	}
		
	
	//TODO: remove DEBUG
	private static PlainTextExporter exporter = new PlainTextExporter();
	
	public String toString()
	{
		return exporter.exportCodeLine(this);
	}
}
