package org.deft.operation.replacetabs;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.text.TextRepresentation;

/**
 * Operation that replaces tabs with a number of spaces. The number of spaces which
 * are used to replace a tab are specified by the ReplaceTabsConfiguration.
 */
public class ReplaceTabsOperation implements Operation {

	/**
	 * Replaces all tabs in a string. The string is inside the ArtifactRepresentation.
	 */
	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		String text = checkAndUnwrap(representation);
		int tabWidth = getTabWidth(config);
		String newText = replaceTabs(text, tabWidth);
		TextRepresentation tr = new TextRepresentation(newText);
		return tr;
	}
	
	private String checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(TextRepresentation.TYPE)) {
			TextRepresentation tr = (TextRepresentation)rep;
			return tr.getText();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	/**
	 * Gets the number of spaces, which should replace a tab,
	 * from the configuration file.
	 */
	private int getTabWidth(OperationConfiguration config) {
		ReplaceTabsConfiguration rtConfig = cast(config);
		int tabWidth = rtConfig.getTabWidth();
		return tabWidth;
	}
	
	/**
	 * Casts the OperationConfiguration to ReplaceTabsConfiguration or throws
	 * an exception if that is not possible. 
	 */
	private ReplaceTabsConfiguration cast(OperationConfiguration config) {
		if (config instanceof ReplaceTabsConfiguration) {
			ReplaceTabsConfiguration afConfig = (ReplaceTabsConfiguration)config;
			return afConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	/**
	 * Performs the actual tab replacement.
	 */
	private String replaceTabs(String text, int tabWidth) {
		String spaces = createTabReplacementString(tabWidth);
		String newText = text.replaceAll("\t", spaces);
		return newText;
	}
	
	/**
	 * Creates the replacement string
	 */
	private String createTabReplacementString(int tabWidth) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < tabWidth; i++) {
			sb.append(" ");
		}
		return sb.toString();
	}
	

	/**
	 * Creates the initial configuration object. 
	 */
	@Override
	public ReplaceTabsConfiguration createConfiguration() {
		return new ReplaceTabsConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return TextRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return TextRepresentation.TYPE;
	}

}
