package org.deft.operationchain.ecoreuml2attributeslisting;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.text.StyleContext;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.operationchain.ecoreuml2attributeslisting.style.LocalStyle;
import org.deft.operationchain.ecoreuml2attributeslisting.textcompute.AttributesExplanationBuilder;
import org.deft.operationchain.ecoreuml2attributeslisting.textcompute.AttributesTermBuilder;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.representation.ecore.EcoreRepresentation;
import org.deft.representation.styleddocument.StyledDefinitionListingRepresentation;
import org.deft.representation.styleddocument.data.DefinitionListing;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

public class EcoreUml2AttributesListingOperation implements Operation {

	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		EObject modelRoot = checkAndUnwrap(representation);
		EcoreUml2AttributesListingConfiguration listingConfig = checkAndUnwrap(config);
		String className = listingConfig.getClassName();
		EClass eClassToDocument = getEClass(modelRoot, className);

		DefinitionListing listing = new DefinitionListing("No additional attributes");
		createStyledListingElements(eClassToDocument, listing);
		StyledDefinitionListingRepresentation rep = new StyledDefinitionListingRepresentation(listing);
		return rep;
	}

	
	
	private void createStyledListingElements(EClass eClass, DefinitionListing listing) {
		StyleContext context = new LocalStyle().getStyleContext();

		List<EAttribute> attributes = new LinkedList<EAttribute>(eClass.getEAttributes());
		Collections.sort(attributes, new NameComparator());
		AttributesTermBuilder tb = new AttributesTermBuilder();
		AttributesExplanationBuilder eb = new AttributesExplanationBuilder();
		for (EAttribute attribute : attributes) {
			TokenizedStyledDocument termDoc = new TokenizedStyledDocument(context);
			tb.addAttributesTerm(attribute, termDoc);

			TokenizedStyledDocument explDoc = new TokenizedStyledDocument(context);
			eb.addAttributesExplanation(attribute, explDoc);
			listing.addDefinition(termDoc, explDoc);
		}
	}
	
	private class NameComparator implements Comparator<EAttribute> {

		@Override
		public int compare(EAttribute a1, EAttribute a2) {
			String name1 = a1.getName();
			String name2 = a2.getName();
			return name1.compareTo(name2);
		}
		
	}
	
	private EClass getEClass(EObject modelRoot, String className) {
		for (EObject eo : modelRoot.eContents()) {
			if (eo instanceof EClass) {
				EClass eClass = (EClass)eo;
				String name = eClass.getName();
				if (name.equals(className)) {
					return eClass;
				}
			}
		}
		return null;
	}
	
	
	

	
	
	private EObject checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreInstanceRepresentation.TYPE)) {
			EcoreInstanceRepresentation eir = (EcoreInstanceRepresentation)rep;
			return eir.getModelRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private EcoreUml2AttributesListingConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof EcoreUml2AttributesListingConfiguration) {
			EcoreUml2AttributesListingConfiguration u2aConfig = (EcoreUml2AttributesListingConfiguration)config;
			return u2aConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}


	@Override
	public OperationConfiguration createConfiguration() {
		return new EcoreUml2AttributesListingConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return StyledDefinitionListingRepresentation.TYPE;
	}
}
