package org.deft.projecttype.openoffice;

import org.deft.RepositoryFacade;
import org.deft.integrator.AbstractEditorChapterDocument;
import org.deft.integrator.ChapterDocRepositoryListener;
import org.deft.integrator.EditorIntegrator;
import org.deft.integrator.ErrorMessageIntegrator;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.integrator.ArtifactReferencePosition;

import ag.ion.bion.officelayer.document.IDocument;
import ag.ion.bion.officelayer.text.ITextCursor;
import ag.ion.bion.officelayer.text.ITextDocument;
import ag.ion.bion.officelayer.text.ITextRange;
import ag.ion.bion.officelayer.text.IViewCursor;

/**
 * Represents an ODFText document in an open OpenOffice editor. Modifications
 * performed on the document are realized via the OpenOffice API. 
 */
public class OdfTextChapterDocument extends AbstractEditorChapterDocument {

	private IDocument document;
	private boolean isIntegrated;
	private ChapterDocRepositoryListener listener = new ChapterDocRepositoryListener(this);
	private ErrorMessageIntegrator uncompIntegrator = new ErrorMessageIntoODFTextIntegrator();
	
	private RepositoryFacade repository = RepositoryFacade.instance();

	public OdfTextChapterDocument(Chapter chapter, IDocument document) {
		super(chapter);
		this.document = document;
	}
	
	public ITextDocument getDocument() {
		return (ITextDocument)document;
	}

	
	public boolean areArtifactsIntegrated() {
		return isIntegrated;
	}




	
	/**
	 * Returns the current position of the cursor.
	 */
	@Override
	public ArtifactReferencePosition getCursorPosition() {
		IViewCursor cursor = getDocument().getViewCursorService().getViewCursor();
		try {
			ITextRange cursorPosition = cursor.getStartTextRange();
			OdfTextArtifactReferencePosition pos = new OdfTextArtifactReferencePosition(this, cursorPosition);
			return pos;
		}
		catch (com.sun.star.uno.RuntimeException re) {
			ITextRange cursorPosition = tryAlternativeWayToGetCursorPosition();
			OdfTextArtifactReferencePosition pos = new OdfTextArtifactReferencePosition(this, cursorPosition);
			return pos;
		}
	}
	

	private ITextRange tryAlternativeWayToGetCursorPosition() {
		ITextCursor textCursor = getDocument().getTextService().getCursorService().getTextCursor();
		ITextRange cursorPosition = textCursor.getStart();
		return cursorPosition;
	}
	
	
	@Override
	public void scrollToRepresentation(ArtifactReference reference) {
		OdfTextArtifactReferencePosition position = getPosition(reference.getId());
		if (position != null) {
			scrollToTextRange(position.getTextRange());
		}
	}
	
	
	private OdfTextArtifactReferencePosition getPosition(String referenceId) {
		ArtifactReference reference = repository.getArtifactReference(referenceId);
		String integratorId = reference.getIntegratorId();
		EditorIntegrator integrator = repository.getExtensionService().getEditorIntegrator(integratorId);
		if (integrator != null) {
			ArtifactReferencePosition position = integrator.getPositionForRepresentation(this, referenceId);
			return (OdfTextArtifactReferencePosition)position;
		}
		return null;
	}
	
	
	private void scrollToTextRange(ITextRange textRange) {
		if (textRange != null) {
			ITextDocument textDocument = getDocument();
			textDocument.getViewCursorService().getViewCursor().goToRange(textRange, false);
		}
	}
	

	@Override
	public void addChapterDocRepositoryListener() {
		repository.addRepositoryUpdateListener(listener);
	}
	
	@Override
	public void removeChapterDocRepositoryListener() {
		repository.removeRepositoryUpdateListener(listener);
	}

	@Override
	public ErrorMessageIntegrator getErrorMessageIntegrator() {
		return uncompIntegrator;
	}

}
