package org.deft.projecttype.openoffice;

import org.deft.InsertIdentifierHandler;
import org.deft.chaptereditor.ChapterEditorInput;
import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.workspaceresources.WorkspaceManager;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Display;

import ag.ion.bion.officelayer.document.IDocument;
import ag.ion.bion.workbench.office.editor.ui.editors.OfficeEditor;

import com.sun.star.awt.KeyEvent;
import com.sun.star.awt.KeyModifier;
import com.sun.star.awt.XExtendedToolkit;
import com.sun.star.awt.XKeyHandler;
import com.sun.star.awt.XToolkit;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.lang.EventObject;
import com.sun.star.uno.UnoRuntime;

/**
 * The representation of the Eclipse editor which contains the OpenOffice instance.
 * 
 * This class mainly overrides the save method of the OpenOffice editor provided
 * by the NOA framework. The new save method removes stale references from the repository
 * (i.e. it removes the references which should be in the opened file according to the repository,
 * but are in fact not there anymore). Then the file is saved.
 * 
 * Furthermore the key combination Alt+I is defined here in order to open the Insert Identifier
 * operation chain gui. This is a hack until I know how to make Eclipse receive the key event
 * and react on the specified key binding (in plugin.xml)
 */
public class OpenOfficeChapterEditorPart extends OfficeEditor implements ChapterEditorPart {
	
	public static final String ID = "org.deft.chaptereditor.openoffice"; //$NON-NLS-1$
	private static InsertKeyHandler handler = null;
	
	private WorkspaceManager workspaceManager = WorkspaceManager.instance();
	private OdfTextChapterDocument chapterDocument;

	@Override
	public void doSave(IProgressMonitor monitor) {
		getChapterDocument().removeNonexistingReferencesFromRepository();
		performActualSave(monitor);
	}

	@Override
	public OdfTextChapterDocument getChapterDocument() {
		if (chapterDocumentDoesNotYetExist()) {
			instantiateChapterDocument();
		}
		if (handler == null) {
			addKeyHandler(getDocument());
		}
		return chapterDocument;
	}
	
	private boolean chapterDocumentDoesNotYetExist() {
		return chapterDocument == null;
	}
	
	private void instantiateChapterDocument() {
		Chapter chapter = getChapter();
		IDocument document = getDocument();
		chapterDocument = new OdfTextChapterDocument(chapter, document);
	}
	
	private void performActualSave(IProgressMonitor monitor) {
		super.doSave(monitor);
		Chapter chapter = getChapter();
		workspaceManager.saveWorkspaceCopyToRepository(chapter);
	}
	
	public Chapter getChapter() {
		ChapterEditorInput input = (ChapterEditorInput)getEditorInput();
		Chapter chapter = input.getChapter();
		return chapter;
	}
	
	private void addKeyHandler(IDocument document) {
		XWindowPeer myWindowPeer = (XWindowPeer)UnoRuntime.queryInterface(XWindowPeer.class, document.getFrame().getXFrame().getContainerWindow());
		XToolkit myToolkit = myWindowPeer.getToolkit();
		XExtendedToolkit myExtToolkit = (XExtendedToolkit)
		UnoRuntime.queryInterface (XExtendedToolkit.class, myToolkit);
		handler = new InsertKeyHandler();
		myExtToolkit.addKeyHandler(handler);
	}
	
	private class InsertKeyHandler implements XKeyHandler {
		public boolean keyPressed(KeyEvent k) {
			if(k.Modifiers == KeyModifier.MOD2 && k.KeyChar == 'i') {
				Display.getDefault().asyncExec(new Runnable() {

					@Override
					public void run() {
						try {
							new InsertIdentifierHandler().execute(null);
						} catch (ExecutionException e) {
							e.printStackTrace();
						}
					}
					
				});
				return true;
			}
			return false;
		}

		public boolean keyReleased(KeyEvent arg0) {
			return false;
		}
		
		public void disposing(EventObject arg0) {
		}
	}
	
}
