package org.deft.repository.artifacttype;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseFileExtensionToArtifactTypeMappingRegistry implements FileExtensionToArtifactTypeMappingRegistry {
	
	private static final String MAPPING_EXTENSION_POINT_NAME = "org.deft.repository.artifacttype.fileextensionmapping";
		
	private Map<String, FileExtensionMappingConfig> registeredMappings = new HashMap<String, FileExtensionMappingConfig>(); 

	public EclipseFileExtensionToArtifactTypeMappingRegistry() {
		getAndCacheRegisteredEditors();
	}
	
	private void getAndCacheRegisteredEditors() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(MAPPING_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			FileExtensionMappingConfig config = new FileExtensionMappingConfig(ce);
			registeredMappings.put(config.getArtifactTypeId(), config);
		}
	}
	
	@Override
	public List<String> getFileExtensions(String artifactTypeId) {
		List<String> fileExtensions = new LinkedList<String>();
		FileExtensionMappingConfig config = registeredMappings.get(artifactTypeId);
		if (config != null) {
			fileExtensions.addAll(config.getFileExtensions());
		}
		return fileExtensions;
	}
	
	

	@Override
	public List<String> getArtifactTypeIds(String fileExtension) {
		List<String> artifactTypeIds = new LinkedList<String>();
		for (FileExtensionMappingConfig config : registeredMappings.values()) {
			List<String> fileExtensions = config.getFileExtensions();
			if (fileExtensions.contains(fileExtension)) {
				artifactTypeIds.add(config.getArtifactTypeId());
			}
		}
		return artifactTypeIds;
	}



	
	
	/**
	 * This class offers convenient access to FileExtensionMappings by
	 * encapsulating the Eclipse internal extension mechanism. There is
	 * one FileExtensionMappingConfig for each concrete fileextensionmapping.
	 * 
	 * The config contains the id of the artifact type, and a list of file
	 * extensions of files that represent the respective artifact type.
	 */
	private class FileExtensionMappingConfig {
	
		private IConfigurationElement ce;
		
		public FileExtensionMappingConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		/**
		 * Returns the id of the artifact type
		 */
		public String getArtifactTypeId() {
			String id = ce.getAttribute("artifacttype_id");
			return id;
		}
		
		/**
		 * Returns the list of file extensions for the artifact type
		 */
		public List<String> getFileExtensions() {
			List<String> fileExtensions = new LinkedList<String>();
			IConfigurationElement[] artifactTypeFileExtensionConfigElements = ce.getChildren();
			for (IConfigurationElement fileExtensionCe : artifactTypeFileExtensionConfigElements) {
				String fileExtension = fileExtensionCe.getAttribute("extensionname");
				fileExtensions.add(fileExtension);
			}
			return fileExtensions;
		}

	}



}