package org.deft.repository.artifacttype;

import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.deft.repository.fragmentsource.OriginalFragmentLocation;

public class MultiArtifactDependencies {
	
	private Node artifactNode;
	
	public MultiArtifactDependencies(OriginalFragmentLocation mainLocation) {
		this.artifactNode = new Node();
		this.artifactNode.location = mainLocation.getFullLocation();
		this.artifactNode.id = mainLocation.getName();
	}
	
	
	public List<String> getChildIdsFromRoot() {
		List<String> childIds = new LinkedList<String>();
		for (Node child : artifactNode.children) {
			childIds.add(child.id);
		}
		return childIds;
	}
	
	public List<String> getDirectChildIds(String id) {
		List<String> childIds = new LinkedList<String>();
		if (doesIdExist(id)) {
			Node node = getById(id);
			for (Node child : node.children) {
				childIds.add(child.id);
			}
		}
		return childIds;
	}
	
	
	public void addDependentLocations(OriginalFragmentLocation location, 
			List<OriginalFragmentLocation> dependentLocations) {
		Node node = getByLocation(location.getFullLocation());
		if (node != null) {
			for (OriginalFragmentLocation depLoc : dependentLocations) {
				String depLocString = depLoc.getFullLocation();
				Node n = null;
				if (locationExists(depLocString)) {
					n = getByLocation(depLocString);
				} else {
					n = new Node();
					n.location = depLocString;
					String newId = getUnusedId(depLoc.getName());
					n.id = newId;
				}
				//check if n has already been added to the node, this can happen
				//under special circumstances with direct and indirect dependencies
				if (!node.children.contains(n)) {
					node.children.add(n);	
				}
			}
		} else {
			throw new IllegalArgumentException("Could not build dependencies because file at " + location.getFullLocation() + " is unknown.");
		}
	}
	

	
	private boolean locationExists(String locationString) {
		return getByLocation(locationString) != null;
	}
	
	public List<String> getDependentLocations() {
		List<Node> descendants = artifactNode.getDescendantNodes();
		Set<String> locations = new LinkedHashSet<String>();
		for (Node descendant : descendants) {
			String location = descendant.location;
			locations.add(location);
		}
		return new LinkedList<String>(locations);
	}
	
	
	public List<String> getAllIds() {
		List<String> ids = new LinkedList<String>();
		ids.add(artifactNode.id);
		for (Node node : artifactNode.getDescendantNodes()) {
			ids.add(node.id);
		}
		return ids;
	}
	
	public List<String> getAllIdsExceptForMainArtifact() {
		List<String> ids = new LinkedList<String>();
		for (Node node : artifactNode.getDescendantNodes()) {
			ids.add(node.id);
		}
		return ids;
	}
	
	public String getLocationForId(String id) {
		Node node = getById(id);
		if (node == null) {
			return null;
		} else {
			return node.location;
		}
	}
	
	
	private Node getById(String id) {
		if (artifactNode.id.equals(id)) {
			return artifactNode;
		} else {
			for (Node node : artifactNode.getDescendantNodes()) {
				if (node.id.equals(id)) {
					return node;
				}
			}
		}
		return null;
	}
	
	private Node getByLocation(String location) {
		if (artifactNode.location.equals(location)) {
			return artifactNode;
		} else {
			for (Node node : artifactNode.getDescendantNodes()) {
				if (node.location.equals(location)) {
					return node;
				}
			}
		}
		return null;
	}
	

	private boolean doesIdExist(String id) {
		Node node = getById(id);
		return node != null;
	}
	
	private String buildId(String name, int count) {
		return name + "_" + count;
	}
	
	private String getUnusedId(String name) {
		if (!doesIdExist(name)) {
			return name;
		} else {
			int i = 0;
			while(true) {
				i++;
				String id = buildId(name, i);
				if (!doesIdExist(id)) {
					return id;
				}
			}
		}
	}
	
	
	
	
	private class Node {
		private List<Node> children = new LinkedList<Node>();
		private String location;
		private String id;
		
		private List<Node> getDescendantNodes() {
			List<Node> descendants = new LinkedList<Node>();
			for (Node child : children) {
				descendants.add(child);
				descendants.addAll(child.getDescendantNodes());
			}
			return descendants;
		}
		
	}
	
	
	public String toString() {
		String string = toString(artifactNode, 0);
		return string;
	}
	
	private String toString(Node node, int indent) {
		StringBuilder sb = new StringBuilder();
		sb.append(indentString(indent)).append(node.id).append(": ").append(node.location);
		if (!node.children.isEmpty()) {
			sb.append("\n");
			for (Node child : node.children) {
				String childString = toString(child, indent + 4);
				sb.append(childString);
				sb.append("\n");
			}
		}
		return sb.toString();
	}
	
	private String indentString(int indent) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < indent; i++) {
			sb.append(" ");
		}
		return sb.toString();
	}

}
