package org.deft.repository.artifacttype.ecore;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deft.repository.fragmentsource.FileBasedFragmentLocation;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.ENamedElementImpl;
import org.eclipse.emf.ecore.resource.Resource;

/**
 * The EcoreCrossReferenceResolver analyzes Ecore models and Ecore diagrams for
 * references to other Ecore files and returns them. 
 */
public class EcoreCrossReferenceResolver {

	public List<OriginalFragmentLocation> getReferencedLocations(
			Resource resource, FileBasedFragmentLocation origLocation) {
		Map<EObject, Collection<EStructuralFeature.Setting>> map 
				= org.eclipse.emf.ecore.util.EcoreUtil.ExternalCrossReferencer.find(resource);
		Set<FileBasedFragmentLocation> locations = getLocations(map, origLocation);
		return new LinkedList<OriginalFragmentLocation>(locations);
	}
	
	public List<OriginalFragmentLocation> getReferencedLocations(
			EPackage modelRoot, FileBasedFragmentLocation origLocation) {
		Map<EObject, Collection<EStructuralFeature.Setting>> map 
				= org.eclipse.emf.ecore.util.EcoreUtil.ExternalCrossReferencer.find(modelRoot);
		Set<FileBasedFragmentLocation> locations = getLocations(map, origLocation);
		return new LinkedList<OriginalFragmentLocation>(locations);
	}
	
	private Set<FileBasedFragmentLocation> getLocations(
			Map<EObject, Collection<EStructuralFeature.Setting>> map, 
			FileBasedFragmentLocation origLocation) {
		Set<FileBasedFragmentLocation> locations = new LinkedHashSet<FileBasedFragmentLocation>();
		for (EObject eo : map.keySet()) {
			if (eo instanceof ENamedElementImpl) {
				ENamedElementImpl ec = (ENamedElementImpl)eo;
				URI uri = ec.eProxyURI();
				if (uri != null) {
					if (uri.isFile()) {
						String fileName = uri.toFileString();
						FileBasedFragmentLocation location = origLocation.appendOrReplace(fileName);
						locations.add(location);
					} 
					if (uri.isPlatform()) {
						String uriPath = uri.path();
						String relPath = uriPath;
						if (uriPath.startsWith("/resource/")) {
							relPath = uriPath.replaceFirst("^/resource/", "");
						} else if (uriPath.startsWith("/plugin/")) {
							relPath = uriPath.replaceFirst("^/plugin/", "");
						}
						FileBasedFragmentLocation uriLoc = findFileForPath(origLocation, relPath);
						if (uriLoc != null) {
							locations.add(uriLoc);
						}
					}
				}
			}
		}
		return locations;
	}
	
	/**
	 * Searches the file specified in relPath. relPath is a relative path. The
	 * relative path is appended to startLoc and it is tested if the file exists. If it
	 * does not exist, relPath is appended to the parent directory of startLoc, and so on.
	 * When finally an ancestor directory is found for which relPath points to a file,
	 * the full location to the file is returned. If the root is reached and nothing is found,
	 * null is returned.
	 * This method is used to find files which are referenced using a platform:/resource uri.
	 * Outside the running eclipse workspace there is no way to resove this uri, so this kind of
	 * brute force search is performed. Note that it is possible that a wrong file is returned,
	 * though this is very unlikely.
	 */
	public FileBasedFragmentLocation findFileForPath(FileBasedFragmentLocation startLoc, String relPath) {
		FileBasedFragmentLocation searchRoot = startLoc;
		while (searchRoot != null) {
			FileBasedFragmentLocation loc = searchRoot.appendOrReplace(relPath);
			if (loc.exists()) {
				return loc;
			} else {
				searchRoot = searchRoot.getParent();
			}
		}
		return null;
	}

}
