package org.deft.repository.artifacttype.ecore;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.fragmentsource.FileBasedFragmentLocation;
import org.deft.util.FileUtil;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

/**
 * This class takes an Ecore or Ecore diagram from the repository and puts it into
 * a temporary directory for further processing (loading a diagram requires a file,
 * a stream is currently not sufficient).
 * If the ecore model or diagram depends on other models, those are written to the
 * temp dir as well. The model references are "normalized", i.e. all models and diagrams
 * will be written into one directory. The internal references are updated if necessary
 * (only in the temp file copies, the originals in the repository stay untouched).   
 */
public class EcoreTempFileRepresentation implements ArtifactRepresentation {

	//temp dir for the models and diagrams
	private File tempDir;
	
	//the absolute path of the model or diagram to be written in the temp file
	private String path;
	private List<String> basicPaths = new LinkedList<String>();
	
	public static final String TYPE = "org.deft.representation.tempfile";
	private EcoreCrossReferenceResolver resolver = new EcoreCrossReferenceResolver();
	
	public EcoreTempFileRepresentation(String tempDir) {
		this.tempDir = new File(tempDir);
	}
	
	public EcoreTempFileRepresentation(File tempDir) {
		this.tempDir = tempDir;
	}
	
	public String getPath() {
		return path;
	}
	
	public List<String> getPathsForBasicArtifacts() {
		return new LinkedList<String>(basicPaths);
	}
	
	@Override
	public String getType() {
		return TYPE;
	}

	/**
	 * Writes the artifact (model or diagram) and all its referenced models into the temp dir and returns
	 * the name of the file in which the artifact has been written.
	 * 
	 * Currently the references are directly manipulated in the XML files. Later this should be changed,
	 * things might be more stable when the resource api is used.
	 */
	public void writeArtifactAndDependentFiles(Artifact artifact, int revision) throws IOException {
		// create resource set and resource 
		ResourceSet resourceSet = new ResourceSetImpl();

		// Register XML resource factory
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("xmi", 
				new XMIResourceFactoryImpl());

		
		String content = new String(artifact.getArtifactContent(revision));
		List<String> references = getReferences(content);
		Map<String, String> refToDependentFileIdMap = mapReferencesToDependentFile(references, artifact, revision);
		//write main file and save its path so it can be accessed via getPath()
		File file = writeArtifact(content, artifact.getName(), refToDependentFileIdMap);
		file.deleteOnExit();
		this.path = file.getAbsolutePath();
		List<String> depenentFileIds = artifact.getDependentFileIds(revision);
		for (String fileId : depenentFileIds) {
			String dependentFileContent = new String(artifact.getContent(fileId, revision));
			List<String> newReferences = getReferences(dependentFileContent);
			Map<String, String> newMap = mapReferencesToDependentFile(newReferences, artifact, revision);
			File writtenFile = writeArtifact(dependentFileContent, fileId, newMap);
			writtenFile.deleteOnExit();
			this.basicPaths.add(writtenFile.getAbsolutePath());

		}
	}
	
	private File writeArtifact(String content, String fileName, Map<String, String> refToDependentFileIdMap) {
		
		for (String ref : refToDependentFileIdMap.keySet()) {
			String referencedFileId = refToDependentFileIdMap.get(ref);
			//all patterns found so far which contain references, there might be more...
			content = content.replaceAll("\\\"ecore:(.*?)\\\"\\shref=\\\"" + ref + "#/",  "\"ecore:$1\" href=\"" + referencedFileId + "#/");
			content = content.replaceAll("eSuperTypes=\\\"" + ref + "#/", "eSuperTypes=\"" + referencedFileId + "#/");
			content = content.replaceAll("\\\"platform:/resource/" + ref + "#/", "\"" + referencedFileId + "#/");
			content = content.replaceAll("\\s" + ref, " " + referencedFileId);

		}
		String directoryPath = tempDir.getAbsolutePath();
		File destFile = new File(directoryPath + "/" + fileName);
		FileUtil.saveToFile(content.getBytes(), destFile, true);
		return destFile;
	}
	
	
	public List<FileBasedFragmentLocation> getOriginalFragmentLocations(
			FileBasedFragmentLocation baseLocation, String content) {
		List<FileBasedFragmentLocation> locations = new LinkedList<FileBasedFragmentLocation>();
		List<String> references = getReferences(content);
		for (String s : references) {
			if (s.startsWith("platform:/resource/")) {
				s = s.replaceFirst("^platform:/resource/", "");
				FileBasedFragmentLocation loc = resolver.findFileForPath(baseLocation, s);
				locations.add(loc);
			} else {
				//normal file reference
				FileBasedFragmentLocation location = baseLocation.appendOrReplace(s);
				locations.add(location);
			}
		}
		return locations;
	} 
	
	public List<String> getReferences(String content) {
		List<String> references = new LinkedList<String>();
		//collect all references to external files
		references.addAll(getForRegex("<element.*?href=\\\"(.*?)#/", content));
		references.addAll(getForRegex("eType=\\\"ecore:EClass\\s(.*?)#/", content));
		references.addAll(getForRegex("eType=\\\"ecore:EEnum\\s(.*?)#/", content));
		references.addAll(getForRegex("eSuperTypes=\\\"(.*?)#/", content));
		references.addAll(getForRegex("eSuperTypes=\\\"[^\\s]*\\s(.*?)#/", content));
		List<String> clearedReferences = new LinkedList<String>();
		//clean references from file or platform uri starts
		for (String ref : references) {
			if (ref.startsWith("platform:/resource/")) {
				ref = ref.replaceFirst("^platform:/resource/", "");
			} else if (ref.startsWith("file:///")) {
				ref = ref.replaceFirst("^file:///", "");
			}
			clearedReferences.add(ref);
		}
		return clearedReferences;
	}
		
	private List<String> getForRegex(String regex, String content) {
		List<String> references = new LinkedList<String>();
		Pattern p = Pattern.compile(regex);
		Matcher m = p.matcher(content);
		while (m.find()) {
			String ref = m.group(1);
			if (!ref.isEmpty()) {
				references.add(ref);
			}
		}
		return references;
	}
	
	public Map<String, String> mapReferencesToDependentFile(List<String> references,
			Artifact artifact, int revision) {
		Map<String, String> map = new HashMap<String, String>();
		for (String ref : references) {
			if (!map.containsKey(ref)) {//test, so we don't compute already known stuff again
				String matchingLocationId = getDepIdForLocationWithBestMatch(ref, artifact, revision);
				map.put(ref, matchingLocationId);
			}
		}
		return map;
	}
	
	
	/**
	 * Checks for all dependent files of the artifact if the ref string matches the 
	 * original location. If there are matches for multiple locations, the longest
	 * path wins. The return value is the id of the dependent file which was matched.
	 */
	private String getDepIdForLocationWithBestMatch(String ref, Artifact artifact, int revision) {
		int longestPathLength = 0;
		String depIdForBestMatch = null;
		ref = ref.replaceAll("\\.\\./", "");
		for (String depId : artifact.getDependentFileIds(revision)) {
			String path = artifact.getOriginalLocation(depId, revision);
			path = path.replaceAll("\\\\", "/");
			if (path.endsWith(ref) && path.length() > longestPathLength) {
				longestPathLength = path.length();
				depIdForBestMatch = depId;
			}
		}
		return depIdForBestMatch;
	}

	
}
