package org.deft.repository.ast.serialize;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.deft.repository.ast.Token;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.annotation.astname.AstNameInformation;
import org.deft.repository.ast.decoration.Templates;
import org.deft.util.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


public class XmlTreeNodeSerializer implements ITreeNodeSerializer {

	private static final Logger logger = Logger.getLogger(XmlTreeNodeSerializer.class);
	
	public TreeNodeRoot read(InputStream is) {
		try {
			DocumentBuilder parser = XmlUtil.getDocumentBuilder();
			InputStream iStream = is;
			if (!(is instanceof BufferedInputStream)) {
				iStream = new BufferedInputStream(is);
			}
			Document doc = parser.parse(iStream);
			TreeNodeRoot tree = convertToTreeNode(doc);
			return tree;
		} catch (SAXException e) {
			logger.warn("Could not read stream", e);
			return null;
		} catch (IOException e) {
			logger.warn("Could not read stream", e);
			return null;
		} catch (ParserConfigurationException e) {
			logger.warn("Could not instantiate XML parser", e);
			return null;
		}
	}

	public void write(TreeNodeRoot root, OutputStream os) {
		Document doc = convertToXml(root);
		OutputStream oStream = os;
		boolean isBuffered = (os instanceof BufferedOutputStream); 
		if (!isBuffered) {
			oStream = new BufferedOutputStream(os);
		}
		XmlUtil.outputPrettyXml(doc, oStream);
		if (!isBuffered) {
			try {
				oStream.close();
			} catch (IOException ioe) {
				logger.warn("Could not write stream", ioe);
			}
		}
	}
	
	
	public TreeNodeRoot convertToTreeNode(Document doc) {
		Element e = doc.getDocumentElement();
		TreeNode tn = convertToTreeNode(e);
		return new TreeNodeRoot(tn);
	}
	
	public TreeNode convertToTreeNode(Element e) {	
		TreeNode tn = convertElement(e); 
		NodeList nl = e.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			if (n instanceof Element) {
				Element eChild = (Element)nl.item(i);
				TreeNode tnChild = convertToTreeNode(eChild);
				tn.addChild(tnChild);
			}
		}
		return tn;
	}
	
	private TreeNode convertElement(Element e) {
		TreeNode tn = null;
		if (e.getAttribute("token").equals("true")) {
			tn = convertTokenNode(e);
		} else {
			tn = new TreeNode(e.getNodeName());
		}
		String name = e.getAttribute("name"); 
		if (!name.equals("")) {
			AstNameInformation ani = new AstNameInformation(name);
			tn.addInformation(ani);
		}
		return tn;
	}
	
	private TokenNode convertTokenNode(Element e) {
		try {
			String attOffset = e.getAttribute("offset");
			String attLine = e.getAttribute("line");
			String attCol = e.getAttribute("col");
			String text = e.getTextContent();

			int offset = Integer.valueOf(attOffset);
			int line = Integer.valueOf(attLine);
			int col = Integer.valueOf(attCol);
			Token t = new Token(line, col, offset, text);
			TokenNode tn = new TokenNode(e.getNodeName(), t);
			return tn;
		} catch (Exception ex) {
			logger.warn("Could not convert Element e to TokenNode", ex);
			return null;
		}
	}
	
	
	public Document convertToXml(TreeNodeRoot root) {
        Document doc = XmlUtil.makeDocument();
        Element eRoot = doConvert(doc, root.getFirstChild());
        doc.appendChild(eRoot);
        return doc;
    }

    private Element doConvert(Document doc, TreeNode node) {
        Element eNode = makeElement(doc, node);
        for (TreeNode child : node.getChildren()) {
            Element eChild = doConvert(doc, child);
            eNode.appendChild(eChild);
        }
        return eNode;
    }

    private Element makeElement(Document doc, TreeNode node) {
        Element e = null;
        try {
            e = doc.createElement(node.getName());
        }
        catch (Exception ex) {
        	logger.warn("Could not make XML Element from TreeNode " + node, ex);
        }
        AstNameInformation info = (AstNameInformation)node.getInformation(Templates.ASTNAME);
        if (info != null) {
            e.setAttribute("name", info.getAstName());
        }
        if (node instanceof TokenNode) {
            e.setTextContent(((TokenNode)node).getToken().getText());
            e.setAttribute("token", "true");
            setAttribute(e, "line", node.getStartLine());
            setAttribute(e, "col", node.getStartCol());
            setAttribute(e, "offset", node.getOffset());
        }
        return e;
    }

    private void setAttribute(Element e, String attName, int value) {
        e.setAttribute(attName, String.valueOf(value));
    }    
	

}
