package org.deft.repository.chapterdocument.latex;

import java.io.IOException;
import java.io.PushbackReader;
import java.io.StringReader;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import net.sourceforge.texlipse.model.OutlineNode;
import net.sourceforge.texlipse.texparser.LatexLexer;
import net.sourceforge.texlipse.texparser.LatexParser;
import net.sourceforge.texlipse.texparser.lexer.LexerException;

import org.deft.repository.chaptertype.ChapterDocument;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.PathFragmentFilter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.integrator.ArtifactReferencePosition;

public class LatexChapterDocument implements ChapterDocument {

	private Chapter chapter;
	
	public LatexChapterDocument(Chapter chapter) {
		if (!chapter.getTypeId().equals("latex")) {
			throw new IllegalArgumentException("Chapter is no LaTeX chapter, but " + chapter.getTypeId());
		}
		this.chapter = chapter;
	}
	

	public byte[] getContentAsByteArray() {
		return chapter.getChapterContent();
	}
	
	public String getContentAsString() {
		String content = new String(getContentAsByteArray());
		return content;
	}
	
	
	public Chapter getChapter() {
		return chapter;
	}

	private LatexParser loadChapterAndParse() {
		String input = new String(chapter.getChapterContent());
		LatexLexer lexer = new LatexLexer(new PushbackReader(new StringReader(input), 1024));
		LatexParser parser = new LatexParser();
		try {
			parser.parse(lexer, false);
			return parser;
		} catch (LexerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	public List<LatexChapterDocument> getDirectlyAndIndirectlyIncludedLatexChapters() {
		List<LatexChapterDocument> chapterDocs = new LinkedList<LatexChapterDocument>();
		getLatexChaptersRecursively(chapterDocs);
		return chapterDocs;
	}
	
	private void getLatexChaptersRecursively(List<LatexChapterDocument> foundDocuments) {
		List<String> chapPaths = getIncludedLatexChapterDocumentNames();
		for (String path : chapPaths) {
			Chapter c = getChapter(path);
			if (c != null && !listContainsDocument(foundDocuments, c)) { //do not follow circular dependencies
				LatexChapterDocument doc = new LatexChapterDocument(c);
				foundDocuments.add(doc);
				doc.getLatexChaptersRecursively(foundDocuments);
			}
		}
	}
	
	private boolean listContainsDocument(List<LatexChapterDocument> list, Chapter chap) {
		for (LatexChapterDocument doc : list) {
			if (doc.getChapter().equals(chap)) {
				return true;
			}
		}
		return false;
	}
	
	
	private List<String> getIncludedLatexChapterDocumentNames() {
		List<String> names = new LinkedList<String>();
		LatexParser parser = loadChapterAndParse();
		List<OutlineNode> inputs = parser.getInputs();
		for (OutlineNode input : inputs) {
			String name = input.getName();
			names.add(name);
		}
		return names;
	}

	
	public List<BibtexChapterDocument> getBibtexChapters() {
		List<BibtexChapterDocument> bibtexChapterDocs = getDirectBibtexChapters();
		
		for (LatexChapterDocument latexDoc : getDirectlyAndIndirectlyIncludedLatexChapters()) {
			List<BibtexChapterDocument> bibChaps = latexDoc.getDirectBibtexChapters();
			bibtexChapterDocs.addAll(bibChaps);
		}
		return bibtexChapterDocs;
	}
	
	private List<BibtexChapterDocument> getDirectBibtexChapters() {
		List<BibtexChapterDocument> bibtexChapterDocs = new LinkedList<BibtexChapterDocument>();
		List<String> chapPaths = getIncludedBibtexChapterDocumentNames();
		for (String path : chapPaths) {
			Chapter c = getChapter(path);
			if (c != null) {
				BibtexChapterDocument doc = new BibtexChapterDocument(c);
				bibtexChapterDocs.add(doc);
			}
		}
		return bibtexChapterDocs;
	}
	
		
	private List<String> getIncludedBibtexChapterDocumentNames() {
		LatexParser parser = loadChapterAndParse();
		String[] bibs = parser.getBibs();
		if (bibs == null) {
			return Collections.emptyList();
		} else {
			List<String> bibsList = Arrays.asList(bibs);
			return bibsList;
		}
	}
	
	
	private Chapter getChapter(String path) {
		FragmentFilter filter = new PathFragmentFilter(path, FragmentType.CHAPTER);
		Project project = this.chapter.getProject();
		List<Fragment> fragments = project.getFragments(filter);
		if (!fragments.isEmpty()) {
			Chapter chapter = (Chapter)fragments.get(0);
			return chapter;
		}
		return null;
	}


	@Override
	public ArtifactReferencePosition removeRepresentation(ArtifactReference reference) {
		return null;
	}


	@Override
	public void insertRepresentation(ArtifactReference reference, ArtifactReferencePosition pos) {
	}
	

	
}
