package org.deft.repository.check;

import java.io.InputStream;

import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.exception.DeftParseException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;

/**
 * Checks whether a specific fragment (e.g. file) in a fragment location (e.g. file system)
 * is of a specific type. This check is executed prior importing a fragment into the repository
 * or before using the fragment to update the corresponding artifact in the repository.
 * 
 * The check assumes that the provided artifact type exists. 
 * 
 * If the fragment is not valid, an InvalidArtifactError is created which can be used
 * for further error reporting and handling.
 */
public class ArtifactValidCheck extends DeftCheck {

	private ArtifactType artifactType;
	private FragmentSource fragmentSource;
	private OriginalFragmentLocation location;
	
	private DeftParseException parseException; 
	
	public ArtifactValidCheck(ArtifactType artifactType, 
			FragmentSource fragmentSource, OriginalFragmentLocation location) {
		this.artifactType = artifactType;
		this.fragmentSource = fragmentSource;
		this.location = location;
	}
	
	
	@Override
	public void perform() {
		clearErrors();
		parseException = null;
		if (!isFragmentValid()) {
			InvalidArtifactError error = new InvalidArtifactError(artifactType.getTypeId(), location, parseException);
			addError(error);
		}
		
	}

	private boolean isFragmentValid() {
		Validator validator = artifactType.getValidator();
		InputStream is = fragmentSource.getInputStream(location);
		try {
			validator.validate(is);
		}
		catch (DeftParseException dpe) {
			parseException = dpe;
			return false;
		}
		return true;
	}
	
}
