package org.deft.repository.check;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;

/**
 * Checks whether a specific fragment (e.g. file) in a fragment location (e.g. file system)
 * is of a specific type. This check is executed prior importing a fragment into the repository
 * or before using the fragment to update the corresponding artifact in the repository.
 * 
 * The check assumes that the provided artifact type exists. 
 * 
 * If the fragment is not valid, an InvalidArtifactError is created which can be used
 * for further error reporting and handling.
 */
public class DependentArtifactsAvailableCheck extends DeftCheck {

	private ArtifactType artifactType;
	private FragmentSource fragmentSource;
	private OriginalFragmentLocation location;
	
	public DependentArtifactsAvailableCheck(ArtifactType artifactType, 
			FragmentSource fragmentSource, OriginalFragmentLocation location) {
		this.artifactType = artifactType;
		this.fragmentSource = fragmentSource;
		this.location = location;
	}
	
	
	@Override
	public void perform() {
		clearErrors();

		List<OriginalFragmentLocation> dependencies = getResolvableDependencies(location);
		List<OriginalFragmentLocation> missingDependencies = getMissingLocations(dependencies);

		if (!missingDependencies.isEmpty()) {
			MissingDependencyError error = new MissingDependencyError(location, missingDependencies);
			addError(error);
		}
		
	}

	
	private List<OriginalFragmentLocation> getResolvableDependencies(OriginalFragmentLocation location) {
		List<OriginalFragmentLocation> locations = new LinkedList<OriginalFragmentLocation>();
		locations.add(location);
		if (fragmentSource.doesLocationExist(location)) {
			Validator validator = artifactType.getValidator();
			List<OriginalFragmentLocation> directDependencies = validator.getDependencyLocations(fragmentSource, location);
			for (OriginalFragmentLocation dependency : directDependencies) {
				List<OriginalFragmentLocation> dependenciesOfDependency = getResolvableDependencies(dependency);
				locations.addAll(dependenciesOfDependency);
			}
		}
		return locations;
	}
	
	
	private List<OriginalFragmentLocation> getMissingLocations(List<OriginalFragmentLocation> locations) {
		List<OriginalFragmentLocation> missingLocations = new LinkedList<OriginalFragmentLocation>();
		for (OriginalFragmentLocation location : locations) {
			boolean exists = fragmentSource.doesLocationExist(location);
			if (!exists) {
				missingLocations.add(location);
			}
		}
		return missingLocations;
	}
	
	
}
