package org.deft.repository.fragmentsource.filesystem;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.deft.repository.fragmentsource.FileBasedFragmentLocation;

public class FileSystemFragmentLocation implements FileBasedFragmentLocation {

	private File file;
	
	public FileSystemFragmentLocation(Object location) {
		if (location instanceof File) {
			this.file = (File)location;
		}
	}
	
	public FileSystemFragmentLocation(File file) {
		this.file = file;
	}
	
	public File getFile() {
		return file;
	}
	
	@Override
	public String getFullLocation() {
		String path = file.getAbsolutePath();
		return path;
	}
	
	@Override
	public String toString() {
		return "Location: " + getFullLocation(); 
	}

	@Override
	public String getName() {
		return file.getName();
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof FileSystemFragmentLocation) {
			FileSystemFragmentLocation loc = (FileSystemFragmentLocation)o;
			boolean equals = getFullLocation().equals(loc.getFullLocation());
			return equals;
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return getFullLocation().hashCode();
	}

	/**
	 * Appends a relative path to an absolute path or replaces
	 * the old path with an absolute path.
	 * If the absolute path denotes a file (which it should always),
	 * the actual file name will be removed before the relative path
	 * is appended.
	 */
	@Override
	public FileSystemFragmentLocation appendOrReplace(String path) {
		File pathFile = new File(path);
		if (pathFile.isAbsolute()) {
			FileSystemFragmentLocation location = new FileSystemFragmentLocation(pathFile);
			return location;
		} else {
			FileSystemFragmentLocation location = appendRelative(path);
			return location;
		}
	}
	

	private FileSystemFragmentLocation appendRelative(String path) {
		try {
			File f = file;
			if (f.isFile()) {
				f = f.getParentFile();//f is now the directory which contains the file
			}
			File newFile = new File(f, path).getCanonicalFile();
			FileSystemFragmentLocation newLocation = new FileSystemFragmentLocation(newFile);
			return newLocation;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public FileBasedFragmentLocation getParent() {
		File parentFile = file.getParentFile();
		if (parentFile != null) {
			FileBasedFragmentLocation parentLoc = new FileSystemFragmentLocation(parentFile);
			return parentLoc;
		}
		return null;
	}

	@Override
	public List<FileBasedFragmentLocation> listFilesAndDirectories() {
		List<FileBasedFragmentLocation> content = new LinkedList<FileBasedFragmentLocation>();
		content.addAll(listDirectories());
		content.addAll(listFiles());
		return content;
	}

	@Override
	public List<FileBasedFragmentLocation> listDirectories() {
		List<FileBasedFragmentLocation> content = new LinkedList<FileBasedFragmentLocation>();
		if (isDirectory()) {
			File[] dirs = file.listFiles(new DirFilter());
			Arrays.sort(dirs);
			for (File dir : dirs) {
				FileBasedFragmentLocation loc = new FileSystemFragmentLocation(dir);
				content.add(loc);
			}
		}
		return content;
	}

	@Override
	public List<FileBasedFragmentLocation> listFiles() {
		List<FileBasedFragmentLocation> content = new LinkedList<FileBasedFragmentLocation>();
		if (isDirectory()) {
			File[] files = file.listFiles(new RealFileFilter());
			Arrays.sort(files);
			for (File file : files) {
				FileBasedFragmentLocation loc = new FileSystemFragmentLocation(file);
				content.add(loc);
			}
		}
		return content;
	}

	@Override
	public boolean isFile() {
		boolean isFile = file.isFile();
		return isFile;
	}

	@Override
	public boolean isDirectory() {
		boolean isDirectory = file.isDirectory();
		return isDirectory;
	}
	
	private class DirFilter implements FileFilter {
		@Override
		public boolean accept(File pathname) {
			boolean isDir = pathname.isDirectory();
			return isDir;
		}
	}
	
	private class RealFileFilter implements FileFilter {
		@Override
		public boolean accept(File pathname) {
			boolean isFile = pathname.isFile();
			return isFile;
		}
	}

	@Override
	public boolean exists() {
		return file.exists();
	}
}
