package org.deft.repository.repositorymanager;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;

/**
 * The IConstraintsManager is a part of the DEFT repository. It offers various
 * checks from which one can tell whether specific operations on the 
 * repository will be allowed or not. 
 * 
 * Implementations of this interface are used by the repository to check if method
 * calls have valid parameters or not. Furthermore, the results of the IConstraintsManager
 * can be propagated by the repository to interested parties, such as the UI.
 */
public interface ConstraintsManager {
	
	/**
	 * Checks if a project with a given name exists.
	 * 
	 * @param name the name of the project
	 * @return <code>true</code> if there is a project with such a name, 
	 * otherwise <code>false</code>
	 */
	public boolean projectExists(String name);
	
	/**
	 * Checks if a folder can contain a fragment of a given type.
	 * 
	 * Folders are not allowed to contain fragments of a different type, e.g.
	 * both Artifacts and Images. Therefore the type of fragments that a folder may
	 * contain must be identified and compared to the fragmentType parameter.
	 * 
	 * @param parent the folder to be checked
	 * @param fragmentType a type of fragment, such as chapter or code file
	 * @return <code>true</code> if the folder can contain a fragment of the type,
	 * otherwise <code>false</code>
	 */
	public boolean canContainFragmentType(Folder parent, FragmentType fragmentType);
	
	/**
	 * Checks if a folder can contain a fragment of the given name.
	 * 
	 * A folder can contain a fragment with a given name if it does not
	 * already have a fragment with that name, or if multiple fragments
	 * with the same name are allowed.
	 * 
	 * @param parent the folder for which to check if the child name is allowed
	 * @param childName the name for which to check if it can occur in the parent folder
	 * @return <code>true</code> if it is possible to have a fragment with the specified
	 * name in the folder, otherwise <code>false</code>
	 */
	public boolean canContainFragmentName(Folder parent, String childName);
	
	/**
	 * Checks if a project can contain a fragment of the given name.
	 * 
	 * This method is similar to {@link #canContainFragmentName(Folder, String)}. 
	 * Fragments need not necessarily be in a folder, they can also appear detachedly
	 * at the "root". However, it is still necessary to check if the "root" is allowed
	 * to contain additional fragments with a given name. The types of fragments (Chapters,
	 * Artifacts, ...) are treated independently of each other.
	 *   
	 * @param project the project for which to check the fragment name
	 * @param fragmentType the type of fragment for which to check if a child name is allowed
	 * @param childName the name for which to check if it can occur
	 * @return <code>true</code> if it is possible to have a fragment with the specified
	 * type and name in the "virtual root folder", otherwise <code>false</code>
	 */
	public boolean canContainFragmentNameAtRoot(Project project, FragmentType fragmentType, String childName);
	
	/**
	 * Checks if a Folder can contain a specific fragment.
	 * 
	 * A folder can only contain fragments from the same project.
	 * 
	 * A folder can contain a fragment with a given name if it does not
	 * already have a fragment with that name, or if multiple fragments
	 * with the same name are allowed.
	 * 
	 * If child is already contained in parent, <code>false</code> is returned.
	 * 
	 * If child is a folder, it must have the same type as parent, and it must not currently be
	 * an ancestor of parent or even the same as parent (i.e. you can't move a folder into 
	 * itself or one of its child folders).
	 * 
	 * If the folder is <code>null</code>, it will be checked if the fragment can be
	 * added to the root of its project. 
	 *   
	 * @param parent the folder for which to check if the child is allowed, or the project root,
	 * if it is <code>null</code>
	 * @param child the fragment for which to check if it can occur in the parent folder
	 * @return <code>true</code> if it is possible to have child in the folder, 
	 * otherwise <code>false</code>
	 */
	public boolean canContainFragment(Folder parent, Fragment child);

	
	/**
	 * Checks if the project root can contain a specific fragment.
	 * 
	 * The project root can only contain fragments from the same project.
	 * 
	 * The project root can contain a fragment with a given name and type if it does not
	 * already have a fragment with that name and type, or if multiple fragments
	 * with the same name are allowed.
	 * 
	 * If child is already contained in the project root, <code>false</code> is returned.
	 * 
	 * @param project the project for which to check if the child is allowed at the root
	 * @param fragmentType the type of fragment for which to check if a child is allowed at the root
	 * @param child the fragment for which to check if it can occur in the project
	 * @return <code>true</code> if it is possible to have child in the project root
	 * otherwise <code>false</code>
	 */
	public boolean canContainFragmentAtRoot(Project project, FragmentType fragmentType, Fragment child);
	
	/**
	 * Checks if there is a fragment with a given name in a folder.
	 * 
	 * @param parent the folder in which to look for a fragment with the name
	 * @param childName the name to look for
	 * @return <code>true</code>if the folder contains a fragment with the name, 
	 * otherwise <code>false</code>
	 */
	public boolean containsFragmentName(Folder parent, String childName);	
	
	/**
	 * Checks if there is a fragment with a given type and name at the root of the project.
	 * 
	 * @param project the project for which to check the fragment name
	 * @param fragmentType the type of fragment for which to check if a child name is allowed
	 * @param childName the name to look for
	 * @return <code>true</code>if the folder contains a fragment with the name, 
	 * otherwise <code>false</code>
	 */
	public boolean containsFragmentNameAtRoot(Project project, FragmentType fragmentType, String childName);

	/**
	 * Checks if a chapter can be added to a tutorial.
	 * 
	 * A chapter can be added if it has not already been added to the tutorial.
	 * 
	 * @param tutorial the tutorial to which one would like to add the chapter
	 * @param chapter the chapter to be added
	 * @return <code>true</code>, if the chapter is not yet in the tutorial, 
	 * otherwise <code>false</code>
	 */
	public boolean canContainChapter(Tutorial tutorial, Chapter chapter);
	
	/**
	 * Checks if a chapter already exists in a tutorial.
	 * @param tutorial the tutorial for which to check for the existing chapter
	 * @param chapter the chapter for which to check in the tutorial
	 * @return <code>true</code> if the chapter has already been added to the tutorial,
	 * otherwise <code>false</code>.
	 */
	public boolean containsChapter(Tutorial tutorial, Chapter chapter);
	
}
