package org.deft.repository.repositorymanager;

import java.io.File;
import java.util.List;

import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.event.RepositoryUpdateListener;
import org.deft.repository.exception.DeftCrossProjectRelationException;
import org.deft.repository.exception.DeftIncompatibleNameException;
import org.deft.repository.exception.DeftIncompatibleTypeException;
import org.deft.repository.exception.DeftInvalidArgumentException;
import org.deft.repository.exception.DeftNullArgumentException;
import org.deft.repository.exception.DeftProjectAlreadyExistsException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.integrator.FileIntegrator;
import org.deft.repository.integrator.Integrator;


public interface ContentManagementService {

	/**
	 * Creates a new project. The type of the project defines which chapter types are allowed. 
	 * For example, a latex project can have latex chapters and bibtex chapters.
	 * 
	 * @param name the name of the project
	 * @param projectType the type of project (xhtml, opendocument, latex, ...) 
	 * 
	 * @return a new project with the specified name
	 * @throws DeftProjectAlreadyExistsException if there is already a project
	 * with the specified name
	 * @throws DeftNullArgumentException if name is null
	 */
	public Project createProject(String name, String projectType) 
			throws DeftProjectAlreadyExistsException,
			DeftNullArgumentException;
	
	
	/**
	 * Returns the type of project (xhtml, opendocument, latex, ...) 
	 * @param project the project for which to get type
	 * @return the type of the project
	 * @throws DeftInvalidArgumentException if the project cannot be found in the repository
	 */
	public String getProjectType(Project project) throws DeftInvalidArgumentException;
	
	/**
	 * Returns a temporary directory specifically for a certain project.
	 * This directory can be used for project specific data.
	 * 
	 * For example, (generated) images cannot be directly inserted into a LaTeX file, they must be
	 * stored in the file system and referenced from there. The project specific temporary directory
	 * is the right place to do that.
	 */
	public File getTempDirectory(Project project) throws DeftInvalidArgumentException;
	
	/**
	 * Creates a new chapter within a project. The chapter lies at the root position
	 * of the project, i.e. it is not contained in a folder.
	 *  
	 * @param project the project to create the chapter in
	 * @param name the name of the chapter
	 * @param chapterType the type of the chapter
	 * @return a new chapter with the specified name and type
	 * @throws DeftNullArgumentException if either project or name are null
	 * @throws DeftIncompatibleNameException if the chapter name is not allowed at
	 * the root
	 * @throws DeftInvalidArgumentException if the project cannot be found in the repository
	 */
	public Chapter createChapter(Project project, String name, String chapterType)
			throws DeftNullArgumentException, 
			DeftIncompatibleNameException,
			DeftInvalidArgumentException;
	
	
	/**
	 * Creates a new chapter within in a folder. 
	 *  
	 * @param parent the folder to create the chapter in
	 * @param name the name of the chapter
	 * @param chapterType the type of the chapter
	 * @return a new chapter with the specified name and type
	 * @throws DeftNullArgumentException if either parent or name are null
	 * @throws DeftInvalidArgumentException if the folder cannot be found in the repository
	 * @throws DeftIncompatibleTypeException if the specified folder cannot contain chapters
	 * @throws DeftIncompatibleNameException if the folder cannot contain a chapter with the given name
	 */
	public Chapter createChapter(Folder parent, String name, String chapterType)
			throws DeftNullArgumentException,
			DeftInvalidArgumentException,
			DeftIncompatibleTypeException,
			DeftIncompatibleNameException;
	
	
	/**
	 * Returns the type of the chapter (XHTML, Docbook, ...). The type is determined
	 * by the parent project. All chapters of a project have the same type.
	 * @param chapter the chapter for which to get the type
	 * @return the chapter type
	 * @throws DeftInvalidArgumentException if the chapter cannot be found in the repository
	 */
	public String getChapterType(Chapter chapter)
			throws DeftInvalidArgumentException;

	
	/**
	 * Creates a new tutorial within a project. The tutorial lies at the root position
	 * of the project, i.e. it is not contained in a folder.
	 *  
	 * @param project the project to create the tutorial in
	 * @param name the name of the tutorial
	 * @return a new tutorial with the specified name
	 * @throws DeftNullArgumentException if either project or name are null
	 * @throws DeftInvalidArgumentException if the project cannot be found in the repository
	 * @throws DeftIncompatibleNameException if the project root cannot contain a tutorial with the given name
	 */
	public Tutorial createTutorial(Project project, String name)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException,
			DeftIncompatibleNameException;
	
	
	/**
	 * Creates a new tutorial within in a folder. 
	 *  
	 * @param parent the folder to create the tutorial in
	 * @param name the name of the tutorial
	 * @return a new tutorial with the specified name
	 * @throws DeftNullArgumentException if either parent or name are null
	 * @throws DeftInvalidArgumentException if the chapter cannot be found in the repository
	 * @throws DeftIncompatibleTypeException if the specified folder cannot contain tutorials
	 * @throws DeftIncompatibleNameException if the folder cannot contain a tutorial with the given name
	 */
	public Tutorial createTutorial(Folder parent, String name)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException,
			DeftIncompatibleTypeException,
			DeftIncompatibleNameException;
	

	public Artifact importArtifact(Project project, FragmentSource fragmentSource, 
			OriginalFragmentLocation location);
	
	public Artifact importArtifact(Project project, String artifactTypeId, 
			FragmentSource fragmentSource, OriginalFragmentLocation location, String name);
	
	public Artifact importArtifact(Project project, String artifactTypeId, 
			FragmentSource fragmentSource, OriginalFragmentLocation location, 
			String name, int revision);

	
	public Artifact importArtifact(Folder parent, FragmentSource fragmentSource, 
			OriginalFragmentLocation location);
	
	public Artifact importArtifact(Folder parent, String artifactTypeId, 
			FragmentSource fragmentSource, OriginalFragmentLocation location, String name);
	
	public Artifact importArtifact(Folder parent, String artifactTypeId, 
			FragmentSource fragmentSource, OriginalFragmentLocation location, 
			String name, int revision, String... dependentArtifactIds);

	
	public void updateArtifact(Artifact artifact, int revision);
	
	public void updateArtifactWith(Artifact artifact, FragmentSource fragmentSource, 
			OriginalFragmentLocation location, int revision);
		
	/**
	 * Creates a new folder within a project. The folder lies at the root position
	 * of the project, i.e. it is not contained in another folder.
	 * 
	 * It must be specified what kind of fragments the folder should be
	 * able to contain. If, for example, the folder should contain
	 * artifacts, its fragType must be FragmentType.ARTIFACT.
	 * 
	 * For all kinds of fragments there is a corresponding fragment type defined in enum
	 * FragmentType.
	 * 
	 * @param project the project to create the folder in
	 * @param fragType the type of the folder
	 * @param name the name of the folder
	 * @return a new folder with the specified name and type
	 */
	public Folder createFolder(Project project, FragmentType fragType, String name)
			throws DeftNullArgumentException,
			DeftInvalidArgumentException,
			DeftIncompatibleNameException;
	
	/**
	 * Creates a new folder as child of an existing folder.
	 * 
	 * The type of the new folder is the same as the type of the existing folder.
	 * 
	 * @param parent the folder under which to create the new folder
	 * @param name the name of the new folder
	 * @return the new folder
	 */
	public Folder createFolder(Folder parent, String name)
			throws DeftNullArgumentException,
			DeftInvalidArgumentException,
			DeftIncompatibleNameException;
	
	/**
	 * Moves a fragment to a folder. If null is passed as target folder, the fragment
	 * will be moved to the root. 
	 * 
	 * @param fragment the fragment to move
	 * @param target the folder to which to move the fragment
	 */
	public void moveToFolder(Fragment fragment, Folder target)
			throws DeftNullArgumentException,
			DeftCrossProjectRelationException,
			DeftIncompatibleTypeException,
			DeftIncompatibleNameException;
	
	/**
	 * Returns the parent folder of a fragment, or null if
	 * the fragment is at the project root and has no parent folder.
	 * 
	 * @param fragment the fragment for which to find the parent folder
	 * @return the parent folder or null if there is no parent folder
	 * @throws DeftNullArgumentException if fragment is null
	 */
	public Folder getParentFolder(Fragment fragment)
		throws DeftNullArgumentException;
	
	/**
	 * Returns all child fragments of a folder. If the folder is empty,
	 * an empty list is returned.
	 * 
	 * @param folder the folder for which to get all children
	 * @return a list of all children of the folder
	 * @throws DeftNullArgumentException if folder is null
	 */
	public List<Fragment> getChildren(Folder folder) 
			throws DeftNullArgumentException,
			DeftInvalidArgumentException;
	
	
	/**
	 * Returns all fragments of a specific type from the project root
	 * (i.e., for fragments which have no parent folder). If there are no
	 * fragments of the wanted type, an empty list is returned.
	 *  
	 * @param project the project for which to get the child fragments
	 * @param fragmentType the type of fragments to retrieve
	 * @return a list of fragments of the specified type from the project root
	 */
	public List<Fragment> getChildren(Project project, FragmentType fragmentType)
			throws DeftNullArgumentException,
			DeftInvalidArgumentException;
	
	/**
	 * Returns the project to which a fragment belongs.
	 * 
	 * @param fragment the fragment for which to return the project
	 * @return the project of the fragment
	 */
	public Project getProjectOf(Fragment fragment)
			throws DeftNullArgumentException,
			DeftInvalidArgumentException;
	
	/**
	 * Returns a list of all projects. If no projects exist, an empty list
	 * is returned.
	 * 
	 * @return a list of all projects
	 */
	public List<Project> getProjects();
	
	/**
	 * Returns the project with the given name. If no such project exists,
	 * null is returned.
	 * 
	 * @param name the name of the searched project
	 * @return the project with the given name, or null
	 * @throws DeftNullArgumentException
	 */
	public Project getProject(String name)
			throws DeftNullArgumentException;
	

	/**
	 * Returns the project with the given id. If no such project exists,
	 * null is returned.
	 * 
	 * @param id the id of the searched project
	 * @return the project with the given id, or null
	 * @throws DeftNullArgumentException
	 */
	public Project getProjectById(String id);


	
	/**
	 * Returns a list of all representations of an artifact.
	 * If the artifact has no representations, an empty list is returned.
	 * 
	 * @param artifact the artifact for which to find the representations
	 * @return a list of representations of the artifact
	 * @throws DeftNullArgumentException if artifact is null
	 */
	public List<ArtifactRepresentation> getRepresentationsForArtifact(Artifact artifact)
			throws DeftNullArgumentException,
			DeftInvalidArgumentException;

	/**
	 * Adds a chapter to a tutorial. If the chapter has already been added to the tutorial, nothing happens.
	 *   
	 * @param tutorial the tutorial to which to add the chapter
	 * @param chapter the chapter to be added to the tutorial
	 * @throws DeftNullArgumentException if tutorial or chapter are null
	 * @throws DeftCrossProjectRelationException if tutorial and chapter belong to different projects
	 */
	public void addChapterToTutorial(Tutorial tutorial, Chapter chapter)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException, 
			DeftCrossProjectRelationException;
	
	/**
	 * Removes a chapter from a tutorial. If the chapter is not part
	 * of the tutorial, nothing happens.
	 *   
	 * @param tutorial the tutorial from which to remove the chapter
	 * @param chapter the chapter to be removed from the tutorial
	 * @throws DeftNullArgumentException if tutorial or chapter are null
	 * @throws DeftCrossProjectRelationException if tutorial and chapter are from different projects 
	 */
	public void removeChapterFromTutorial(Tutorial tutorial, Chapter chapter)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException;
	
	/**
	 * Returns all chapters of a tutorial. If the tutorial has no chapters,
	 * an empty list is returned.
	 * 
	 * @param tutorial the tutorial for which to get the chapters
	 * @return a list of chapters of the tutorial
	 * @throws DeftNullArgumentException if tutorial is null
	 */
	public List<Chapter> getChaptersForTutorial(Tutorial tutorial)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException;
	
	/**
	 * Returns all tutorials in which a chapter is contained. If the
	 * chapter is not part of any tutorial, an empty list is
	 * returned.
	 * 
	 * @param chapter the chapter for which to get the tutorials
	 * @return a list of tutorials the chapter is in
	 * @throws DeftNullArgumentException if chapter is null
	 */
	public List<Tutorial> getTutorialsForChapter(Chapter chapter)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException;
	
	/**
	 * Returns the fragment with a given id. If there is no fragment with that id,
	 * null is returned.
	 * 
	 * @param id the id of the fragment to be retrieved
	 * @return the fragment with the given id, or null
	 * @throws DeftNullArgumentException if id is null
	 */
	public Fragment getFragment(String id)
			throws DeftNullArgumentException;
	
	
	/**
	 * Returns a list of fragments from a project. Only fragments
	 * matching the filter are contained in the list. If the filter is 
	 * null, all fragments of the project are returned. If no fragments match,
	 * an empty list is returned.
	 * 
	 * Please note: in contrast to {@link #getChildren(Project, FragmentType)}
	 * ALL fragments are considered, not only the fragments from the project
	 * root.
	 * 
	 * @param project the project for which to get fragments
	 * @param filter a filter specifying what fragments to return, or <code>null</code>
	 * @return a list of fragments
	 * @throws DeftNullArgumentException if project is null
	 */
	public List<Fragment> getFragments(Project project, FragmentFilter filter)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException;
	
	
	/**
	 * Returns the fragment type that the folder can contain.
	 * 
	 * A folder has the fragment type {@link FragmentType#FOLDER}. A folder that
	 * can contain artifacts, for example, has the fragment type affiliation
	 * {@link FragmentType#ARTIFACT}.
	 * 
	 * @param folder the folder for which to get the type of fragments it can contain
	 * @return the type of fragments that the folder can contain
	 * @throws DeftNullArgumentException if folder is null
	 */	
	public FragmentType getFragmentTypeAffiliation(Folder folder)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException;
	
	/**
	 * Sets the position of a chapter in a tutorial. The first position is 0,
	 * the last position is listsize-1.
	 * 
	 * A tutorial has an ordered list of chapters. The order determines 
	 * the reading order when the tutorial is exported (to HTML, or PDF, for example).
	 * This method changes the chapter order of the tutorial.
	 * 
	 * If the chapter is not part of the tutorial, nothing happens. The same holds if 
	 * the position is less than 0 or greater than the number of chapters in the tutorial.
	 * Otherwise the chapter will be moved to the according position in the list.
	 *  
	 * @param tutorial the tutorial in which to move the chapter
	 * @param chapter the chapter to move in the tutorial
	 * @param position the position to move the chapter to
	 * @throws DeftNullArgumentException if tutorial or chapter are null
	 * @throws DeftCrossProjectRelationException if tutorial and chapter are from different projects
	 */
	public void setChapterPosition(Tutorial tutorial, Chapter chapter, int position)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException,
			DeftCrossProjectRelationException;
	
	
	/**
	 * Gets the position of a chapter in a tutorial. The first position is 0.
	 * 
	 * A tutorial has an ordered list of chapters. The order determines 
	 * the reading order when the tutorial is exported (to HTML, or PDF, for example).
	 * 
	 * If the chapter is not part of the tutorial, -1 is returned.
	 *  
	 * @param tutorial the tutorial for which to get the position of the chapter
	 * @param chapter the chapter for which to get the position
	 * @throws DeftNullArgumentException if tutorial or chapter are null
	 * @throws DeftCrossProjectRelationException if tutorial and chapter are from different projects
	 */
	public int getChapterPosition(Tutorial tutorial, Chapter chapter);
	
	
	/**
	 * Renames a project.
	 * @param project the project to be renamed
	 * @param newName the new name of the project
	 * @throws DeftNullArgumentException if project or newName are null
	 * @throws DeftProjectAlreadyExistsException if there is already a project with the given name
	 */
	public void rename(Project project, String newName)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException, 
			DeftProjectAlreadyExistsException;
	
	/**
	 * Renames a fragment.
	 * @param fragment the fragment to be renamed
	 * @param newName the new name of the fragment
	 * @throws DeftNullArgumentException if fragment or newName are null
	 */
	public void rename(Fragment fragment, String newName)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException, 
			DeftIncompatibleNameException;
	

	
	/**
	 * Removes a project including all its fragments. If some fragment could not be
	 * removed for any reason, removal will stop immediately and false will be returned.
	 * 
	 * @param project the project to be removed
	 * @return true if all fragments and the project could be deleted, otherwise false
	 * @throws DeftNullArgumentException if project is null
	 */
	public boolean removeProject(Project project)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException;
	

	/**
	 * Removes a fragment. 
	 * 
	 * If the fragment has relations to other fragments, such as a chapter being part of a tutorial, those 
	 * relationships are also removed. 
	 * 
	 * If the fragment is a folder, all of its descendants will also be removed. If one of the descendants
	 * could not be removed for any reason, removal will stop immediately and false will be returned.
	 * 
	 * @param fragment the fragment to be removed
	 * @return true if the fragment and all its descendants could be deleted, otherwise false
	 * @throws DeftNullArgumentException if fragment is null
	 */	
	public boolean removeFragment(Fragment fragment)
			throws DeftNullArgumentException, 
			DeftInvalidArgumentException;
	
	
	public void addRepositoryUpdateListener(RepositoryUpdateListener listener);
	public void removeRepositoryUpdateListener(RepositoryUpdateListener listener);
	

	/**
	 * Checks if a project with a given name exists.
	 * 
	 * @param name the name of the project
	 * @return <code>true</code> if there is a project with such a name, 
	 * otherwise <code>false</code>
	 */
	public boolean projectExists(String name);
	
	/**
	 * Checks if a folder can contain a fragment of a given type.
	 * 
	 * Folders are not allowed to contain fragments of a different type, e.g.
	 * both Artifacts and Images. Therefore the type of fragments that a folder may
	 * contain must be identified and compared to the fragmentType parameter.
	 * 
	 * @param parent the folder to be checked
	 * @param fragmentType a type of fragment, such as chapter or code file
	 * @return <code>true</code> if the folder can contain a fragment of the type,
	 * otherwise <code>false</code>
	 */
	public boolean canContainFragmentType(Folder parent, FragmentType fragmentType);
	
	/**
	 * Checks if a folder can contain a fragment of the given name.
	 * 
	 * A folder can contain a fragment with a given name if it does not
	 * already have a fragment with that name, or if multiple fragments
	 * with the same name are allowed.
	 * 
	 * @param parent the folder for which to check if the child name is allowed
	 * @param childName the name for which to check if it can occur in the parent folder
	 * @return <code>true</code> if it is possible to have a fragment with the specified
	 * name in the folder, otherwise <code>false</code>
	 */
	public boolean canContainFragmentName(Folder parent, String childName);
	
	/**
	 * Checks if a project can contain a fragment of the given name.
	 * 
	 * This method is similar to {@link #canContainFragmentName(Folder, String)}. 
	 * Fragments need not necessarily be in a folder, they can also appear detachedly
	 * at the "root". However, it is still necessary to check if the "root" is allowed
	 * to contain additional fragments with a given name. The types of fragments (Chapters,
	 * Artifacts, ...) are treated independently of each other.
	 *   
	 * @param project the project for which to check the fragment name
	 * @param fragmentType the type of fragment for which to check if a child name is allowed
	 * @param childName the name for which to check if it can occur
	 * @return <code>true</code> if it is possible to have a fragment with the specified
	 * type and name in the "virtual root folder", otherwise <code>false</code>
	 */
	public boolean canContainFragmentName(Project project, FragmentType fragmentType, String childName);
	
	/**
	 * Checks if a Folder can contain a specific fragment.
	 * 
	 * A folder can only contain fragments from the same project.
	 * 
	 * A folder can contain a fragment with a given name if it does not
	 * already have a fragment with that name, or if multiple fragments
	 * with the same name are allowed.
	 * 
	 * If child is already contained in parent, <code>false</code> is returned.
	 * 
	 * If child is a folder, it must have the same type as parent, and it must not currently be
	 * an ancestor of parent or even the same as parent (i.e. you can't move a folder into 
	 * itself or one of its child folders).
	 * 
	 * If the folder is <code>null</code>, it will be checked if the fragment can be
	 * added to the root of its project. 
	 *   
	 * @param parent the folder for which to check if the child is allowed, or the project root,
	 * if it is <code>null</code>
	 * @param child the fragment for which to check if it can occur in the parent folder
	 * @return <code>true</code> if it is possible to have child in the folder, 
	 * otherwise <code>false</code>
	 */
	public boolean canContainFragment(Folder parent, Fragment child);

	/**
	 * Checks if the project root can contain a specific fragment.
	 * 
	 * The project root can only contain fragments from the same project.
	 * 
	 * The project root can contain a fragment with a given name and type if it does not
	 * already have a fragment with that name and type, or if multiple fragments
	 * with the same name are allowed.
	 * 
	 * If child is already contained in the project root, <code>false</code> is returned.
	 * 
	 * @param project the project for which to check if the child is allowed at the root
	 * @param fragmentType the type of fragment for which to check if a child is allowed at the root
	 * @param child the fragment for which to check if it can occur in the project
	 * @return <code>true</code> if it is possible to have child in the project root
	 * otherwise <code>false</code>
	 */
	public boolean canContainFragmentAtRoot(Project project, FragmentType fragmentType, Fragment child);
	
	/**
	 * Checks if there is a fragment with a given name in a folder.
	 * 
	 * @param parent the folder in which to look for a fragment with the name
	 * @param childName the name to look for
	 * @return <code>true</code>if the folder contains a fragment with the name, 
	 * otherwise <code>false</code>
	 */
	public boolean containsFragmentName(Folder parent, String childName);	
	
	/**
	 * Checks if there is a fragment with a given type and name at the root of the project.
	 * 
	 * @param project the project for which to check the fragment name
	 * @param fragmentType the type of fragment for which to check if a child name is allowed
	 * @param childName the name to look for
	 * @return <code>true</code>if the folder contains a fragment with the name, 
	 * otherwise <code>false</code>
	 */
	public boolean containsFragmentNameAtRoot(Project project, FragmentType fragmentType, String childName);

	/**
	 * Checks if a chapter can be added to a tutorial.
	 * 
	 * A chapter can be added if it has not already been added to the tutorial.
	 * 
	 * @param tutorial the tutorial to which one would like to add the chapter
	 * @param chapter the chapter to be added
	 * @return <code>true</code>, if the chapter is not yet in the tutorial, 
	 * otherwise <code>false</code>
	 */
	public boolean canContainChapter(Tutorial tutorial, Chapter chapter);
	
	/**
	 * Checks if a chapter already exists in a tutorial.
	 * @param tutorial the tutorial for which to check for the existing chapter
	 * @param chapter the chapter for which to check in the tutorial
	 * @return <code>true</code> if the chapter has already been added to the tutorial,
	 * otherwise <code>false</code>.
	 */
	public boolean containsChapter(Tutorial tutorial, Chapter chapter);
	

	public byte[] getChapterContent(Chapter chapter);
	public byte[] getArtifactContent(Artifact artifact);
	public byte[] getArtifactContent(Artifact artifact, int revision);
	public byte[] getContent(Artifact artifact, String dependentFileId);
	public byte[] getContent(Artifact artifact, String dependentFileId, int revision);
	public String getOriginalLocation(Artifact artifact);
	public String getOriginalLocation(Artifact artifact, int revision);
	public String getOriginalLocation(Artifact artifact, String dependentFileId);
	public String getOriginalLocation(Artifact artifact, String dependentFileId, int revision);
	public List<String> getDependentFileIds(Artifact artifact);
	public List<String> getDependentFileIds(Artifact artifact, int revision);

	
	public void saveChapterContent(Chapter chapter, byte[] content);
	
	
	public ArtifactReference getArtifactReference(String refId);
	public ArtifactReference createArtifactReference(Chapter chapter, Artifact artifact,
			OperationChain operationChain, Integrator integrator);
	public void updateArtifactReference(ArtifactReference reference,
			OperationChain operationChain, Integrator integrator);
	
	public boolean removeArtifactReference(ArtifactReference reference);

	public List<ArtifactReference> getArtifactReferences(Project project);
	public List<ArtifactReference> getArtifactReferences(Chapter chapter);
	public List<ArtifactReference> getArtifactReferences(Artifact artifact);
	
	
	public Chapter getChapter(ArtifactReference reference);
	public Artifact getArtifact(ArtifactReference reference);
	public int getLatestCheckedRevision(ArtifactReference reference);
	public List<ArtifactReference> getUncheckedReferences(Project project);
	public List<ArtifactReference> getUncheckedReferences(Chapter chapter);
	public boolean isUnchecked(ArtifactReference reference);
	public void setArtifactReferenceChecked(ArtifactReference reference);
	public void setArtifactReferenceUnchecked(ArtifactReference reference);
	public ArtifactReferenceUpdateType getUpdateType(ArtifactReference reference);

	
	/**
	 * Returns the OperationChain which belongs to the reference. If the OperationChain
	 * had been configured (some parameters modified) by the user, this configuration
	 * will be restored.
	 *  
	 * @param ref the reference for which to return the OperationChain
	 * @return the configured OperationChain
	 */
	public OperationChain getOperationChain(ArtifactReference ref);
	
	public FileIntegrator getFileIntegrator(ArtifactReference ref);
	
	public void setConfiguration(Integrator integrator, ArtifactReference reference); //hack, remove in next version
	
	public List<Artifact> getArtifactsWithMissingOrigin(Project project);
	
	public List<Artifact> getArtifactsWithChangedOrigin(Project project);

	public int getLatestRevision(Project project);
}
