package org.deft.repository.repositorymanager;

import java.util.List;

import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.event.IContentChangeListener;


/**
 * The ContentManager is a part of the DEFT repository. It is responsible to
 * maintain the project and fragment objects and their properties (names, ids, ...).
 * 
 * This class is ONLY meant to be used by the repository implementation. Methods are public
 * for technical reasons only. No consistency checks are performed here, every input should 
 * have been checked by the repository BEFORE calling methods from the ContentManager.
 */
public interface ContentManager {
	
	/**
	 * Removes all content.
	 */
	public void clear();
	
	/**
	 * Creates a new project.
	 * 
	 * @param name the name of the project
	 * @param projectType the type of the project (xhtml, latex, ...)
	 * @return a new project with the specified name and type
	 */
	public Project createProject(String name, String projectType);
	
	/**
	 * Returns the project type (xhtml, latex, ...).
	 * @param project the project for which to get the type
	 * @return the type of the project
	 */
	public String getProjectType(Project project);
	

	/**
	 * Creates a new chapter within a project.
	 * 
	 * If the chapter should be added to a specific folder, call method
	 * {@link #setParentFolder(Fragment, Folder)} after this method.
	 *  
	 * @param project the project to create the chapter in
	 * @param name the name of the chapter
	 * @param chapterType the type of the chapter
	 * @return a new chapter with the specified name and type
	 */
	public Chapter createChapter(Project project, String name, String chapterType);

	/**
	 * Returns the type of the chapter (XHTML, Docbook, ...). The type is determined
	 * by the parent project. All chapters of a project have the same type.
	 * @param chapter the chapter for which to get the type
	 * @return the chapter type
	 */
	public String getChapterType(Chapter chapter);
	
	/**
	 * Creates a new tutorial within a project.
	 * 
	 * If the tutorial should be added to a specific folder, call method
	 * {@link #setParentFolder(Fragment, Folder)} after this method.
	 *  
	 * @param project the project to create the tutorial in
	 * @param name the name of the tutorial
	 * @return a new tutorial with the specified name
	 */
	public Tutorial createTutorial(Project project, String name);
	
	/**
	 * Creates a new artifact within a project.
	 * 
	 * If the artifact should be added to a specific folder, call method
	 * {@link #setParentFolder(Fragment, Folder)} after this method.
	 */
	public Artifact createArtifact(Project project, String fragmentSourceId, 
			String location, String artifactTypeId, 
			int revision, MultiArtifactDependencies dependencies, String name);
	
	
	
	
	/**
	 * Creates a new folder within a project.
	 * 
	 * It must be specified what kind of fragments the folder should be
	 * able to contain. If, for example, the folder should contain
	 * artifacts, its <code>fragType</code> must be FragmentType.ARTIFACT.
	 * 
	 * For all kinds of fragments there is a corresponding fragment type defined.
	 * 
	 * @param project the project to create the folder in
	 * @param fragType the type of the folder
	 * @param name the name of the folder
	 * @return a new folder with the specified name and type
	 */
	public Folder createFolder(Project project, FragmentType fragType, String name);
	
	/**
	 * Creates a new folder as child of an existing folder.
	 * 
	 * The type of the new folder is the same as the type of the existing folder.
	 * 
	 * @param parent the folder under which to create the new folder
	 * @param name the name of the new folder
	 * @return the new folder
	 */
	public Folder createFolder(Folder parent, String name);
	
	/**
	 * Moves a fragment to a folder.
	 * 
	 * @param fragment the fragment to move
	 * @param parentFolder the folder to which to move the fragment
	 */
	public void setParentFolder(Fragment fragment, Folder parentFolder);
	
	/**
	 * Returns the parent folder of a fragment, or <code>null</code> if
	 * the fragment resides at the project root and has no parent folder.
	 * 
	 * @param fragment the fragment for which to find the parent folder
	 * @return the parent folder or <code>null</code> if there is no parent folder
	 */
	public Folder getParentFolder(Fragment fragment);
	
	/**
	 * Returns all child fragments of a folder. If the folder is empty,
	 * an empty list is returned.
	 * 
	 * @param folder the folder for which to get all children
	 * @return a list of all children of the folder
	 */
	public List<Fragment> getChildren(Folder folder);
	
	/**
	 * Returns all fragments of a specific type from the project root
	 * (i.e., the fragments have no parent folder). This includes folders
	 * with the appropriate affiliation. (see {@link #getFragmentTypeAffiliation(Folder)}).
	 * If there are no fragments of the wanted type, an empty list is returned.
	 * 
	 * If fragmentType is FOLDER, all root folders (i.e. folders which have
	 * no parent folder) are returned.
	 *  
	 * @param project the project for which to get the child fragments
	 * @param fragmentType the type of fragments to retrieve
	 * @return a list of fragments with the specified type from the project root
	 */
	public List<Fragment> getChildren(Project project, FragmentType fragmentType);
	
	/**
	 * Returns the project to which a fragment belongs.
	 * 
	 * @param fragment the fragment for which to return the project
	 * @return the project of the fragment
	 */
	public Project getProjectOf(Fragment fragment);
	
	/**
	 * Returns a list of all projects. If no projects exist, an empty list
	 * is returned.
	 * 
	 * @return a list of all projects
	 */
	public List<Project> getProjects();
	
	/**
	 * Returns the project with the given name. If no such project exists,
	 * <code>null</code> is returned.
	 * 
	 * @param name the name of the searched project
	 * @return the project with the given name, or <code>null</code>
	 */
	public Project getProject(String name);

	/**
	 * Returns the Artifact to which the occurrence refers.
	 * @param occurrence the occurrence for which to find the 
	 * associated artifact
	 * @return the artifact that a occurrence refers to
	 */
//	public Artifact getBaseArtifact(ArtifactOccurrence occurrence);
	
	
	/**
	 * Returns a list of all occurrences of an artifact.
	 * If the artifact has no occurrences, an empty list is returned.
	 * 
	 * @param artifact the artifact for which to find the occurrences
	 * @return a list of occurrences of the artifact
	 */
//	public List<ArtifactOccurrence> getOccurrencesForArtifact(Artifact artifact);

	/**
	 * Adds a chapter to a tutorial. 
	 *   
	 * @param tutorial the tutorial to which to add the chapter
	 * @param chapter the chapter to be added to the tutorial
	 */
	public void addChapterToTutorial(Tutorial tutorial, Chapter chapter);
	
	/**
	 * Removes a chapter from a tutorial. If the chapter is not part
	 * of the tutorial, nothing happens.
	 *   
	 * @param tutorial the tutorial from which to remove the chapter
	 * @param chapter the chapter to be removed from the tutorial
	 */
	public void removeChapterFromTutorial(Tutorial tutorial, Chapter chapter);
	
	/**
	 * Returns all chapters of a tutorial. If the tutorial has no chapters,
	 * an empty list is returned.
	 * 
	 * @param tutorial the tutorial for which to get the chapters
	 * @return a list of chapters of the tutorial
	 */
	public List<Chapter> getChaptersForTutorial(Tutorial tutorial);
	
	/**
	 * Returns all tutorials in which a chapter is contained. If the
	 * chapter does not participate in any tutorial, an empty list is
	 * returned.
	 * 
	 * @param chapter the chapter for which to get the tutorials
	 * @return a list of tutorials the chapter is in
	 */
	public List<Tutorial> getTutorialsForChapter(Chapter chapter);
	
	/**
	 * Returns the fragment with a given id. If there is no fragment with that id,
	 * <code>null</code> is returned.
	 * 
	 * @param id the id of the fragment to be retrieved
	 * @return the fragment with the given id, or <code>null</code>
	 */
	public Fragment getFragment(String id);
	
	
	/**
	 * Returns the ArtifactOccurrence with a given id. 
	 * If there is no occurrence with that id,
	 * <code>null</code> is returned.
	 * 
	 * @param id the id of the occurrence to be retrieved
	 * @return the occurrence with the given id, or <code>null</code>
	 */
//	public ArtifactOccurrence getOccurrence(String id);
	
	
	/**
	 * Returns a list of fragments from a project. Only fragments
	 * matching the filter are contained in the list. If the filter is 
	 * <code>null</code>, all fragments of the project are returned.
	 * 
	 * Please note: in contrast to {@link #getChildren(Project, FragmentType)}
	 * ALL fragments are considered, not only the fragments from the project
	 * root.
	 * 
	 * @param project the project for which to get fragments
	 * @param filter a filter specifying what fragments to return, or <code>null</code>
	 * @return a list of fragments
	 */
	public List<Fragment> getFragments(Project project, FragmentFilter filter);
	
	
	/**
	 * Returns the fragment type that the folder can contain.
	 * 
	 * A folder has the fragment type {@link FragmentType#FOLDER}. A folder that
	 * can contain artifacts, for example, has the fragment type affiliation
	 * {@link FragmentType#ARTIFACT}.
	 * 
	 * @param folder the folder for which to get the type of fragments it can contain
	 * @return the type of fragments that the folder can contain
	 */	
	public FragmentType getFragmentTypeAffiliation(Folder folder);
	
	/**
	 * Sets the position of a chapter in a tutorial. The first position is 0,
	 * the last position is listsize-1.
	 * 
	 * A tutorial has an ordered list of chapters. The order determines 
	 * the reading order when the tutorial is exported (to HTML, or PDF, for example).
	 * This method changes the chapter order of the tutorial.
	 * 
	 * If the chapter is not part of the tutorial, nothing happens. The same holds if 
	 * the position is less than 0 or greater than the number of chapters in the tutorial.
	 * Otherwise the chapter will be moved to the according position in the list.
	 *  
	 * @param tutorial the tutorial in which to move the chapter
	 * @param chapter the chapter to move in the tutorial
	 * @param position the position to move the chapter to
	 */
	public void setChapterPosition(Tutorial tutorial, Chapter chapter, int position);
	
	
	/**
	 * Gets the position of a chapter in a tutorial. The first position is 0.
	 * 
	 * A tutorial has an ordered list of chapters. The order determines 
	 * the reading order when the tutorial is exported (to HTML, or PDF, for example).
	 * 
	 * If the chapter is not part of the tutorial, -1 is returned.
	 *  
	 * @param tutorial the tutorial for which to get the position of the chapter
	 * @param chapter the chapter for which to get the position
	 */
	public int getChapterPosition(Tutorial tutorial, Chapter chapter);
	
	
	
	public void rename(Project project, String newName);
	public void rename(Fragment fragment, String newName);
	
	public void remove(Fragment fragment);
	public void remove(Project project);

	
	public ArtifactReference getArtifactReference(String refId);

	public ArtifactReference createArtifactReference(Chapter chapter, Artifact artifact,
			String operationChainId, String integratorId);
	public void removeArtifactReference(ArtifactReference reference);
	public void clearArtifactOfArtifactReference(ArtifactReference reference);
	
	public List<ArtifactReference> getArtifactReferences(Project project);
	public List<ArtifactReference> getArtifactReferences(Chapter chapter);
	public List<ArtifactReference> getArtifactReferences(Artifact artifact);
	
	public Chapter getChapter(ArtifactReference reference);
	public Artifact getArtifact(ArtifactReference reference);

	public int getLatestCheckedRevision(ArtifactReference reference);
	public void setLatestCheckedRevision(ArtifactReference reference, int latestRevision);
	public ArtifactReferenceUpdateType getUpdateType(ArtifactReference reference);
	public void setUpdateType(ArtifactReference reference, ArtifactReferenceUpdateType updateType);
	
	public void addContentChangeListener(IContentChangeListener listener);
	public void removeContentChangeListener(IContentChangeListener listener);
	
	public String getOriginalLocation(Artifact artifact, int revision);
	public String getOriginalLocation(Artifact artifact, String dependentFileId, int revision);


	
	
	public void addRevisionIndex(Artifact artifact, int revision, String location, MultiArtifactDependencies dependencies);
	public void removeRevision(Artifact artifact, int revision);	
	
	public List<Integer> getRevisionIndexes(Artifact artifact);
	public int getLatestRevision(Project project);
	public int getLatestRevision(Artifact artifact);
	public List<String> getDependentFileIds(Artifact artifact, int revision);

}
