package org.deft.repository.repositorymanager;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.deft.operation.OperationChain;
import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.integrator.Integrator;
import org.w3c.dom.Document;

/**
 * The DataStorageManager writes and reads the contents of projects
 * and their fragments into and from the repository. This only includes
 * pure content data (such as the content of a Chapter). Metadata such as
 * names and ids is maintained by the ContentManager. 
 * 
 * The boolean return types indicate if the method was executed successfully.
 * External factors (e.g. files to be deleted are opened in an external editor)
 * might prevent the methods from successful execution. The calling method
 * must know if the DataStorageManager was unable to perform a certain task,
 * in order to take appropriate measures.
 */
public interface DataStorageManager {
	
	/**
	 * Writes data regarding the project into the repository.
	 */
	public boolean createProject(Project project);
	
	/**
	 * Returns the temporary directory for the project.
	 */
	public File getTempDirectory(Project project);
	
	/**
	 * Removes all project related information from the repository.
	 */
	public boolean removeProject(Project project);

	/**
	 * Adds the content of an artifact to the repository under a certain revision number.
	 * 
	 * @param artifact the artifact whose content to add
	 * @param revision the revision (version) of the content to be added.
	 * Each artifact can contain multiple revisions, which represent the
	 * evolution of its content.
	 */
	public boolean createArtifactContent(Artifact artifact, int revision, 
			String originalLocation, MultiArtifactDependencies dependencies)
			throws IOException;
	
	/**
	 * Removes the content of a specific revision of an artifact.
	 */
	public boolean removeArtifactContent(Artifact artifact, int revision);
	
	/**
	 * Removes the content of all revisions of an artifact.
	 */
	public boolean removeArtifactContent(Artifact artifact);
	

	/**
	 * Adds the content of a chapter to the repository.
	 */
	public boolean createChapterContent(Chapter chapter);
	
	/**
	 * Removes the content of a chapter to the repository.
	 */
	public boolean removeChapterContent(Chapter chapter);
	

	/**
	 * Adds the specific configuration of an ArtifactReference to the repository. 
	 *
	 * @param reference the reference for which to add the contents
	 * @param chain the OperationChain whose configuration is the actual data to be stored
	 * @param integrator the Integrator whose configuration is the actual data to be stored
	 */
	public boolean createArtifactReferenceContent(ArtifactReference reference, 
			OperationChain chain, Integrator integrator);
	
	/**
	 * Removes the configuration of an ArtifactReference from the repository.
	 */
	public boolean removeArtifactReferenceContent(ArtifactReference reference);
	
	/**
	 * Returns an XML document representing the operation chain configuration of the ArtifactReference 
	 */
	public Document getArtifactReferenceOperationChainConfigContent(ArtifactReference reference);
	
	/**
	 * Returns an XML document representing the integrator configuration of the ArtifactReference 
	 */
	public Document getArtifactReferenceIntegratorConfigContent(ArtifactReference reference);
	
	
	
	public InputStream getInputStream(Chapter chapter);
	public File getFile(Chapter chapter);
	
	public InputStream getInputStream(Artifact artifact, int revision);
	public InputStream getInputStream(Artifact artifact, String dependentFileId, int revision);
	
	public OutputStream getOutputStream(Chapter chapter);
	
	/**
	 * Returns if the data (e.g. file) from which the artifact was created
	 * still exists at the same location that is stored in the Artifact.
	 */
	public boolean isArtifactOriginStillExisting(Artifact artifact);
	
	/**
	 * Returns whether the data (e.g. file) from which the artifact was
	 * created is different from the artifact content in a specific revision.
	 * If the original data (file) is missing, false is returned.
	 */
	public boolean hasArtifactOriginChanged(Artifact artifact, int revision);

}
