package org.deft.repository.repositorymanager.actions;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;

public abstract class RemoveAction extends AbstractRepositoryAction {

	protected ContentManager contentManager;
	private Project project;

	public RemoveAction(ContentManager cm, Project project) {
		this.contentManager = cm;
		this.project = project;
	}
	
	protected Project getProject() {
		return project;
	}
	
	/**
	 * This method creates and returns remove actions that have to be executed before THIS remove action
	 * can be executed.
	 * 
	 * Sometimes an entity from the repository cannot be removed because other entities depend on it.
	 * For example, a tutorial cannot be removed, while it contains chapters, or a project cannot be
	 * deleted while it has chapters, artifacts, and the like. The depending entities have to be
	 * removed first. The actions returned here will take care of that.
	 * 
	 * Implementations of that method must ensure that the list contains the actions in the order in
	 * which they can be executed. There are no additional checks. If the order is wrong, for example a
	 * folder removal is scheduled before the removal of an artifact in that folder, the repository will
	 * throw an exception or become invalid.
	 * 
	 * Furthermore, implementations must ensure that there is only one RemoveAction created for any entity
	 * in the repository. For example, if a chapter is in a tutorial, and both chapter and tutorial are to
	 * be deleted (happens if the whole project is being removed), both the {@link RemoveChapterAction} and 
	 * the {@link RemoveTutorialAction} will produce a {@link RemoveChapterFromTutorialAction}. However, only
	 * one RemoveChapterFromTutorialAction must be returned. The helper method 
	 * {@link #getCopyWithoutDuplicateActions(List)} should be used to remove duplicate actions. 
	 */
	public List<RemoveAction> getPriorRemoveActions() {
		return Collections.emptyList();
	}
	
	/**
	 * Creates a copy of a list of RemoveActions, in which duplicate actions are removed. The first
	 * action remains, duplicate actions that come later are removed.
	 *  
	 * A RemoveAction is considered a duplicate of another action if {@link #equals(Object)} returns true.
	 */
	public List<RemoveAction> getCopyWithoutDuplicateActions(List<RemoveAction> actions) {
		List<RemoveAction> actionsWithoutDuplicates = new LinkedList<RemoveAction>();
		for (RemoveAction action : actions) {
			if (!actionsWithoutDuplicates.contains(action)) {
				actionsWithoutDuplicates.add(action);
			}
		}
		return actionsWithoutDuplicates;
	}
	

	/**
	 * Helper method to create concrete RemoveActions from a generic fragment.
	 */
	public static RemoveAction createFragmentRemoveAction(Fragment fragment, ContentManager cm, DataStorageManager dsm) {
		if (fragment instanceof Chapter) {
			return new RemoveChapterAction(cm, dsm, (Chapter)fragment);
		} else if (fragment instanceof Artifact) {
			return new RemoveArtifactAction(cm, dsm, (Artifact)fragment);
		} else if (fragment instanceof Tutorial) {
			return new RemoveTutorialAction(cm, (Tutorial)fragment);
		} else if (fragment instanceof Folder) {
			return new RemoveFolderAction(cm, dsm, (Folder)fragment);
		}
		return null;
	}

	
}
