package org.deft.repository.repositorymanager.actions;

import java.io.IOException;
import java.util.List;

import org.deft.operation.OperationChain;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;
import org.deft.repository.service.ExtensionService;
import org.w3c.dom.Document;

public class UpdateArtifactWithAction extends AbstractRepositoryAction {

	private ContentManager contentManager;
	private DataStorageManager dataStorageManager;
	private Artifact artifact;
	private ArtifactType artifactType;
	private FragmentSource fragmentSource;
	private OriginalFragmentLocation location;
	private int revision;
	private ExtensionService extensionService;

	public UpdateArtifactWithAction(ExtensionService es, ContentManager cm, DataStorageManager dsm,
			Artifact artifact, ArtifactType artifactType, FragmentSource fragmentSource, 
			OriginalFragmentLocation location, int revision) {
		this.extensionService = es;
		this.contentManager = cm;
		this.dataStorageManager = dsm;
		this.artifact = artifact;
		this.artifactType = artifactType;
		this.fragmentSource = fragmentSource;
		this.location = location;
		this.revision = revision;
	}
	
	
	@Override
	public void execute() {
		String oldName = artifact.getName();
		MultiArtifactDependencies dependencies = setupDependencies();
		contentManager.addRevisionIndex(artifact, revision, location.getFullLocation(), dependencies);
		contentManager.rename(artifact, location.getName());
		try {
			executeSuccessful = dataStorageManager.createArtifactContent(artifact, revision, location.getFullLocation(), dependencies);
			setUpdateStatusForAllReferences();
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
			executeSuccessful = false;
		}
		if (!executeSuccessful) {
			dataStorageManager.removeArtifactContent(artifact, revision);
			contentManager.removeRevision(artifact, revision);
			contentManager.rename(artifact, oldName);
		}
	}
	
	
	private MultiArtifactDependencies setupDependencies() {
		Validator validator = artifactType.getValidator();
		MultiArtifactDependencies dependencies = validator.getMultiArtifactDependencies(fragmentSource, location);
		return dependencies;
	}
	
	
	private void setUpdateStatusForAllReferences() {
		List<ArtifactReference> references = contentManager.getArtifactReferences(artifact);
		for (ArtifactReference reference : references) {
			setUpdateStatus(reference);
		}
	}
	
	private void setUpdateStatus(ArtifactReference reference) {
		String chainId = reference.getOperationChainId();
		OperationChain chain = extensionService.getOperationChain(chainId);
		Document doc = dataStorageManager.getArtifactReferenceOperationChainConfigContent(reference);
		chain.getChainConfigurations().loadConfigurationForOperationChain(doc);
		//
		int oldRevision = contentManager.getLatestCheckedRevision(reference);
		ArtifactRepresentation oldRep = chain.executeOperationsOnRevision(artifact, oldRevision);
		ArtifactRepresentation newRep = chain.executeOperationsOnRevision(artifact, revision);
		ArtifactReferenceUpdateType updateType = getUpdateType(oldRep, newRep);
		contentManager.setUpdateType(reference, updateType);
	}
	
	
	private ArtifactReferenceUpdateType getUpdateType(ArtifactRepresentation oldRep, 
			ArtifactRepresentation newRep) {
		if (newRep == null) {
			return ArtifactReferenceUpdateType.UNCOMPUTABLEUPDATE;
		} else if (oldRep.equals(newRep)) {
			return ArtifactReferenceUpdateType.NOUPDATE;
		} else if (!oldRep.equals(newRep)) {
			return ArtifactReferenceUpdateType.NORMALUPDATE;
		}
		return null;
	}

	@Override
	public FragmentUpdateEvent createEvent(boolean direct) {
		FragmentUpdateEvent event = new FragmentUpdateEvent(artifact, UpdateType.UPDATED, direct);
		return event;
	}
}
