package org.deft.repository.repositorymanager.impl;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.event.IContentChangeListener;
import org.deft.repository.repositorymanager.ConstraintsManager;
import org.deft.repository.repositorymanager.ContentManager;

/**
 * Abstract superclass for a logic (e.g. prolog) based ConstraintsManager. The repository
 * content is transformed into a knowledge based which can be reasoned upon according to
 * some specified rules which encode the repository consistency constraints. 
 * These rules are usually also encoded in the knowledge base.
 */
public abstract class ConstraintsManagerLogicBasedImpl implements ConstraintsManager {

	protected ContentManager contentManager;
	private boolean contentChanged = true;
	
	public ConstraintsManagerLogicBasedImpl(ContentManager contentManager) {
		this.contentManager = contentManager;
		contentManager.addContentChangeListener(new IContentChangeListener() {

			@Override
			public void contentChanged() {
				contentChanged = true;
			}

		});
	}
	
	
	@Override
	public boolean canContainChapter(Tutorial tutorial, Chapter chapter) {
		updateIfNeeded();
		boolean retValue = checkCanContainChapter(tutorial, chapter);
		return retValue;
	}

	@Override
	public boolean canContainFragment(Folder parent, Fragment child) {
		updateIfNeeded();
		boolean retValue = checkCanContainFragment(parent, child);
		return retValue;
	}
	
	
	@Override
	public boolean canContainFragmentAtRoot(Project project, FragmentType fragmentType, Fragment child) {
		updateIfNeeded();
		boolean retValue = checkCanContainFragmentAtRoot(project, fragmentType, child);
		return retValue;
	}

	@Override
	public boolean canContainFragmentName(Folder parent, String childName) {
		updateIfNeeded();
		boolean retValue = checkCanContainFragmentName(parent, childName);
		return retValue;
	}

	@Override
	public boolean canContainFragmentNameAtRoot(Project project, 
			FragmentType fragmentType, String childName) {
		updateIfNeeded();
		boolean retValue = checkCanContainFragmentName(project, 
				fragmentType, childName);
		return retValue;
	}

	@Override
	public boolean canContainFragmentType(Folder parent,
			FragmentType fragmentType) {
		updateIfNeeded();
		boolean retValue = checkCanContainFragmentType(parent, fragmentType);
		return retValue;
	}

	@Override
	public boolean containsChapter(Tutorial tutorial, Chapter chapter) {
		updateIfNeeded();
		boolean retValue = checkContainsChapter(tutorial, chapter);
		return retValue;
	}

	@Override
	public boolean containsFragmentName(Folder parent, String childName) {
		updateIfNeeded();
		boolean retValue = checkContainsFragmentName(parent, childName);
		return retValue;
	}

	@Override
	public boolean containsFragmentNameAtRoot(Project project,
			FragmentType fragmentType, String childName) {
		updateIfNeeded();
		boolean retValue = checkContainsFragmentNameAtRoot(project,
				fragmentType, childName);
		return retValue;
	}
	

	
	@Override
	public final boolean projectExists(String name) {
		updateIfNeeded();
		boolean retValue = checkProjectExists(name);
		return retValue;
	}
	

	
	private void updateIfNeeded() {
		if (contentChanged) {
			clearFacts();
			setupFacts();
			contentChanged = false;
		}
	}
	
	protected abstract void setupFacts();
	protected abstract void clearFacts();
	protected abstract boolean checkProjectExists(String name);
	protected abstract boolean checkCanContainChapter(Tutorial tutorial, Chapter chapter);
	protected abstract boolean checkCanContainFragment(Folder parent, Fragment child);
	protected abstract boolean checkCanContainFragmentAtRoot(Project project, FragmentType fragmentType, Fragment child);
	protected abstract boolean checkCanContainFragmentName(Folder parent, String childName);
	protected abstract boolean checkCanContainFragmentName(Project project, 
			FragmentType fragmentType, String childName);
	protected abstract boolean checkCanContainFragmentType(Folder parent, FragmentType type);
	protected abstract boolean checkContainsChapter(Tutorial tutorial, Chapter chapter);
	protected abstract boolean checkContainsFragment(Folder parent, Fragment child);
	protected abstract boolean checkContainsFragmentName(Folder parent, String childName);
	protected abstract boolean checkContainsFragmentNameAtRoot(Project project, 
			FragmentType fragmentType, String childName);
	protected abstract boolean checkContainsFragmentType(Folder parent, FragmentType type);



}
