package org.deft.repository.repositorymanager.impl;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Identifiable;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.resource.ResourceHandlerProvider;

import alice.tuprolog.InvalidTheoryException;
import alice.tuprolog.MalformedGoalException;
import alice.tuprolog.NoMoreSolutionException;
import alice.tuprolog.NoSolutionException;
import alice.tuprolog.Prolog;
import alice.tuprolog.SolveInfo;
import alice.tuprolog.Struct;
import alice.tuprolog.Theory;


public class ConstraintsManagerTuPrologImpl extends ConstraintsManagerLogicBasedImpl {
	
	protected Prolog prolog;
	private Theory rulesTheory;
	private Theory factTheory;
	private List<Struct> predicateList = new LinkedList<Struct>();
	
	public ConstraintsManagerTuPrologImpl(ContentManager contentManager, ResourceHandlerProvider provider) {
		super(contentManager);
		URL fragmentsFileUrl = provider.getAbsoluteResourceUrl("org.deft.repository", "resources/fragmentconstraints.pl");
		prolog = new Prolog();
		try {
			//the mainTheory contains only the rules from the fragmentconstraints file
			rulesTheory = new Theory(fragmentsFileUrl.openStream());
			prolog.setTheory(rulesTheory);
		} 
		catch (FileNotFoundException fnfe) {
			fnfe.printStackTrace();
		} 
		catch (IOException ioe) {
			ioe.printStackTrace();
		}
		catch (InvalidTheoryException ite) {
			ite.printStackTrace();
		}
		
	}
	
	@Override
	public void setupFacts() {
		for (Project project : contentManager.getProjects()) {
			addProjectContent(project);
		}
		commitBatchUpdate();
	}
	
	/**
	 * Gets all elements from the project root and adds them recursively to the
	 * knowledge base.
	 * 
	 * @param project the project for which to add the contents
	 */
	private void addProjectContent(Project project) {
		addProjectPredicate(project);
		List<Fragment> lFragments = new LinkedList<Fragment>();
		lFragments.addAll(contentManager.getFragments(project, FragmentFilter.CHAPTERFILTER)); 
		lFragments.addAll(contentManager.getFragments(project, FragmentFilter.ARTIFACTFILTER));
		lFragments.addAll(contentManager.getFragments(project, FragmentFilter.TUTORIALFILTER));
		lFragments.addAll(contentManager.getFragments(project, FragmentFilter.FOLDERFILTER));
		//add parent child relationships
		for (Fragment fragment : lFragments) {
			addTree(fragment);
		}
		//add tutorial chapter relationships
		for (Fragment fTut : contentManager.getFragments(project, FragmentFilter.TUTORIALFILTER)) {
			Tutorial tutorial = (Tutorial)fTut;
			for (Chapter chapter : contentManager.getChaptersForTutorial(tutorial)) {
				addTutorialChapterPredicate(tutorial, chapter);
			}
		}
	}
	
	/**
	 * Adds all predicates for a fragment to the knowledgebase and then recursively for
	 * its child fragments, in case fragment is a folder.
	 * 
	 * @param fragment the fragment to add (including its descendants)
	 */
	private void addTree(Fragment fragment) {
		addFragmentPredicate(fragment);
		if (fragment instanceof Folder) {
			Folder folder = (Folder)fragment;
			addFolderTypePredicate(folder);
			for (Fragment child : contentManager.getChildren(folder)) {
				addParentChildPredicate(folder, child);
				addTree(child);
			}
		}
	}
	
	/**
	 * Computes the strings from which to create a fragment predicate. These
	 * are then used to have a predicate created and marked for batch update.
	 * 
	 * @param fragment
	 */
	private void addFragmentPredicate(Fragment fragment) {
		String idString = getIdString(fragment);
		String type = fragment.getFragmentType().getName();
		String name = getNameString(fragment.getName());
		String projectIdString = getIdString(contentManager.getProjectOf(fragment));
		addBatchPredicateRule("fragment", idString, name, type);
		addBatchPredicateRule("belongsToProject", idString, projectIdString);
	}
	
	private void addProjectPredicate(Project project) {
		String idString = getIdString(project);
		String name = getNameString(project.getName());
		
		addBatchPredicateRule("project", idString, name);	
	}
	
	/**
	 * Computes the strings from which to create a parent predicate, denoting
	 * the parent child relationship between two fragments. These strings
	 * are then used to have a predicate created and marked for batch update.
	 *  
	 * @param parent
	 * @param child
	 */
	private void addParentChildPredicate(Fragment parent, Fragment child) {
		String parentIdString = getIdString(parent); 
		String childIdString = getIdString(child);
		addBatchPredicateRule("parent", parentIdString, childIdString);
	}
	
	/**
	 * Computes the strings from which to create a predicate for the type
	 * of a folder (A folder can contain only fragments of the same type
	 * such as Artifacts or other folders for Artifacts). These strings
	 * are then used to have a predicate created and marked for batch update.
	 *  
	 * @param parent
	 * @param child
	 */
	private void addFolderTypePredicate(Folder folder) {
		String folderIdString = getIdString(folder);
		String folderType = folder.getFragmentTypeAffiliation().getName();
		addBatchPredicateRule("folderType", folderIdString, folderType);
	}	
	
	/**
	 * Computes the strings from which to create an inTutorial predicate,
	 * denoting the relationship between chapters that have been added
	 * to tutorials.
	 *  
	 * @param tutorial
	 * @param chapter
	 */
	private void addTutorialChapterPredicate(Tutorial tutorial, Chapter chapter) {
		String tutorialIdString = getIdString(tutorial); 
		String chapterIdString = getIdString(chapter);
		addBatchPredicateRule("inTutorial", tutorialIdString, chapterIdString);	
	}
	
	
	
	
	/**
	 * Creates a predicate from a name and a number of arguments and adds
	 * it to a list. All predicates from this list will be added to the
	 * knowledgebase (the "theory") when commitBatchUpdate is executed.
	 *  
	 * @param predName
	 * @param argName
	 */
	private void addBatchPredicateRule(String predName, String ... argName) {
		predicateList.add(makePredicateTerm(predName, argName));
	}	
	
	/**
	 * Creates a predicate from a name and a number of arguments.
	 * 
	 * @param name
	 * @param params
	 * @return the predicate
	 */
	private Struct makePredicateTerm(String name, String ... params) {
		StringBuilder sb = new StringBuilder();
		sb.append(name + ("( "));
		Struct[] structs = new Struct[params.length];
		for (int i = 0; i < params.length; i++) {
			structs[i] = new Struct(params[i]);
			sb.append(params[i] + " ");		
		}
		sb.append(")");
		return new Struct(name, structs);
	}
	
	
	/**
	 * Adds all predicates from the predicateList to the knowledgebase
	 * (called theory in TuProlog terms). The predicates in the predicateList
	 * stem from method addBatchPredicateRule().
	 * After the predicates have been added to the theory the predicateList
	 * is cleared to be ready for the next batch update.
	 */
	private void commitBatchUpdate() {
		Struct list = new Struct("[]");
		for (Struct s : predicateList) {
			list.append(s);
		}
		try {
			factTheory = new Theory(list);
			prolog.addTheory(factTheory);
			predicateList.clear();
		} catch (InvalidTheoryException ite) {
			ite.printStackTrace();
		}		
	}
	
	/**
	 * Removes all predicates representing the repository's fragments.
	 * Only the rules from the fragmentconstraints.pl file remain in the
	 * knowledgebase.
	 */
	public void clearFacts() {
		try {
			prolog.setTheory(rulesTheory);
		} catch (InvalidTheoryException ite) {
			ite.printStackTrace();
		}
	}


	
	private boolean doQuery(String query) {
		assert predicateList.isEmpty() : "A batch update has been executed but not commited.";
        try {
        	SolveInfo info = prolog.solve(query);
        	return info.isSuccess();
        }
        catch (MalformedGoalException mge) {
        	mge.printStackTrace();
        }
        return false;		
	}
	
	private String getIdString(Identifiable identifiable) {
		return "id_" + identifiable.getId().replaceAll("-", "_");
	}
	
	private String getNameString(String name) {
		//String newName = "\'" + name + "\'";
		String newName = name;
		String isoName = convertToIso(newName);
		String escapedName = isoName.replaceAll("'", "''");
//		escapedName = escapedName.replaceAll('%', arg1)
		return escapedName;
	}

	/**
	 * Fragment names may contain foreign characters (e.g. Japanese letters...).
	 * TuProlog cannot handle them, so transform them first.
	 * 
	 * @param string fragment name with potential foreign characters
	 * @return ISO-8859-1 string where foreign characters
	 */
	private String convertToIso(String string) {
	    try {
	        byte[] utf8 = string.getBytes("UTF-8");
	        return new String(utf8, "ISO-8859-1");
	    } catch (UnsupportedEncodingException e) {
	    	e.printStackTrace(); 
	    }
	    //never come here
	    assert false;
	    return null;
	}

	
	protected void debug(String query) {
		try {
			System.out.println("------- Query: " + query + " ------- ");
			SolveInfo info = prolog.solve(query);
			System.out.println(info.getSolution());
			while (prolog.hasOpenAlternatives()) {
				info = prolog.solveNext();
				System.out.println(info.getSolution());
			}
				
		} catch (MalformedGoalException e) {
			e.printStackTrace();
		} catch (NoSolutionException e) {
			System.out.println("No solution");
		} catch (NoMoreSolutionException e) {
			e.printStackTrace();
		}
	}
	

	@Override
	protected boolean checkCanContainChapter(Tutorial tutorial, Chapter chapter) {
		String queryString = "canContainChapter(" + getIdString(tutorial) + ","
				+ getIdString(chapter) + ").";
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkCanContainFragment(Folder parent, Fragment child) {
		String queryString = "";
		if (parent == null) {
			queryString = "canContainFragmentAtRoot(" + getIdString(child) + ").";
		} else {
			queryString = "canContainFragment(" + getIdString(parent) + ","
					+ getIdString(child) + ").";
		}
		
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkCanContainFragmentAtRoot(Project project, 
			FragmentType fragmentType, Fragment child) {
		String queryString = "canContainFragmentAtRoot(" + getIdString(project) + ","
				+ fragmentType.getName() + "," + "\'" + getIdString(child) + "\').";
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkCanContainFragmentName(Folder parent, String childName) {
		String queryString = "canContainFragmentName(" + getIdString(parent) + "," 
		+ "\'"  + getNameString(childName) + "\').";
		boolean result = doQuery(queryString);
		return result;		
	}
	
	@Override
	protected boolean checkCanContainFragmentName(Project project,
			FragmentType fragmentType, String childName) {
		String queryString = "canContainFragmentNameAtRoot(" + getIdString(project) + "," 
				+ fragmentType.getName() + "," + "\'" + getNameString(childName) + "\').";
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkCanContainFragmentType(Folder parent, FragmentType fragmentType) {
		String queryString = "canContainFragmentType(" + getIdString(parent) + ","
				+ fragmentType.getName() + ").";
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkContainsChapter(Tutorial tutorial, Chapter chapter) {
		String queryString = "containsChapter(" + getIdString(tutorial) + ","
				+ getIdString(chapter) + ").";
		boolean result = doQuery(queryString);
		return result;

	}
	
	@Override
	protected boolean checkContainsFragment(Folder parent, Fragment child) {
		String queryString = "containsFragment(" + getIdString(parent) + ","
				+ getIdString(child) + ").";
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkContainsFragmentName(Folder parent, String childName) {
		String queryString = "containsFragmentName(" + getIdString(parent) + "," 
			+ "\'"  + getNameString(childName) + "\').";
		return doQuery(queryString);
	}
	
	@Override
	protected boolean checkContainsFragmentNameAtRoot(Project project,
			FragmentType fragmentType, String childName) {
		String queryString = "containsFragmentNameAtRoot(" + getIdString(project) + ","
				+ fragmentType.getName() + "," + "\'" + getNameString(childName) + "\').";
//		
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkContainsFragmentType(Folder parent, FragmentType fragmentType) {
		String queryString = "containsFragmentType(" + getIdString(parent) + ","
				+ fragmentType.getName() + ").";
		boolean result = doQuery(queryString);
		return result;
	}
	
	@Override
	protected boolean checkProjectExists(String name) {
		String queryString = "projectExists('" + getNameString(name) + "').";
		return doQuery(queryString);		
	}
	

}
