package org.deft.repository.repositorymanager.impl;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.operation.OperationChain;
import org.deft.operation.OperationChainConfiguration;
import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.chaptertype.ChapterAwareResourceHandlerProvider;
import org.deft.repository.chaptertype.ChapterTypeResourceHandler;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Project;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.integrator.Integrator;
import org.deft.repository.integrator.IntegratorConfiguration;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;
import org.deft.repository.repositorymanager.OptionsService;
import org.deft.repository.service.ExtensionService;
import org.deft.util.FileUtil;
import org.deft.util.StreamUtil;
import org.deft.util.XmlUtil;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.w3c.dom.Document;

/**
 * The DataStorageManagerFileSystemImpl is, as the name suggests, a DataStorageManager
 * which uses the file system to store repository content. 
 */
public class DataStorageManagerFileSystemImpl implements DataStorageManager {

	private ContentManager contentManager;
	private OptionsService options;
	private ChapterAwareResourceHandlerProvider resourceHandlerProvider;
	private ExtensionService extensionService;
	
	/**
	 * Map that stores for all fragments (fragment ids) the streams that have been opened
	 * for that fragment. Files can't be deleted if they are open, but if we know all
	 * open streams, we can close them prior deleting the corresponding file.
	 */
	private Map<String, List<Closeable>> streamMap = new HashMap<String, List<Closeable>>();
	
	public DataStorageManagerFileSystemImpl(ContentManager contentManager,
			OptionsService options, ChapterAwareResourceHandlerProvider resourceHandlerProvider,
			ExtensionService extensionService) {
		this.contentManager = contentManager;
		this.options = options;
		this.resourceHandlerProvider = resourceHandlerProvider;
		this.extensionService = extensionService;
	}
	
	@Override
	public boolean createProject(Project project) {
		IPath projectFolder = computeProjectFolder(project);
		boolean folderCreated = createFolder(projectFolder);
		return folderCreated;
	}
	
	@Override
	public File getTempDirectory(Project project) {
		IPath projectFolder = computeProjectFolder(project);
		IPath tempFolder = projectFolder.append("temp");
		File tempDir = tempFolder.toFile();
		if (!tempDir.exists()) {
			tempDir.mkdirs();
		}
		return tempDir;
	}
	
	@Override
	public boolean removeProject(Project project) {
		IPath projectFolder = computeProjectFolder(project);
		File f = projectFolder.toFile();
		if (!f.exists()) {
			// Tried to delete project folder, but it did not exist
			return true;
		} else {
			if (isNotEmpty(f)) {
				boolean success = forceDeleteContents(f);
				if (!success) {
					throw new IllegalStateException("The project folder cannot be deleted because it is not empty, and its contents cannot be deleted");
				} else {
					return success;
				}
			}
			boolean result = f.delete();
			return result;
		}
	}
	
	private boolean isNotEmpty(File f) {
		return f.list().length != 0;
	}
	
	private boolean forceDeleteContents(File file) {
		if (file.isDirectory()) {
			for (File f : file.listFiles()) {
				forceDeleteContents(f);
			}
		}
		boolean result = file.delete();
		return result;
	}
	
	

	@Override
	public boolean createArtifactContent(Artifact artifact, int revision,
			String originalLocation, MultiArtifactDependencies dependencies) throws IOException {
		String sourceId = artifact.getFragmentSourceId();
		FragmentSource fragmentSource = extensionService.getFragmentSource(sourceId);
		IPath destinationFolder = computePathOfFolderForFragment(artifact, revision);
		boolean folderCreated = createFolder(destinationFolder);
		if (folderCreated) {
			IPath mainFilePath = computeArtifactLocation(artifact, revision);
			saveMainFile(mainFilePath, fragmentSource, originalLocation);
			saveDependencyFiles(artifact, revision, fragmentSource, dependencies, destinationFolder);
			return true;
		} else {
			return false;
		}
	}

	private void saveMainFile(IPath mainFilePath, FragmentSource source, String locationString) throws IOException {
		OriginalFragmentLocation location = source.createLocationObject(locationString);
		byte[] content = getContentFrom(source, location);
		BufferedOutputStream bos = getBufferedOutputStream(mainFilePath);
		writeBinaryArtifactContentAndCloseStream(bos, content);
	}
	
	private void saveDependencyFiles(Artifact artifact, int revision,
			FragmentSource source, MultiArtifactDependencies dependencies, 
			IPath destinationFolder) throws IOException {
		List<String> ids = dependencies.getAllIdsExceptForMainArtifact();
		for (String id : ids) {
			String locationString = dependencies.getLocationForId(id);
			OriginalFragmentLocation location = source.createLocationObject(locationString);
			byte[] content = getContentFrom(source, location);
			IPath destinationFilePath = computeArtifactDependencyLocation(artifact, id, revision);
			BufferedOutputStream bos = getBufferedOutputStream(destinationFilePath);
			writeBinaryArtifactContentAndCloseStream(bos, content);
		}
	}

	

	
	private byte[] getContentFrom(FragmentSource fragmentSource, OriginalFragmentLocation location) {
		BufferedInputStream bis = getBufferedInputStream(fragmentSource, location);
		byte[] binaryArtifactContent = getBinaryContentAndCloseStream(bis);
		return binaryArtifactContent;
	}
	
	private BufferedInputStream getBufferedInputStream(FragmentSource source,
			OriginalFragmentLocation location) {
		InputStream is = source.getInputStream(location);
		if (is == null) {
			return null;
		}
		if (is instanceof BufferedInputStream) {
			return (BufferedInputStream)is;
		}
		BufferedInputStream bis = new BufferedInputStream(is);
		return bis;
	}
	
	private byte[] getBinaryContentAndCloseStream(BufferedInputStream bis) {
		try {
			byte[] contents = new byte[bis.available()];
			bis.read(contents, 0, contents.length);
			bis.close();
			return contents;
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}
	
	
	
	private BufferedOutputStream getBufferedOutputStream(IPath destFile) {
		try {
			FileOutputStream fos = new FileOutputStream(destFile.toFile());
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			return bos;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	private void writeBinaryArtifactContentAndCloseStream(BufferedOutputStream bos, 
			byte[] data) throws IOException {
		bos.write(data);
		bos.flush();
		bos.close();
	}
	
	
	
	@Override
	public boolean removeArtifactContent(Artifact artifact, int revision) {
		IPath destinationFile = computeArtifactLocation(artifact, revision);
		File f = destinationFile.toFile();
		if (fileIsAlreadyDeleted(f)) {
			return true;
		} else {
			boolean result = f.delete();
			return result;
		}
	}
	
	private boolean fileIsAlreadyDeleted(File f) {
		return !f.exists();
	}

	
	@Override
	public boolean removeArtifactContent(Artifact artifact) {
		IPath folder = computePathOfFolderForFragment(artifact);
		File f = folder.toFile();
		//directory (f) might be deleted due to a bug or by the user...
		if (f.exists()) {
			boolean result = doRemoveContentFrom(f, artifact);
			return result;
		}
		//if the directory did not exist in the first place, the artifact content did neither 
		return true;
	}

	
	private boolean doRemoveContentFrom(File directory, Artifact artifact) {
		//directory is a folder which contains several files
		//with the code (one for each revision)
		File[] files = FileUtil.getFilesFromDirectory(directory);
		for (File fRevisionFile : files) {
			boolean revisionDeleted = fRevisionFile.delete();
			if (!revisionDeleted) {
				closeStreams(artifact);
			}
			revisionDeleted = fRevisionFile.delete();
			//revisionDeleted should be true here, if not, there is some
			//error...we will not be able to delete the folder, the repository
			//will have to take some appropriate actions
		}
		//revisions are deleted, now delete folder
		//delete recursively in case there is something else inside the folder, e.g. svn metadata
		boolean result = FileUtil.deleteDirectory(directory);
		return result;
	}
	
	
	
	

	@Override
	public boolean createChapterContent(Chapter chapter) {
		IPath destinationFile = computePathOfFolderForFragment(chapter);
		String chapterType = contentManager.getChapterType(chapter);
		ChapterTypeResourceHandler chapterTypeHandler = resourceHandlerProvider.getChapterTypeResourceHandler(chapterType);
		URL templateUrl = chapterTypeHandler.getChapterTemplateUrl();
		boolean result = FileUtil.saveToFile(templateUrl, destinationFile.toFile(), true);
		return result;
	}
	


	
	@Override
	public boolean removeChapterContent(Chapter chapter) {
		IPath path = computePathOfFolderForFragment(chapter);
		File f = path.toFile();
		if (fileIsAlreadyDeleted(f)) {
			return true;
		}
		boolean result = f.delete();
		if (!result) {
			closeStreams(chapter);
			result = f.delete();
		}
		return result;
	}
	

	
	private IPath computeProjectFolder(Project project) {
		String repositoryPath = options.getRepositoryLocation();
		IPath projectPath = new Path(repositoryPath);
		projectPath = projectPath.append(project.getId());
		return projectPath;
	}

	private IPath computePathOfFolderForFragment(Fragment fragment) {
		Project project = contentManager.getProjectOf(fragment);
		IPath fragmentPath = computeProjectFolder(project);
		fragmentPath = fragmentPath.append(fragment.getId());
		return fragmentPath;
	}
	
	
	private IPath computePathOfFolderForFragment(Fragment fragment, int revision) {
		IPath fragmentPath = computePathOfFolderForFragment(fragment);
		fragmentPath = fragmentPath.append("rev" + revision);
		return fragmentPath;
	}
	

	private boolean createFolder(IPath path) {
		File f = path.toFile();
		//if f exists and is a directory, we are already done
		//if f exists and is a file, we can't create a directory
		if (f.exists()) {
			return f.isDirectory();
		}
		//if f does not exist, create it
		boolean result = f.mkdirs();
		return result;
	}
	
	
	private IPath computeRevisionLocation(Artifact artifact, int revision) {
		IPath location = computePathOfFolderForFragment(artifact).append("rev" +  revision);
		return location;
	}

	private IPath computeArtifactLocation(Artifact artifact, int revision) {
		IPath location = computeRevisionLocation(artifact, revision).append("_artifact_");
		return location;
	}

	private IPath computeArtifactDependencyLocation(Artifact artifact, String dependentFileId, int revision) {
		IPath location = computeRevisionLocation(artifact, revision).append(dependentFileId);
		return location;
	}

	
	@Override
	public boolean createArtifactReferenceContent(ArtifactReference reference, 
			OperationChain chain, Integrator integrator) {
		Chapter chapter = contentManager.getChapter(reference);
		Artifact artifact = contentManager.getArtifact(reference);
		Project project = contentManager.getProjectOf(chapter);
		createReferencesFolderIfNecessary(project);
		if (referencesFolderExists(project)) {
			OperationChainConfiguration chainConfig = chain.getChainConfigurations();
			IntegratorConfiguration integratorConfig = integrator.getIntegratorConfiguration();
			
			IPath chainConfigLocation = computeReferenceOperationChainConfigPath(reference);
			Document chainXmlDoc = chainConfig.getConfigurationAsXml(reference.getId(), artifact, chapter);
			writeXml(chainXmlDoc, chainConfigLocation);
			
			
			IPath integratorConfigLocation = computeReferenceIntegratorConfigPath(reference);
			Document integratorXmlDoc = integratorConfig.getConfigurationAsXml(reference.getId(), artifact, chapter);
			writeXml(integratorXmlDoc, integratorConfigLocation);
			return true;
		}
		return false;
	}
	
	
	private void createReferencesFolderIfNecessary(Project project) {
		IPath path = computeReferencesFolderPath(project);
		File folder = path.toFile();
		if (!folder.exists()) {
			folder.mkdirs();
		}
	}
	
	
	private IPath computeReferencesFolderPath(Project project) {
		IPath referencesFolderPath = computeProjectFolder(project);
		referencesFolderPath = referencesFolderPath.append("references");
		return referencesFolderPath;
	}
	
	private boolean referencesFolderExists(Project project) {
		IPath referencesFolderPath = computeProjectFolder(project);
		File folder = referencesFolderPath.toFile();
		return folder.exists() && folder.isDirectory();
	}
		
	
	private IPath computeReferenceOperationChainConfigPath(ArtifactReference reference) {
		Chapter chapter = contentManager.getChapter(reference);
		Project project = contentManager.getProjectOf(chapter);
		IPath referencesFolderPath = computeReferencesFolderPath(project);
		IPath referencePath = referencesFolderPath.append(reference.getId() + "-chain");
		return referencePath;
	}
	
	
	private IPath computeReferenceIntegratorConfigPath(ArtifactReference reference) {
		Chapter chapter = contentManager.getChapter(reference);
		Project project = contentManager.getProjectOf(chapter);
		IPath referencesFolderPath = computeReferencesFolderPath(project);
		IPath referencePath = referencesFolderPath.append(reference.getId() + "-integrator");
		return referencePath;
	}
	

	
	
	private void writeXml(Document doc, IPath location) {
		try {
			BufferedOutputStream bos = getBufferedOutputStream(location);
			XmlUtil.outputPrettyXml(doc, bos);
			bos.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	
	@Override
	public boolean removeArtifactReferenceContent(ArtifactReference reference) {
		Chapter chapter = contentManager.getChapter(reference);
		Project project = contentManager.getProjectOf(chapter);
		if (referenceFolderExists(project)) {//might be deleted due to bugs or by the user for some reason
			IPath referenceChainConfigPath = computeReferenceOperationChainConfigPath(reference);
			File referenceChainConfigFile = referenceChainConfigPath.toFile();
			boolean referenceChainConfigDeleted = false;
			if (fileIsAlreadyDeleted(referenceChainConfigFile)) {
				referenceChainConfigDeleted = true;
			} else {
				referenceChainConfigDeleted = referenceChainConfigFile.delete();
			}
			
			IPath referenceIntegratorConfigPath = computeReferenceIntegratorConfigPath(reference);
			File referenceIntegratorConfigFile = referenceIntegratorConfigPath.toFile();
			boolean referenceIntegratorConfigDeleted = false;
			if (fileIsAlreadyDeleted(referenceIntegratorConfigFile)) {
				referenceIntegratorConfigDeleted = true;
			} else {
				referenceIntegratorConfigDeleted = referenceIntegratorConfigFile.delete();
			}
			
			
			if (referenceChainConfigDeleted && referenceIntegratorConfigDeleted && referenceFolderIsEmpty(project)) {
				boolean success = deleteReferenceFolder(project);
				return success;
			} else {
				return referenceChainConfigDeleted;
			}

		} else {
			//reference folder is deleted, so the reference is as well
			return true;
		}
	}
	
	
	private boolean referenceFolderExists(Project project) {
		IPath referencePath = computeReferencesFolderPath(project);
		File f = referencePath.toFile();
		return f.exists();
	}
	
	private boolean referenceFolderIsEmpty(Project project) {
		IPath referencePath = computeReferencesFolderPath(project);
		File f = referencePath.toFile();
		String[] files = f.list();
		boolean isEmpty =  files.length == 0;
		return isEmpty;
	}
	
	
	private boolean deleteReferenceFolder(Project project) {
		IPath referencePath = computeReferencesFolderPath(project);
		File f = referencePath.toFile();
		boolean success = f.delete();
		return success;
	}
	
	
	
	public Document getArtifactReferenceOperationChainConfigContent(ArtifactReference reference) {
		IPath location = computeReferenceOperationChainConfigPath(reference);
		Document doc = readXml(location);
		return doc;
	}
	
	public Document getArtifactReferenceIntegratorConfigContent(ArtifactReference reference) {
		IPath location = computeReferenceIntegratorConfigPath(reference);
		Document doc = readXml(location);
		return doc;
	}
	
	
	private Document readXml(IPath location) {
		if (location.toFile().exists()) {
			Document doc = XmlUtil.loadXmlFromFile(location.toFile());
			return doc;
		} else {
			return null;
		}
	}
	
	
	
	
	
	

	@Override
	public InputStream getInputStream(Chapter chapter) {
		IPath fragmentPath = computePathOfFolderForFragment(chapter);
		return getInputStream(chapter, fragmentPath);
	}
	
	@Override
	public File getFile(Chapter chapter) {
		IPath fragmentPath = computePathOfFolderForFragment(chapter);
		return fragmentPath.toFile();
	}

	
	
	@Override
	public InputStream getInputStream(Artifact artifact, int revision) {
		IPath path = computeArtifactLocation(artifact, revision);
		return getInputStream(artifact, path);
	}
	
	@Override
	public InputStream getInputStream(Artifact artifact, String dependentFileId, int revision) {
		IPath path = computeArtifactDependencyLocation(artifact, dependentFileId, revision);
		return getInputStream(artifact, dependentFileId, path);
	}
	
	private InputStream getInputStream(Artifact artifact, String dependentFileId, IPath path) {
		if (path != null && path.toFile().exists() && path.toFile().isFile()) {
			try {
				BufferedInputStream bi = new BufferedInputStream(
						new FileInputStream(path.toFile()));
				recordStream(artifact, dependentFileId, bi);
				return bi;
			} catch (FileNotFoundException fnfe) {
				fnfe.printStackTrace();
			}
		}
		return null;		
	}
	
	private InputStream getInputStream(Fragment fragment, IPath path) {
		if (path != null && path.toFile().exists() && path.toFile().isFile()) {
			try {
				BufferedInputStream bi = new BufferedInputStream(
						new FileInputStream(path.toFile()));
				recordStream(fragment, bi);
				return bi;
			} catch (FileNotFoundException fnfe) {
				fnfe.printStackTrace();
			}
		}
		return null;		
	}
	
	private void recordStream(Artifact artifact, String dependentFileId, Closeable stream) {
		String id = artifact.getId() + "/" + dependentFileId;
		List<Closeable> list = streamMap.get(id);
		if (list == null) {
			list = new LinkedList<Closeable>();
			streamMap.put(id, list);
		}
		list.add(stream);
	}
	
	private void recordStream(Fragment fragment, Closeable stream) {
		String id = fragment.getId();
		List<Closeable> list = streamMap.get(id);
		if (list == null) {
			list = new LinkedList<Closeable>();
			streamMap.put(id, list);
		}
		list.add(stream);
	}

	private void closeStreams(Fragment fragment) {
		String id = fragment.getId();
		List<Closeable> list = streamMap.get(id);
		if (list != null) {
			for (Closeable c : list) {
				try {
					c.close();
				} catch (IOException e) {					
					e.printStackTrace();
					//exit here, do not remove the fragment
					//id from the stream map if a stream
					//could not be closed
					return;
				}
			}
		}
		streamMap.remove(id);
	}

	@Override
	public OutputStream getOutputStream(Chapter chapter) {
		IPath fragmentPath = computePathOfFolderForFragment(chapter);
		if (fragmentPath != null && fragmentPath.toFile().exists()
				&& fragmentPath.toFile().isFile()) {
			try {
				FileOutputStream fos = new FileOutputStream(fragmentPath.toFile());
				recordStream(chapter, fos);
				return fos;
			} catch (FileNotFoundException fnfe) {
				fnfe.printStackTrace();
			}
		}
		return null;
	}
	
	@Override
	public boolean isArtifactOriginStillExisting(Artifact artifact) {
		String location = artifact.getOriginalLocation();
		File file = new File(location);
		boolean exists = file.isFile() && file.exists();
		return exists;
	}
	
	@Override
	public boolean hasArtifactOriginChanged(Artifact artifact, int revision) {
		if (isArtifactOriginStillExisting(artifact)) {
			InputStream artifactStream = getInputStream(artifact, revision);
			String location = artifact.getOriginalLocation();
			File file = new File(location);
			try {
				InputStream fileStream = new FileInputStream(file);
				boolean identical = StreamUtil.isContentIdentical(artifactStream, fileStream);
				if (!identical) {
					return true;
				}
				List<String> dependentFileIds = contentManager.getDependentFileIds(artifact, revision);
				for (String dependentFileId : dependentFileIds) {
					identical = isDependentFileOriginIdentical(artifact, dependentFileId, revision);
					if (!identical) {
						return true;
					}
				}
				return false;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				return true;
			}
		}
		return false;
	}
	
	private boolean isDependentFileOriginIdentical(Artifact artifact, String dependentFileId, int revision) {
		InputStream artifactStream = getInputStream(artifact, dependentFileId, revision);
		String location = contentManager.getOriginalLocation(artifact, dependentFileId, revision);
		File file = new File(location);
		if (file.exists() && file.isFile()) {
			try {
				InputStream fileStream = new FileInputStream(file);
				boolean identical = StreamUtil.isContentIdentical(artifactStream, fileStream);
				return identical;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				return true;
			}
		}
		return false;
	}


}
