package org.deft.repository.repositorymanager.impl;

import static org.deft.repository.datamodel.FragmentType.ARTIFACT;
import static org.deft.repository.datamodel.FragmentType.CHAPTER;
import static org.deft.repository.datamodel.FragmentType.TUTORIAL;

import java.util.List;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.exception.DeftCrossProjectRelationException;
import org.deft.repository.exception.DeftIncompatibleNameException;
import org.deft.repository.exception.DeftIncompatibleTypeException;
import org.deft.repository.exception.DeftInvalidArgumentException;
import org.deft.repository.exception.DeftNullArgumentException;
import org.deft.repository.exception.DeftProjectAlreadyExistsException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.repositorymanager.ConstraintsManager;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.ValidationManagerContentManagementService;

	
public class ValidationManagerContentManagementServiceOAWImpl implements ValidationManagerContentManagementService {

	private ConstraintsManager constraintsManager;
	private ContentManager contentManager;
	
	public ValidationManagerContentManagementServiceOAWImpl(
			ContentManager contentManager, ConstraintsManager constraintsManager) {
		this.contentManager = contentManager;
		this.constraintsManager = constraintsManager;
	}
	
	
	private boolean isValidProject(Project project) {
		List<Project> projects = contentManager.getProjects();
		boolean isValidProject = projects.contains(project);
		return isValidProject;
	}
	
	
	private boolean isValidFragment(Fragment fragment) {
		Fragment fragmentFromRep = contentManager.getFragment(fragment.getId());
		boolean isValidFragment = fragment.equals(fragmentFromRep);
		return isValidFragment;
	}
	

	
	private boolean belongToSameProject(Fragment fragment1, Fragment fragment2) {
		Project p1 = contentManager.getProjectOf(fragment1);
		Project p2 = contentManager.getProjectOf(fragment2);
		boolean belongToSameProject = p1.equals(p2);
		return belongToSameProject;
	}
	
	private boolean canRename(Fragment fragment, String newName) {
		//if new name is same as old name the "renaming" can take place without problems
		if (fragment.getName().equals(newName)) {
			return true;
		}
		
		Folder parent = contentManager.getParentFolder(fragment);
		if (parent == null) {
			boolean canContainName = constraintsManager.canContainFragmentName(parent, newName);
			return canContainName;
		} else {
			Project project = contentManager.getProjectOf(fragment);
			FragmentType type = fragment.getFragmentType();
			boolean canContainName = constraintsManager.canContainFragmentNameAtRoot(project, type, newName);
			return canContainName;
		}
	}
	
	
	
	//////////// start generated code (copied here by hand, but no changes were done) //////////////////////////////////
	
	

	public void createProject(String name) throws DeftNullArgumentException,
			DeftProjectAlreadyExistsException {

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (constraintsManager.projectExists(name)) {
			throw new DeftProjectAlreadyExistsException(name);
		}
	}

	public void getChapterType(Project project)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}
	}

	public void createChapter(Project project, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}

		if (!constraintsManager.canContainFragmentNameAtRoot(project, CHAPTER, name)) {
			throw new DeftIncompatibleNameException(project, "chapter", name);
		}
	}

	public void createChapter(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleTypeException, DeftIncompatibleNameException {

		if (parent == null) {
			throw new DeftNullArgumentException("parent");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidFragment(parent)) {
			throw new DeftInvalidArgumentException(parent);
		}

		if (!constraintsManager.canContainFragmentType(parent, CHAPTER)) {
			throw new DeftIncompatibleTypeException(parent, "chapter");
		}

		if (!constraintsManager.canContainFragmentName(parent, name)) {
			throw new DeftIncompatibleNameException(parent, name);
		}
	}

	public void getChapterType(Chapter chapter)
			throws DeftInvalidArgumentException {

		if (!isValidFragment(chapter)) {
			throw new DeftInvalidArgumentException(chapter);
		}
	}

	public void createTutorial(Project project, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}

		if (!constraintsManager.canContainFragmentNameAtRoot(project, TUTORIAL, name)) {
			throw new DeftIncompatibleNameException(project, "tutorial", name);
		}
	}

	public void createTutorial(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleTypeException, DeftIncompatibleNameException {

		if (parent == null) {
			throw new DeftNullArgumentException("parent");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidFragment(parent)) {
			throw new DeftInvalidArgumentException(parent);
		}

		if (!constraintsManager.canContainFragmentType(parent, TUTORIAL)) {
			throw new DeftIncompatibleTypeException(parent, "tutorial");
		}

		if (!constraintsManager.canContainFragmentName(parent, name)) {
			throw new DeftIncompatibleNameException(parent, name);
		}
	}

	public void importArtifact(Project project, String artifactId,
			FragmentSource fragmentSource, OriginalFragmentLocation location,
			String name) throws DeftNullArgumentException,
			DeftInvalidArgumentException, DeftIncompatibleNameException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (artifactId == null) {
			throw new DeftNullArgumentException("artifactId");
		}

		if (fragmentSource == null) {
			throw new DeftNullArgumentException("fragmentSource");
		}

		if (location == null) {
			throw new DeftNullArgumentException("location");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}

		if (!constraintsManager.canContainFragmentNameAtRoot(project, ARTIFACT, name)) {
			throw new DeftIncompatibleNameException(project, "artifact", name);
		}
	}

	public void importArtifact(Folder parent, String artifactTypeId,
			FragmentSource fragmentSource, OriginalFragmentLocation location,
			String name) throws DeftNullArgumentException,
			DeftInvalidArgumentException, DeftIncompatibleTypeException,
			DeftIncompatibleNameException {

		if (parent == null) {
			throw new DeftNullArgumentException("parent");
		}

		if (artifactTypeId == null) {
			throw new DeftNullArgumentException("artifactTypeId");
		}

		if (fragmentSource == null) {
			throw new DeftNullArgumentException("fragmentSource");
		}

		if (location == null) {
			throw new DeftNullArgumentException("location");
		}

		if (!isValidFragment(parent)) {
			throw new DeftInvalidArgumentException(parent);
		}

		if (!constraintsManager.canContainFragmentType(parent, ARTIFACT)) {
			throw new DeftIncompatibleTypeException(parent, "artifact");
		}

		if (!constraintsManager.canContainFragmentName(parent, name)) {
			throw new DeftIncompatibleNameException(parent, name);
		}
	}

	public void createImage(Project project, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}

	}

	public void createImage(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleTypeException, DeftIncompatibleNameException {

		if (parent == null) {
			throw new DeftNullArgumentException("parent");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidFragment(parent)) {
			throw new DeftInvalidArgumentException(parent);
		}


		if (!constraintsManager.canContainFragmentName(parent, name)) {
			throw new DeftIncompatibleNameException(parent, name);
		}
	}

	public void createFolder(Project project, FragmentType fragType, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (fragType == null) {
			throw new DeftNullArgumentException("fragType");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}

		if (!constraintsManager.canContainFragmentNameAtRoot(project, fragType, name)) {
			throw new DeftIncompatibleNameException(project, fragType, name);
		}
	}

	public void createFolder(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException {

		if (parent == null) {
			throw new DeftNullArgumentException("parent");
		}

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}

		if (!isValidFragment(parent)) {
			throw new DeftInvalidArgumentException(parent);
		}

		if (!constraintsManager.canContainFragmentName(parent, name)) {
			throw new DeftIncompatibleNameException(parent, name);
		}
	}

	public void moveToFolder(Folder target, Fragment toMove)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftCrossProjectRelationException, DeftIncompatibleTypeException {

		if (toMove == null) {
			throw new DeftNullArgumentException("toMove");
		}

		if (!isValidFragment(toMove)) {
			throw new DeftInvalidArgumentException(toMove);
		}

		if (!isValidFragment(target)) {
			throw new DeftInvalidArgumentException(target);
		}

		if (!belongToSameProject(toMove, target)) {
			throw new DeftCrossProjectRelationException(target, toMove);
		}

		if (!constraintsManager.canContainFragment(target, toMove)) {
			throw new DeftIncompatibleTypeException(target, toMove);
		}
	}

	public void getParentFolder(Fragment fragment)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (fragment == null) {
			throw new DeftNullArgumentException("fragment");
		}

		if (!isValidFragment(fragment)) {
			throw new DeftInvalidArgumentException(fragment);
		}
	}

	public void getChildren(Folder folder) throws DeftNullArgumentException,
			DeftInvalidArgumentException {

		if (folder == null) {
			throw new DeftNullArgumentException("folder");
		}

		if (!isValidFragment(folder)) {
			throw new DeftInvalidArgumentException(folder);
		}
	}

	public void getChildren(Project project, FragmentType fragmentType)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (fragmentType == null) {
			throw new DeftNullArgumentException("fragmentType");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}
	}

	public void getProjectOf(Fragment fragment)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (fragment == null) {
			throw new DeftNullArgumentException("fragment");
		}

		if (!isValidFragment(fragment)) {
			throw new DeftInvalidArgumentException(fragment);
		}
	}

	public void getProjects() {
	}

	public void getProject(String name) throws DeftNullArgumentException {

		if (name == null) {
			throw new DeftNullArgumentException("name");
		}
	}

	public void addChapterToTutorial(Tutorial tutorial, Chapter chapter)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftCrossProjectRelationException {

		if (tutorial == null) {
			throw new DeftNullArgumentException("tutorial");
		}

		if (chapter == null) {
			throw new DeftNullArgumentException("chapter");
		}

		if (!isValidFragment(tutorial)) {
			throw new DeftInvalidArgumentException(tutorial);
		}

		if (!isValidFragment(chapter)) {
			throw new DeftInvalidArgumentException(chapter);
		}

		if (!belongToSameProject(tutorial, chapter)) {
			throw new DeftCrossProjectRelationException(tutorial, chapter);
		}
	}

	public void removeChapterFromTutorial(Tutorial tutorial, Chapter chapter)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (tutorial == null) {
			throw new DeftNullArgumentException("tutorial");
		}

		if (chapter == null) {
			throw new DeftNullArgumentException("chapter");
		}

		if (!isValidFragment(tutorial)) {
			throw new DeftInvalidArgumentException(tutorial);
		}

		if (!isValidFragment(chapter)) {
			throw new DeftInvalidArgumentException(chapter);
		}
	}

	public void getChaptersForTutorial(Tutorial tutorial)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (tutorial == null) {
			throw new DeftNullArgumentException("tutorial");
		}

		if (!isValidFragment(tutorial)) {
			throw new DeftInvalidArgumentException(tutorial);
		}
	}

	public void getTutorialsForChapter(Chapter chapter)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (chapter == null) {
			throw new DeftNullArgumentException("chapter");
		}

		if (!isValidFragment(chapter)) {
			throw new DeftInvalidArgumentException(chapter);
		}
	}

	public void getFragment(String id) throws DeftNullArgumentException {

		if (id == null) {
			throw new DeftNullArgumentException("id");
		}
	}

	public void getFragments(Project project, FragmentFilter filter)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}
	}

	public void getFragmentTypeAffiliation(Folder folder)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (folder == null) {
			throw new DeftNullArgumentException("folder");
		}

		if (!isValidFragment(folder)) {
			throw new DeftInvalidArgumentException(folder);
		}
	}

	public void setChapterPosition(Tutorial tutorial, Chapter chapter,
			int position) throws DeftNullArgumentException,
			DeftInvalidArgumentException, DeftCrossProjectRelationException {

		if (tutorial == null) {
			throw new DeftNullArgumentException("tutorial");
		}

		if (chapter == null) {
			throw new DeftNullArgumentException("chapter");
		}

		if (!isValidFragment(tutorial)) {
			throw new DeftInvalidArgumentException(tutorial);
		}

		if (!isValidFragment(chapter)) {
			throw new DeftInvalidArgumentException(chapter);
		}

		if (!belongToSameProject(tutorial, chapter)) {
			throw new DeftCrossProjectRelationException(tutorial, chapter);
		}
	}

	public void rename(Project project, String newName)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftProjectAlreadyExistsException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (newName == null) {
			throw new DeftNullArgumentException("newName");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}

		if (constraintsManager.projectExists(newName)) {
			throw new DeftProjectAlreadyExistsException(newName);
		}
	}

	public void rename(Fragment fragment, String newName)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException {

		if (fragment == null) {
			throw new DeftNullArgumentException("fragment");
		}

		if (newName == null) {
			throw new DeftNullArgumentException("newName");
		}

		if (!isValidFragment(fragment)) {
			throw new DeftInvalidArgumentException(fragment);
		}

		if (canRename(fragment, newName)) {
			throw new DeftIncompatibleNameException(fragment, newName);
		}
	}

	public void removeProject(Project project)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (project == null) {
			throw new DeftNullArgumentException("project");
		}

		if (!isValidProject(project)) {
			throw new DeftInvalidArgumentException(project);
		}
	}

	public void removeFragment(Fragment fragment)
			throws DeftNullArgumentException, DeftInvalidArgumentException {

		if (fragment == null) {
			throw new DeftNullArgumentException("fragment");
		}

		if (!isValidFragment(fragment)) {
			throw new DeftInvalidArgumentException(fragment);
		}
	}

}
