package org.deft.repository.workspaceresources;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.deft.repository.Repository;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Project;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;


public class RepositoryWorkspaceNameMatcher {
	
	private ContentManagementService cms = Repository.instance.getContentManagementService();
	
	/**
	 * Returns the name of the project in the Eclipse workspace- 
	 */
	public String computeProjectName(Project project) {
		return project.getId();
	}
	
	/**
	 * Returns the name of the folder in which the actual file copy resides/will reside.
	 */
	public String computeFolderName(Chapter chapter) {
		return chapter.getId();
	}
	
	/**
	 * Returns the name of the chapter workspace copy. 
	 */
	public String computeFileName(Chapter chapter) {
		String nameWithExtension = chapter.getName() + "." + chapter.getTypeId();
		return nameWithExtension;
	}
	
	
	public String getChaptersMainFolderName() {
		return "chapters";
	}
	
	
	
	/**
	 * Returns for a DEFT project corresponding IProject in the Eclipse workspace 
	 * if it exists, or null if it doesn't exist. 
	 */
	public IProject getEclipseProjectInWorkspace(Project deftProject) {
		String eclipseProjectName = computeProjectName(deftProject);
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject projectResource = root.getProject(eclipseProjectName);
		if (projectResource.exists()) {
			if (!projectResource.isOpen()) {
				//project must be open, otherwise there will be exceptions
				try {
					projectResource.open(null);
				} catch (CoreException ce) {
					ce.printStackTrace();
				}
			}
			return projectResource;
		}
		return null;
	}
	
	
	public boolean hasCorrespondingEclipseProject(Project deftProject) {
		return getEclipseProjectInWorkspace(deftProject) != null;
	}
	
	/**
	 * Returns for an Eclipse workspace project the corresponding DEFT project 
	 * if it exists, or null if it doesn't exist. 
	 */
	public Project getDeftProject(IProject eclipseProject) {
		String eclipseProjectName = eclipseProject.getName();
		Project deftProject = cms.getProjectById(eclipseProjectName);
		return deftProject;
	}
	
	
	public boolean hasCorrespondingDeftProject(IProject eclipseProject) {
		return getDeftProject(eclipseProject) != null;
	}
	

	/**
	 * Returns the workspace copy of a DEFT chapter if there is one, otherwise
	 * returns null. 
	 * 
	 * @param deftProject the project to which the chapter belongs. This parameter
	 * has been introduced so we can still get the file if the chapter has already
	 * been deleted and this method is called to retrieve the IFile to be deleted.
	 * 
	 * @param deftChapter the chapter for which to retrieve the corresponding IFile
	 */
	public IFile getEclipseChapterInWorkspace(Project deftProject, Chapter deftChapter) {
		if (hasCorrespondingEclipseProject(deftProject)) {
			IProject eclipseProject = getEclipseProjectInWorkspace(deftProject);
			String mainFolderName = getChaptersMainFolderName();
			IFolder mainFolder = eclipseProject.getFolder(mainFolderName);
			if (mainFolder.exists()) {
				String chapterFolderName = computeFolderName(deftChapter);
				IFolder chapterFolder = mainFolder.getFolder(chapterFolderName);
				if (chapterFolder.exists()) {
					String eclipseChapterName = computeFileName(deftChapter);
					IFile eclipseChapter = chapterFolder.getFile(eclipseChapterName);
					if (eclipseChapter.exists()) {
						return eclipseChapter;
					}
				}
			}
		} 
		return null;
	}
	
	
	public boolean hasCorrespondingEclipseChapter(Chapter deftChapter) {
		return getEclipseChapterInWorkspace(deftChapter.getProject(), deftChapter) != null;
	}
	
	
	public Chapter getDeftChapter(IFile eclipseChapter) {
		IFolder folder = (IFolder)eclipseChapter.getParent();
		String deftChapterId = folder.getName();
		Fragment chapterFragment = cms.getFragment(deftChapterId);
		return (Chapter)chapterFragment;
	}
	
	public boolean hasCorrespondingDeftChapter(IFile eclipseChapter) {
		return getDeftChapter(eclipseChapter) != null;
	}
	
	
	public List<IProject> getEclipseProjectsInWorkspace() {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject[] eclipseProjects = root.getProjects();
		List<IProject> eclipseProjectsList = Arrays.asList(eclipseProjects);
		return eclipseProjectsList;
	}
	
	public List<IFile> getEclipseChaptersInWorkspace(IProject eclipseProject) {
		List<IFile> chapterFiles = new LinkedList<IFile>();
		IFolder mainChapterFolder = eclipseProject.getFolder(getChaptersMainFolderName());
		if (mainChapterFolder != null) {
			List<IFolder> chapterFolders = getChapterFolders(mainChapterFolder);
			for (IFolder chapterFolder : chapterFolders) {
				IFile file = getFirstFileFromFolder(chapterFolder);
				chapterFiles.add(file);
			}
		}
		return chapterFiles;
	}
		
	
	private List<IFolder> getChapterFolders(IFolder mainChapterFolder) {
		try {
			List<IFolder> chapterFolders = new LinkedList<IFolder>();
			if (mainChapterFolder.exists()) {
				IResource[] childResources = mainChapterFolder.members();
				for (IResource child : childResources) {
					if (child instanceof IFolder) {
						chapterFolders.add((IFolder)child);
					}
				}
			}
			return chapterFolders;
		} catch (CoreException e) {
			e.printStackTrace();
		}
		//null will be returned if something severe happens. we don't want
		//to return an empty list and pretend nothing of importance happened
		return null;
	}

	
	/**
	 * Returns the first file in a folder. It is presumed that there is
	 * only one file. 
	 */
	private IFile getFirstFileFromFolder(IFolder folder) {
		try {
			IResource[] resources = folder.members();
			for (IResource resource : resources) {
				if (resource instanceof IFile) {
					return (IFile)resource;
				}
			}
		} catch (CoreException e) {
			e.printStackTrace();
		}
		return null;
	}

}
