package org.deft.repository.workspaceresources;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.deft.repository.Repository;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.ProjectUpdateEvent;
import org.deft.repository.event.SimpleRepositoryUpdateListener;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

/**
 * The workspace manager is responsible to copy chapters from the repository into the "Eclipse"
 * workspace. This is necessary because some editors rely on the Eclipse resources framework (with
 * Interfaces such as IFile), which works only with files within the Eclipse workspace.
 * 
 * The workspace manager also deletes chapters when they are deleted from the repository. Furthermore,
 * upon start and end of DEFT the workspace is synchronized with the DEFT repository.
 * 
 * It would have been possible to use links instead of copies, but the decision has been in favor of 
 * copies for two reasons.
 * 
 *   1) The repository API abstracts from files in a file system. However, links require files as targets.
 *   2) If Eclipse crashes during editing, the edited file might be left in a corrupt state. Since all edit
 *      operations are done on a copy, the repository contents will not be affected, though.
 *      
 * When a file copy is created, first a project (IProject) is created, which has the same name as the project
 * of the chapter in the repository. Within the project there is a folder called "chapters", in which a
 * folder with a unique name is created, which contains the actual file copy. The reason why the folder 
 * has been introduced is that the name of the file must be the name of the chapter, because some editors 
 * write the filename into the editor tabs. Filenames might not be unique, though, therefore the uniquely named 
 * folders exist.  
 */
public class WorkspaceManager {
	
	private static final ContentManagementService cms = Repository.instance.getContentManagementService();
	
	private static WorkspaceManager instance;
	
	private RepositoryWorkspaceNameMatcher matcher = new RepositoryWorkspaceNameMatcher();
	private boolean doSynchronize;
	
	private WorkspaceManager() {
		clearWorkspace();
		addRepositoryListener();
		synchronizeWorkspace();
		doSynchronize = false;
	}
	
	public static WorkspaceManager instance() {
		if (instance == null) {
			instance = new WorkspaceManager();
		}
		return instance;
	}
	
	
	public IProject getEclipseProjectInWorkspace(Project deftProject) {
		syncIfScheduled();
		if (!matcher.hasCorrespondingEclipseProject(deftProject)) {
			createEclipseProject(deftProject);
		}
		return matcher.getEclipseProjectInWorkspace(deftProject);
	}
	
	public Project getDeftProject(IProject eclipseProject) {
		syncIfScheduled();
		return matcher.getDeftProject(eclipseProject);
	}
	
	public IFile getEclipseChapterInWorkspace(Chapter deftChapter) {
		if (!matcher.hasCorrespondingEclipseChapter(deftChapter)) {
			createEclipseChapter(deftChapter);
		}
		Project deftProject = deftChapter.getProject();
		IFile file = getEclipseChapterInWorkspace(deftProject, deftChapter);
		return file;
	}
	
	private IFile getEclipseChapterInWorkspace(Project deftProject, Chapter deftChapter) {
		return matcher.getEclipseChapterInWorkspace(deftProject, deftChapter);
	}
	
	public Chapter getDeftChapter(IFile eclipseChapter) {
		syncIfScheduled();
		return matcher.getDeftChapter(eclipseChapter);
	}
	
	
	
	
	/**
	 * Adds a listener to react on repository events in order to delete workspace copies when
	 * the corresponding chapter or project is deleted.
	 */
	private void addRepositoryListener() {
		UpdateListener listener = new UpdateListener();
		cms.addRepositoryUpdateListener(listener);
	}

	
	private void refresh() {
		try {
			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			workspace.getRoot().refreshLocal(IResource.DEPTH_INFINITE, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Makes the workspace reflect the DEFT repository contents (projects and chapters) 
	 */
	public void synchronizeWorkspace() {
		refresh();
		deleteWorkspaceResourcesWithoutCorrespondingDeftCounterpart();
		createMissingWorkspaceResourcesForDeftRepositoryContent();
		doSynchronize = false;
	}

	
	private void syncIfScheduled() {
		if (doSynchronize) {
			synchronizeWorkspace();
		}
	}
	
	
	private void deleteWorkspaceResourcesWithoutCorrespondingDeftCounterpart() {
		List<IProject> eclipseProjects = matcher.getEclipseProjectsInWorkspace();
		for (IProject eclipseProject : eclipseProjects) {
			if (matcher.hasCorrespondingDeftProject(eclipseProject)) {
				deleteEclipseChaptersWithoutDeftChapter(eclipseProject);
			} else {
				deleteEclipseProject(eclipseProject);
			}
		}
	}
	
 
	private void deleteEclipseChaptersWithoutDeftChapter(IProject eclipseProject) {
		List<IFile> eclipseChapters = matcher.getEclipseChaptersInWorkspace(eclipseProject);
		for (IFile eclipseChapter : eclipseChapters) {
			if (!matcher.hasCorrespondingDeftChapter(eclipseChapter)) {
				deleteEclipseChapter(eclipseChapter);
			}
		}
	}
	
	
	private void deleteEclipseProject(IProject eclipseProject) {
		try {
			eclipseProject.delete(true, true, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	
	private void deleteEclipseChapter(IFile eclipseChapter) {
		try {
			IFolder parentFolder = (IFolder)eclipseChapter.getParent();
			parentFolder.delete(true, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	
	
	private void createMissingWorkspaceResourcesForDeftRepositoryContent() {
		List<Project> deftProjects = cms.getProjects();
		for (Project deftProject : deftProjects) {
			if (matcher.hasCorrespondingEclipseProject(deftProject)) {
				createMissingEclipseChapters(deftProject);
			} else {
				createEclipseProject(deftProject);
			}
		}
	}
	
	private void createEclipseProject(Project deftProject) {
		String projectResourceName = matcher.computeProjectName(deftProject);
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject projectResource = root.getProject(projectResourceName);
		try {
			projectResource.create(null);
			projectResource.open(null);
		} catch (CoreException ce) {
			ce.printStackTrace();
		}
		createMissingEclipseChapters(deftProject);
	}
	
	private void createMissingEclipseChapters(Project deftProject) {
		for (Fragment chapterFragment : deftProject.getFragments(FragmentFilter.CHAPTERFILTER)) {
			Chapter deftChapter = (Chapter)chapterFragment;
			if (!matcher.hasCorrespondingEclipseChapter(deftChapter)) {
				createEclipseChapter(deftChapter);
			}
		}
	}
	
	
	private void createEclipseChapter(Chapter deftChapter) {
		Project deftProject = deftChapter.getProject();
		IProject eclipseProject = matcher.getEclipseProjectInWorkspace(deftProject);
		String mainFolderName = matcher.getChaptersMainFolderName();
		IFolder mainChapterFolder = getOrCreateFolder(eclipseProject, mainFolderName);
		createEclipseChapter(mainChapterFolder, deftChapter);
	}
	
	private void createEclipseChapter(IFolder mainChapterFolder, Chapter deftChapter) {
		String folderName = matcher.computeFolderName(deftChapter);
		IFolder chapterFolder = getOrCreateFolder(mainChapterFolder, folderName);
		String fileName = matcher.computeFileName(deftChapter);
		IFile chapterFile = chapterFolder.getFile(fileName);
		saveChapterContentsToChapterFile(deftChapter, chapterFile);
	}
	
	
	/**
	 * Returns the IFolder object with the given name from a project. If
	 * the IFolder does not exist, it is created first. 
	 */
	private IFolder getOrCreateFolder(IContainer container, String folderName) {
		IFolder folder = container.getFolder(new Path(folderName));
		if (!folder.exists()) {
			try {
				folder.create(true, true, null);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
		return folder;
	}
	

	
	/**
	 * Copies the content of a chapter into an IFile.
	 */
	private void saveChapterContentsToChapterFile(Chapter chapter, IFile chapterFile) {
		InputStream chapterInputStream = getChapterInputStream(chapter);
		saveStreamInFile(chapterInputStream, chapterFile);
		try {
			chapterInputStream.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
	

	/**
	 * Returns an InputStream which contains the contents of the chapter. 
	 */
	private InputStream getChapterInputStream(Chapter chapter) {
		byte[] content = Repository.instance.getContentManagementService().getChapterContent(chapter);
		InputStream is = new ByteArrayInputStream(content);
		return is;
	}


	/**
	 * Saves the InputStream to an IFile 
	 */
	private void saveStreamInFile(InputStream is, IFile chapterFile) {
		try {
			if (chapterFile.exists()) {
				chapterFile.delete(true, null);
			}
			chapterFile.create(is, true, null);
			try {
				Thread.sleep(300);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} catch (CoreException ce) {
			ce.printStackTrace();
		}
	}
	
	
	
	
	
	public void saveWorkspaceCopyToRepository(Chapter chapter) {
		IFile chapterWorkspaceCopy = getEclipseChapterInWorkspace(chapter);
		byte[] content = getFileContent(chapterWorkspaceCopy);
		cms.saveChapterContent(chapter, content);
	}
	
	private byte[] getFileContent(IFile file) {
		try {
			InputStream is = file.getContents(true);
			byte[] contents = new byte[is.available()];
			is.read(contents, 0, contents.length);
			is.close();
			return contents;
		} catch (CoreException ce) {
			ce.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}
	
	
	
	
	public void addFileToEclipseProject(IProject project, IPath path) {
		IFile iFile = project.getFile(path);
		try {
			iFile.create(new ByteArrayInputStream(new byte[]{}), true, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	public void clearWorkspace() {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject[] eclipseProjects = root.getProjects();
		for (IProject eProject : eclipseProjects) {
			deleteEclipseProject(eProject);
		}
	}
	
	
	/**
	 * Listener that listens for repository changes in order to remove workspace copies of chapters
	 * and projects when the corresponding chapters and projects have been deleted from the repository.
	 */
	private class UpdateListener extends SimpleRepositoryUpdateListener {

		@Override
		public void projectCreated(ProjectUpdateEvent event) {
			syncIfScheduled();
			Project project = event.getProject();
			createEclipseProject(project);
		}
		
		@Override
		public void fragmentCreated(FragmentUpdateEvent event) {
			syncIfScheduled();
			Fragment fragment = event.getFragment();
			if (fragment instanceof Chapter) {
				Chapter chapter = (Chapter)fragment;
				createEclipseChapter(chapter);
			}
		}
		
		@Override
		public void fragmentRemoved(FragmentUpdateEvent event) {
			syncIfScheduled();
			Fragment fragment = event.getFragment();
			if (fragment instanceof Chapter) {
				Chapter chapter = (Chapter)fragment;
				Project project = event.getProject();
				IFile eclipseChapter = getEclipseChapterInWorkspace(project, chapter);
				deleteEclipseChapter(eclipseChapter);
				refresh();
			}
		}
		
		@Override
		public void projectRemoved(ProjectUpdateEvent event) {
			syncIfScheduled();
			Project project = event.getProject();
			IProject eclipseProject = getEclipseProjectInWorkspace(project);
			deleteEclipseProject(eclipseProject);
		}

		@Override
		public void contentLoaded() {
			//don't call synchronize, we don't want to delete the
			//projects which we have just used. There might still be open
			//streams which we cannot close because we don't have the stream objects
			//here
			createMissingWorkspaceResourcesForDeftRepositoryContent();
			doSynchronize = false;
		}
		
		@Override
		public void repositoryUpdated() {
		}
		
	}

}
