package org.deft.representation.identifiers.data;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public abstract class Scope {
	
	private String name;
	private Scope parent;
	private List<Scope> children = new LinkedList<Scope>();
	
	public Scope(String name) {
		if (name == null) {
			throw new IllegalArgumentException("The name must not be null.");
		}
		if (name.equals("")) {
			throw new IllegalArgumentException("The name must not be empty.");
		}
		this.name = name;
	}
	
	
	public abstract String getIconPath();
	public abstract String getIconPluginId();
	
	public abstract String getAsIdString();
	public abstract Scope getDescendantScopeById(String id);
	public abstract char getDelimiter();
	
	
	

	public void addChildScope(Scope scope) {
		children.add(scope);
		scope.parent = this;
	}
	
	public Scope getParentScope() {
		return parent;
	}
	
	public Scope getRootScope() {
		Scope parent = getParentScope();
		while (parent != null) {
			Scope newParent = parent.getParentScope();
			if (newParent == null) {
				return parent;
			} else {
				parent = newParent;
			}
		}
		return this;
	}
	
	public List<Scope> getChildScopes() {
		return new LinkedList<Scope>(children);
	}
	
	public String getName() {
		return name;
	}
	
	
	public List<Scope> getThisAndDescendentScopesAsList() {
		List<Scope> list = new LinkedList<Scope>();
		list.add(this);
		for (Scope child : children) {
			List<Scope> childScopeList = child.getThisAndDescendentScopesAsList();
			list.addAll(childScopeList);
		}
		return list;
	}
			
	
	/**
	 * Returns a list with all ancestor scopes of this scope.
	 * 
	 * The root is at the beginning of the list and the direct parent
	 * is at the end of the list.
	 */
	public List<Scope> getAncestorScopesAsList() {
		Scope s = getParentScope(); 
		if (s != null) {
			List<Scope> list = s.getAncestorScopesAsList();
			list.add(s);
			return list;
		}
		return new LinkedList<Scope>();
	}
	
	/**
	 * Returns the fullLocalName.
	 * The fullLocalName is in many cases just the name
	 * of the scope (usually a named scope such as a class).
	 * But in some cases this method
	 * should be overridden, for example with methods.
	 * The fullLocalName of a method includes also
	 * a parameter list.
	 */
	public String getFullLocalName() {
		return getName();
	}
	
	
	
	/**
	 * Returns the fully qualified name of this scope.
	 * The fully qualified name consists of the fullLocalName
	 * of each ancestor scope (starting from the root),
	 * separated by a . (dot).
	 */
	public String getFullyQualifiedName() {
		List<Scope> idents = getAncestorScopesAsList();
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < idents.size(); i++) {
			Scope ident = idents.get(i);
			sb.append(ident.getFullLocalName());
			sb.append(getDelimiter());
		}
		sb.append(this.getFullLocalName());
		String fqName = sb.toString();
		return fqName;
	}

	/**
	 * Returns the scope with the relative qualified name.
	 * 
	 * Example:
	 * Method is called from a class called MyClass with the parameter MyClass.x()
	 * Returned is the method under MyClass with the fullLocalName x()
	 * 
	 * Note that in order to find the scope for an absolute name you need
	 * to call this method from the root scope
	 */
	public Scope findScope(String relativeQualifiedName) {
		List<String> elements = getNameElements(relativeQualifiedName);
		if (!elements.isEmpty()) {
			String head = elements.remove(0);
			if (head.equals(getFullLocalName())) {
				Scope ident = findScope(elements);
				return ident;
			}
		}
		return null;
	}
	
	/**
	 * Recursive helper method.
	 * Tries to find the child scope that matches the next
	 * element of the qualified name. 
	 */
	private Scope findScope(List<String> nameElements) {
		if (nameElements.isEmpty()) {
			return this;
		} else {
			String head = nameElements.remove(0);
			for (Scope child : children) {
				if (head.equals(child.getFullLocalName())) {
					Scope ident = child.findScope(nameElements);
					return ident;
				}
			}
		}
		return null;
	}

	
	/**
	 * Transforms the qualified name into a list of named elements.
	 * The separator character is the . (dot).
	 */
	protected List<String> getNameElements(String qualifiedName) {
		String delimiter = getDelimiterAndEscapeIfNecessary();
		String[] elements = qualifiedName.split(delimiter);
		List<String> list = Arrays.asList(elements);
		//we need to remove elements in other methods
		//so we cannot keep the current list which does not support removal
		return new LinkedList<String>(list);
	}
	
	private String getDelimiterAndEscapeIfNecessary() {
		char delimiter = getDelimiter();
		if (delimiter == '.') {
			return "\\.";
		}
		if (delimiter == '\\') {
			return "\\\\";
		}
		return new Character(delimiter).toString();
	}
	
}
