package org.deft.representation.identifiers.ecore;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.representation.identifiers.data.Scope;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.gmf.runtime.notation.impl.EdgeImpl;
import org.eclipse.gmf.runtime.notation.impl.NodeImpl;

public class EcoreDiagramToIdentTransformer {

	private Map<EClass, List<EReference>> classRefMap = new HashMap<EClass, List<EReference>>();
	private Map<EReference, String> referenceIdMap = new HashMap<EReference, String>();
	
	public Scope transform(List<Diagram> diagrams) {
		classRefMap.clear();
		if (diagrams.size() == 1) {
			Diagram d = diagrams.get(0);
			EcoreDiagramScope scope = transform(d);
			return scope;
		} else {
			EcoreDiagramCollectionScope scope = new EcoreDiagramCollectionScope();
			for (Diagram d : diagrams) {
				EcoreDiagramScope ds = transform(d);
				scope.addChildScope(ds);
			}
			return scope;
		}
	}
	
	private EcoreDiagramScope transform(Diagram diagram) {
		findAllReferences(diagram);
		EcoreDiagramScope scope = new EcoreDiagramScope(diagram.getName());
		List<NodeImpl> diagramAndNodeImpls = getDiagramsAndNodeImplsWithBasicElements(diagram.eContents());
		fill(scope, diagramAndNodeImpls);
		return scope;
	}
	
	/**
	 * Unlike Ecore models, Ecore diagrams do not have references contained in
	 * classes. References are stored separately. Therefore we first search all
	 * diagram edges which represent references and store the references together with
	 * their source eclass. Later, when classes are processed, the stored class-reference
	 * relationship is used to build the correct scope hierarchy.
	 */
	private void findAllReferences(Diagram diagram) {
		Iterator<EObject> it = diagram.eAllContents();
		while (it.hasNext()) {
			EObject eo = it.next();
			if (eo instanceof EdgeImpl) {
				EdgeImpl ei = (EdgeImpl) eo;
				EObject basicEdge = ei.basicGetElement();
				if (basicEdge instanceof EReference) {
					EReference ref = (EReference)basicEdge;
					View v = (View)ei.getSource();
					if (v instanceof NodeImpl) {
						NodeImpl ni = (NodeImpl)v;
						EObject basicNode = ni.basicGetElement();
						if (basicNode instanceof EClass) {
							EClass ec = (EClass)basicNode;
							List<EReference> list = classRefMap.get(ec);
							if (list == null) {
								list = new LinkedList<EReference>();
								classRefMap.put(ec,  list);
							}
							list.add(ref);
							String id = getId(eo);
							referenceIdMap.put(ref, id);
						}
					}
				}
			}
		}
	}
	
	public List<NodeImpl> getDiagramsAndNodeImplsWithBasicElements(List<EObject> contentList) {
		List<NodeImpl> list = new LinkedList<NodeImpl>();
		for (EObject eo : contentList) {
			if (eo instanceof NodeImpl) {
				NodeImpl child = (NodeImpl)eo;
				if (child instanceof Diagram) {
					list.add(child);
				} else if (child.basicGetElement() != null) {
					list.add(child);
				} else {
					List<NodeImpl> recursive = getDiagramsAndNodeImplsWithBasicElements(child.eContents());
					list.addAll(recursive);
				}
			}
		}
		return list;
	}
	
	private void fill(Scope scope, NodeImpl vi) {
		fill(scope, getDiagramsAndNodeImplsWithBasicElements(vi.eContents()));
	}
	
	private void fill(Scope scope, List<NodeImpl> contents) {
		for (NodeImpl ni : contents) {
			EObject be = ni.basicGetElement();
			if (be instanceof EPackage) {
				try {
					EcorePackageIdentifier epi = createPackageIdentifier(ni, scope);
					scope.addChildScope(epi);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (be instanceof EClass) {
				try {
					EcoreClassIdentifier eci = createClassIdentifier(ni, scope);
					scope.addChildScope(eci);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (be instanceof EEnum) {
				try {
					EcoreEnumIdentifier eei = createEnumIdentifier(ni, scope);
					scope.addChildScope(eei);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (be instanceof EEnumLiteral) {
				try {
					EcoreEnumLiteralIdentifier eeli = createEnumLiteralIdentifier(ni, scope);
					scope.addChildScope(eeli);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (be instanceof EAttribute) {
				try {
					EcoreAttributeIdentifier eai = createAttributeIdentifier(ni, scope);
					scope.addChildScope(eai);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (be instanceof EAnnotation) {
				try {
					EcoreAnnotationIdentifier eai = createAnnotationIdentifier(ni, scope);
					scope.addChildScope(eai);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (be instanceof EOperation) {
				try {
					EcoreOperationIdentifier eoi = createOperationIdentifier(ni, scope);
					scope.addChildScope(eoi);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (be instanceof EParameter) {
				try {
					EcoreParameterIdentifier epi = createParameterIdentifier(ni, scope);
					scope.addChildScope(epi);
				} 
				catch (Exception e) {e.printStackTrace();}
			} 
		}
	}
	
	private String getId(EObject eo) {
		Resource r = eo.eResource();
		if (r instanceof XMIResource) {
			XMIResource xmir = (XMIResource)r;
			String id = xmir.getID(eo);
			return id;
		}
		return null;
	}
	
	private EcorePackageIdentifier createPackageIdentifier(NodeImpl ni, Scope parent) {
		EPackage ep = (EPackage)ni.basicGetElement();
		String name = ep.getName();
		String id = getId(ni);
		EcorePackageIdentifier ident = new EcorePackageIdentifier(name, id);
		fill(ident, ni);
		return ident;
	}
	
	private EcoreClassIdentifier createClassIdentifier(NodeImpl ni, Scope parent) {
		EClass ec = (EClass)ni.basicGetElement();
		String name = ec.getName();
		if (name != null && !name.equals("")) {
			String id = getId(ni);
			EcoreClassIdentifier ident = new EcoreClassIdentifier(name, id);
			addExistingReferences(ec, ident);
			fill(ident, ni);
			return ident;		
		}
		return new EcoreClassIdentifier("ERROR", "ERROR");
	}
	
	private void addExistingReferences(EClass ec, EcoreClassIdentifier classIdent) {
		List<EReference> eRefs = classRefMap.get(ec);
		if (eRefs != null) {
			for (EReference er : eRefs) {
				String name = er.getName();
				String id = referenceIdMap.get(er);
				EClassifier eType = er.getEType();
				String type = eType.getName();
				int lowerBound = er.getLowerBound();
				int upperBound = er.getUpperBound();
				EcoreReferenceIdentifier refIdent = new EcoreReferenceIdentifier(
						name, id, type, lowerBound, upperBound);
				classIdent.addChildScope(refIdent);
			}
		}
	}
	
	private EcoreEnumIdentifier createEnumIdentifier(NodeImpl ni, Scope parent) {
		EEnum ee = (EEnum)ni.basicGetElement();
		String name = ee.getName();
		String id = getId(ni);
		EcoreEnumIdentifier ident = new EcoreEnumIdentifier(name, id);
		fill(ident, ni);
		return ident;		
	}
	
	private EcoreEnumLiteralIdentifier createEnumLiteralIdentifier(NodeImpl ni, Scope parent) {
		EEnumLiteral eel = (EEnumLiteral)ni.basicGetElement();
		String name = eel.getName();
		String id = getId(ni);
		EcoreEnumLiteralIdentifier ident = new EcoreEnumLiteralIdentifier(name, id);
		fill(ident, ni);
		return ident;		
	}
	
	private EcoreAttributeIdentifier createAttributeIdentifier(NodeImpl ni, Scope parent) {
		EAttribute ea = (EAttribute)ni.basicGetElement();
		String name = ea.getName();
		String id = getId(ni);
		EClassifier eType = ea.getEType();
		String type = null;
		if (eType != null) {
			type = eType.getName();
		}
		int lowerBound = ea.getLowerBound();
		int upperBound = ea.getUpperBound();
		EcoreAttributeIdentifier ident = new EcoreAttributeIdentifier(name, id, type, lowerBound, upperBound);
		fill(ident, ni);
		return ident;
	}
	

		
	private EcoreOperationIdentifier createOperationIdentifier(NodeImpl ni, Scope parent) {
		EOperation eo = (EOperation)ni.basicGetElement();
		String name = eo.getName();
		String id = getId(ni);
		EClassifier eType = eo.getEType();
		String type = "";
		if (eType != null) {
			type = eType.getName();
		}
		int lowerBound = eo.getLowerBound();
		int upperBound = eo.getUpperBound();
		EcoreOperationIdentifier ident = new EcoreOperationIdentifier(name, id, type, lowerBound, upperBound);
		//no parameters, because those are not part of the diagram
		return ident;
	}
	
	
	private EcoreParameterIdentifier createParameterIdentifier(NodeImpl ni, Scope parent) {
		EParameter ep = (EParameter)ni.basicGetElement();
		String name = ep.getName();
		String id = getId(ni);
		EClassifier eType = ep.getEType();
		String type = eType.getName();
		EcoreParameterIdentifier ident = new EcoreParameterIdentifier(name, id, type);
		fill(ident, ni);
		return ident;
	}
	
	
	private EcoreAnnotationIdentifier createAnnotationIdentifier(NodeImpl ni, Scope parent) {
		EAnnotation ea = (EAnnotation)ni.basicGetElement();
		String name = ea.getSource();
		String id = getId(ni);
		EcoreAnnotationIdentifier ident = new EcoreAnnotationIdentifier(name, id);
		return ident;
	}

}
