package org.deft.representation.styleddocument.conversion;

import java.awt.Color;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import org.deft.representation.styleddocument.data.StyledToken;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;
import com.lowagie.text.rtf.RtfWriter2;

/**
 * The RtfMapping creates an RTF-string from plain text
 * and additionally provided formatting information.
 * 
 * The actual RTF string generation is delegated to the iText library. 
 */
public class RtfMapping implements PlainText2StyledTextMapping {
	
	private Document doc;
	private OutputStream outStream;
	
	private Chunk currentString;
	private Font currentFont;

	private String styledText = "";
	
	public RtfMapping(){
		// empty
	}

	@Override
	public void finish() {
		doc.close();
		styledText = outStream.toString();
	}

	@Override
	public String getStyledText() {
		return styledText;
	}


	@Override
	public void init() {
		doc = new Document();
		outStream = new ByteArrayOutputStream();
		RtfWriter2.getInstance(doc, outStream);
		doc.open();
	}
	
	
	@Override
	public void nextToken(StyledToken token) {
		nextString(token.getText());
		if (token.isBold()) setBold();
		if (token.isItalic()) setItalic();
		if (token.isUnderlined()) setUnderline();
		if (token.isStrikeThrough()) setStrikeThrough();
		if (token.getFontFamily() != null) {
			if (token.getFontFamily().equals("Courier New")) {
				setMonospace();
			}
		}
		Color c = token.getColor();
		setForeground(c.getRed(), c.getGreen(), c.getBlue());
	}
	

	@Override
	public void nextString(String nextString) {
		//the RTF library does not like windows linebreaks, so replace
		//them with unix linebreaks
		nextString = nextString.replaceAll("\r\n", "\n");
		if(currentString != null){
			try {
				currentString.setFont(currentFont);
				doc.add(currentString);
			} catch (DocumentException e) {
				e.printStackTrace();
			}
		}
		currentString = new Chunk(nextString);
		// default style
		currentFont = new Font();
		currentFont.setStyle(Font.NORMAL);
		currentFont.setColor(0, 0, 0);
	}

	@Override
	public void setBold() {
		currentFont.setStyle(currentFont.getStyle() | Font.BOLD);
	}

	@Override
	public void setForeground(int r, int g, int b) {
		currentFont.setColor(r, g, b);
	}

	@Override
	public void setItalic() {
		currentFont.setStyle(currentFont.getStyle() | Font.ITALIC);
	}

	@Override
	public void setUnderline() {
		currentFont.setStyle(currentFont.getStyle() | Font.UNDERLINE);
	}

	@Override
	public void setStrikeThrough() {
		currentFont.setStyle(currentFont.getStyle() | Font.STRIKETHRU);
	}

	@Override
	public void setMonospace() {
		currentFont.setFamily("Courier");
	}
	
}
