package org.deft.representation.styleddocument.data;

import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Style;
import javax.swing.text.StyleContext;

import org.deft.representation.styleddocument.conversion.PlainText2StyledTextMapping;
import org.deft.util.StringUtil;

/**
 * A TokenizedStyledDocument is a StyledDocument which memorizes which strings were
 * inserted at which offsets and with which Style. In the DefaultStyledDocument this
 * information is lost and one has to work at character level.
 * 
 * The strings and their related information are stored in a StyledToken.
 */
@SuppressWarnings("serial")
public class TokenizedStyledDocument extends DefaultStyledDocument implements InlineContent {

	private List<List<StyledToken>> styledTokens = new LinkedList<List<StyledToken>>();
	
	public TokenizedStyledDocument() {
	}
	
	public TokenizedStyledDocument(StyleContext styles) {
		super(styles);
	}
	
	
	@Override
	public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
		super.insertString(offs, str, a);
		if (a instanceof Style) {
			Style style = (Style)a;
			StyledToken styledToken = new StyledToken(str, offs, style);
			
			List<StyledToken> newTokens = splitIfNecessary(styledToken);

			for (int i = 0; i < newTokens.size(); i++) {
				StyledToken t = newTokens.get(i);
				List<StyledToken> lastLine = getLastLineTokens();
				
				if (t.getText().equals("\n")) {
					List<StyledToken> newLine = new LinkedList<StyledToken>();
					styledTokens.add(newLine);
				} else {
					lastLine.add(t);
				}
			}
			
		}
	}
	
	private List<StyledToken> splitIfNecessary(StyledToken token) {
		List<StyledToken> tokens = new LinkedList<StyledToken>();
		String text = token.getText();
		StringTokenizer tokenizer = new StringTokenizer(text, "\n", true);
		while (tokenizer.hasMoreElements()) {
			String stringToken = tokenizer.nextToken();
			StyledToken t = new StyledToken(stringToken, 0, token.getStyle());
			tokens.add(t);
		}
		return tokens;
	}
	
	private List<StyledToken> getLastLineTokens() {
		if (styledTokens.isEmpty()) {
			List<StyledToken> firstLine = new LinkedList<StyledToken>();
			styledTokens.add(firstLine);
		}
		return styledTokens.get(styledTokens.size() - 1);
	}
	
	public List<List<StyledToken>> getLines() {
		//create a copy;
		List<List<StyledToken>> lines = new LinkedList<List<StyledToken>>();
		for (int i = 0; i < styledTokens.size(); i++) {
			List<StyledToken> line = getLine(i);
			lines.add(line);
		}
		return lines;
	}
	
	public List<StyledToken> getLine(int line) {
		List<StyledToken> tokens = styledTokens.get(line);
		return new LinkedList<StyledToken>(tokens); //copy
	}
	
	
	

	
	private boolean isEmptyLine(List<StyledToken> line) {
		for (StyledToken token : line) {
			if (!token.isWhitespace()) {
				return false;
			}
		}
		return true;
	}
	
	
	public String getMarkedUpText(PlainText2StyledTextMapping mapping) {
		mapping.init();
		int lineCount = getLines().size();
		for (int i = 0; i < lineCount; i++) {
			List<StyledToken> line = getLines().get(i);
			for (StyledToken st : line) {
				mapping.nextToken(st);
			}
			if (i < lineCount - 1) {
				StyledToken linebreakToken = new StyledToken("\n", 0, getDefaultStyle());
				mapping.nextToken(linebreakToken);
			}
		}
		mapping.nextString("");
		mapping.finish();
		String markedupString = mapping.getStyledText();
		return markedupString;
	}
	
	
	public boolean equals(Object o) {
		if (o instanceof TokenizedStyledDocument) {
			TokenizedStyledDocument doc2 = (TokenizedStyledDocument)o;
			List<List<StyledToken>> st1 = this.getLines();
			List<List<StyledToken>> st2 = doc2.getLines();
			boolean equals = st1.equals(st2);
			return equals;
		} else {
			return false;
		}
	}
	
	
	public Style getDefaultStyle() {
		StyleContext context = new StyleContext();
		Style defaultStyle = context.getStyle(StyleContext.DEFAULT_STYLE);
		return defaultStyle;
	}
	
	public TokenizedStyledDocument multilineTrim(int firstLineOffset) {
		List<List<StyledToken>> styledTokensCopy = getStyledTokensCopyWithFirstLineOffset(firstLineOffset);
		int leadingSpacesCount = getMinLeadingSpacesCount(styledTokensCopy);
		removeLeadingCharacters(styledTokensCopy, leadingSpacesCount);
		
		TokenizedStyledDocument doc = createStyledDocFromTokens(styledTokensCopy);

		return doc;
	}

	/**
	 * returns the minimum number of leading spaces of all lines
	 */
	private int getMinLeadingSpacesCount(List<List<StyledToken>> styledTokensCopy) {
		int minCount = Integer.MAX_VALUE;
		for (int i = 0; i < getLines().size(); i++) {
			List<StyledToken> line = styledTokensCopy.get(i);
			if (!isEmptyLine(line)) {
				int leadingSpacesCount = getLeadingSpacesCount(line);
				minCount = Math.min(minCount, leadingSpacesCount);
			}
		}
		return minCount;
	}
	
	private int getLeadingSpacesCount(List<StyledToken> line) {
		int length = 0;
		for (StyledToken token : line) {
			if (token.isWhitespace()) {
				length += token.getText().length();
			} else {
				length += StringUtil.getLeadingSpacesCount(token.getText());
				break;
			}
		}
		return length;
	}
	
	private List<List<StyledToken>> getStyledTokensCopyWithFirstLineOffset(int firstLineOffset) {
		List<List<StyledToken>> styledTokensCopy = getLines();
		String leadingSpaces = StringUtil.makeSpaces(firstLineOffset);
		StyledToken leadingSpacesToken = new StyledToken(leadingSpaces, 0, getDefaultStyle());
		styledTokensCopy.get(0).add(0, leadingSpacesToken);
		return styledTokensCopy;
	}

	private void removeLeadingCharacters(List<List<StyledToken>> lines, int leadingSpacesCount) {
		for (List<StyledToken> line : lines) {
			removeLeadingCharactersFromLine(line, leadingSpacesCount);
		}
	}
	
	private void removeLeadingCharactersFromLine(List<StyledToken> line, int leadingSpacesCount) {
		int charsToRemove = leadingSpacesCount;
		List<StyledToken> lineCopy = new LinkedList<StyledToken>(line);
		for (StyledToken token : lineCopy) {
			if (token.getText().length() <= charsToRemove) {
				line.remove(token);
				charsToRemove -= token.getText().length();
			} else {
				String text = token.getText();
				String newText = text.substring(charsToRemove);
				StyledToken newToken = new StyledToken(newText, 0, token.getStyle());
				line.remove(token);
				line.add(0, newToken);
				break;
			}
		}
	}
	
	
	private TokenizedStyledDocument createStyledDocFromTokens(List<List<StyledToken>> styledTokensCopy) {
		TokenizedStyledDocument doc = new TokenizedStyledDocument();
		try {
			int lastLineIndex = styledTokensCopy.size() - 1;
			for (int l = 0; l <= lastLineIndex; l++) {
				List<StyledToken> line = styledTokensCopy.get(l);
				for (StyledToken token : line) {
					doc.insertString(doc.getLength(), token.getText(), token.getStyle());
				}
				if (l != lastLineIndex) {
					doc.insertString(doc.getLength(), "\n", getDefaultStyle());
				}
			}
			return doc;
		} catch (BadLocationException be) {
			be.printStackTrace();
		}
		return null;
	}
}
