package org.deft.representation.table.data;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class Table {

	private List<TableRow> rows = new LinkedList<TableRow>();
	private boolean isLocked = false;
	private List<Map<Integer, TableCell>> columnMaps;
	
	private List<TableCell> colHeaders = new LinkedList<TableCell>();
	private List<TableCell> rowHeaders = new LinkedList<TableCell>();
	
	public Table() {
	}
	
	public Table(String[][] cells) {
		for (int i = 0; i < cells.length; i++) {
			TableRow row = new TableRow();
			addRow(row);
			for (int j = 0; j < cells[i].length; j++) {
				String text = "";
				try {
					text = cells[i][j];
				}
				catch (Exception e) {}
				TableCell cell = new TableCell(text);
				row.addCell(cell);
			}
		}
		lockTable();
	}
	
	
	public void lockTable() {
		isLocked = false;
		columnMaps = getColumnMapsForAllRows();
		isLocked = true;
	}
	
	public void unlockTable() {
		isLocked = false;
	}
	
	
	public boolean hasColumnHeaders() {
		return !colHeaders.isEmpty();
	}

	
	
	public boolean hasRowHeaders() {
		return !rowHeaders.isEmpty();
	}
	
	
	public TableCell getColumnHeader(int col) {
		return colHeaders.get(col - 1);
	}
	
	
	public TableCell getRowHeader(int row) {
		return rowHeaders.get(row - 1);
	}
	
	
	public void addRow(TableRow row) {
		rows.add(row);
	}
	
	
	public void addRowHeaderCell(TableCell cell) {
		rowHeaders.add(cell);
	}
	
	public void setRowHeaderCells(List<TableCell> cells) {
		rowHeaders.clear();
		if (cells != null) {
			rowHeaders.addAll(cells);
		}
	}
	
	
	public void addColumnHeaderCell(TableCell cell) {
		colHeaders.add(cell);
	}

	public void setColumnHeaderCells(List<TableCell> cells) {
		colHeaders.clear();
		if (cells != null) {
			colHeaders.addAll(cells);
		}
	}

	
	
	public int getRowCount() {
		return rows.size();
	}
	
	public int getColumnCount() {
		if (rows.isEmpty()) {
			return 0; 
		} else {
			TableRow firstRow = rows.get(0);
			int colCount = 0;
			for (TableCell cell : firstRow.getCells()) {
				int cellColSpan = cell.getColSpan();
				colCount += cellColSpan;
			}
			return colCount;
		}
	}
	

	public List<TableCell> getCellsStartingInRow(int rowNumber) {
		List<TableCell> cells = new LinkedList<TableCell>();
		if (rowNumber <= getRowCount()) {
			List<TableCell> rowContent = rows.get(rowNumber - 1).getCells();
			cells.addAll(rowContent);
		}
		
		return cells;
	}
	
	
	
	public List<TableCell> getCellsStartingInColumn(int colNumber) {
		List<TableCell> cells = new LinkedList<TableCell>();
		for (Map<Integer, TableCell> map : getColumnMapsForAllRows()) {
			TableCell cell = map.get(colNumber);
			if (cell != null && !cells.contains(cell)) {
				cells.add(cell);
			}
		}
		return cells;
	}
	
	
	/**
	 * Returns all cells found on a specific row. This includes both the cells that
	 * start on the row and the cells that start on a previous row and have a rowspan
	 * to reach this row.  
	 */
	public List<TableCell> getCellsContainedInRow(int row) {
		List<TableCell> cells = new LinkedList<TableCell>();
		for (int col = 1; col <= getColumnCount(); col++) {
			TableCell cell = getTableCell(row, col);
			if (!cells.contains(cell)) {
				cells.add(cell);
			}
		}
		return cells;
	}
		
	
	
	public List<TableCell> getCellsContainedInColumn(int col) {
		List<TableCell> cells = new LinkedList<TableCell>();
		for (int row = 1; row <= getRowCount(); row++) {
			TableCell cell = getTableCell(row, col);
			if (!cells.contains(cell)) {
				cells.add(cell);
			}
		}
		return cells;
	}

	
	
	/**
	 * Returns a list of maps indicating the cell columns for each row.
	 * 
	 * Each map in the list represents a table row. The map has columns as key
	 * and the table cell starting at this column as value. If a table cell spans
	 * multiple columns, only the first column is in the map.
	 * If a table cell spans multiple rows, it is entered in all corresponding maps.
	 * 
	 * So it is possible to check for each row which cells are contained at which column.
	 */
	private List<Map<Integer, TableCell>> getColumnMapsForAllRows() {
		if (!isLocked) {
			List<Map<Integer, TableCell>> columnMaps = new LinkedList<Map<Integer, TableCell>>();
			//add an empty map for each row
			for (int r = 1; r <= getRowCount(); r++) {
				columnMaps.add(new HashMap<Integer, TableCell>());
			}
			//add for each row the newly encountered cells 
			for (int r = 1; r <= getRowCount(); r++) {
				TableRow row = rows.get(r - 1);
				Map<Integer, TableCell> map = columnMaps.get(r - 1);
				for (TableCell cell : row.getCells()) {
					int nextCol = computeLowestAvailableCol(map);
					map.put(nextCol, cell);
					//if the cell spans multiple rows, add an entry to the other maps as well
					for (int span = 2; span <= cell.getRowSpan(); span++) {
						Map<Integer, TableCell> m = columnMaps.get(r-2+span);
						m.put(nextCol, cell);
					}
				}
				
			}
			return columnMaps;
		} else {
			return this.columnMaps;
		}
	}
	
	
	
	private int computeLowestAvailableCol(Map<Integer, TableCell> cellsMap) {
		if (!cellsMap.keySet().contains(1)) {
			return 1;
		} else {
			List<Integer> colStarts = new LinkedList<Integer>(cellsMap.keySet());
			Collections.sort(colStarts);
			for (int col : colStarts) {
				TableCell cell = cellsMap.get(col);
				int nextPossibleCol = col + cell.getColSpan();
				if (!colStarts.contains(nextPossibleCol)) {
					return nextPossibleCol;
				}
			}
		}
		return -1;
	}
	

	
	public TableCell getTableCell(int row, int column) {
		if (row < 1 || row > getRowCount()) {
			throw new IndexOutOfBoundsException("Row index out of bounds: " + row);
		}
		if (column < 1 || column > getColumnCount()) {
			throw new IndexOutOfBoundsException("Column index out of bounds: " + column);
		}

		List<Map<Integer, TableCell>> maps = getColumnMapsForAllRows();
		Map<Integer, TableCell> map = maps.get(row - 1);
		int columnStartIndex = getThisOrNextSmallerColumnFromMap(map.keySet(), column); 
		TableCell cell = map.get(columnStartIndex);
		return cell;
	}
	
	private int getThisOrNextSmallerColumnFromMap(Set<Integer> columns, int searchedColumn) {
		List<Integer> cols = new LinkedList<Integer>(columns);
		Collections.sort(cols);
		int previousCol = 0;
		for (int c : cols) {
			if (c > searchedColumn) {
				return previousCol;
			}
			previousCol = c;
		}
		return previousCol;
	}
	
	
	public int getStartingRow(TableCell cell) {
		for (int row = 1; row < getRowCount(); row++) {
			List<TableCell> cells = getCellsContainedInColumn(row);
			if (cells.contains(cell)) {
				return row;
			}
		}
		return -1;
	}
	
	
	public int getStartingColumn(TableCell cell) {
		for (int col = 1; col <= getColumnCount(); col++) {
			List<TableCell> cells = getCellsContainedInColumn(col);
			if (cells.contains(cell)) {
				return col;
			}
		}
		return -1;
	}
	
	
	
	public int getNumberOfColumnInRow(TableCell cell, int row) {
		List<TableCell> cells = getCellsContainedInRow(row);
		int number = cells.indexOf(cell) + 1;
		return number;
	}
	
	
	public int getNumberOfRowInColumn(TableCell cell, int column) {
		List<TableCell> cells = getCellsContainedInColumn(column);
		int number = cells.indexOf(cell) + 1;
		return number;
	}
	
	
	public boolean equals(Object o) {
		if (o instanceof Table) {
			Table table = (Table)o;
			boolean equals = (rows.equals(table.rows));
			return equals;
		} else {
			return false;
		}
	}
	
}
