package org.deft.requirementsontology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.collections15.multimap.MultiHashMap;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLCommentAnnotation;
import org.semanticweb.owl.model.OWLConstant;
import org.semanticweb.owl.model.OWLDataProperty;
import org.semanticweb.owl.model.OWLDataPropertyExpression;
import org.semanticweb.owl.model.OWLDataSomeRestriction;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLIndividual;
import org.semanticweb.owl.model.OWLObjectProperty;
import org.semanticweb.owl.model.OWLObjectPropertyExpression;
import org.semanticweb.owl.model.OWLObjectSomeRestriction;
import org.semanticweb.owl.model.OWLTypedConstant;

import edu.uci.ics.jung.graph.DirectedOrderedSparseMultigraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.OrderedSparseMultigraph;

public abstract class AbstractROModel
{
	/**
	 * contains individuals as vertices and object properties as edges
	 */
	private Graph<IndividualVertex,ObjectPropertyEdge> currentGoalModelGraph;
	/**
	 * contains individuals that have data properties and these properties
	 */
	private MultiHashMap<IndividualVertex,DataProperty> currentDataProperties;
	/**
	 * contains all defined class names of the requirements ontology
	 */
	private Set<String> ontologyClassNames;
	/**
	 * contains all defined root class names of the requirements ontology
	 */
	private Set<String> ontologyRootClassNames;
	/**
	 * contains all defined class names of the requirements ontology without the attribute class an its subclasses
	 */
	private Set<String> ontologyNoAttributeClassNames;
	/**
	 * contains all defined attribute class names of the requirements ontology and the names of its subclasses
	 */
	private Set<String> ontologyAttributeClassNames;
	/**
	 * contains all individual names of the requirements ontology
	 */
	private Set<String> ontologyIndividualNames;
	/**
	 * contains the names of all defined object properties of the requirements ontology
	 */
	private Set<String> ontologyObjectPropertyNames;
	/**
	 * contains the names of all defined data properties of the requirements ontology
	 */
	private Set<String> ontologyDataPropertyNames;
	/**
	 * contains the names of all possible main types of an individual of the requirements ontology
	 */
	private Set<String> ontologyMainTypeNames;
	/**
	 * contains the accessor to the loaded requirements ontology
	 */
	private OntologyAccessor ontologyAccessor;
	
////////// constructors
	
	/**
	 * creates a requirements ontology model with a complete goal model graph (to a given ontology accessor)
	 */
	public AbstractROModel(OntologyAccessor ontologyAccessor)
	{
		this.ontologyAccessor = ontologyAccessor;
		this.initialize();
	}
	// end constructor

////////// end constructors
	
////////// initializing the requirements ontology model
	
	/**
	 * initializes the requirements ontology model
	 * (inserts ontology details,
	 * inserts ontology objects)
	 */
	public void initialize()
	{
		this.initializeGoalModelGraph();
		this.currentDataProperties = new MultiHashMap<IndividualVertex,DataProperty>();
		this.ontologyClassNames = new TreeSet<String>();
		this.ontologyRootClassNames = new TreeSet<String>();
		this.ontologyNoAttributeClassNames = new TreeSet<String>();
		this.ontologyAttributeClassNames = new TreeSet<String>();
		this.ontologyObjectPropertyNames = new TreeSet<String>();
		this.ontologyDataPropertyNames = new TreeSet<String>();
		this.ontologyIndividualNames = new TreeSet<String>();
		this.ontologyMainTypeNames = new TreeSet<String>();

		// order important: 
		// 1. ontology class names must be inserted before individuals can be inserted!
		// 2. individuals must be inserted before object/data properties can be inserted!
		this.insertOntologyDetails();
		this.insertAll();
	}
	// end initialize
	
	/**
	 * initializes the goal model graph of the requirements ontology model
	 */
	protected abstract void initializeGoalModelGraph();

	/**
	 * initializes the goal model graph of the requirements ontology model as an ordered graph
	 */
	public void initializeStandardGoalModelGraph()
	{
		this.currentGoalModelGraph = new OrderedSparseMultigraph<IndividualVertex,ObjectPropertyEdge>();
	}
	// end initializeStandardGoalModelGraph
	
	/**
	 * initializes the goal model graph of the requirements ontology model as an directed ordered graph
	 */
	public void initializeDirectedGoalModelGraph()
	{
		this.currentGoalModelGraph = new DirectedOrderedSparseMultigraph<IndividualVertex,ObjectPropertyEdge>();
	}
	// end initializeDirectedGoalModelGraph
		
////////// end initializing the requirements ontology model	
	
////////// inserting ontology details
	
	/**
	 * inserts ontology details
	 * (inserts all simple class names of the requirements ontology,
	 * inserts all simple class names of the requirements ontology (without the attribute class and its subclasses),
	 * inserts all simple attribute class names of the requirements ontology (with its subclasses),
	 * inserts all simple object property names of the requirements ontology,
	 * inserts all simple data property names of the requirements ontology,
	 * inserts all simple individual names of the requirments ontology)
	 */
	private void insertOntologyDetails()
	{
		this.insertOntologyClassNames();
		this.insertOntologyRootClassNames();
		this.insertOntologyNoAttributeClassNames();
		this.insertOntologyAttributeClassNames();
		this.insertOntologyObjectPropertyNames();
		this.insertOntologyDataPropertyNames();
		this.insertOntologyIndividualNames();
		this.insertOntologyMainTypeNames();
	}
	// end insertOntologyDetails
	
	/**
	 * inserts all simple class names of the requirements ontology
	 */
	private void insertOntologyClassNames()
	{
		Set<String> classNameSet = new TreeSet<String>();
		Set<OWLClass> referencedClassSet = ontologyAccessor.getOwlClasses();
			
		for (OWLClass referencedClass : referencedClassSet)
		{
			classNameSet.add(referencedClass.toString());
		}
		// end for

		this.ontologyClassNames.addAll(classNameSet);
	}
	// end insertOntologyClassNames
	
	/**
	 * inserts all simple root class names of the requirements ontology (without "Thing")
	 */
	private void insertOntologyRootClassNames()
	{
		Set<String> classNameSet = new TreeSet<String>();
		Set<OWLClass> rootClassNameSet = ontologyAccessor.getOwlRootClasses();
		
		for (OWLClass rootClass : rootClassNameSet)
		{
			classNameSet.add(rootClass.toString());
		}
		// end for

		this.ontologyRootClassNames.addAll(classNameSet);
	}
	// end insertRootClassNames
	
	/**
	 * inserts all simple class names (without the attribute class and its subclasses) of the requirements ontology 
	 */
	private void insertOntologyNoAttributeClassNames()
	{
		Set<String> classNameSet = new TreeSet<String>();
		Set<OWLDescription> classDescriptionSet = new TreeSet<OWLDescription>();
		
		Set<OWLClass> rootClassSet = ontologyAccessor.getOwlRootClasses();
		
		for (OWLClass rootClass : rootClassSet)
		{
			if (rootClass.toString().equals("Attribute") == false)
			{
				classDescriptionSet.add(rootClass);
				classDescriptionSet.addAll(ontologyAccessor.getAllOwlSubclasses(rootClass.toString()));
			}
			// end if
		}
		// end for
		
		for (OWLDescription classDescription : classDescriptionSet)
		{
			classNameSet.add(classDescription.toString());
		}
		// end for

		this.ontologyNoAttributeClassNames.addAll(classNameSet);
	}
	// end insertOntologyNoAttributeClassNames
	
	/**
	 * inserts all simple attribute class names (with its subclasses) of the requirements ontology
	 * inserts also the class "Complete" that is a object-intersection of "Checked" and "Implemented"
	 */
	private void insertOntologyAttributeClassNames()
	{
		Set<String> classNameSet = new TreeSet<String>();
		Set<OWLDescription> classDescriptionSet = new TreeSet<OWLDescription>();
		
		classDescriptionSet.add(ontologyAccessor.getOwlClass("Attribute"));
		classDescriptionSet.addAll(ontologyAccessor.getAllOwlSubclasses("Attribute"));
		classDescriptionSet.add(ontologyAccessor.getOwlClass("Complete"));
				
		for (OWLDescription classDescription : classDescriptionSet)
		{
			classNameSet.add(classDescription.toString());
		}
		// end for

		this.ontologyAttributeClassNames.addAll(classNameSet);
	}
	// end insertOntologyAttributeClassNames	
	
	/**
	 * inserts all simple object property names of the requirements ontology
	 */
	private void insertOntologyObjectPropertyNames()
	{
		Set<String> objectPropertyNameSet = new TreeSet<String>();	
		Set<OWLObjectProperty> referencedObjectPropertySet = this.ontologyAccessor.getOwlObjectProperties();
		
		for (OWLObjectProperty referencedObjectProperty : referencedObjectPropertySet)
		{
			objectPropertyNameSet.add(referencedObjectProperty.toString());
		}
		// end for

		this.ontologyObjectPropertyNames.addAll(objectPropertyNameSet);
	}
	// end insertOntologyObjectPropertyNames	
	
	/**
	 * inserts all simple data property names of the requirements ontology
	 */
	private void insertOntologyDataPropertyNames()
	{
		Set<String> dataPropertyNameSet = new TreeSet<String>();	
		Set<OWLDataProperty> referencedDataPropertySet = this.ontologyAccessor.getOwlDataProperties();
		
		for (OWLDataProperty referencedDataProperty : referencedDataPropertySet)
		{
			dataPropertyNameSet.add(referencedDataProperty.toString());
		}
		// end for

		this.ontologyDataPropertyNames.addAll(dataPropertyNameSet);
	}
	// end insertOntologyDataPropertyNames
	
	/**
	 * inserts all simple individual names of the requirments ontology
	 */
	private void insertOntologyIndividualNames()
	{
		Set<String> individualNameSet = new TreeSet<String>();
		
		for (String className : this.ontologyClassNames)
		{
			Set<OWLIndividual> individualSet = this.ontologyAccessor.getOwlIndividuals(className);
			
			for (OWLIndividual individual : individualSet)
			{
				individualNameSet.add(individual.toString());
			}
			// end for
		}
		// end for
		
		this.ontologyIndividualNames.addAll(individualNameSet);
	}
	// end insertOntologyIndividualNames
	
	/**
	 * inserts all possible main type names of an individual of the requirements ontology
	 */
	private void insertOntologyMainTypeNames()
	{
		this.ontologyMainTypeNames.add("Problem");
		this.ontologyMainTypeNames.add("Challenge");
		this.ontologyMainTypeNames.add("Goal");
		this.ontologyMainTypeNames.add("Requirement");
		this.ontologyMainTypeNames.add("Story");
		this.ontologyMainTypeNames.add("SoftMetric");
		this.ontologyMainTypeNames.add("TestCase");
		this.ontologyMainTypeNames.add("Exception");
		this.ontologyMainTypeNames.add("Refinement");
		this.ontologyMainTypeNames.add("Source");
		this.ontologyMainTypeNames.add("Stakeholder");
	}
	// end insertOntologyMainTypeNames

////////// end inserting ontology details

////////// inserting objects to the requirements ontology model	

	/**
	 * inserts all objects into the requirements ontology model
	 * (inserts all individual vertices into the goal model graph,
	 * inserts all object property edges into the goal model graph,
	 * inserts all data properties into the data property map) 
	 */
	public void insertAll()
	{
		this.insertIndividuals();
		this.insertObjectProperties();
		this.insertDataProperties();		
	}
	// end insertAll
	
	/**
	 * inserts all individual vertices into the goal model graph
	 */
	public void insertIndividuals()
	{
		for (String individualName : this.ontologyIndividualNames)
		{
			this.insertIndividual(individualName);	
		}
		// end for
	}
	// end insertAllIndividuals

	/**
	 * inserts all object property edges into the goal model graph
	 * inserts an object property edge only if source individual vertex and target individual vertex exist already in the goal model graph 
	 */
	public void insertObjectProperties()
	{
		if (this.currentGoalModelGraph.getVertexCount() > 0)
		{
			Collection<IndividualVertex> individualCollection = this.currentGoalModelGraph.getVertices();
			
			for (IndividualVertex individual : individualCollection)
			{
				this.insertObjectPropertiesForIndividual(individual.getSimpleName());
			}
			// end if
		}
		// end if
	}
	// end insertAllObjectProperties

	/**
	 * inserts all data properties into the data properties map
	 * inserts a data property only if the source individual exists already in the goal model graph
	 */
	public void insertDataProperties()
	{	 
		if (this.currentGoalModelGraph.getVertexCount() > 0)
		{
			Collection<IndividualVertex> individualCollection = this.currentGoalModelGraph.getVertices();
			
			if (individualCollection.isEmpty() == false)
			{
				for (IndividualVertex individual : individualCollection)
				{
					this.insertDataPropertiesForIndividual(individual.getSimpleName());
				}
				// end for
			}
			// end if
		}
		// end if
	}
	// end insertAllDataProperties	
	
	/**
	 * inserts an individual vertex into the goal model graph (to a given simple individual name)
	 * inserts an individual vertex only if the individual vertex does not already exists in the goal model graph
	 * inserts an individual vertex only if the individual does exists in the requirements ontology
	 */
	public void insertIndividual(String simpleIndividualName)
	{
		Collection<IndividualVertex> individualCollection = this.getIndividuals();
		boolean individualAlreadyInGraph = false;
		
		if (this.ontologyIndividualNames.contains(simpleIndividualName) == false)
		{
			throw new IllegalArgumentException("individual <" + simpleIndividualName + "> does not exists in the requirements ontology");
		}
		// end if
		
		for (IndividualVertex individualInGraph : individualCollection)
		{
			if (individualInGraph.getSimpleName().equals(simpleIndividualName))
			{
				individualAlreadyInGraph = true;
			}
			// end if
		}
		// end for
		
		if (individualAlreadyInGraph == false)
		{
			String individualEssetialType = this.getIndividualEssentialType(simpleIndividualName);
			String individualMainType = this.getIndividualMainType(simpleIndividualName);
			Set<String> individualComments = this.getIndividualComments(simpleIndividualName);
			Set<String> individualAttributes = this.getIndividualAttributes(simpleIndividualName);
			ArrayList<MultiHashMap<String,String>> individualData = this.getIndividualData(simpleIndividualName);
		
			this.currentGoalModelGraph.addVertex(new IndividualVertex(simpleIndividualName,individualEssetialType,individualMainType,individualComments,individualAttributes,individualData.get(0),individualData.get(1),individualData.get(2),individualData.get(3),individualData.get(4),individualData.get(5),individualData.get(6),individualData.get(7)));
		}
		// end if		
	}
	// end insertIndividual
	
	/**
	 * inserts all object property edges into the goal model graph (to a given simple object property name)
	 * inserts an object property edge only if the object property edge between source individual and target individual does not already exists in the goal model graph
	 * inserts an object property edge only if source individual vertex and target individual vertex exist already in the goal model graph
	 */
	protected abstract void insertObjectProperty(String simpleObjectProperty);
	
	/**
	 * inserts all data properties into the data property map (to a given simple data property name)
	 * inserts a data property only if the data property of the individual does not already exists in the data property map
	 * inserts a data property only if the source individual exists already in the goal model graph
	 */
	public void insertDataProperty(String simpleDataPropertyName)
	{
		Collection<IndividualVertex> individualCollection = this.currentGoalModelGraph.getVertices();
		
		for (IndividualVertex sourceIndividual : individualCollection)
		{
			Collection<DataProperty> dataPropertyCollection = this.currentDataProperties.get(sourceIndividual);
			boolean dataPropertyAlreadyInMap = false;
			
			if (dataPropertyCollection != null)
			{
				for (DataProperty dataProperty : dataPropertyCollection)
				{
					if (dataProperty.getSimpleName().equals(simpleDataPropertyName))
					{
						dataPropertyAlreadyInMap = true;
					}
					// end if
				}
				// end for
			}
			// end if
			
			if (dataPropertyAlreadyInMap == false)
			{
				Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap = ontologyAccessor.getOwlDataProperties(sourceIndividual.getSimpleName());
				
				if (dataPropertyMap != null)
				{
					for (OWLDataPropertyExpression dataProperty : dataPropertyMap.keySet())
					{
						if (dataProperty.toString().equals(simpleDataPropertyName))
						{
							Set<OWLConstant> constantSet = dataPropertyMap.get(dataProperty);
							
							for (OWLConstant constant : constantSet)
							{
								this.currentDataProperties.put(sourceIndividual, new DataProperty(dataProperty.toString(),constant));
							}
							// end for
						}
						// end if
					}
					// end for		
				}
				// end if
			}
			// end if
		}
		// end for
	}
	// end insertDataProperty	

	/**
	 * inserts an individual vertex into the goal model graph (to a given simple individual name)
	 * inserts only outgoing object properties of the given individual
	 * inserts an object property edge only if the target individual vertex already exists in the goal model graph
	 * inserts all data properties of the individual into the data property map
	 */
	protected abstract void insertIndividualWithProperties(String simpleIndividualName);

	/**
	 * inserts all individual vertices that belong to a specific class or one of its subclasses into the goal model graph (to a given simple class name)
	 */
	public void insertIndividuals(String simpleClassName)
	{
		Set<OWLIndividual> individualSet = this.ontologyAccessor.getAllOwlIndividuals(simpleClassName);
				
		for (OWLIndividual individual : individualSet)
		{
			this.insertIndividual(individual.toString());
		}
		// end for
	}
	// end insertAllIndividuals

	/**
	 * inserts all individual vertices that belong to a specific class or one of its subclasses into the goal model graph (to a given simple class name)
	 * inserts only outgoing object properties of the inserted individuals
	 * inserts an object property edge only if the target individual already exists in the goal model graph
	 * inserts all data properties of the individual into the data property map
	 */
	public void insertIndividualsWithProperties(String simpleClassName)
	{
		Set<OWLIndividual> individualSet = this.ontologyAccessor.getAllOwlIndividuals(simpleClassName);
		
		for (OWLIndividual individual : individualSet)
		{
			this.insertIndividual(individual.toString());
		}
		// end for
		
		for (OWLIndividual individual : individualSet)
		{
			this.insertIndividualWithProperties(individual.toString());
		}
		// end for		
	}
	// end insertAllIndividualsWithProperties
	
	/**
	 * inserts all object property edges into the goal model graph (to a given simple object property name)
	 * inserts also the source/target individual vertices if they does not exist into the goal model graph
	 */
	public void insertObjectPropertyWithIndividuals(String simpleObjectPropertyName)
	{
		Set<OWLIndividual> sourceIndividualSet = new TreeSet<OWLIndividual>();
				
		for (String individualName : this.ontologyIndividualNames)
		{
			sourceIndividualSet.add(this.ontologyAccessor.getOwlIndividual(individualName));
		}
		// end for
		
		Set<OWLIndividual> wantedIndividualSet = new TreeSet<OWLIndividual>();
		
		for (OWLIndividual sourceIndividual : sourceIndividualSet)
		{
			Map<OWLObjectPropertyExpression,Set<OWLIndividual>> objectPropertyMap = ontologyAccessor.getOwlObjectProperties(sourceIndividual.toString());
			
			if (objectPropertyMap != null)
			{
				for (OWLObjectPropertyExpression objectProperty : objectPropertyMap.keySet())
				{
					if (objectProperty.toString().equals(simpleObjectPropertyName))
					{
						Set<OWLIndividual> targetIndividualSet = objectPropertyMap.get(objectProperty);
						wantedIndividualSet.add(sourceIndividual);
						wantedIndividualSet.addAll(targetIndividualSet);
					}
					// end if
				}
				// end for
			}
			// end if
		}
		// end for
		
		for (OWLIndividual wantedIndividual : wantedIndividualSet)
		{
			this.insertIndividual(wantedIndividual.toString());
		}
		// end for
		
		this.insertObjectProperty(simpleObjectPropertyName);
	}
	// end insertObjectPropertyWithIndividuals
	
	/**
	 * inserts all data properties into the data property map (to a given simple data property name)
	 * inserts also the source individual vertices if they does not exist into the goal model graph
	 */
	public void insertDataPropertyWithIndividuals(String simpleDataPropertyName)
	{
		Set<OWLIndividual> sourceIndividualSet = new TreeSet<OWLIndividual>();
		
		for (String individualName : this.ontologyIndividualNames)
		{
			sourceIndividualSet.add(this.ontologyAccessor.getOwlIndividual(individualName));
		}
		// end for
		
		Set<OWLIndividual> wantedIndividualSet = new TreeSet<OWLIndividual>();
		
		for (OWLIndividual sourceIndividual : sourceIndividualSet)
		{
			Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap = this.ontologyAccessor.getOwlDataProperties(sourceIndividual.toString());
			
			if (dataPropertyMap != null)
			{
				for (OWLDataPropertyExpression dataProperty : dataPropertyMap.keySet())
				{
					if (dataProperty.toString().equals(simpleDataPropertyName))
					{
						wantedIndividualSet.add(sourceIndividual);
					}
					// end if
				}
				// end for
			}
			// end if
		}
		// end for
		
		for (OWLIndividual wantedIndividual : wantedIndividualSet)
		{
			this.insertIndividual(wantedIndividual.toString());
		}
		// end for
		
		this.insertDataProperty(simpleDataPropertyName);
	}
	// end insertDataPropertyWithIndividuals
	
	/**
	 * inserts all object property edges into the goal model graph (to a given simple individual name)
	 * inserts an object property edge only if source/target individual vertices exist already in the goal model graph
	 */
	protected abstract void insertObjectPropertiesForIndividual(String sourceIndividualName);

	/**
	 * inserts all data properties into the data property map (to a given simple individual name)
	 * inserts a data property only if the source individual exists already in the goal model graph
	 */
	public void insertDataPropertiesForIndividual(String simpleIndividualName)
	{
		Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap = ontologyAccessor.getOwlDataProperties(simpleIndividualName);
		
		if (dataPropertyMap != null)
		{
			for (OWLDataPropertyExpression dataProperty : dataPropertyMap.keySet())
			{
				Set<OWLConstant> constantSet = dataPropertyMap.get(dataProperty);
				
				for (OWLConstant constant : constantSet)
				{
					if (this.getIndividual(simpleIndividualName) != null)
					{
						Collection<DataProperty> dataPropertyCollection = this.currentDataProperties.get(this.getIndividual(simpleIndividualName));
						boolean dataPropertyAlreadyInMap = false;
						
						if (dataPropertyCollection != null)
						{
							for (DataProperty currentDataProperty : dataPropertyCollection)
							{
								if ((currentDataProperty.getSimpleName().equals(dataProperty.toString())) &&
									(currentDataProperty.getDataValue().equals(constant.getLiteral())))
								{
									dataPropertyAlreadyInMap = true;
								}
								// end if
							}
							// end for
						}
						// end if
						
						if (dataPropertyAlreadyInMap == false)
						{
							this.currentDataProperties.put(this.getIndividual(simpleIndividualName), new DataProperty(dataProperty.toString(),constant));
						}
						// end if
					}
					// end if
				}
				// end for
			}
			// end for	
		}
		// end if
	}
	// end insertDataProperties	
	
////////// end inserting objects to the requirements ontology model		

////////// removing objects from the requirements ontology model
	
	/**
	 * removes an existing individual vertex from the goal model graph (to a given simple individual name)
	 * removes all existing object properties of the individual from the goal model graph
	 * removes all existing data properties of the individual from the data property map
	 */
	public void removeIndividual(String simpleIndividualName)
	{
		Collection<IndividualVertex> individualCollection = this.currentGoalModelGraph.getVertices();
		Collection<IndividualVertex> wantedIndividualSet = new HashSet<IndividualVertex>();
		
		for (IndividualVertex individual : individualCollection)
		{
			if (individual.getSimpleName().equals(simpleIndividualName))
			{
				wantedIndividualSet.add(individual);
			}
			// end if
		}
		// end for
		
		for (IndividualVertex individual : wantedIndividualSet)
		{
			currentDataProperties.remove(individual);
			this.currentGoalModelGraph.removeVertex(individual);
		}
		// end for
	}
	// end removeIndividual
	
	/**
	 * removes all existing object properties from the goal model graph (to a given simple object property name)
	 */
	public void removeObjectProperty(String simpleObjectPropertyName)
	{
		Collection<ObjectPropertyEdge> objectPropertyCollection = this.currentGoalModelGraph.getEdges();
		Collection<ObjectPropertyEdge> wantedObjectPropertySet = new HashSet<ObjectPropertyEdge>();
		
		for (ObjectPropertyEdge objectProperty : objectPropertyCollection)
		{
			if (objectProperty.getSimpleName().equals(simpleObjectPropertyName))
			{
				wantedObjectPropertySet.add(objectProperty);	
			}
			// end if
		}
		// end for
		
		for (ObjectPropertyEdge objectProperty : wantedObjectPropertySet)
		{
			this.currentGoalModelGraph.removeEdge(objectProperty);
		}
		// end for	
	}
	// end removeObjectProperty
	
	/**
	 * removes all existing object properties from the goal model graph
	 */
	public void removeObjectProperties()
	{
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection = this.currentGoalModelGraph.getEdges();
		Collection<ObjectPropertyEdge> wantedObjectPropertySet = new HashSet<ObjectPropertyEdge>();
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection)
		{
			wantedObjectPropertySet.add(objectPropertyEdge);
		}
		// end for
		
		for (ObjectPropertyEdge objectProperty : wantedObjectPropertySet)
		{
			this.currentGoalModelGraph.removeEdge(objectProperty);
		}
		// end for		
	}
	// end removeObjectProperties
	
	/**
	 * removes all existing data properties form the data property map (to a given simple data property name)
	 */
	public void removeDataProperty(String simpleDataPropertyName)
	{
		Collection<IndividualVertex> individualCollection = this.currentGoalModelGraph.getVertices();
		
		for (IndividualVertex individual : individualCollection)
		{
			Collection<DataProperty> dataPropertyCollection = this.currentDataProperties.get(individual);
			Collection<DataProperty> wantedDataPropertySet = new HashSet<DataProperty>();
			
			if (dataPropertyCollection != null)
			{
				for (DataProperty dataProperty : dataPropertyCollection)
				{
					if (dataProperty.getSimpleName().equals(simpleDataPropertyName))
					{
						wantedDataPropertySet.add(dataProperty);
					}
					// end if
				}
				// end for
				
				dataPropertyCollection.removeAll(wantedDataPropertySet);
				
				if ((dataPropertyCollection.size() > 0) == false)
				{
					this.currentDataProperties.remove(individual);
				}
				// end if
			}
			// end if
		}
		// end for
	}
	// end removeDataProperty
	
	/**
	 * removes all existing data properties from the data property map
	 */
	public void removeDataProperties()
	{
		this.currentDataProperties.clear();
	}
	// end removeDataProperties
	
	/**
	 * removes all existing individual vertices from the goal model graph
	 * (removes all individuals and object properties from the goal model graph,
	 * removes all data properties from the data property map)
	 */
	public void removeIndividuals()
	{
		this.removeAll();
	}
	// end removeIndividuals
	
	/**
	 * removes all existing individual vertices that belong to a specific class or one of its subclasses from the goal model graph (to a given simple class name)
	 * removes all existing object properties of the individual from the goal model graph
	 * removes all existing data properties of the individual from the data property map
	 */
	public void removeIndividuals(String simpleClassName)
	{
		Collection<IndividualVertex> individualCollection = this.getIndividuals(simpleClassName);
		Set<String> individualNameSet = new TreeSet<String>();
		
		for (IndividualVertex individual : individualCollection)
		{
			individualNameSet.add(individual.getSimpleName());
		}
		// end for
		
		for (String individualName : individualNameSet)
		{
			this.removeIndividual(individualName);
		}
		// end for
	}
	// end removeIndividuals
	
	/**
	 * removes unconnected individual vertices from the goal model graph
	 * removes an individual vertex if it is neither a source indivdual nor a target individual of any object properties from the goal model graph
	 * removes all existing data properties of the unconnected individuals from the data property map
	 */
	public void removeUnconnectedIndividuals()
	{
		Set<String> sourceIndividuals = new TreeSet<String>();
		Set<String> targetIndividuals = new TreeSet<String>();
		Set<String> allIndividualNames = new TreeSet<String>();
		
		for (IndividualVertex sourceIndividual : this.getIndividuals())
		{
			allIndividualNames.add(sourceIndividual.getSimpleName());
			
			for (IndividualVertex targetIndividual : this.getIndividuals())
			{
				if (this.getObjectProperties(sourceIndividual.getSimpleName(),targetIndividual.getSimpleName()).isEmpty() == false)
				{
					sourceIndividuals.add(sourceIndividual.getSimpleName());
					targetIndividuals.add(targetIndividual.getSimpleName());
				}
				// end for
			}
			// end for
		}
		// end for
		
		for (String individualName : allIndividualNames)
		{
			if ((sourceIndividuals.contains(individualName) == false) && (targetIndividuals.contains(individualName) == false))
			{
				this.removeIndividual(individualName);
			}
			// end if
		}
		// end for
	}
	// end removeUnconnectedIndividuals
	
	/**
	 * removes all objects from the requirements ontology model
	 * (removes all individuals and object properties from the goal model graph,
	 * removes all data properties from the data property map)
	 */
	public void removeAll()
	{
		this.initializeGoalModelGraph();
		this.currentDataProperties = new MultiHashMap<IndividualVertex,DataProperty>();
	}
	// end removeAll
	
////////// end removing objects from the requirements ontology model
	
	
////////// getting current objects from the requirements ontology model	

	/**
	* returns the current goal model graph from the requirements ontology model
	*/
	public Graph<IndividualVertex,ObjectPropertyEdge> getGoalModelGraph()
	{
		return this.currentGoalModelGraph;
	}
	// end getGoalModelGraph
	
	/**
	* returns all current individuals from the goal model graph
	*/
	public Collection<IndividualVertex> getIndividuals()
	{
		return this.currentGoalModelGraph.getVertices();
	}
	// end getAllIndividuals
	
	/**
	* returns the all current data properties from the requirements ontology model
	*/
	public MultiHashMap<IndividualVertex,DataProperty> getDataProperties()
	{
		return this.currentDataProperties;
	}
	// end getAllDataProperties
	
	/**
	* returns all current individual vertices that belong to a specific class or one of its subclasses (to a given simple class name)
	*/
	public Collection<IndividualVertex> getIndividuals(String simpleClassName)
	{
		Set<String> classNameSet = new TreeSet<String>();
		Set<OWLDescription> wantedClassSet = this.ontologyAccessor.getAllOwlSubclasses(simpleClassName);
		wantedClassSet.add(this.ontologyAccessor.getOwlClass(simpleClassName));
		
		for (OWLDescription wantedClass : wantedClassSet)
		{
			classNameSet.add(wantedClass.toString());
		}
		// end for
		
		Collection<IndividualVertex> individualCollection = this.getIndividuals();
		Collection<IndividualVertex> wantedIndividualSet = new HashSet<IndividualVertex>();
		
		for (IndividualVertex individual : individualCollection)
		{
			if (classNameSet.contains(individual.getEssentialType()))
			{
				wantedIndividualSet.add(individual);
			}
			// end if
		}
		// end for
		
		return wantedIndividualSet;
	}
	// end getIndividuals	

	/**
	 * returns an individual vertex from the goal model graph (to a given simple individual name)
	 */
	public IndividualVertex getIndividual(String simpleIndividualName)
	{
		Collection<IndividualVertex> individualCollection = this.currentGoalModelGraph.getVertices();
		IndividualVertex wantedIndividual = null;
		
		for (IndividualVertex individual : individualCollection)
		{
			if (individual.getSimpleName().equals(simpleIndividualName))
			{
				wantedIndividual = individual;
			}
			// end if
		}
		// end for
		
		return wantedIndividual;
	}
	// end getIndividual
	
	/**
	 * returns true if the individual exists in the goal model graph (to a given simple individual name)
	 */
	public boolean containsIndividual(String simpleIndividualName)
	{
		Collection<IndividualVertex> individualCollection = this.currentGoalModelGraph.getVertices();
		boolean individualAlreadyInGraph = false;
		
		for (IndividualVertex individual : individualCollection)
		{
			if (individual.getSimpleName().equals(simpleIndividualName))
			{
				individualAlreadyInGraph = true;
			}
			// end if
		}
		// end for
	
		return individualAlreadyInGraph;
	}
	// end containsIndividual	
	
	/**
	 * prints the main information of the requirements ontology model
	 */
	public void printInfo()
	{
		System.out.println("----------");
		System.out.println("----------");
		System.out.println("RO model:");
		System.out.println("----------");
		System.out.println("individual count: " + this.currentGoalModelGraph.getVertexCount());
		System.out.println("object property count: " + this.currentGoalModelGraph.getEdgeCount());
		System.out.println("data property count: " + this.currentDataProperties.totalSize());
		System.out.println("----------");
		System.out.println("individuals and object properties:\n" + this.currentGoalModelGraph.toString());
		System.out.println("data properties:\n" + this.currentDataProperties.toString());
		System.out.println("----------");
		System.out.println("----------");
	}
	// end printInfo

	/**
	 * returns all current outgoing object property edges of an individual vertex from the goal model graph (to a given simple individual name)
	 */
	public Collection<ObjectPropertyEdge> getObjectProperties(String simpleIndividualName)
	{
		return this.currentGoalModelGraph.getOutEdges(this.getIndividual(simpleIndividualName));
	}
	// end getAllObjectProperties
	
	/**
	 * returns all current data properties of an individual vertex from the data property map (to a given simple individual name)
	 */
	public Collection<DataProperty> getDataProperties(String simpleIndividualName)
	{
		if (this.getIndividual(simpleIndividualName) == null)
		{
			return null;
		}
		else
		{
			if (this.currentDataProperties.get(this.getIndividual(simpleIndividualName)) == null)
			{
				return new ArrayList<DataProperty>();
			}
			else
			{
				return this.currentDataProperties.get(this.getIndividual(simpleIndividualName));				
			}
			// end if else
		}
		// end if else
	}
	// end getAllObjectProperties	
	
	/**
	 * returns all current object property edges between a source individual vertex and a target individual vertex from the goal model graph (to a given simple source individual name and a simple target individual name)
	 */
	public Collection<ObjectPropertyEdge> getObjectProperties(String simpleSourceIndividualName, String simpleTargetIndividualName)
	{
		if (this.ontologyIndividualNames.contains(simpleSourceIndividualName) == false)
		{
			throw new IllegalArgumentException("individual <" + simpleSourceIndividualName + "> does not exists in the requirements ontology");
		}
		// end if
		
		if (this.ontologyIndividualNames.contains(simpleTargetIndividualName) == false)
		{
			throw new IllegalArgumentException("individual <" + simpleTargetIndividualName + "> does not exists in the requirements ontology");
		}
		// end if
		
		return this.currentGoalModelGraph.findEdgeSet(this.getIndividual(simpleSourceIndividualName),this.getIndividual(simpleTargetIndividualName));
	}
	// end getAllObjectProperties
	
////////// end getting current objects from the requirements ontology model

////////// getting ontology details
	
	/**
	 * returns the accessor to the loaded requirements ontology
	 */
	public OntologyAccessor getOntologyAccessor()
	{
		return this.ontologyAccessor;
	}
	// end getOntologyAccessor

	/**
	 * returns all simple class names of the requirement ontology
	 */
	public Set<String> getOntologyClassNames()
	{
		return this.ontologyClassNames;
	}
	// end getOntologyClassNames

	/**
	 * returns all simple object property names of the requirement ontology
	 */
	public Set<String> getOntologyObjectPropertyNames()
	{
		return this.ontologyObjectPropertyNames;
	}
	// end getOntologyObjectPropertyNames
	
	/**
	 * returns all simple data property names of the requirement ontology
	 */
	public Set<String> getOntologyDataPropertyNames()
	{
		return this.ontologyDataPropertyNames;
	}
	// end getOntologyDataPropertyNames
	
	/**	
	 * returns all simple individual names of the requirements ontology
	 */
	public Set<String> getOntologyIndividualNames()
	{
		return this.ontologyIndividualNames;
	}
	// end getOntologyIndividualNames
	
	/**
	 * returns all simple root class names of the requirements ontology (without "Thing")
	 */
	public Set<String> getOntologyRootClassNames()
	{
		return this.ontologyRootClassNames;
	}
	// end getOntologyRootClassNames
	
	/**
	 * returns all simple attribute class names (with its subclasses) of the requirements ontology
	 */
	public Set<String> getOntologyAttributeClassNames()
	{
		return this.ontologyAttributeClassNames;
	}
	// end getOntologyAttributeClassNames
	
	/**
	 * returns all simple class names (without the attribute class and its subclasses) of the requirements ontology
	 */
	public Set<String> getOntologyNoAttributeClassNames()
	{
		return this.ontologyNoAttributeClassNames;
	}
	// end getOntologyNoAttributeClassNames
	
	/**
	 * returns all simple main type names of the requirements ontology
	 */
	public Set<String> getOntologyMainTypeNames()
	{
		return this.ontologyMainTypeNames;
	}
	// end getOntologyMainTypeNames
	
	/**
	 * returns all comments of an individual (to a given simple individual name)
	 */
	public Set<String> getIndividualComments(String simpleIndividualName)
	{
		Set<OWLCommentAnnotation> commentAnnotationSet = this.ontologyAccessor.getOwlCommentAnnotations(simpleIndividualName);
		Set<String> comments = new TreeSet<String>();
		
		for (OWLCommentAnnotation commentAnnotation : commentAnnotationSet)
		{
			comments.add(commentAnnotation.getAnnotationValue().toString());
		}
		// end for
		
		return comments;
	}
	// end getIndividualComments
	
	/**
	 * returns the essential type of an individual (to a given simple individual name)
	 * essential type could be all classes but one of the attribute classes
	 * if the individual belongs just to an attribute class the essential type would be empty
	 */
	public String getIndividualEssentialType(String simpleIndividualName)
	{
		Set<OWLDescription> typeSet = this.ontologyAccessor.getOwlTypes(simpleIndividualName);
		String essentialType = new String();
		
		for (OWLDescription type : typeSet)
		{
			if ((this.ontologyAttributeClassNames.contains(type.toString()) == false) &&
				(type.isOWLThing() == false))
			{
				essentialType = type.toString();
			}
			// end if
		}
		// end for
		
		return essentialType;
	}
	// end getIndividualEssentialType
	
	/**
	 * returns the main type of an individual (to a given simple individual name)
	 * main type could be one of the classes "Problem", "Challenge", "Goal", "Requirement", "Story", "SoftMetric", "TestCase", "Exception", "Refinement", "Source" or "Stakeholder"
	 * if the individual belongs just to an attribute class the main type would be empty
	 */
	public String getIndividualMainType(String simpleIndividualName)
	{
		String mainType = new String();
		
		if (this.belongsToClassOrSubclass(simpleIndividualName,"Problem") == true)
		{
			mainType = "Problem";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Challenge") == true)
		{
			mainType = "Challenge";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Goal") == true)
		{
			mainType = "Goal";
		}		
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Requirement") == true)
		{
			mainType = "Requirement";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Story") == true)
		{
			mainType = "Story";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"SoftMetric") == true)
		{
			mainType = "SoftMetric";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"TestCase") == true)
		{
			mainType = "TestCase";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Exception") == true)
		{
			mainType = "Exception";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Refinement") == true)
		{
			mainType = "Refinement";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Source") == true)
		{
			mainType = "Source";
		}
		else if (this.belongsToClassOrSubclass(simpleIndividualName,"Stakeholder") == true)
		{
			mainType = "Stakeholder";
		}
		// end if else if
		
		return mainType;
	}
	// end getIndividualMainType
	
	/**
	 * returns true if the individual belongs to a specific class or one of its subclasses (to a given simple individual name and a given simple class name)
	 * return value depends on the essential type of the individual
	 * if the individual belongs just to an attribute class the return value would be false 
	 */
	public boolean belongsToClassOrSubclass(String simpleIndividualName, String simpleClassName)
	{
		boolean belongsToClassOrSubclass = false;
		Set<String> classNameSet = new TreeSet<String>();
		Set<OWLDescription> wantedClassSet = this.ontologyAccessor.getAllOwlSubclasses(simpleClassName);
		wantedClassSet.add(this.ontologyAccessor.getOwlClass(simpleClassName));
		String essentialType = this.getIndividualEssentialType(simpleIndividualName);
		
		for (OWLDescription wantedClass : wantedClassSet)
		{
			classNameSet.add(wantedClass.toString());
		}
		// end for
		
		for (String className : classNameSet)
		{
			if (className.equals(essentialType))
			{
				belongsToClassOrSubclass = true;
			}
			// end if
		}
		// end for
		
		return belongsToClassOrSubclass;
	}
	// end belongsToClassOrSubclass
	
	/**
	 * returns the attributes of an individual (to a given simple individual name)
	 */
	public Set<String> getIndividualAttributes(String simpleIndividualName)
	{
		Set<OWLDescription> typeSet = this.ontologyAccessor.getOwlTypes(simpleIndividualName);
		Set<String> attributeNameSet = new TreeSet<String>();
		
		for (OWLDescription type : typeSet)
		{
			if (this.ontologyAttributeClassNames.contains(type.toString()))
			{
				attributeNameSet.add(type.toString());
			}
			// end if
		}
		// end for
		
		return attributeNameSet;
	}
	// end getIndividualAttributes
	
	/**
	 * returns the formal defined attributes of a class (to a given simple class name)
	 * formal attribute == object/data-some-restriction of a class
	 */
	public MultiHashMap<String,String> getFormalAttributes(String simpleClassName)
	{
		Set<OWLDescription> someRestrictionMap = this.ontologyAccessor.getOwlSomeRestrictions(simpleClassName);
		MultiHashMap<String,String> formalAttributes = new MultiHashMap<String,String>();
		
		for (OWLDescription someRestriction : someRestrictionMap)
		{
			if (someRestriction instanceof uk.ac.manchester.cs.owl.OWLObjectSomeRestrictionImpl)
			{
				formalAttributes.put(((OWLObjectSomeRestriction) someRestriction).getProperty().toString(),((OWLObjectSomeRestriction) someRestriction).getFiller().toString());
			}
			else if (someRestriction instanceof uk.ac.manchester.cs.owl.OWLDataSomeRestrictionImpl)
			{
				formalAttributes.put(((OWLDataSomeRestriction) someRestriction).getProperty().toString(),((OWLDataSomeRestriction) someRestriction).getFiller().toString());
			}
			// end if else
		}
		// end for
		
		return formalAttributes;
	}
	// end getFormalAttributes
	
	/**
	 * returns the data of an individual (to a given simple individual name)
	 * (returns all formal defined attributes of the essential type class of the individual,
	 * returns all formal defined attributes of the individual (and the target individual/value if available),
	 * returns all object properties of the individual that do not belong to a formal attribute (object-some-restriction) of the individual, 
	 * returns all object properties (and target individual) of the individual that do not belong to a formal attribute (object-some-restriction) of the individual,
	 * returns all data properties of the individual that do not belong to a formal attribute (data-some-restriction) of the individual,
	 * returns all data properties (and target data value) of the individual that do not belong to a formal attribute (data-some-restriction) of the individual)
	 */
	public ArrayList<MultiHashMap<String,String>> getIndividualData(String simpleIndividualName)
	{
		Map<OWLObjectPropertyExpression,Set<OWLIndividual>> objectPropertyMap = this.ontologyAccessor.getOwlObjectProperties(simpleIndividualName);
		Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap = this.ontologyAccessor.getOwlDataProperties(simpleIndividualName);
		MultiHashMap<String,String> formalAttributes = this.getFormalAttributes(this.getIndividualEssentialType(simpleIndividualName));
		MultiHashMap<String,String> formalAttributesWithContent = new MultiHashMap<String,String>();
		MultiHashMap<String,String> additionalObjectProperties = new MultiHashMap<String,String>();
		MultiHashMap<String,String> additionalObjectPropertiesWithContent = new MultiHashMap<String,String>();
		MultiHashMap<String,String> additionalDataProperties = new MultiHashMap<String,String>();
		MultiHashMap<String,String> additionalDataPropertiesWithContent = new MultiHashMap<String,String>();
		MultiHashMap<String,String> incomingObectProperties = new MultiHashMap<String,String>();
		MultiHashMap<String,String> incomingObjectPropertiesWithContent = new MultiHashMap<String,String>();
		ArrayList<MultiHashMap<String,String>> individualData = new ArrayList<MultiHashMap<String,String>>(8);
		
		for (String formalAttribute : formalAttributes.keySet())
		{
			Collection<String> formalAttributeValueTypes = formalAttributes.get(formalAttribute);

			for (String formalAttributeValueType : formalAttributeValueTypes)
			{
				if (this.ontologyObjectPropertyNames.contains(formalAttribute))
				{
					boolean formalAttributeAsserted = false;
					
					if (objectPropertyMap != null)
					{
						for (OWLObjectPropertyExpression objectProperty : objectPropertyMap.keySet())
						{
							if (objectProperty.toString().equals(formalAttribute))
							{
								for (OWLIndividual individual : objectPropertyMap.get(objectProperty))
								{
									if (this.belongsToClassOrSubclass(individual.toString(),formalAttributeValueType) == true)
									{
										formalAttributeAsserted = true;
										formalAttributesWithContent.put(formalAttribute + "-" + formalAttributeValueType, individual.toString());
										
//											if (this.getIndividualEssentialType(individual.toString()).equals(formalAttributeValueType))
//											{
//												formalAttributesWithContent.put(formalAttribute + "-" + formalAttributeValueType, individual.toString());
//											}
//											else
//											{
//												formalAttributesWithContent.put(formalAttribute + "-" + formalAttributeValueType, individual.toString() + " (" + this.getIndividualEssentialType(individual.toString()) + ")");
//											}
//											// end if else
									}
									// end if
								}
								// end for								
							}
							// end if
						}
						// end for
					}
					// end if
					if (formalAttributeAsserted == false)
					{
						formalAttributesWithContent.put(formalAttribute + "-" + formalAttributeValueType, "");
					}
					// end if
				}
				else if (this.ontologyDataPropertyNames.contains(formalAttribute))
				{
					boolean formalAttributeAsserted = false;
					
					if (dataPropertyMap != null)
					{
						for (OWLDataPropertyExpression dataProperty : dataPropertyMap.keySet())
						{
							if (dataProperty.toString().equals(formalAttribute))
							{
								for (OWLConstant constant : dataPropertyMap.get(dataProperty))
								{
									if (constant.isTyped())
									{
										if (formalAttributeValueType.equals(((OWLTypedConstant) constant).getDataType().toString()))
										{
											formalAttributesWithContent.put(formalAttribute + "-" + formalAttributeValueType, constant.toString());
										}
										// end if
									}
									// end if
								}
								// end for
							}
							// end for
						}
						// end for
					}
					// end if
					
					if (formalAttributeAsserted == false)
					{
						formalAttributesWithContent.put(formalAttribute + "-" + formalAttributeValueType, "");
					}
					// end if
				}
				// end if else if
			}
			// end for
		}
		// end for
		
		if (objectPropertyMap != null)
		{
			for (OWLObjectPropertyExpression objectProperty : objectPropertyMap.keySet())
			{
				if (formalAttributes.containsKey(objectProperty.toString()))
				{
					Set<String> keySet = new TreeSet<String>();
					Set<String> individualNameSet = new TreeSet<String>();
					
					for (String formalAttributeValue : formalAttributes.get(objectProperty.toString()))
					{
						keySet.add(objectProperty.toString() + "-" + formalAttributeValue);
					}
					// end for
					
					for (String key : keySet)
					{
						individualNameSet.addAll(formalAttributesWithContent.get(key));
					}
					// end for
				
					for (OWLIndividual individual : objectPropertyMap.get(objectProperty))
					{
						if (individualNameSet.contains(individual.toString()) == false)
						{
							if (this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()) != null)
							{
								if (additionalObjectProperties.get(objectProperty.toString()).contains(this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()).toString()) == false)
								{
									additionalObjectProperties.put(objectProperty.toString(), this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()).toString());
								}
								// end if
							}
							else
							{
								if (additionalObjectProperties.get(objectProperty.toString()).contains("-") == false)
								{
									additionalObjectProperties.put(objectProperty.toString(), "-");
								}
								// end if
							}
							// end if else
							
							if (this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()) != null)
							{
								additionalObjectPropertiesWithContent.put(objectProperty.toString() + "-" + this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()).toString(), individual.toString());
							}
							else
							{
								additionalObjectPropertiesWithContent.put(objectProperty.toString() + "-" + "-", individual.toString());
							}
							// end if else
						}
						// end if
					}
					// end for
				}
				else
				{
					if (this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()) != null)
					{
						additionalObjectProperties.put(objectProperty.toString(), this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()).toString());
					}
					else
					{
						additionalObjectProperties.put(objectProperty.toString(), "-");
					}
					// end if else
					
					for (OWLIndividual individual : objectPropertyMap.get(objectProperty))
					{
						if (this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()) != null)
						{
							additionalObjectPropertiesWithContent.put(objectProperty.toString() + "-" + this.ontologyAccessor.getOwlObjectPropertyRange(objectProperty.toString()), individual.toString());
						}
						else
						{
							additionalObjectPropertiesWithContent.put(objectProperty.toString() + "-" + "-", individual.toString());
						}
						// end if else
					}
					// end for					
				}
				// end if else
			}
			// end for
		}
		// end if
		
		if (dataPropertyMap != null)
		{
			for (OWLDataPropertyExpression dataProperty : dataPropertyMap.keySet())
			{
				if (formalAttributes.containsKey(dataProperty.toString()))
				{
					Set<String> keySet = new TreeSet<String>();
					Set<String> individualNameSet = new TreeSet<String>();					
				
					for (String formalAttributeValue : formalAttributes.get(dataProperty.toString()))
					{
						keySet.add(dataProperty.toString() + "-" + formalAttributeValue);
					}
					// end for
					
					for (String key : keySet)
					{
						individualNameSet.addAll(formalAttributesWithContent.get(key));
					}
					// end for					
										
					for (OWLConstant constant : dataPropertyMap.get(dataProperty))
					{
						if (individualNameSet.contains(constant.toString()) == false)
						{

							if (this.ontologyAccessor.getOwlDataPropertyRange(dataProperty.toString()) != null)
							{
								if (additionalDataProperties.get(dataProperty.toString()).contains(this.ontologyAccessor.getOwlDataPropertyRange(dataProperty.toString()).toString()) == false)
								{
									additionalDataProperties.put(dataProperty.toString(), this.ontologyAccessor.getOwlDataPropertyRange(dataProperty.toString()).toString());
								}
								// end if
							}
							else
							{
								if (additionalDataProperties.get(dataProperty.toString()).contains("-") == false)
								{
									additionalDataProperties.put(dataProperty.toString(), "-");
								}
								// end if
							}
							// end if else
							
							additionalDataPropertiesWithContent.put(dataProperty.toString() + "-" + this.ontologyAccessor.getOwlDataPropertyRange(dataProperty.toString()).toString(), constant.getLiteral());						
						}
						// end if
					}
					// end for
				}
				else
				{
					additionalDataProperties.put(dataProperty.toString(), this.ontologyAccessor.getOwlDataPropertyRange(dataProperty.toString()).toString());
					
					for (OWLConstant constant : dataPropertyMap.get(dataProperty))
					{
						additionalDataPropertiesWithContent.put(dataProperty.toString() + "-" + this.ontologyAccessor.getOwlDataPropertyRange(dataProperty.toString()).toString(), constant.getLiteral());
					}
					// end for	
				}
				// end if else
			}
			// end for
		}
		// end if
		
		for (String sourceIndividual : this.ontologyIndividualNames)
		{
			objectPropertyMap = this.ontologyAccessor.getOwlObjectProperties(sourceIndividual);
			
			if (objectPropertyMap != null)
			{
				for (OWLObjectPropertyExpression objectProperty : objectPropertyMap.keySet())
				{
					for (OWLIndividual targetIndividual : objectPropertyMap.get(objectProperty))
					{
						if (targetIndividual.toString().equals(simpleIndividualName))
						{
							if (incomingObectProperties.get(objectProperty.toString()) != null)
							{
								if (incomingObectProperties.get(objectProperty.toString()).contains(this.getIndividualEssentialType(sourceIndividual)) == false)
								{
									incomingObectProperties.put(objectProperty.toString(),this.getIndividualEssentialType(sourceIndividual));
									incomingObjectPropertiesWithContent.put(objectProperty.toString() + "-" + this.getIndividualEssentialType(sourceIndividual), sourceIndividual);
								}
								else
								{
									incomingObjectPropertiesWithContent.put(objectProperty.toString() + "-" + this.getIndividualEssentialType(sourceIndividual), sourceIndividual);
								}
								// end if else
							}
							else
							{
								incomingObectProperties.put(objectProperty.toString(),this.getIndividualEssentialType(sourceIndividual));
								incomingObjectPropertiesWithContent.put(objectProperty.toString() + "-" + this.getIndividualEssentialType(sourceIndividual), sourceIndividual);
							}
							// end if else
						}
						// end if
					}
					// end for
				}
				// end for
			}
			// end if
		}
		// end for

		individualData.add(0,formalAttributes);
		individualData.add(1,formalAttributesWithContent);
		individualData.add(2,additionalObjectProperties);
		individualData.add(3,additionalObjectPropertiesWithContent);
		individualData.add(4,additionalDataProperties);
		individualData.add(5,additionalDataPropertiesWithContent);
		individualData.add(6,incomingObectProperties);
		individualData.add(7,incomingObjectPropertiesWithContent);
		
		return individualData;
	}
	// end getIndividualData
	
	/**
	 * creates a refinement forest (set of refinement trees) to use tree layout visualization 
	 * (removes all objects from the requirements ontology model,
	 * inserts all individual vertices,
	 * removes all individual vertices that belongs to "Stakeholder" (essential type),
	 * inserts all object property edges "refinesTo", "hasRefinementSource" and "hasRefinementTarget",
	 * removes all unconnected individual vertices (individuals that are neither a source nor a target of an object property) 
	 */
	public void createRefinementForest()
	{
		this.removeAll();
		this.insertIndividuals();
//		this.removeIndividuals("Stakeholder");
		this.insertObjectProperty("refinesTo");
//		this.insertObjectProperty("isRefinementOf");
//		this.insertObjectProperty("hasRefinementSource");
//		this.insertObjectProperty("hasRefinementTarget");
		this.removeUnconnectedIndividuals();
		
		// before creating a delegate forest you have to transform undirected edges into directed edges and generate a directed ordered sparse multigraph
		//return new DelegateForest<IndividualVertex,ObjectPropertyEdge>((DirectedOrderedSparseMultigraph<IndividualVertex,ObjectPropertyEdge>) this.getGoalModelGraph());
	}
	// end creatsRefinementForest
		
////////// end getting ontology details	
}
// end class AbstractROModel