package org.deft.requirementsontology;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import edu.uci.ics.jung.algorithms.layout.KKLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.algorithms.layout.TreeLayout;
import edu.uci.ics.jung.algorithms.layout.util.RandomLocationTransformer;
import edu.uci.ics.jung.graph.Forest;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.decorators.AbstractEdgeShapeTransformer;
import edu.uci.ics.jung.visualization.renderers.Renderer.VertexLabel.Position;

public class GoalModelVisualizer
{
	/**
	 * contains the requirements ontology model
	 */
	private AbstractROModel	roModel;

////////// constructors
	
	/**
	 * creates a visualizer for a goal model graph of the requirements ontology model (to a given requirements ontology model)
	 */
	public GoalModelVisualizer(AbstractROModel roModel)
	{
		this.roModel = roModel;
	}
	// end constructor
	
////////// end constructors

////////// visualizing the goal model graph
	
	/**
	 * creates an image of the goal model graph (to a given file name, graphic format, graphic size, layout type)
	 * available graphic formats -> ImageIO.getWriterFormatNames() 
	 */
	public void createImage(String fileName, String graphicFormat, Dimension graphicSize, VisualizationLayoutType layoutType)
	{
		VisualizationViewer<IndividualVertex,ObjectPropertyEdge> visualizationComponent = this.getVisualizationComponent(graphicSize,layoutType);
		BufferedImage image = this.getImage(visualizationComponent);
		
		try
		{
			boolean graphicFormatAvailable = false;
			
			for (String formatName : ImageIO.getWriterFormatNames())
			{
				if (formatName.equals(graphicFormat))
				{
					graphicFormatAvailable = true;
				}
				// end if
			}
			// end for
			
			if (graphicFormatAvailable == true)
			{
				ImageIO.write(image,graphicFormat,new File(fileName));
				
//				OutputStream outputStream = new FileOutPutStream(new File(fileName));
//				ImageIO.wirte(image,graphicFormat,outputStream);
//				outputStream.flush();
//				outputStream.close();
			}
			else
			{
				System.out.println("graphic format not available");
			}
			// end if else
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		// end try catch	
	}
	// end createImage
	
	/**
	 * returns a buffered image of the goal model graph (to a given graphic size, layout type)
	 */
	public BufferedImage getImage(Dimension graphicSize, VisualizationLayoutType layoutType)
	{
		VisualizationViewer<IndividualVertex,ObjectPropertyEdge> visualizationComponent = this.getVisualizationComponent(graphicSize,layoutType);
		return this.getImage(visualizationComponent);
	}
	// end getImage
	
	/**
	 * creates a JFrame that contains the goal model graph (to a given frame size)
	 */
	public void createJFrame(Dimension frameSize, VisualizationLayoutType layoutType)
	{
		// visualization component
		VisualizationViewer<IndividualVertex,ObjectPropertyEdge> visualizationComponent = this.getVisualizationComponent(frameSize,layoutType);
		visualizationComponent.setGraphMouse(new DefaultModalGraphMouse<IndividualVertex,ObjectPropertyEdge>());

		// frame
		JFrame frame = new JFrame("Requirements Ontology Goal Model Graph");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(visualizationComponent);
		
		frame.pack();
		frame.setVisible(true);
	}
	// end createJFrame

	/**
	 * returns an rgb image of the goal model graph (to a given visualization component)
	 */
	private BufferedImage getImage(VisualizationViewer<IndividualVertex,ObjectPropertyEdge> visualizationComponent)
	{
		int width = visualizationComponent.getPreferredSize().width;
		int height = visualizationComponent.getPreferredSize().height;
		BufferedImage image = new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);
		Graphics2D graphic = image.createGraphics();
		SwingUtilities.paintComponent(graphic, visualizationComponent, new JPanel(),0,0,width,height);
		
//		visualizationComponent.paint(graphic);
//		graphic.dispose();

		return image;
	}
	// end getImage
	
	/**
	 * returns a visualization component that contains the layout of the goal model graph (to a given component size and a given visualization layout type)
	 * supports a standard tree layout (VisualizationLayoutType.TREE) -> creates refinement forrest out of the goal model graph
	 * supports a standard graph layout (VisualizationLayoutType.GRAPH)
	 */
	private VisualizationViewer<IndividualVertex,ObjectPropertyEdge> getVisualizationComponent(Dimension componentSize, VisualizationLayoutType layoutType)
	{
		Layout<IndividualVertex,ObjectPropertyEdge> layout = null;
		Dimension layoutSize = new Dimension(componentSize.width,componentSize.height);
//		Dimension layoutSize = new Dimension(componentSize.width-200,componentSize.height-300);
		
		if ((this.roModel.getGoalModelGraph() instanceof edu.uci.ics.jung.graph.OrderedSparseMultigraph) &&
				(layoutType.equals(VisualizationLayoutType.TREE)))
		{
			throw new IllegalArgumentException("requirements ontology model does not support tree layout visualization");
		}	
		else if ((this.roModel.getGoalModelGraph() instanceof edu.uci.ics.jung.graph.DirectedOrderedSparseMultigraph) &&
				(layoutType.equals(VisualizationLayoutType.TREE)))
		{
			try
			{
				// layout
				layout = new TreeLayout<IndividualVertex,ObjectPropertyEdge>((Forest<IndividualVertex, ObjectPropertyEdge>) ((DirectedROModel) this.roModel).getGoalModelForest(),200,150);
			}
			catch (Exception e)
			{
				throw new IllegalArgumentException("requirements ontology model does not support tree layout visualization");
			}
			// end try catch
		}		
		else
		{
			// layout
			layout = new KKLayout<IndividualVertex,ObjectPropertyEdge>(this.roModel.getGoalModelGraph());
			
			layout.setSize(new Dimension(layoutSize.width-500,layoutSize.height-500));
			
			// vertex positions
			((KKLayout<IndividualVertex,ObjectPropertyEdge>) layout).setLengthFactor(1.8);
			layout.setInitializer(new RandomLocationTransformer<IndividualVertex>(layoutSize,0));
			
//			for (IndividualVertex vertex : roModel.getGoalModelGraph().getVertices())
//			{
//				if (vertex.getSimpleName().equals("ConnectPeople"))
//				{
//					((FRLayout<IndividualVertex,ObjectPropertyEdge>)layout).setLocation(vertex,500,500);
//				}
//				// end if
//			}
//			//end for
//
//			VertexPositionTransformer<IndividualVertex,Point2D> vertexPositions = new VertexPositionTransformer<IndividualVertex,Point2D>(roModel,layout.getSize());
//			layout.setInitializer(vertexPositions);
		}
		// end if else if
		
		// visualization component
		VisualizationViewer<IndividualVertex,ObjectPropertyEdge> visualizationComponent = new VisualizationViewer<IndividualVertex,ObjectPropertyEdge>(layout);
		visualizationComponent.setPreferredSize(componentSize);
		
		// vertices
		visualizationComponent.getRenderContext().setVertexFillPaintTransformer(new VertexFillPaintTransformer<IndividualVertex,Paint>());
		visualizationComponent.getRenderContext().setVertexLabelTransformer(new VertexLabelTransformer<IndividualVertex,String>());
		visualizationComponent.getRenderContext().setVertexFontTransformer(new VertexFontTransformer<IndividualVertex,Font>());
		visualizationComponent.getRenderContext().setVertexShapeTransformer(new VertexShapeTransformer<IndividualVertex,Shape>());
		visualizationComponent.getRenderer().getVertexLabelRenderer().setPosition(Position.E);
		
		// edges
		visualizationComponent.getRenderContext().setEdgeFontTransformer(new EdgeFontTransformer<ObjectPropertyEdge,Font>());
		visualizationComponent.getRenderContext().setEdgeStrokeTransformer(new EdgeStrokeTransformer<ObjectPropertyEdge,Stroke>());
		visualizationComponent.getRenderContext().setEdgeLabelTransformer(new EdgeLabelTransformer<ObjectPropertyEdge,String>());
		visualizationComponent.getRenderContext().getEdgeLabelRenderer().setRotateEdgeLabels(false);
		visualizationComponent.getRenderContext().setLabelOffset(5);  // default = 10		
		AbstractEdgeShapeTransformer<IndividualVertex,ObjectPropertyEdge> edgeShapeTransformer = (AbstractEdgeShapeTransformer<IndividualVertex,ObjectPropertyEdge>) visualizationComponent.getRenderContext().getEdgeShapeTransformer();
		edgeShapeTransformer.setControlOffsetIncrement(20);
		
		// background
		visualizationComponent.setBackground(Color.WHITE);
		
		return visualizationComponent;
	}
	// end createVisualizationComponent

////////// end visualizing the goal model graph
}
// end class GoalModelVisualizer