package org.deft.requirementsontology;

import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.collections15.multimap.MultiHashMap;

public class IndividualVertex
{
	/**
	 * contains the simple name of the individual
	 */
	private String simpleName;
	/**
	 * contains the essential type of the individual
	 * essential type is the class of the individual
	 */
	private String essentialType;
	/**
	 * contains the main type of the individual
	 * main type could be one of the classes "Problem", "Challenge", "Goal", "Requirement", "Story", "SoftMetric", "TestCase", "Exception", "Refinement", "Source" or "Stakeholder"
	 */
	private String mainType;
	/**
	 * contains all comment annotations for the individual
	 */
	private Set<String> comments;
	/**
	 * contains the attributes of the individual
	 */
	private Set<String> attributes;
	/**
	 * contains all formal defined attributes of the essential type class of the individual
	 * formal attribute == object/data-some-restriction of the class
	 * does not contain the target individual (object property) or the data value (data property)
	 * <object/data property of the object/data-some-restriction, target type of the object/data-some-restriction> 
	 */
	private MultiHashMap<String,String> formalAttributes;
	/**
	 * contains all formal defined attributes of the individual (and the target individuals/values if available)
	 * <object/data property of the object/data-some-restriction - target type of the object/data-some-restriction, target individual/value>
	 */
	private MultiHashMap<String,String> formalAttributesWithContent;
	/**
	 * contains all object properties of the individual that do not belong to a formal attribute (object-some-restriction) of the individual
	 * (does not contain the target individuals)
	 * <object property, range class of the object property>
	 */
	private MultiHashMap<String,String> additionalObjectProperties;
	/**
	 * contains all object properties (and target individuals) of the individual that do not belong to a formal attribute (object-some-restriction) of the individual
	 * <object property - range class of the object property, target individuals>  
	 */
	private MultiHashMap<String,String> additionalObjectPropertiesWithContent;
	/**
	 * contains all data properties of the individual that do not belong to a formal attribute (data-some-restriction) of the individual
	 * (does not contain the target data values)
	 * <data property, range class of the data property>
	 */
	private MultiHashMap<String,String> additionalDataProperties;
	/**
	 * contains all data properties (and target data values) of the individual that do not belong to a formal attribute (data-some-restriction) of the individual
	 * <data property - range class of the data property, data values>
	 */
	private MultiHashMap<String,String> additionalDataPropertiesWithContent;
	/**
	 * contains all incoming object properties of the individual
	 * (does not contain the source individual)
	 * <object property, essential type class of the source individual>
	 */
	private MultiHashMap<String,String> incomingObjectProperties;
	/**
	 * contains all object properties (and source individuals) of the individual
	 * <object property - essential type class of the source individual, source individuals>
	 */
	private MultiHashMap<String,String> incomingObjectPropertiesWithContent;
	
	
////////// constructors	
	
	/**
	 * creates an individual vertex
	 */
	public IndividualVertex(String simpleIndividualName, String essentialType, String mainType, Set<String> comments, Set<String> attributes, MultiHashMap<String,String> formalAttributes, MultiHashMap<String,String> formalAttributesWithContent, MultiHashMap<String,String> additionalObjectProperties, MultiHashMap<String,String> additionalObjectPropertiesWithContent, MultiHashMap<String,String> additionalDataProperties, MultiHashMap<String,String> additionalDataPropertiesWithContent, MultiHashMap<String,String> incomingObjectProperties, MultiHashMap<String,String> incomingObjectPropertiesWithContent)
	{
		this.simpleName = simpleIndividualName;
		this.essentialType = essentialType;
		this.mainType = mainType;
		this.comments = comments;
		this.attributes = attributes;
		this.formalAttributes = formalAttributes;
		this.formalAttributesWithContent = formalAttributesWithContent;
		this.additionalObjectProperties = additionalObjectProperties;
		this.additionalObjectPropertiesWithContent = additionalObjectPropertiesWithContent;
		this.additionalDataProperties = additionalDataProperties;
		this.additionalDataPropertiesWithContent = additionalDataPropertiesWithContent;
		this.incomingObjectProperties = incomingObjectProperties;
		this.incomingObjectPropertiesWithContent = incomingObjectPropertiesWithContent;
	}
	// end constructor

////////// end constructors

////////// accessing the individual vertex convenient	

	/**
	 * returns the simple name of the individual
	 */
	public String getSimpleName()
	{
		return this.simpleName;
	}
	// end getSimpleName

	/**
	 * returns the essential type of the individual
	 */
	public String getEssentialType()
	{
		return this.essentialType;
	}
	// end getEssentialType
	
	/**
	 * returns the main type of the individual
	 */
	public String getMainType()
	{
		return this.mainType;
	}
	// end getMainType
	
	/**
	 * returns the comments for the individual
	 */
	public Set<String> getComments()
	{
		return this.comments;
	}
	// end getComments

	/**
	 * returns the attributes of the individual
	 */
	public Set<String> getAttributes()
	{
		return this.attributes;
	}
	// end getAttributes
	
	/**
	 * returns the names of the formal defined attributes of the individual
	 */
	public Set<String> getFormalAttributeNames()
	{
		return this.formalAttributes.keySet();
	}
	// end getFormalAttributeNames
	
	/**
	 * returns the target types of a formal defined attributes of the individual (to a given formal attribute name)
	 */
	public Set<String> getFormalAttributeTypes(String formalAttributeName)
	{
		Set<String> formalAttributeTypes = new TreeSet<String>();
		
		if (this.formalAttributes.get(formalAttributeName) != null)
		{
			for (String formalAttributeType : this.formalAttributes.get(formalAttributeName))
			{
				formalAttributeTypes.add(formalAttributeType);
			}
			// end for
			
			return formalAttributeTypes;
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getFormalAttributeType
	
	/**
	 * returns the target individuals of a formal defined attributes of the individual (to a given formal attribute name and a given formal attribute target type)
	 */
	public Set<String> getFormalAttributeIndividuals(String formalAttributeName, String formalAttributeType)
	{
		Set<String> formalAttributeIndividuals = new TreeSet<String>();
		
		if (this.formalAttributesWithContent.get(formalAttributeName + "-" + formalAttributeType) != null)
		{
			for (String formalAttributeIndividual : this.formalAttributesWithContent.get(formalAttributeName + "-" + formalAttributeType))
			{
				formalAttributeIndividuals.add(formalAttributeIndividual);
			}
			// end for
			
			return formalAttributeIndividuals;	
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getFormalAttributeIndividuals
	
	/**
	 * returns the names of the additional object properties of the individual
	 */
	public Set<String> getAdditionalObjectPropertyNames()
	{
		return this.additionalObjectProperties.keySet();
	}
	// end getAdditionalObjectPropertyNames
	
	/**
	 * returns the target type of an additional object property of the individual (to a given additional object property name)
	 */
	public String getAdditionalObjectPropertyType(String additionalObjectPropertyName)
	{
		Set<String> additionalObjectPropertyTypes = new TreeSet<String>();
		
		if (this.additionalObjectProperties.get(additionalObjectPropertyName) != null)
		{
			for (String additionalObjectPropertyType : this.additionalObjectProperties.get(additionalObjectPropertyName))
			{
				additionalObjectPropertyTypes.add(additionalObjectPropertyType);
			}
			// end for

			return additionalObjectPropertyTypes.toArray()[0].toString();	
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getAdditionalObjectPropertyTypes
	
	/**
	 * returns the target individuals of an additional object property of the individual (to a given additional object property name and a given additional object property target type)
	 */
	public Set<String> getAdditionalObjectPropertyIndividuals(String additionalObjectPropertyName, String additonalObjectPropertyType)
	{
		Set<String> additionalObjectPropertyIndividuals = new TreeSet<String>();
		
		if (this.additionalObjectPropertiesWithContent.get(additionalObjectPropertyName + "-" + additonalObjectPropertyType) != null)
		{
			for (String additionalObjectPropertyIndividual : this.additionalObjectPropertiesWithContent.get(additionalObjectPropertyName + "-" + additonalObjectPropertyType))
			{
				additionalObjectPropertyIndividuals.add(additionalObjectPropertyIndividual);
			}
			// end for
			
			return additionalObjectPropertyIndividuals;
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getAdditonalObjectPropertIndividuals

	/**
	 * returns the names of the additional data properties of the individual
	 */
	public Set<String> getAdditionalDataPropertyNames()
	{
		return this.additionalDataProperties.keySet();
	}
	// end getAdditionalDataPropertyNames
	
	/**
	 * returns the target type of an additional data property of the individual (to a given additional data property name)
	 */
	public String getAdditionalDataPropertyType(String additionalDataPropertyName)
	{
		Set<String> additionalDataPropertyTypes = new TreeSet<String>();
		
		if (this.additionalDataProperties.get(additionalDataPropertyName) != null)
		{
			for (String additionalDataPropertyType : this.additionalDataProperties.get(additionalDataPropertyName))
			{
				additionalDataPropertyTypes.add(additionalDataPropertyType);
			}
			// end for

			return additionalDataPropertyTypes.toArray()[0].toString();	
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getAdditionalDataPropertyType
	
	/**
	 * returns the values of an additional data property of the individual (to a given additional data property name and a given additional data property target type)
	 */
	public Set<String> getAdditionalDataPropertyValues(String additionalDataPropertyName, String additonalDataPropertyType)
	{
		Set<String> additionalDataPropertyValues = new TreeSet<String>();
		
		if (this.additionalDataPropertiesWithContent.get(additionalDataPropertyName + "-" + additonalDataPropertyType) != null)
		{
			for (String additionalDataPropertyValue : this.additionalDataPropertiesWithContent.get(additionalDataPropertyName + "-" + additonalDataPropertyType))
			{
				additionalDataPropertyValues.add(additionalDataPropertyValue);
			}
			// end for
			
			return additionalDataPropertyValues;
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getAdditonalDataPropertValues
	
	/**
	 * returns the names of the incoming object properties of the individual
	 */
	public Set<String> getIncomingObjectPropertyNames()
	{
		return this.incomingObjectProperties.keySet();
	}
	// end getIncomingObjectPropertyNames	
	
	/**
	 * returns the source types of an incoming object property of the individual (to a given incoming object property name)
	 */
	public Set<String> getIncomingObjectPropertySourceIndividualTypes(String incomingObjectPropertyName)
	{
		Set<String> incomingObjectPropertySourceIndividualTypes = new TreeSet<String>();
		
		if (this.incomingObjectProperties.get(incomingObjectPropertyName) != null)
		{
			for (String incomingObjectPropertySourceIndividualType : this.incomingObjectProperties.get(incomingObjectPropertyName))
			{
				incomingObjectPropertySourceIndividualTypes.add(incomingObjectPropertySourceIndividualType);
			}
			// end for

			return incomingObjectPropertySourceIndividualTypes;	
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getIncomingObjectPropertyTypes
	
	/**
	 * returns the source individuals of an incoming object property of the individual (to a given additional object property name and a given additional object property target type)
	 */
	public Set<String> getIncomingObjectPropertySourceIndividuals(String incomingObjectPropertyName, String incomingObjectPropertyType)
	{
		Set<String> incomingObjectPropertySourceIndividuals = new TreeSet<String>();
		
		if (this.incomingObjectPropertiesWithContent.get(incomingObjectPropertyName + "-" + incomingObjectPropertyType) != null)
		{
			for (String incomingObjectPropertySourceIndividual : this.incomingObjectPropertiesWithContent.get(incomingObjectPropertyName + "-" + incomingObjectPropertyType))
			{
				incomingObjectPropertySourceIndividuals.add(incomingObjectPropertySourceIndividual);
			}
			// end for
			
			return incomingObjectPropertySourceIndividuals;
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getIncomingObjectPropertySourceIndividuals
	
////////// end accessing the individual vertex convenient
	
////////// accessing the individual vertex
	
	/**
	 * returns the formal attributes of the individual
	 */
	public MultiHashMap<String,String> getFormalAttributes()
	{
		return this.formalAttributes;
	}
	// end getFormalAttributes
	
	/**
	 * returns the formal attributes (with target individual/value if available) of the individual
	 */
	public MultiHashMap<String,String> getFormalAttributesWithContent()
	{
		return this.formalAttributesWithContent;
	}
	// end getFormalAttributesWithContent

	/**
	 * returns the object properties of the individual that do not belong to an formal attribute (object-some-restriction) of the individual
	 */
	public MultiHashMap<String,String> getAdditionalObjectProperties()
	{
		return this.additionalObjectProperties;
	}
	// end getAdditionalObjectProperties
	
	/**
	 * returns the object properties (and target individuals) of the individual that do not belong to an formal attribute (object-some-restriction) of the individual
	 */
	public MultiHashMap<String,String> getAdditionalObjectPropertiesWithContent()
	{
		return this.additionalObjectPropertiesWithContent;
	}
	// end getAdditionalObjectPropertiesWithContent

	/**
	 * returns the data properties of the individual that do not belong to an formal attribute (data-some-restriction) of the individual
	 */
	public MultiHashMap<String,String> getAdditionalDataProperties()
	{
		return this.additionalDataProperties;
	}
	// end getAdditionalDataProperties
	
	/**
	 * returns the data properties (and target data values) of the individual that do not belong to an formal attribute (data-some-restriction) of the individual
	 */
	public MultiHashMap<String,String> getAdditionalDataPropertiesWithContent()
	{
		return this.additionalDataPropertiesWithContent;
	}
	// end getAdditionalDataPropertiesWithContent
	
	/**
	 * returns the incoming object properties of the individual
	 */
	public MultiHashMap<String,String> getIncomingObjectProperties()
	{
		return this.incomingObjectProperties;
	}
	// end getIncomingObjectProperties
	
	/**
	 * returns the incoming object properties (and source individuals) of the individual  
	 */
	public MultiHashMap<String,String> getIncomingObjectPropertiesWithContent()
	{
		return this.incomingObjectPropertiesWithContent;
	}
	// end getIncomingObjectPropertiesWithContent
	
	public String toString()
	{
		return this.getSimpleName();
	}
	// end toString
	
////////// end accessing the individual vertex
}
// end class Individual