package org.deft.requirementsontology;

import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.semanticweb.owl.apibinding.OWLManager;
import org.semanticweb.owl.io.StreamInputSource;
import org.semanticweb.owl.model.OWLAnnotation;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLCommentAnnotation;
import org.semanticweb.owl.model.OWLConstant;
import org.semanticweb.owl.model.OWLDataFactory;
import org.semanticweb.owl.model.OWLDataProperty;
import org.semanticweb.owl.model.OWLDataPropertyExpression;
import org.semanticweb.owl.model.OWLDataRange;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLIndividual;
import org.semanticweb.owl.model.OWLObjectProperty;
import org.semanticweb.owl.model.OWLObjectPropertyExpression;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyCreationException;
import org.semanticweb.owl.model.OWLOntologyManager;

public class OntologyAccessor
{
	/**
	 * contains an owl manager
	 */
	private OWLOntologyManager manager;
	/**
	 * contains an owl data factory
	 */
	private OWLDataFactory factory;
	/**
	 * contains the requirements ontology
	 */
	private OWLOntology ontology;
	/**
	 * contains the file of the requirements ontology
	 */
	private File file;
	/**
	 * contains the string with the uri of the requirements ontology
	 */
	private String ontologyUriString;
	
////////// constructors
	
	/**
	 * creates an ontology accessor from a specific owl file
	 */
	public OntologyAccessor(String fileName)
	{
		this.manager = OWLManager.createOWLOntologyManager();
		this.factory = this.manager.getOWLDataFactory();
		this.loadOntologyFromFile(fileName);
	}
	// end constructor
	
	/**
	 * creates an ontology accessor from an owl ontology
	 */
	public OntologyAccessor(OWLOntology ontology)
	{
		this.manager = OWLManager.createOWLOntologyManager();
		this.factory = manager.getOWLDataFactory();
		this.loadOntology(ontology);
	}
	// end constructor
	
	/**
	 * creats an ontology accessor from an input stream
	 */
	public OntologyAccessor(InputStream inputStream)
	{
		this.manager = OWLManager.createOWLOntologyManager();
		this.factory = manager.getOWLDataFactory();
		this.loadOntologyFromStream(inputStream);
	}
	// end constructor
	
////////// end constructors

////////// loading an ontology
	
	/**
	 * loads an ontology from a specific file
	 */
	private void loadOntologyFromFile(String fileName)
	{
		try
		{
			this.file = new File(fileName);
			this.ontology = manager.loadOntologyFromPhysicalURI(file.toURI());
			this.ontologyUriString = this.ontology.getURI().toString();
		}
		catch (OWLOntologyCreationException e)
		{
			System.out.println("failed to load ontology from file " + file.toURI() + "\n" + e.getMessage());
		}
		// end try catch
	}
	// end loadOntology

	/**
	 * loads an ontology from an owl ontology
	 */
	private void loadOntology(OWLOntology ontology)
	{
		this.ontology = ontology;
		this.ontologyUriString = ontology.getURI().toString();
	}
	// end loadOntology
	
	/**
	 * loads an ontology from an input stream
	 */
	private void loadOntologyFromStream(InputStream stream)
	{
		try
		{
			StreamInputSource source = new StreamInputSource(stream);
			this.ontology = manager.loadOntology(source);
			this.ontologyUriString = ontology.getURI().toString();
		}
		catch (OWLOntologyCreationException e)
		{
			System.out.println("failed to load ontology from input stream\n" + e.getMessage());
		}
		// end try catch		
	}
	// end loadOntologyFromStream
	
////////// end loading an ontology
	
////////// accessing the ontology
	
	/**
	 * returns the absolute path of the file of the ontology file
	 * returns null if the ontology was not created from a file
	 */
	public String getFilePath()
	{
		return this.file.getAbsolutePath();
	}
	// end getFilename
	
	/**
	 * returns the name of the file of the ontology
	 * returns null if the ontology was not loaded from a file
	 */
	public String getFileName()
	{
		return this.file.getPath();
	}
	// end getFileName
	
	/**
	 * returns the loaded ontology
	 */
	public OWLOntology getOwlOntology()
	{
		return this.ontology;
	}
	// end getOwlOntology
	
	/**
	 * returns the URI to a given simple class/property/individual name
	 */
	public URI getUri(String simpleClassOrPropertyOrIndividualName)
	{
		URI uri = URI.create(this.ontologyUriString + "#" + simpleClassOrPropertyOrIndividualName);
		return uri;
	}
	// end getUri
	
	/**
	 * returns the URI string to a given simple class/property/individual name
	 */
	public String getUriString(String simpleClassOrPropertyOrIndividualName)
	{
		URI uri = this.getUri(simpleClassOrPropertyOrIndividualName);
		return uri.toString();
	}
	// end getUriString
	
	/**
	 * returns the simple name to a given URI string
	 */
	public String getSimpleName(String uriString)
	{
		int positionOfChar = uriString.indexOf('#');
		String simpleName = uriString.substring(positionOfChar + 1);
		return simpleName;
	}
	// end getSimpleName

	/**
	 * returns the owl class to a given simple class name
	 * creates a new owl class if it does not exists in the ontology
	 */
	public OWLClass getOwlClass(String simpleClassName)
	{
		URI classUri = this.getUri(simpleClassName);
		OWLClass owlClass = factory.getOWLClass(classUri);
		return owlClass;
	}
	// end getOwlClass

	/**
	 * returns the owl individual to a given simple individual name
	 * creates a new owl individual if it does not exists in the ontology
	 */
	public OWLIndividual getOwlIndividual(String simpleIndividualName)
	{
		URI individualUri = this.getUri(simpleIndividualName);
		OWLIndividual individual = factory.getOWLIndividual(individualUri);
		return individual;
	}
	// end getOwlIndividual
	
	/**
	 * returns the owl object property to a given simple object property name
	 * creates a new owl object property if it does not exists in the ontology
	 */
	public OWLObjectProperty getOwlObjectProperty(String simpleObjectPropertyName)
	{
		URI objectPropertyUri = this.getUri(simpleObjectPropertyName);
		OWLObjectProperty objectProperty = factory.getOWLObjectProperty(objectPropertyUri);
		return objectProperty;
	}
	// end getOwlObjectProperty
	
	/**
	 * returns the owl data property to a given simple data property name
	 * creates a new owl data property if it does not exists in the ontology
	 */
	public OWLDataProperty getOwlDataProperty(String simpleDataPropertyName)
	{
		URI dataPropertyUri = this.getUri(simpleDataPropertyName);
		OWLDataProperty dataProperty = factory.getOWLDataProperty(dataPropertyUri);
		return dataProperty;
	}
	// end getOwlDataProperty

	/**
	 * returns a set of all individuals that belong to a specific class (to a given simple class name)
	 */
	public Set<OWLIndividual> getOwlIndividuals(String simpleClassName)
	{
		OWLClass owlClass = this.getOwlClass(simpleClassName);
		Set<OWLIndividual> individualSet = owlClass.getIndividuals(ontology);
		return individualSet;
	}
	// end getOwlIndividuals
		
	/**
	 * returns a map of all object properties that belong to a specific individual (to a given simple individual name)
	 */
	public Map<OWLObjectPropertyExpression,Set<OWLIndividual>> getOwlObjectProperties(String simpleIndividualName)
	{
		OWLIndividual individual = this.getOwlIndividual(simpleIndividualName);
		Map<OWLObjectPropertyExpression,Set<OWLIndividual>> objectPropertyMap = individual.getObjectPropertyValues(this.ontology);
		return objectPropertyMap;
	}
	// end getOwlObjectProperties
	
	/**
	 * returns a map of all data properties that belong to a specific individual (to a given simple individual name)
	 */
	public Map<OWLDataPropertyExpression,Set<OWLConstant>> getOwlDataProperties(String simpleIndividualName)
	{
		OWLIndividual individual = this.getOwlIndividual(simpleIndividualName);
		Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap = individual.getDataPropertyValues(ontology);
		return dataPropertyMap;
	}
	// end getOwlDataProperties

	/**
	 * returns the set of class types of a specific individual (to a given simple individual name)
	 */
	public Set<OWLDescription> getOwlTypes(String simpleIndividualName)
	{
		OWLIndividual individual = this.getOwlIndividual(simpleIndividualName);
		Set<OWLDescription> typeSet = individual.getTypes(ontology);
		return typeSet;
	}
	// end getOwlTypes
	
	/**
	 * returns true if an individual is of a specific class type (to the given simple individual/class name)
	 */
	public boolean isOwlType(String simpleIndividualName, String simpleClassName)
	{
		Set<OWLDescription> typeSet = this.getOwlTypes(simpleIndividualName);
		boolean isType = false;
		
		for (OWLDescription type : typeSet)
		{
			if (type.toString().equals(simpleClassName) == true)
			{
				isType = true;
			}
			// end if
		}
		// end for
		
		return isType;
	}
	// end isOwlType
	
	/**
	 * returns true if an object property is symmetric (to a given simple object property name)
	 * if the object property does not exists the return value would be false 
	 */
	public boolean isSymmetricObjectProperty(String simpleObjectPropertyName)
	{
		boolean isSymmetric = false;
		Set<OWLObjectProperty> objectPropertySet = this.ontology.getReferencedObjectProperties();
		
		for (OWLObjectProperty objectProperty : objectPropertySet)
		{
			if (objectProperty.toString().equals(simpleObjectPropertyName))
			{
				isSymmetric = objectProperty.isSymmetric(this.ontology);
			}
			// end if
		}
		// end for
		
		return isSymmetric;
	}
	// end isSymmetricObjectProperty
		
	/**
	 * returns a set of all subclasses of a specific class (subsuming all subclasses)
	 * (to a given simple class name)
	 */
	public Set<OWLDescription> getAllOwlSubclasses(String simpleClassName)
	{
		Set<OWLDescription> subclassSet = this.getOwlSubclasses(simpleClassName);
		
		if (subclassSet.isEmpty() == false)
		{
			TreeSet<OWLDescription> tempSubclassNameSet = new TreeSet<OWLDescription>();
			
			for (OWLDescription classDescription : subclassSet)
			{
				tempSubclassNameSet.add(classDescription);
			}
			// end for
			
			for (OWLDescription classDescription : tempSubclassNameSet)
			{
				subclassSet.addAll(this.getAllOwlSubclasses(classDescription.toString()));
			}
			// end for
		}
		// end if
		
		return subclassSet;
	}
	// end getAllOwlSubclasses

	/**
	 * returns a set of the direct subclasses of a specific class (to a given simple class name)
	 */
	public Set<OWLDescription> getOwlSubclasses(String simpleClassName)
	{
		OWLClass owlClass = this.getOwlClass(simpleClassName);
		Set<OWLDescription> subclassSet = owlClass.getSubClasses(this.ontology);
		return subclassSet;
	}
	// end getOwlSubclasses
	
	/**
	 * returns a set of all superclasses of a specific class (subsuming all superclasses)
	 * (to a given simple class name)
	 */
	public Set<OWLDescription> getAllOwlSuperclasses(String simpleClassName)
	{
		Set<OWLDescription> superclassSet = this.getOwlSuperclasses(simpleClassName);
		
		if (superclassSet.isEmpty() == false)
		{
			TreeSet<OWLDescription> tempSuperclassNameSet = new TreeSet<OWLDescription>();
			
			for (OWLDescription classDescription : superclassSet)
			{
				tempSuperclassNameSet.add(classDescription);
			}
			// end for
			
			for (OWLDescription classDescription : tempSuperclassNameSet)
			{
				if ((classDescription instanceof uk.ac.manchester.cs.owl.OWLClassImpl) == true)
				{
					superclassSet.addAll(this.getAllOwlSuperclasses(classDescription.toString()));
				}
				// end if
			}
			// end for
		}
		// end if
		
		return superclassSet;
	}
	// end getAllOwlSuperclasses

	/**
	 * returns a set of the direct superclasses of a specific class (to a given simple class name)
	 */
	public Set<OWLDescription> getOwlSuperclasses(String simpleClassName)
	{
		OWLClass owlClass = this.getOwlClass(simpleClassName);
		Set<OWLDescription> superclassSet = owlClass.getSuperClasses(this.ontology);
		return superclassSet;
	}
	// end getOwlSuperclasses
	
	/**
	 * returns a set of all object/data-some-restrictions of a specific class (to a given simple class name)
	 */
	public Set<OWLDescription> getOwlSomeRestrictions(String simpleClassName)
	{
		Set<OWLDescription> someRestrictionSet = this.getAllOwlSuperclasses(simpleClassName);
	
		if (someRestrictionSet.isEmpty() == false)
		{
			TreeSet<OWLDescription> tempObjectSomeRestrictionSet = new TreeSet<OWLDescription>();
			
			for (OWLDescription classDescription : someRestrictionSet)
			{
				tempObjectSomeRestrictionSet.add(classDescription);
			}
			// end for
			
			for (OWLDescription classDescription : tempObjectSomeRestrictionSet)
			{
				if ((classDescription instanceof uk.ac.manchester.cs.owl.OWLObjectSomeRestrictionImpl) == false &&
					(classDescription instanceof uk.ac.manchester.cs.owl.OWLDataSomeRestrictionImpl) == false)
				{
					someRestrictionSet.remove(classDescription);
				}
				// end if
			}
			// end if
		}
		// end if
		
		return someRestrictionSet;
	}
	// end getOwlSomeRestrictions
	
	/**
	 * returns a set of all object/data-some-restrictions of a specific class (subsuming all some-restrictions of all leaf classes)
	 * (to a given simple class name)
	 */
	public Set<OWLDescription> getAllOwlSomeRestrictions(String simpleClassName)
	{
		Set<OWLDescription> subclassNameSet = this.getAllOwlSubclasses(simpleClassName);
		Set<OWLDescription> someRestrictionSet = new TreeSet<OWLDescription>();
		
		if (subclassNameSet.isEmpty() == true)
		{
			someRestrictionSet = this.getOwlSomeRestrictions(simpleClassName);
		}
		else
		{
			for (OWLDescription classDescription : subclassNameSet)
			{
				someRestrictionSet.addAll(this.getOwlSomeRestrictions(classDescription.toString()));
			}
			// end for
		}
		// end if else

		return someRestrictionSet;
	}
	// end getAllOwlSomeRestrictions
	
	/**
	 * returns a set of all individuals that belongs to a specific class or one of its subclasses (to a given simple class name)
	 */
	public Set<OWLIndividual> getAllOwlIndividuals(String simpleClassName)
	{
		Set<OWLDescription> classNameSet = this.getAllOwlSubclasses(simpleClassName);
		classNameSet.add(this.getOwlClass(simpleClassName));
		Set<OWLIndividual> individualSet = new TreeSet<OWLIndividual>();
		
		for (OWLDescription className : classNameSet)
		{
			individualSet.addAll(this.getOwlIndividuals(className.toString()));
		}
		// end for
		
		return individualSet;
	}
	// end getAllOwlIndividuals

	/**
	 * returns a set of the domain classes of an object property (to a given object property name)
	 */
	public Set<OWLDescription> getOwlObjectPropertyDomains(String simpleObjectPropertyName)
	{
		OWLObjectProperty objectProperty = this.getOwlObjectProperty(simpleObjectPropertyName);
		Set<OWLDescription> domainClassSet = objectProperty.getDomains(this.ontology);
		return domainClassSet;
	}
	// end getOwlObjectPropertyDomains
	
	/**
	 * returns just one domain class of an object property (to a given object property name)
	 */
	public OWLDescription getOwlObjectPropertyDomain(String simpleObjectPropertyName)
	{
		OWLObjectProperty objectProperty = this.getOwlObjectProperty(simpleObjectPropertyName);
		Set<OWLDescription> domainClassSet = objectProperty.getDomains(this.ontology);
		
		if (domainClassSet.toArray().length > 0)
		{
			return (OWLDescription) domainClassSet.toArray()[0];
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getOwlObjectPropertyDomain

	/**
	 * returns a set of the range classes of an object property (to a given object property name)
	 */
	public Set<OWLDescription> getOwlObjectPropertyRanges(String simpleObjectPropertyName)
	{
		OWLObjectProperty objectProperty = this.getOwlObjectProperty(simpleObjectPropertyName);
		Set<OWLDescription> rangeClassSet = objectProperty.getRanges(this.ontology);
		return rangeClassSet;
	}
	// end getOwlObjectPropertyRanges
	
	/**
	 * returns just one range class of an object property (to a given object property name)
	 */
	public OWLDescription getOwlObjectPropertyRange(String simpleObjectPropertyName)
	{
		OWLObjectProperty objectProperty = this.getOwlObjectProperty(simpleObjectPropertyName);
		Set<OWLDescription> rangeClassSet = objectProperty.getRanges(this.ontology);
		
		if (rangeClassSet.toArray().length > 0)
		{
			return (OWLDescription) rangeClassSet.toArray()[0];
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getOwlObjectPropertyRange
	
	/**
	 * returns a set of the domain classes of a data property (to a given data property name)
	 */
	public Set<OWLDescription> getOwlDataPropertyDomains(String simpleDataPropertyName)
	{
		OWLDataProperty dataProperty = this.getOwlDataProperty(simpleDataPropertyName);
		Set<OWLDescription> domainClassSet = dataProperty.getDomains(this.ontology);
		return domainClassSet;
	}
	// end getOwlDataPropertyDomains	

	/**
	 * returns just one domain class of a data property (to a given data property name)
	 */
	public OWLDescription getOwlDataPropertyDomain(String simpleDataPropertyName)
	{
		OWLDataProperty dataProperty = this.getOwlDataProperty(simpleDataPropertyName);
		Set<OWLDescription> domainClassSet = dataProperty.getDomains(this.ontology);
		
		if (domainClassSet.toArray().length > 0)
		{
			return (OWLDescription) domainClassSet.toArray()[0];
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getOwlDataPropertyDomains
	
	/**
	 * returns a set of the range datatypes of a data property (to a given data property name)
	 */
	public Set<OWLDataRange> getOwlDataPropertyRanges(String simpleDataPropertyName)
	{
		OWLDataProperty dataProperty = this.getOwlDataProperty(simpleDataPropertyName);
		Set<OWLDataRange> rangeDataTypeSet = dataProperty.getRanges(this.ontology);
		return rangeDataTypeSet;
	}
	// end getOwlDataPropertyRanges
	
	/**
	 * returns just one range datatype of a data property (to a given data property name)
	 */
	public OWLDataRange getOwlDataPropertyRange(String simpleDataPropertyName)
	{
		OWLDataProperty dataProperty = this.getOwlDataProperty(simpleDataPropertyName);
		Set<OWLDataRange> rangeDataTypeSet = dataProperty.getRanges(this.ontology);
		
		if (rangeDataTypeSet.toArray().length > 0)
		{
			return (OWLDataRange) rangeDataTypeSet.toArray()[0];
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getOwlDataPropertyRange
	
	/**
	 * returns a set of all object properties that are defined (TBox) in the requirements ontology
	 */
	public Set<OWLObjectProperty> getOwlObjectProperties()
	{
		return this.ontology.getReferencedObjectProperties();
	}
	// end getOwlObjectProperties
	
	/**
	 * returns a set of all data properties that are defined (TBox) in the requirements ontology
	 */
	public Set<OWLDataProperty> getOwlDataProperties()
	{
		return this.ontology.getReferencedDataProperties();
	}
	// end getOwlDataProperties
	
	/**
	 * returns a set of all classes that are defined (TBox) in the requirements ontology (without "Thing")
	 */
	public Set<OWLClass> getOwlClasses()
	{
		Set<OWLClass> wantedClasses = new TreeSet<OWLClass>();
		Set<OWLClass> referencedClassesSet = this.ontology.getReferencedClasses();
		
		for (OWLClass referencedClass : referencedClassesSet)
		{
			if (referencedClass.isOWLThing() == false)
			{
				wantedClasses.add(referencedClass);
			}
			// end if
		}
		// end for
		
		return wantedClasses;
	}
	// end getOwlClasses
	
	/**
	 * returns a set of all root classes (classes without superclasses and without equivalent classes)
	 */
	public Set<OWLClass> getOwlRootClasses()
	{
		Set<OWLClass> referencedClassSet = this.getOwlClasses();
		Set<OWLClass> wantedClassSet = new TreeSet<OWLClass>();
		
		for (OWLClass referencedClass : referencedClassSet)
		{
			if ((this.hasOwlSuperclass(referencedClass) == false) &&
				(referencedClass.getEquivalentClasses(ontology).isEmpty() == true))
			{
				wantedClassSet.add(referencedClass);
			}
			// end if
		}
		// end for
		
		return wantedClassSet;
	}
	// end getAllOwlRootClasses
	
	/**
	 * returns a set of all comment annotations that belongs to a specific individual (to a given simple individual name)
	 */
	@SuppressWarnings("rawtypes")
	public Set<OWLCommentAnnotation> getOwlCommentAnnotations(String simpleIndividualName)
	{
		OWLIndividual individual = this.getOwlIndividual(simpleIndividualName);
		Set<OWLAnnotation> annotationSet = individual.getAnnotations(this.ontology);
		Set<OWLCommentAnnotation> commentAnnotations = new TreeSet<OWLCommentAnnotation>();
		
		for (OWLAnnotation annotation : annotationSet)
		{
			if ((annotation instanceof uk.ac.manchester.cs.owl.OWLCommentAnnotationImpl) == true)
			{
				commentAnnotations.add((OWLCommentAnnotation) annotation);
			}
			// end if
		}
		// end for
		
		return commentAnnotations;
	}
	// end getOwlCommentAnnotations
	
	/**
	 * returns true if a class has a superclass that is not an object/data-some-restriction (to a given owl class)
	 */
	private boolean hasOwlSuperclass(OWLClass owlClass)
	{
		Set<OWLDescription> descriptionSet = owlClass.getSuperClasses(ontology);
		
		for (OWLDescription description : descriptionSet)
		{
			if (description instanceof uk.ac.manchester.cs.owl.OWLClassImpl)
			{
				return true;
			}
			// end if
		}
		// end for
		
		return false;
	}
	// end hasOwlSuperclass
	
////////// end accessing the ontology
}
// end class OntologyAccessor