package org.deft.resource.impl;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;

import org.deft.resource.ResourceHandler;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

public class SimpleJavaResourceHandler implements ResourceHandler {

	protected IPath resourceDirectory;
	private String resourceHandlerId;
	
	public SimpleJavaResourceHandler(String resourceHandlerId) {
		this.resourceHandlerId = resourceHandlerId;
		IPath locationOfThisClass = new Path(computeDiskLocation()); //the parent folder of this class
		IPath projectRoot = locationOfThisClass.removeLastSegments(5); //this must be updated if this class is moved
		resourceDirectory = projectRoot.append(resourceHandlerId);
	}


	
	public URL getResourceUrl(String resource) {
        String location = resourceDirectory.append(resource).toOSString();
		try {
			URL url = new File(location).toURI().toURL();
			return url;
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
        return null;
    }
	

	public String getResourceHandlerId() {
		return resourceHandlerId;
	}
	
	
    protected String computeDiskLocation() {
        boolean inJar = false;
        Class<SimpleJavaResourceHandler> c = SimpleJavaResourceHandler.class;
        //change dots in class (dots are package separators such as in java.lang.String) to slashes
        //and add ".class", this results in a file name (incl. path)
        String pathName = c.getName().replace('.', '/') + ".class";
        //ask classloader for absolute path of the file
        ClassLoader loader = c.getClassLoader();
        URL url = (loader != null) ? loader.getResource(pathName) : ClassLoader.getSystemResource(pathName);
        String path = url.getPath();
        //url.getPath() returns the desired path to the JavaAppResourceHandler.class, 
        //special characters are, however, escaped
        //e.g. %20 for a space; we need to unescape the url path to be useful
        //However, before we unescape we have to apply a little "hack": 
        //Any + signs will be unescaped to spaces, therefore we escape + signs to %2b first.
        //When the string is then decoded, %2b will result in the + sign again
        path = path.replaceAll("\\+", "%2b");
		try {
			path = URLDecoder.decode(path, "UTF-8");
		} catch (UnsupportedEncodingException uee) {
			//this should never happen
		}
		//depending on the operating system we now have a String like 
		//"/C:/foo/bar/deft/JavaAppResourceHandler.class" (Windows)
		//or "/home/foo/bar/deft/JavaAppResourceHandler.class" (Unix). 
		//If JavaAppResourceHandler is run from a jar file, the path looks like
		//"file:/C:/foo/bar/deft.jar!/deft/JavaAppResourceHandler.class" or
		//"file:/home/foo/bar/deft.jar!/deft/JavaAppResourceHandler.class" respectively.
		//
		//Therefore, to compute the path of JavaAppResourceHandler, we have to take care 
		//how paths are organized by the operating system.
		//For paths on Windows we have to remove everything from the start of the current 
		//path string up to and including the first slash ("/"). 
		//On Unix systems, where the slash denotes the file system root, the leading slash must not be
		//removed. We check on whether to remove the first slash by what the default separator is.
		boolean removeFirstSlash = !File.separator.equals("/");
        path = path.substring(path.indexOf("/") + (removeFirstSlash ? 1 : 0), path.length());
        //if class is in a jar file, the path string now has a form like
        //"C:/foo/bar/deft.jar!/deft/JavaAppResourceHandler.class"
        //We need to cut off everything behind the deft.jar, that is, everything starting from "!/"
        if (path.contains("!/")) {
            path = path.substring(0, path.indexOf("!/"));
            inJar = true;
        }
        File f = new File(path);
        //remove the JavaAppResourceHandler.class (or the deft.jar) (we need the main directory only)
        f = f.getParentFile();
        //if the file was not in a jar, the deft package directory is still part of the
        //path, remove it, too
        if (!inJar) {
            f = f.getParentFile();
        }
        //we're done
        return f.getAbsolutePath();
    }	

}

