package org.deft.util;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.gmf.runtime.emf.core.resources.GMFResource;
import org.eclipse.gmf.runtime.notation.NotationPackage;
import org.eclipse.jface.viewers.TreeViewer;

public class EcoreUtil {
	
	public static Resource getDiagramResource(File file) {
		URI uri = URI.createFileURI(file.getAbsolutePath());
		Resource resource = new GMFResource(uri);
		if (!resource.isLoaded()) {
			try {
				//without registering resource loading will fail
				registerEmfResourceFactories();
				resource.load(null);
			} catch (IOException e) {
				String msg = "The given path '" + file.getAbsolutePath()
						+ "' does not lead to a valid EMF/Ecore resource.";
				throw new IllegalArgumentException(msg, e);
			}
		}
		return resource;
	}
	
	public static void registerEmfResourceFactories() throws IOException {

		if (!Platform.isRunning()) {
			/* Probably register the Ecore resource for EMF. */
			if (Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
					.get(EcorePackage.eNS_PREFIX) == null) {

				Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
						.put(EcorePackage.eNS_PREFIX,
								new EcoreResourceFactoryImpl() {
									public Resource createResource(URI uri) {
										XMIResource xmiResource = new XMIResourceImpl(
												uri);
										return xmiResource;
									}
								});
			}
			// no else.

			if (EPackage.Registry.INSTANCE.getEPackage(EcorePackage.eNS_URI) == null) {
				EPackage.Registry.INSTANCE.put(EcorePackage.eNS_PREFIX,
						EcorePackage.eINSTANCE);
			}
			// no else.

			/* Probably register the notation resource for EMF. */
			if (Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
					.get(NotationPackage.eNS_PREFIX) == null) {

				Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
						.put(NotationPackage.eNS_PREFIX,
								new XMIResourceFactoryImpl() {
									public Resource createResource(URI uri) {
										XMIResource xmiResource = new XMIResourceImpl(
												uri);
										return xmiResource;
									}
								});
			}
			// no else.

			if (EPackage.Registry.INSTANCE.getEPackage(NotationPackage.eNS_URI) == null) {
				EPackage.Registry.INSTANCE.put(NotationPackage.eNS_PREFIX,
						NotationPackage.eINSTANCE);
			}
			// no else.
		}
		// no else.
	}
	
	public static EPackage loadEcoreModel(File file) {
		try {
			registerEmfResourceFactories();
			ResourceSet resourceSet = new ResourceSetImpl();
			XMIResource resource = (XMIResource) resourceSet.getResource(
			URI.createFileURI(file.getAbsolutePath()), true);
			resource.load(Collections.EMPTY_MAP);
			EPackage ecorePackage = (EPackage)resource.getContents().get(0);
			return ecorePackage;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	public static EPackage loadEcoreModel(InputStream is) {
		try {
			registerEmfResourceFactories();
		} catch (IOException e) {
			e.printStackTrace();
		}
		URI ecoreURI = URI.createURI("/dummy.ecore");
		Resource resource = new XMIResourceImpl(ecoreURI);
		try {
			resource.unload();
			resource.load(is, Collections.EMPTY_MAP);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		EPackage ecorePackage = (EPackage)resource.getContents().get(0);
		return ecorePackage;
	}
	
	
	
	/**
	 * Loads a dynamic ecore model instance from an InputStream. In Eclipse, Dynamic Ecore models are usually
	 * saved in .xmi files. The metamodels have no generated files, they are built / loaded in memory.
	 * 
	 * @param ecorePackage the root element of the ecore model which describes the model instance to be loaded 
	 * @param is a stream containing the content of the model instance to be loaded
	 * 
	 * @return the root element of the model instance
	 */
	public static EObject loadDynamicEcoreInstance(EPackage ecorePackage, InputStream is) {

		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("xmi", 
				new XMIResourceFactoryImpl());
			
		// register package in local resource registry
		resourceSet.getPackageRegistry().put(ecorePackage.getNsURI(), ecorePackage);

		//create empty resource
		Resource resource = resourceSet.createResource(URI.createURI("/dummy.xmi"));
		resource.unload();
			
		//load stream contents into resource 
		try {
			resource.load(is, null);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
			
		return resource.getContents().get(0);
	}
	
	
	
	public static EObject loadRegisteredEcoreInstance(File file) {
		URI fileUri = URI.createFileURI(file.getAbsolutePath());
		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.createResource(fileUri);
		EObject root = resource.getContents().get(0);
		return root;
	}
	
	
	public static EObject loadRegisteredEcoreInstance(InputStream is) {
		EObject eo = EcoreUtil.loadRegisteredEcoreInstance("xmi", is);
		return eo;
	}

	public static EObject loadRegisteredEcoreInstance(String registeredExtension, String content) {
		InputStream is = new ByteArrayInputStream(content.getBytes());
		EObject eo = loadRegisteredEcoreInstance(registeredExtension, is);
		return eo;
	}
	

	public static EObject loadRegisteredEcoreInstance(String registeredExtension, InputStream is) {
		ResourceSet resourceSet = new ResourceSetImpl();
			
		//create empty resource
		Resource resource = resourceSet.createResource(URI.createURI("/dummy." + registeredExtension));
		resource.unload();
			
		//load stream contents into resource 
		try {
			resource.load(is, Collections.EMPTY_MAP);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		
		EList<EObject> contents = resource.getContents();
		return contents.get(0);
	}
	
	
	public static void setupEcoreTreeViewer(EObject modelRoot, TreeViewer treeViewer) {
		ComposedAdapterFactory adapterFactory = new ComposedAdapterFactory(ComposedAdapterFactory.Descriptor.Registry.INSTANCE);
		adapterFactory.addAdapterFactory(new ResourceItemProviderAdapterFactory());
		adapterFactory.addAdapterFactory(new EcoreItemProviderAdapterFactory());
		adapterFactory.addAdapterFactory(new ReflectiveItemProviderAdapterFactory());
		AdapterFactoryContentProvider contentProvider = new AdapterFactoryContentProvider(adapterFactory);
		treeViewer.setContentProvider(contentProvider);
		treeViewer.setLabelProvider(new AdapterFactoryLabelProvider(adapterFactory));
		treeViewer.setInput(modelRoot);
	}
	
}
