package org.deft.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.LinkedList;
import java.util.List;

public class StringUtil {
	
	/**
	 * Removes the max possible number of leading spaces from each line of a multiline 
	 * string so that the relative indenting still stays intact. 
	 * 
	 * Example:
	 * 
	 *     void x() {
	 *         int i = 0;
	 *     }
	 *     
	 * results in
	 * 
	 * void x() {
	 *     int i = 0;
	 * }
	 * 
	 * The parameter firstLineCol specifies the starting column of the first line.
	 */
	public static String multilineTrim(int firstLineCol, String string) {
		String leadingSpaces = makeSpaces(firstLineCol - 1); //col numbering starts with 1, so subtract 1
		String s = leadingSpaces + string;
		int removableSpaceCount = findMinLeadingSpacesCountInMultiLineString(s);
		String shortenedString = removeFromEveryLine(s, removableSpaceCount);
		return shortenedString;
	}
	
	
	/**
	 * Creates a String of spaces of a given length.
	 */
	public static String makeSpaces(int count) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < count; i++) {
			sb.append(" ");
		}
		return sb.toString();
	}
	
	
	/**
	 * Counts for all lines of a multiline string the leading spaces
	 * and returns the smallest found value. The is is number of spaces
	 * by which every line can safely be shortened. Empty lines are not
	 * taken into consideration, because they don't affect the number of
	 * spaces by which the lines of the multiline string can be shortened.
	 */
	public static int findMinLeadingSpacesCountInMultiLineString(String string) {
		int minCount = string.length();
		for (String line : splitIntoLines(string)) {
			if (!isEmptyLine(line)) {
				int leadingSpacesCount = getLeadingSpacesCount(line);
				minCount = Math.min(minCount, leadingSpacesCount);
			}
		}
		return minCount;
	}
	
	/**
	 * Returns true if a string consists only of whitespace.
	 * 
	 * It does this by removing all whitespace and then checking
	 * if there is anything left.
	 */
	private static boolean isEmptyLine(String string) {
		return string.trim().isEmpty();
	}
	
	/**
	 * Returns the leading spaces of a string. 
	 */
	public static int getLeadingSpacesCount(String line) {
		int count = 0;
		for (int i = 0; i < line.length(); i++) {
			if (line.charAt(i) == ' ') {
				count++;
			} else {
				return count;
			}
		}
		return count;
	}
	
	
	/**
	 * Removes a number of leading characters (which should be spaces)
	 * from every line of a multiline string. If a line has less characters
	 * than the number of chars to remove, the line is kept intact. Such
	 * lines are expected to consist only of spaces, otherwise there is a bug.
	 */
	private static String removeFromEveryLine(String string, int removeCount) {
		StringBuilder sb = new StringBuilder();
		int i = 0;
		List<String> lines = splitIntoLines(string);
		for (String line : lines) {
			String shortenedLine = "";
			if (isEmptyLine(line)) {
				shortenedLine = line;
			} else {
				shortenedLine = line.substring(removeCount);
			}
			sb.append(shortenedLine);
			if (i < lines.size() - 1) {
				sb.append("\n");
				i++;
			}
			
		}
		return sb.toString();
	}
	
	
	
	private static List<String> splitIntoLines(String string) {
		List<String> lines = new LinkedList<String>();
		StringReader sr = new StringReader(string);
		BufferedReader br = new BufferedReader(sr);
		String line = "";
		while (line != null) {
			try {
				line = br.readLine();
				if (line != null) {
					lines.add(line);
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return lines;
	}
	

}
